#! /bin/sh

#
# @(#) Tools Installer 
# 

#
# 
#
#
#
#
# ksh
#
# Liste des fonctions 
# grep '^[^ ]* ()' installer_setup.sh | sort
#
#
#
#
#
#
#


### DEBUG ###
# set -x
set -u
### FIN DEBUG ###



###########################################################################
# DisplayTrace
###########################################################################

# 
# Displays message to trace file
# 
# 
# Argument 1 : texte du message
# 
# 

DisplayTrace ()
{
    # RETURN_CODE=0
    
    echo "$1"  >> "${FILE_TRACE}"
    
    # return ${RETURN_CODE}
}


###########################################################################
# DisplayError
###########################################################################

# 
# Displays message to error trace file
# 
# 
# Argument 1 : texte du message
# 
# 
# 

DisplayError ()
{
    # RETURN_CODE=0
    
    echo "$1" >> "${FILE_TRACE_ERROR}"
    
    # return ${RETURN_CODE}
}


###########################################################################
# DisplayTraceAndError
###########################################################################

# 
# Affichage de message de trace (dans fichier de trace) et d'erreur (dans 
# fichier de trace des erreurs)
# 
# 
# Argument 1 : texte du message
# 
# 
# 

DisplayTraceAndError ()
{
    # RETURN_CODE=0
    
    echo "$1"  >> "${FILE_TRACE}"
    echo "$1"  >> "${FILE_TRACE_ERROR}"
    
    # return ${RETURN_CODE}
}


###########################################################################
# DisplayWarning
###########################################################################

# 
# Displays message to warning trace file
# 
# 
# Argument 1 : texte du message
# 
# 
# 

DisplayWarning ()
{
    # RETURN_CODE=0
    
    echo "$1" >> "${FILE_TRACE_WARNING}"
    
    # return ${RETURN_CODE}
}


###########################################################################
# DisplaySyntax
###########################################################################

# 
# 
#


DisplaySyntax ()
{
    echo "Usage:"
    echo "-h        Help"
    echo "-d path   Sets installation directory"
    echo "-v        Version information"
    echo ""

}


###########################################################################
# DisplayVersion
###########################################################################

# 
# 
#

DisplayVersion ()
{
    echo "Product Installer"
    echo "${PRODUCT_DESCRIPTION} (${PRODUCT_COMPONENT_NAME}) ${PRODUCT_VERSION}"
    echo "${PRODUCT_INSTALLER_TYPE} installation (${PRODUCT_LICENCE_TYPE}) "
    
    echo "${PRODUCT_OPERATING_SYSTEM} ${PRODUCT_ARCHITECTURE} "

}


###########################################################################
# CheckVersion
###########################################################################

#
# Controle si la version de la mise-a-jour peut etre appliquee a la version 
# courante
#
# Argument 1 : numero de la version actuellement installee (_VERSION_CURRENT)
# Argument 2 : numero de version de base courante (_VERSION_CURRENT_BASE)
# Argument 3 : numero de la version de la mise-a-jour  (_VERSION_UPDATE)
# Argument 4 : numero de version minimale a partir de laquelle peut
#              s'appliquer la mise-a-jour (_VERSION_UPDATE_BASE)
#
# Attention : 
# Les versions controlees sont sous forme de chaines de caracteres.
# 
#
#
# 
# Warning : _VERSION_UPDATE_BASE not yet used.
#

CheckVersion ()
{
    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> CheckVersion"
    

    if [ $# -ne 4 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else
        _VERSION_CURRENT=$1
        _VERSION_CURRENT_BASE=$2
        _VERSION_UPDATE=$3
        _VERSION_UPDATE_BASE=$4
        
        DisplayTrace "    _VERSION_CURRENT      : ${_VERSION_CURRENT}"
        DisplayTrace "    _VERSION_CURRENT_BASE : ${_VERSION_CURRENT_BASE}"
        DisplayTrace "    _VERSION_UPDATE       : ${_VERSION_UPDATE}"
        DisplayTrace "    _VERSION_UPDATE_BASE  : ${_VERSION_UPDATE_BASE}"
        
        # Separation of the different values that compose the full version number
        # VERSION.RELEASE.MODIFICATION.FIXPAK
        CURRENT_NUMBER_VERSION=`echo ${_VERSION_CURRENT} | ${CUT} -d'.' -f1`
        CURRENT_NUMBER_RELEASE=`echo ${_VERSION_CURRENT} | ${CUT} -d'.' -f2`
        CURRENT_NUMBER_MODIFICATION=`echo ${_VERSION_CURRENT} | ${CUT} -d'.' -f3`
        CURRENT_NUMBER_FIXPAK=`echo ${_VERSION_CURRENT} | ${CUT} -d'.' -f4`

        # Removal of non significative zeroes
        CURRENT_NUMBER_VERSION=`RemoveLeadingZeroes "${CURRENT_NUMBER_VERSION}"`
        CURRENT_NUMBER_RELEASE=`RemoveLeadingZeroes "${CURRENT_NUMBER_RELEASE}"`
        CURRENT_NUMBER_MODIFICATION=`RemoveLeadingZeroes "${CURRENT_NUMBER_MODIFICATION}"`
        CURRENT_NUMBER_FIXPAK=`RemoveLeadingZeroes "${CURRENT_NUMBER_FIXPAK}"`

        DisplayTrace ""
        DisplayTrace "    CURRENT_NUMBER_VERSION      : ${CURRENT_NUMBER_VERSION}"
        DisplayTrace "    CURRENT_NUMBER_RELEASE      : ${CURRENT_NUMBER_RELEASE}"
        DisplayTrace "    CURRENT_NUMBER_MODIFICATION : ${CURRENT_NUMBER_MODIFICATION}"
        DisplayTrace "    CURRENT_NUMBER_FIXPAK       : ${CURRENT_NUMBER_FIXPAK}"
        DisplayTrace ""


        # Separation of the different values that compose the full version number
        # VERSION.RELEASE.MODIFICATION.FIXPAK
        UPDATE_NUMBER_VERSION=`echo ${_VERSION_UPDATE} | ${CUT} -d'.' -f1`
        UPDATE_NUMBER_RELEASE=`echo ${_VERSION_UPDATE} | ${CUT} -d'.' -f2`
        UPDATE_NUMBER_MODIFICATION=`echo ${_VERSION_UPDATE} | ${CUT} -d'.' -f3`
        UPDATE_NUMBER_FIXPAK=`echo ${_VERSION_UPDATE} | ${CUT} -d'.' -f4`

        # Removal of non significative zeroes
        UPDATE_NUMBER_VERSION=`RemoveLeadingZeroes "${UPDATE_NUMBER_VERSION}"`
        UPDATE_NUMBER_RELEASE=`RemoveLeadingZeroes "${UPDATE_NUMBER_RELEASE}"`
        UPDATE_NUMBER_MODIFICATION=`RemoveLeadingZeroes "${UPDATE_NUMBER_MODIFICATION}"`
        UPDATE_NUMBER_FIXPAK=`RemoveLeadingZeroes "${UPDATE_NUMBER_FIXPAK}"`
        
        DisplayTrace ""
        DisplayTrace "    UPDATE_NUMBER_VERSION          : ${UPDATE_NUMBER_VERSION}"
        DisplayTrace "    UPDATE_NUMBER_RELEASE          : ${UPDATE_NUMBER_RELEASE}"
        DisplayTrace "    UPDATE_NUMBER_MODIFICATION     : ${UPDATE_NUMBER_MODIFICATION}"
        DisplayTrace "    UPDATE_NUMBER_FIXPAK           : ${UPDATE_NUMBER_FIXPAK}"
        DisplayTrace ""


        DisplayTrace ""

        # Components of a full version number : 
        # VERSION.RELEASE.MODIFICATION.FIXPAK

        # PRODUCT_VERSION_REQUIRED from product information file not yet used !
        # It should have...
        # (Direct comparison with PRODUCT_VERSION_REQUIRED not used because of
        # sh shell used and not ksh shell)
    
        if [ "${LICENSE_ACTIVATION}" = "yes" ]
        then
            # License activation
            if [ "${_VERSION_CURRENT_BASE}" != "${_VERSION_UPDATE_BASE}" ]
            then
                DisplayTrace "    Base version ${_VERSION_CURRENT_BASE} installed is not the expected one to activate ${_VERSION_UPDATE}"
                RETURN_CODE=8
            else
                if [ ${CURRENT_NUMBER_VERSION} -ne ${UPDATE_NUMBER_VERSION} ]
                then
                    DisplayTrace "Version ${_VERSION_CURRENT} is bad to activate ${_VERSION_UPDATE}."  
                    RETURN_CODE=8
                else
                    if [ ${CURRENT_NUMBER_RELEASE} -ne ${UPDATE_NUMBER_RELEASE} ]
                    then
                        DisplayTrace "Version ${_VERSION_CURRENT} is bad to activate ${_VERSION_UPDATE}."  
                        RETURN_CODE=8
                    else
                        DisplayTrace "Version ${_VERSION_CURRENT} is good to activate ${_VERSION_UPDATE}."  
                    fi
                fi
            fi
        else
            # Installation of a FP
            if [ "${_VERSION_CURRENT_BASE}" != "${_VERSION_UPDATE_BASE}" ]
            then
                DisplayMessage "INZP027E" "Base version %s installed is not the expected one to apply the update %s." "${_VERSION_CURRENT_BASE}" "${_VERSION_UPDATE}"
                DisplayTrace "    Base version ${_VERSION_CURRENT_BASE} installed is not the expected one to apply the update ${_VERSION_UPDATE}"
                RETURN_CODE=8
            else
                if [ ${CURRENT_NUMBER_VERSION} -gt ${UPDATE_NUMBER_VERSION} ]
                then
                    DisplayMessage "INZP021E" "Version %s is too high or equal to apply the update %s." "${_VERSION_CURRENT}" "${_VERSION_UPDATE}"
                    DisplayTrace "    Version ${_VERSION_CURRENT} is too high or equal to apply the update ${_VERSION_UPDATE}"
                    RETURN_CODE=8
                else
                    if [ ${CURRENT_NUMBER_VERSION} -lt ${UPDATE_NUMBER_VERSION} ]
                    then
                        DisplayMessage "INZP022E" "Version %s is too low to apply the update %s." "${_VERSION_CURRENT}" "${_VERSION_UPDATE}"
                        DisplayTrace "Version ${_VERSION_CURRENT} is too low to apply the update ${_VERSION_UPDATE}."  
                        RETURN_CODE=8
                    else
                        # Version numbers identical

                        if [ ${CURRENT_NUMBER_RELEASE} -gt ${UPDATE_NUMBER_RELEASE} ]
                        then
                            DisplayMessage "INZP021E" "Version %s is too high or equal to apply the update %s." "${_VERSION_CURRENT}" "${_VERSION_UPDATE}"
                            DisplayTrace "    Version ${_VERSION_CURRENT} is too high or equal to apply the update ${_VERSION_UPDATE}"
                            RETURN_CODE=8
                        else
                            if [ ${CURRENT_NUMBER_RELEASE} -lt ${UPDATE_NUMBER_RELEASE} ]
                            then
                                DisplayMessage "INZP022E" "Version %s is too low to apply the update %s." "${_VERSION_CURRENT}" "${_VERSION_UPDATE}"
                                DisplayTrace "Version ${_VERSION_CURRENT} is too low to apply the update ${_VERSION_UPDATE}."  
                                RETURN_CODE=8
                            else
                                # Release numbers identical

                                if [ ${CURRENT_NUMBER_MODIFICATION} -gt ${UPDATE_NUMBER_MODIFICATION} ]
                                then
                                    DisplayMessage "INZP021E" "Version %s is too high or equal to apply the update %s." "${_VERSION_CURRENT}" "${_VERSION_UPDATE}"
                                    DisplayTrace "    Version ${_VERSION_CURRENT} is too high or equal to apply the update ${_VERSION_UPDATE}"
                                    RETURN_CODE=8
                                else
                                    if [ ${CURRENT_NUMBER_MODIFICATION} -lt ${UPDATE_NUMBER_MODIFICATION} ]
                                    then
                                        # Modification number greater
                                        DisplayMessage "INZP023I" "Found version %s, update to version %s can be applied." "${_VERSION_CURRENT}" "${_VERSION_UPDATE}"
                                        DisplayTrace "    Found version ${_VERSION_CURRENT}, update to version ${_VERSION_UPDATE} can be applied."
                                        DisplayTrace "    "     
                                        echo ""
                                    else
                                        # Modification numbers identical

                                        if [ ${CURRENT_NUMBER_FIXPAK} -ge ${UPDATE_NUMBER_FIXPAK} ]
                                        then
                                            DisplayMessage "INZP021E" "Version %s is too high or equal to apply the update %s." "${_VERSION_CURRENT}" "${_VERSION_UPDATE}"
                                            DisplayTrace "    Version ${_VERSION_CURRENT} is too high or equal to apply the update ${_VERSION_UPDATE}"
                                            RETURN_CODE=8
                                        else
                                            # Fixpak number greater
                                            DisplayMessage "INZP023I" "Found version %s, update to version %s can be applied." "${_VERSION_CURRENT}" "${_VERSION_UPDATE}"
                                            DisplayTrace "    Found version ${_VERSION_CURRENT}, update to version ${_VERSION_UPDATE} can be applied."
                                            DisplayTrace "    "     
                                            echo ""
                                        fi
                                    fi
                                fi
                            fi
                        fi
                    fi
                fi                         
            fi
        fi
    fi
    
    return ${RETURN_CODE}
}    


###########################################################################
# CheckExistingInstallation
###########################################################################

#
# Retrieve existing installation information
# 
# 
# 
#
# Argument 1 : 
#
# 
# 
# 

CheckExistingInstallation ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> CheckExistingInstallation"

    # Retrieval of informations of the already installed product.
    # There are two ways for retrieving informations on an existing installation:
    #
    # 1. From the file install/product_information.sig
    #
    # 2. From the file install/${LPRODUCT}_inst_settings.cfg
    #    This method is already used by uninstallation processing.
    # 
    # After a complete installation, information about product's version, license, ...
    # are in both files.
    # 
    # For now, method 2 is used.
    # 

    # Directory existence check

    if [ ${RETURN_CODE} -eq 0 ]
    then
        if [ ! -d "${DIR_INSTALLATION}" ]
        then
            DisplayTraceAndError "Directory \"${DIR_INSTALLATION}\" not found."
            DisplayMessage "INZP024E" "Directory %s not found." "${DIR_INSTALLATION}"
            RETURN_CODE=1
        fi
    fi

    # Settings file check
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # File containing informations about the product already present
        FILE_INSTALL_SETTINGS=${DIR_INSTALLATION}/install/${LPRODUCT}_inst_settings.cfg

        if [ ! -f "${FILE_INSTALL_SETTINGS}" ]
        then
            _MESSAGE_1="File ${FILE_INSTALL_SETTINGS} not found"
            _MESSAGE_2="${UPRODUCT} not found in ${DIR_INSTALLATION} directory"
            DisplayTraceAndError "${_MESSAGE_1}"
            DisplayTraceAndError "${_MESSAGE_2}"
            DisplayMessage "INZP001I" "File %s not found" "${FILE_INSTALL_SETTINGS}"
            echo "${_MESSAGE_2}"
            RETURN_CODE=1
        fi
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        EXISTING_PRODUCT_NAME=`GetSettingsVariable "PRODUCT_NAME"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PRODUCT_NAME\" not found in ${FILE_INSTALL_SETTINGS}"
        fi

        # Pas de EXISTING_PRODUCT_COMPONENT_NAME dans le fichier

        EXISTING_PRODUCT_VERSION=`GetSettingsVariable "PRODUCT_VERSION"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PRODUCT_VERSION\" not found in ${FILE_INSTALL_SETTINGS}"
        fi

        EXISTING_PRODUCT_VERSION_REQUIRED=`GetSettingsVariable "PRODUCT_VERSION_REQUIRED"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PRODUCT_VERSION_REQUIRED\" not found in ${FILE_INSTALL_SETTINGS}"
            EXISTING_PRODUCT_VERSION_REQUIRED=""
        fi

        EXISTING_PRODUCT_DESCRIPTION=`GetSettingsVariable "PRODUCT_DESCRIPTION"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PRODUCT_DESCRIPTION\" not found in ${FILE_INSTALL_SETTINGS}"
        fi

        EXISTING_PRODUCT_OPERATING_SYSTEM=`GetSettingsVariable "PRODUCT_OPERATING_SYSTEM"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PRODUCT_OPERATING_SYSTEM\" not found in ${FILE_INSTALL_SETTINGS}"
        fi

        EXISTING_PRODUCT_ARCHITECTURE=`GetSettingsVariable "PRODUCT_ARCHITECTURE"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PRODUCT_ARCHITECTURE\" not found in ${FILE_INSTALL_SETTINGS}"
        fi

        EXISTING_PRODUCT_INSTALLER_TYPE=`GetSettingsVariable "PRODUCT_INSTALLER_TYPE"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PRODUCT_INSTALLER_TYPE}\" not found in ${FILE_INSTALL_SETTINGS}"
        fi

        EXISTING_PRODUCT_LICENCE_TYPE=`GetSettingsVariable "PRODUCT_LICENCE_TYPE"`
        if [ $? -ne 0 ]
        then
            # Information non memorisee
            DisplayTrace "Parameter \"PRODUCT_LICENCE_TYPE\" not found in ${FILE_INSTALL_SETTINGS}"
        fi
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        # If variable not found in settings file (because of old package),
        # search it into the product signature file
        if [ "${EXISTING_PRODUCT_VERSION_REQUIRED}" = "" ]
        then
            FILE_INSTALL_SETTINGS=${DIR_INSTALLATION}/install/product_information.sig

            EXISTING_PRODUCT_VERSION_REQUIRED=`GetSettingsVariable "PRODUCT_VERSION_REQUIRED"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "Parameter \"PRODUCT_VERSION_REQUIRED\" not found in ${FILE_INSTALL_SETTINGS}"
            fi
        fi
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Some informations are not available in the product settings file 
        # (Those are available in the product product information file)
        DisplayTrace "    EXISTING_PRODUCT_NAME                     : ${EXISTING_PRODUCT_NAME}"
        DisplayTrace "    EXISTING_PRODUCT_DESCRIPTION              : ${EXISTING_PRODUCT_DESCRIPTION}"
        # DisplayTrace "    EXISTING_PRODUCT_COMPONENT_NAME           : ${EXISTING_PRODUCT_COMPONENT_NAME}"
        DisplayTrace "    EXISTING_PRODUCT_VERSION                  : ${EXISTING_PRODUCT_VERSION}"
        DisplayTrace "    EXISTING_PRODUCT_VERSION_REQUIRED         : ${EXISTING_PRODUCT_VERSION_REQUIRED}"
        # DisplayTrace "    EXISTING_PRODUCT_DEFAULT_INSTALLATION_DIR : ${EXISTING_PRODUCT_DEFAULT_INSTALLATION_DIR}"
        DisplayTrace "    EXISTING_PRODUCT_OPERATING_SYSTEM         : ${EXISTING_PRODUCT_OPERATING_SYSTEM}"
        DisplayTrace "    EXISTING_PRODUCT_ARCHITECTURE             : ${EXISTING_PRODUCT_ARCHITECTURE}"
        DisplayTrace "    EXISTING_PRODUCT_INSTALLER_TYPE           : ${EXISTING_PRODUCT_INSTALLER_TYPE}"
        DisplayTrace "    EXISTING_PRODUCT_LICENCE_TYPE             : ${EXISTING_PRODUCT_LICENCE_TYPE}"
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckVersion "${EXISTING_PRODUCT_VERSION}" "${EXISTING_PRODUCT_VERSION_REQUIRED}" "${PRODUCT_VERSION}" "${PRODUCT_VERSION_REQUIRED}"
        RETURN_CODE=$?
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# UpdateEnvironmentINST
###########################################################################

#
# Ajout d'une variable initialisee dans l'environnement (installation)
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_environment"
#
#
# Pour l'instant, pas de remplacement de la valeur d'une variable deja presente
#
#
# L'environnement est l'environnement global : /etc/profile 
# La variable est ajoutee dans un paragraphe delimite de la forme suivante :
#
# 
# # Begin product Environment Variable Section
# # Do not edit this section manually
# VARIABLE=valeur
# export VARIABLE
# # End product Environment Variable Section
#
#
# Si le paragraphe n'existe pas il est cree.
# Si la variable est deja declaree dans le paragraphe, sa valeur est remplacee.
# Si la variable n'est pas deja declaree dans le paragraphe, sa declaration y
# est ajoutee.
#
# Il n'y a qu'un seul paragraphe et celui-ci peut declarer plusieurs variables.
#
#
# A faire : mettre les etiquettes de debut et de fin de paragraphe dans le fichier
#            .INI 
# A faire : utiliser des variables pour les etiquettes de debut et de fin de 
#           paragraphe 
#
### DEBUG ###
# >>> Fonction provenant de ${LPRODUCT}_inst.sh
### FIN DEBUG ###
#

UpdateEnvironmentINST ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> UpdateEnvironmentINST"

    
    IDENTIFIER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "variable"` 
    VALUE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "value"` 



    DisplayTrace "    IDENTIFIER  : \"${IDENTIFIER}\"" 
    DisplayTrace "    VALUE       : \"${VALUE}\"" 


    VALUE=`\eval echo "${VALUE}"`


    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    VALUE       : \"${VALUE}\"" 



    # Fichier temporaire vide cree avec les memes droits et proprietaire que
    # l'original
    # >>> A faire ? Ne pas mettre le fichier temporaire au meme endroit que l'original
    # car le FILE_PROFILE est un chemin complet
    FILE_TEMP="${FILE_PROFILE}.$$"
    ${CP} -p "${FILE_PROFILE}" "${FILE_TEMP}"
    > "${FILE_TEMP}"

    # Fichier pour script "sed"
    FILE_SED_TEMP="sed_script.$$"
    > "${FILE_SED_TEMP}"
    
    # Suffixe pour la sauvegarde du fichier original 
    SUFFIX=`date +"%d%m%y_%H%M%S"`

    # Recherche du paragraphe
    ${AWK} '/# Begin ${UPRODUCT}/ , /# End ${UPRODUCT}/ {print}'  ${FILE_PROFILE} > ${FILE_TEMP} 

    if [ -s "${FILE_TEMP}" ]
    then
        DisplayTrace "    Paragraph found"
    
        # Si paragraphe deja existant 
        # Recherche si la variable est deja declaree
        ${GREP} '^'"${IDENTIFIER}=" "${FILE_TEMP}" >/dev/null 2>&1
        CODE_RETOUR=$?

        if [ ${CODE_RETOUR} -eq 0 ]
        then

            DisplayTrace "    Variable already declared"

        
            ${CP} -p ${FILE_PROFILE} ${FILE_PROFILE}.${SUFFIX}
                   
            # Variable deja presente
            ReplaceVariable "${FILE_PROFILE}" "${IDENTIFIER}" "${VALUE}" 
            
            ${RM} ${FILE_SED_TEMP} 
            ${RM} ${FILE_TEMP} 
            
        else
            DisplayTrace "    Undeclared variable"

            # Ajouter la declaration de la variable dans le paragraphe
            # On travaille directement a partir du fichier original car on sait 
            # que le paragraphe s'y trouve deja

            # Les directives "sed" sont dans un fichier a cause de l'utilisation de 
            # variables shell a substituer
            echo '/^# End ${UPRODUCT} /i\' >> ${FILE_SED_TEMP} 
            echo "${IDENTIFIER}=${VALUE}\\" >> ${FILE_SED_TEMP} 
            echo "export ${IDENTIFIER}" >> ${FILE_SED_TEMP} 

            # Attention reutilisation du meme fichier temporaire
            
            ${SED} -f ${FILE_SED_TEMP} "${FILE_PROFILE}" > ${FILE_TEMP}
            
            ${MV} "${FILE_PROFILE}" ${FILE_PROFILE}.${SUFFIX}
            ${MV} ${FILE_TEMP} "${FILE_PROFILE}"
            
            ${RM} ${FILE_SED_TEMP} 
            
        fi

    else
        DisplayTrace "    Paragraph not found"

    
        # Paragraphe inexistant
        # Le paragraphe est ajoute en fin 
        
        ${CP} -p "${FILE_PROFILE}" ${FILE_PROFILE}.${SUFFIX}
        
        echo ""                                         >> "${FILE_PROFILE}"
        echo "# Begin ${UPRODUCT} Environment Variable Section" >> "${FILE_PROFILE}"
        echo "# Do not edit this section manually"      >> "${FILE_PROFILE}"
        echo "${IDENTIFIER}=${VALUE}"                   >> "${FILE_PROFILE}"
        echo "export ${IDENTIFIER}"                     >> "${FILE_PROFILE}" 
        echo "# End ${UPRODUCT} Environment Variable Section"   >> "${FILE_PROFILE}"
        echo ""                                         >> "${FILE_PROFILE}" 

        ${RM} ${FILE_SED_TEMP} 
        ${RM} ${FILE_TEMP} 

    fi

    return ${RETURN_CODE}
}


###########################################################################
# ParseCommandline
###########################################################################

# 
# 
#


ParseCommandline ()
{
    :


}





###########################################################################
# ReadProductInformationFile
###########################################################################

# 
# 
#

ReadProductInformationFile ()
{
    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> ReadProductInformationFile"
    

    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        FILE="$1" 

        DisplayTrace "    FILE : \"${FILE}\"" 
        
        CheckReadableFile "${FILE}"
        
        if [ $? -eq 0 ]
        then
            . "${FILE}" 

            DisplayTrace ""
            DisplayTrace "    Product information: " 
            DisplayTrace "    (file \"${FILE}\")"
            DisplayTrace ""
            DisplayTrace "    PRODUCT_NAME                     : ${PRODUCT_NAME}"
            DisplayTrace "    PRODUCT_DESCRIPTION              : ${PRODUCT_DESCRIPTION}"
            DisplayTrace "    PRODUCT_COMPONENT_NAME           : ${PRODUCT_COMPONENT_NAME}"
            DisplayTrace "    PRODUCT_VERSION                  : ${PRODUCT_VERSION}"
            DisplayTrace "    PRODUCT_VERSION_REQUIRED         : ${PRODUCT_VERSION_REQUIRED}"
            DisplayTrace "    PRODUCT_DEFAULT_INSTALLATION_DIR : ${PRODUCT_DEFAULT_INSTALLATION_DIR}"
            DisplayTrace "    PRODUCT_OPERATING_SYSTEM         : ${PRODUCT_OPERATING_SYSTEM}"
            DisplayTrace "    PRODUCT_ARCHITECTURE             : ${PRODUCT_ARCHITECTURE}"
            DisplayTrace "    PRODUCT_INSTALLER_TYPE           : ${PRODUCT_INSTALLER_TYPE}"
            DisplayTrace "    PRODUCT_LICENCE_TYPE             : ${PRODUCT_LICENCE_TYPE}"
            
            case "${PRODUCT_OPERATING_SYSTEM}" in
            "AIX")
                PRODUCT_PLATFORM="aix"
                ;;

            "Solaris")
                case "${PRODUCT_ARCHITECTURE}" in
                "SPARC")
                    PRODUCT_PLATFORM="sun"
                    ;;
                "X86")
                    PRODUCT_PLATFORM="sun_x86"
                    ;;
                *)
                    DisplayError "Invalid value for PRODUCT_ARCHITECTURE : \"${PRODUCT_ARCHITECTURE}\" "
                    ;;
                esac
                ;;

            "HP-UX")
                PRODUCT_PLATFORM="hpia"
                ;;

            "Linux")
                case "${PRODUCT_ARCHITECTURE}" in
                "Intel")
                    PRODUCT_PLATFORM="linux"
                    ;;
                "PowerPC")
                    PRODUCT_PLATFORM="plinux"
                    ;;
                "s390x")
                    PRODUCT_PLATFORM="zlinux"
                    ;;
                "IA64")
                    PRODUCT_PLATFORM="ialinux"
                    ;;
                *)
                    DisplayError "Invalid value for PRODUCT_ARCHITECTURE : \"${PRODUCT_ARCHITECTURE}\" "
                    ;;
                esac
                ;;

            "Linux LE")
                PRODUCT_PLATFORM="plinuxle"
                ;;

            *)
                DisplayError "Invalid value for PRODUCT_OPERATING_SYSTEM : \"${PRODUCT_OPERATING_SYSTEM}\" "
                ;;
            esac
            
            DisplayTrace ""
            DisplayTrace "    PRODUCT_PLATFORM                 : ${PRODUCT_PLATFORM}"
        fi
    fi        
    return ${RETURN_CODE}

}


###########################################################################
# CheckProductInformation
###########################################################################

# 
# Checking of product information data
#
# SUB_CONTEXT variable set
#
# Check the operating system
#


CheckProductInformation ()
{
    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> CheckProductInformation"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    

        SUB_CONTEXT=""
        case `ConvertToUpperCase "${PRODUCT_INSTALLER_TYPE}"` in
        "COMPLETE")
            SUB_CONTEXT="COMPLETE"
            :
            ;;

        "FIXPAK")
            SUB_CONTEXT="FIXPAK"
            ;;
            
        *)
            DisplayError "Invalid value for PRODUCT_INSTALLER_TYPE : \"${PRODUCT_INSTALLER_TYPE}\" "
            ;;
        esac
        

        case `ConvertToUpperCase "${PRODUCT_LICENCE_TYPE}"` in
        "FULL"|"TRY_AND_BUY"|"WORKGROUP")
            :
            ;;
          
        "NONE")
            # FixPak case
            :
            ;;        
        *)
            DisplayError "Invalid value for PRODUCT_LICENCE_TYPE : \"${PRODUCT_LICENCE_TYPE}\" "
            ;;
        esac
        
        if [ "${PRODUCT_PLATFORM}" != "${PLATFORM}" ]
        then
            DisplayMessage "INZP013E" "This installation package is for a %s on %s system but the current machine is a %s on %s system.\n" "${PRODUCT_OPERATING_SYSTEM}" "${PRODUCT_ARCHITECTURE}" "${OS_NAME}" "${ARCHITECTURE}"
            DisplayTrace "    Invalid package for the current platform "
            RETURN_CODE=1
        fi
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# SearchProductByNameAndVersion
###########################################################################

# 
# Search a product in the product database file using its name (and component)
# and version
#
# Argument 1 : product database file
# Argument 2 : product name
# Argument 3 : product component name
# Argument 4 : product version
# 
# Returns a string to stdout
#
# The product database field separator is not parametered in exclusion regular expression 
#


SearchProductByNameAndVersion ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    # DisplayTrace ""
    # DisplayTrace ">>> SearchProductByNameAndVersion"
    
    

    if [ $# -ne 4 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=4
    else    
        PRODUCT_DATABASE_FILE="$1" 
        SEARCHED_PRODUCT_NAME="$2" 
        SEARCHED_PRODUCT_COMPONENT_NAME="$3"
        SEARCHED_PRODUCT_VERSION="$4" 

        # DisplayTrace "    PRODUCT_DATABASE_FILE : \"${PRODUCT_DATABASE_FILE}\"" 
        # DisplayTrace "    SEARCHED_PRODUCT_NAME : \"${SEARCHED_PRODUCT_NAME}\"" 
        # DisplayTrace "    SEARCHED_PRODUCT_COMPONENT_NAME : \"${SEARCHED_PRODUCT_COMPONENT_NAME}\"" 
        # DisplayTrace "    SEARCHED_PRODUCT_VERSION : \"${SEARCHED_PRODUCT_VERSION}\""  
        
        SEARCHED_PRODUCT_NUMBER_VERSION=`echo ${SEARCHED_PRODUCT_VERSION} | ${CUT} -d'.' -f1`
        SEARCHED_PRODUCT_NUMBER_RELEASE=`echo ${SEARCHED_PRODUCT_VERSION} | ${CUT} -d'.' -f2`
        SEARCHED_PRODUCT_NUMBER_MODIFICATION=`echo ${SEARCHED_PRODUCT_VERSION} | ${CUT} -d'.' -f3`
        SEARCHED_PRODUCT_NUMBER_FIXPAK=`echo ${SEARCHED_PRODUCT_VERSION} | ${CUT} -d'.' -f4`

        # DisplayTrace ""
        # DisplayTrace "    SEARCHED_PRODUCT_NUMBER_VERSION      : ${SEARCHED_PRODUCT_NUMBER_VERSION}"
        # DisplayTrace "    SEARCHED_PRODUCT_NUMBER_RELEASE      : ${SEARCHED_PRODUCT_NUMBER_RELEASE}"
        # DisplayTrace "    SEARCHED_PRODUCT_NUMBER_MODIFICATION : ${SEARCHED_PRODUCT_NUMBER_MODIFICATION}"
        # DisplayTrace "    SEARCHED_PRODUCT_NUMBER_FIXPAK       : ${SEARCHED_PRODUCT_NUMBER_FIXPAK}"
        # DisplayTrace ""

        
        STRING_REG_EXPR="^${SEARCHED_PRODUCT_NAME}"
        STRING_REG_EXPR="${STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${SEARCHED_PRODUCT_COMPONENT_NAME}"
        STRING_REG_EXPR="${STRING_REG_EXPR}${FS_PRODUCT_DATABASE}[^|]*"
        STRING_REG_EXPR="${STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${SEARCHED_PRODUCT_NUMBER_VERSION}"
        STRING_REG_EXPR="${STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${SEARCHED_PRODUCT_NUMBER_RELEASE}"
        STRING_REG_EXPR="${STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${SEARCHED_PRODUCT_NUMBER_MODIFICATION}"
        STRING_REG_EXPR="${STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${SEARCHED_PRODUCT_NUMBER_FIXPAK}"
        
        # DisplayTrace "    STRING_REG_EXPR   : \"${STRING_REG_EXPR}\""
        
        RETURN_STRING=`${GREP} "${STRING_REG_EXPR}" "${PRODUCT_DATABASE_FILE}"`
        RETURN_CODE=$?
        
    fi
    
    echo "${RETURN_STRING}"
    return ${RETURN_CODE}
}


###########################################################################
# SearchProductByName
###########################################################################

# 
# Search a product in the product database file using its name 
# 
#
# Argument 1 : product database file
# Argument 2 : product name
# 
# Returns a string to stdout
#
#

SearchProductByName ()
{
    RETURN_CODE=0
    RETURN_STRING=""
    
    # DisplayTrace ""
    # DisplayTrace ">>> SearchProductByName"
    
    

    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=4
    else    
        _PRODUCT_DATABASE_FILE="$1" 
        _SEARCHED_PRODUCT_NAME="$2" 

        # DisplayTrace "    _PRODUCT_DATABASE_FILE : \"${_PRODUCT_DATABASE_FILE}\" " 
        # DisplayTrace "    _SEARCHED_PRODUCT_NAME : \"${_SEARCHED_PRODUCT_NAME}\" " 
        
        
        
        _STRING_REG_EXPR="^${_SEARCHED_PRODUCT_NAME}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}.*"
        
        # DisplayTrace "    _STRING_REG_EXPR   : \"${_STRING_REG_EXPR}\" "
        
        RETURN_STRING=`${GREP} "${_STRING_REG_EXPR}" "${_PRODUCT_DATABASE_FILE}"`
        RETURN_CODE=$?
        
    fi
    
    echo "${RETURN_STRING}"
    return ${RETURN_CODE}
}


###########################################################################
# SearchProductByNameAndComponent
###########################################################################

# 
# Search a product in the product database file using its name and component 
# name
# 
#
# Argument 1 : product database file
# Argument 2 : product name
# 
# Returns a string to stdout
#
#
#

SearchProductByNameAndComponent ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    # DisplayTrace ""
    # DisplayTrace ">>> SearchProductByNameAndComponent"
    
    

    if [ $# -ne 3 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=4
    else    
        _PRODUCT_DATABASE_FILE="$1" 
        _SEARCHED_PRODUCT_NAME="$2" 
        _SEARCHED_PRODUCT_COMPONENT_NAME="$3"

        # DisplayTrace "    _PRODUCT_DATABASE_FILE : \"${_PRODUCT_DATABASE_FILE}\"" 
        # DisplayTrace "    _SEARCHED_PRODUCT_NAME : \"${_SEARCHED_PRODUCT_NAME}\"" 
        # DisplayTrace "    _SEARCHED_PRODUCT_COMPONENT_NAME : \"${_SEARCHED_PRODUCT_COMPONENT_NAME}\"" 
        
        _STRING_REG_EXPR="^${_SEARCHED_PRODUCT_NAME}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_SEARCHED_PRODUCT_COMPONENT_NAME}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}.*"

        
        # DisplayTrace "    _STRING_REG_EXPR   : \"${_STRING_REG_EXPR}\" "
        
        RETURN_STRING=`${GREP} "${_STRING_REG_EXPR}" "${_PRODUCT_DATABASE_FILE}"`
        RETURN_CODE=$?
        
    fi
    
    echo "${RETURN_STRING}"
    return ${RETURN_CODE}
}



###########################################################################
# SearchProductByNameAndComponentAndDirectory
###########################################################################

# 
# Search a product in the product database file using its name, component 
# name and installation directory
# 
#
# Argument 1 : product database file
# Argument 2 : product name
# Argument 3 : product installation directory
# 
# Returns a string to stdout
#
#
#

SearchProductByNameAndComponentAndDirectory ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    # DisplayTrace ""
    # DisplayTrace ">>> SearchProductByNameAndComponentAndDirectory"
    

    if [ $# -ne 4 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=4
    else    
        _PRODUCT_DATABASE_FILE="$1" 
        _SEARCHED_PRODUCT_NAME="$2" 
        _SEARCHED_PRODUCT_COMPONENT_NAME="$3"
        _SEARCHED_PRODUCT_INSTALLATION_DIRECTORY="$4"

        # DisplayTrace "    _PRODUCT_DATABASE_FILE : \"${_PRODUCT_DATABASE_FILE}\"" 
        # DisplayTrace "    _SEARCHED_PRODUCT_NAME : \"${_SEARCHED_PRODUCT_NAME}\"" 
        # DisplayTrace "    _SEARCHED_PRODUCT_COMPONENT_NAME : \"${_SEARCHED_PRODUCT_COMPONENT_NAME}\"" 
        # DisplayTrace "    _SEARCHED_PRODUCT_INSTALLATION_DIRECTORY : \"${_SEARCHED_PRODUCT_INSTALLATION_DIRECTORY}\"" 
        
        _STRING_REG_EXPR="^${_SEARCHED_PRODUCT_NAME}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_SEARCHED_PRODUCT_COMPONENT_NAME}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}.*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}.*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}.*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}.*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}.*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}.*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}.*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_SEARCHED_PRODUCT_INSTALLATION_DIRECTORY}"

        
        # DisplayTrace "    _STRING_REG_EXPR : \"${_STRING_REG_EXPR}\" "
        
        RETURN_STRING=`${GREP} "${_STRING_REG_EXPR}" "${_PRODUCT_DATABASE_FILE}"`
        RETURN_CODE=$?
    fi
    
    echo "${RETURN_STRING}"
    return ${RETURN_CODE}
}


###########################################################################
# AddProduct
###########################################################################

# 
# Add a product in the product database file 
#
# Argument 1 : product database file
# Argument 2 : product name
# Argument 3 : product component name
# Argument 4 : product version
# Argument 5 : product description
# Argument 6 : product installation directory
# 
#
#


AddProduct ()
{
    RETURN_CODE=0

    
    DisplayTrace ""
    DisplayTrace ">>> AddProduct"
    

    if [ $# -ne 6 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        _PRODUCT_DATABASE_FILE="$1" 
        _PRODUCT_NAME="$2" 
        _PRODUCT_COMPONENT_NAME="$3"
        _PRODUCT_VERSION="$4" 
        _PRODUCT_DESCRIPTION="$5" 
        _PRODUCT_INSTALLATION_DIRECTORY="$6" 

        DisplayTrace "    _PRODUCT_DATABASE_FILE          : \"${_PRODUCT_DATABASE_FILE}\"" 
        DisplayTrace "    _PRODUCT_NAME                   : \"${_PRODUCT_NAME}\"" 
        DisplayTrace "    _PRODUCT_COMPONENT_NAME         : \"${_PRODUCT_COMPONENT_NAME}\"" 
        DisplayTrace "    _PRODUCT_VERSION                : \"${_PRODUCT_VERSION}\""  
        DisplayTrace "    _PRODUCT_DESCRIPTION            : \"${_PRODUCT_DESCRIPTION}\""  
        DisplayTrace "    _PRODUCT_INSTALLATION_DIRECTORY : \"${_PRODUCT_INSTALLATION_DIRECTORY}\""  
        
        _PRODUCT_NUMBER_VERSION=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f1`
        _PRODUCT_NUMBER_RELEASE=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f2`
        _PRODUCT_NUMBER_MODIFICATION=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f3`
        _PRODUCT_NUMBER_FIXPAK=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f4`

        DisplayTrace ""
        DisplayTrace "    _PRODUCT_NUMBER_VERSION      : ${_PRODUCT_NUMBER_VERSION}"
        DisplayTrace "    _PRODUCT_NUMBER_RELEASE      : ${_PRODUCT_NUMBER_RELEASE}"
        DisplayTrace "    _PRODUCT_NUMBER_MODIFICATION : ${_PRODUCT_NUMBER_MODIFICATION}"
        DisplayTrace "    _PRODUCT_NUMBER_FIXPAK       : ${_PRODUCT_NUMBER_FIXPAK}"
        DisplayTrace ""

        
        _STRING_NEW_PRODUCT="${_PRODUCT_NAME}"
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}${_PRODUCT_COMPONENT_NAME}"
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}${_PRODUCT_DESCRIPTION}"
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_VERSION}"
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_RELEASE}"
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_MODIFICATION}"
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_FIXPAK}"
        
        _CURRENT_DATE=`date +"%d/%m/%Y"`
        _CURRENT_TIME=`date +"%H:%M"`
        
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}${_CURRENT_DATE}"
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}${_CURRENT_TIME}"
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}${_PRODUCT_INSTALLATION_DIRECTORY}"
        _STRING_NEW_PRODUCT="${_STRING_NEW_PRODUCT}${FS_PRODUCT_DATABASE}"
        
        DisplayTrace "    _STRING_NEW_PRODUCT   : \"${_STRING_NEW_PRODUCT}\""

        DisplayTrace "    Creating entry in \"${_PRODUCT_DATABASE_FILE}\" "
        
        echo "${_STRING_NEW_PRODUCT}" >> "${_PRODUCT_DATABASE_FILE}"
        RETURN_CODE=$?
        
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# RemoveProduct
###########################################################################

# 
# Remove a product in the product database file 
#
# Argument 1 : product database file
# Argument 2 : product name
# Argument 3 : product component name
# Argument 4 : product version (may be ".*" in order to use any version)
# Argument 5 : product installation directory
# 
# The product database field separator is not parametered in exclusion regular expression 
#


RemoveProduct ()
{
    RETURN_CODE=0

    
    DisplayTrace ""
    DisplayTrace ">>> RemoveProduct"
    

    if [ $# -ne 5 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        _PRODUCT_DATABASE_FILE="$1" 
        _PRODUCT_NAME="$2" 
        _PRODUCT_COMPONENT_NAME="$3"
        _PRODUCT_VERSION="$4" 
        _PRODUCT_INSTALLATION_DIRECTORY="$5" 

        DisplayTrace "    _PRODUCT_DATABASE_FILE          : \"${_PRODUCT_DATABASE_FILE}\"" 
        DisplayTrace "    _PRODUCT_NAME                   : \"${_PRODUCT_NAME}\"" 
        DisplayTrace "    _PRODUCT_COMPONENT_NAME         : \"${_PRODUCT_COMPONENT_NAME}\"" 
        DisplayTrace "    _PRODUCT_VERSION                : \"${_PRODUCT_VERSION}\""  
        DisplayTrace "    _PRODUCT_INSTALLATION_DIRECTORY : \"${_PRODUCT_INSTALLATION_DIRECTORY}\""  

        
        # Fichier temporaire vide cree avec les memes droits et proprietaire que
        # l'original
        _FILE_TEMP="${_PRODUCT_DATABASE_FILE}.$$"
        ${CP} -p "${_PRODUCT_DATABASE_FILE}" "${_FILE_TEMP}"
        > "${_FILE_TEMP}"
        
        if [ "${_PRODUCT_VERSION}" = ".*" ]
        then
            _PRODUCT_NUMBER_VERSION='.*'
            _PRODUCT_NUMBER_RELEASE='.*'
            _PRODUCT_NUMBER_MODIFICATION='.*'
            _PRODUCT_NUMBER_FIXPAK='.*'
        else
            _PRODUCT_NUMBER_VERSION=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f1`
            _PRODUCT_NUMBER_RELEASE=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f2`
            _PRODUCT_NUMBER_MODIFICATION=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f3`
            _PRODUCT_NUMBER_FIXPAK=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f4`      
        fi
        DisplayTrace ""
        DisplayTrace "    _PRODUCT_NUMBER_VERSION      : ${_PRODUCT_NUMBER_VERSION}"
        DisplayTrace "    _PRODUCT_NUMBER_RELEASE      : ${_PRODUCT_NUMBER_RELEASE}"
        DisplayTrace "    _PRODUCT_NUMBER_MODIFICATION : ${_PRODUCT_NUMBER_MODIFICATION}"
        DisplayTrace "    _PRODUCT_NUMBER_FIXPAK       : ${_PRODUCT_NUMBER_FIXPAK}"
        DisplayTrace ""


        _STRING_REG_EXPR="^${_PRODUCT_NAME}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_COMPONENT_NAME}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}[^|]*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_VERSION}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_RELEASE}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_MODIFICATION}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_FIXPAK}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}[^|]*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}[^|]*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_INSTALLATION_DIRECTORY}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}"

        
        DisplayTrace "    _STRING_REG_EXPR   : \"${_STRING_REG_EXPR}\" "
        
        RETURN_STRING=`${GREP} "${_STRING_REG_EXPR}" "${_PRODUCT_DATABASE_FILE}"`
        RETURN_CODE=$?

        if [ "${RETURN_CODE}" -eq 0 ]
        then
            DisplayTrace "    Removing product"
            
            ${GREP} -v "${_STRING_REG_EXPR}" "${_PRODUCT_DATABASE_FILE}" > "${_FILE_TEMP}"
            RETURN_CODE=$?


            if [ "${RETURN_CODE}" -eq 0 ]
            then
                ${MV} "${_FILE_TEMP}" "${_PRODUCT_DATABASE_FILE}"
                RETURN_CODE=$?
                
                if [ "${RETURN_CODE}" -ne 0 ]
                then
                    DisplayError "Error while replacing product database file"
                    RETURN_CODE=4
                fi            
                
            else  
                DisplayError "Error while removing product entry from product database file"
                RETURN_CODE=3
              
            fi    
            
        else
            DisplayError "Product not found in product database file"
            RETURN_CODE=2
        fi    

        ${RM} -f "${_FILE_TEMP}"
        
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# UpdateProductVersion
###########################################################################

# 
# (Warning: obsolete function? (centralized product database not more used))
# 
# Update a product in the product database file using its name, component 
# name, version and installation directory. 
# Update in installation (add the product) and in uninstallation (remove the 
# product)
# 
# - Version 
# - Date (current)
# - Time (current)
# 
# 
#
# Argument 1 : product database file
# Argument 2 : product name
# Argument 3 : product component name
# Argument 4 : product version
# Argument 5 : product installation directory
# Argument 6 : product updated version
# 
#
# The product database field separator is not parametered in exclusion regular expression 
#
#


UpdateProductVersion ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> UpdateProductVersion"
    
    

    if [ $# -ne 6 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        _PRODUCT_DATABASE_FILE="$1" 
        _PRODUCT_NAME="$2" 
        _PRODUCT_COMPONENT_NAME="$3"
        _PRODUCT_VERSION="$4" 
        _PRODUCT_INSTALLATION_DIRECTORY="$5" 
        _PRODUCT_VERSION_NEW="$6" 

        DisplayTrace "    _PRODUCT_DATABASE_FILE          : \"${_PRODUCT_DATABASE_FILE}\"" 
        DisplayTrace "    _PRODUCT_NAME                   : \"${_PRODUCT_NAME}\"" 
        DisplayTrace "    _PRODUCT_COMPONENT_NAME         : \"${_PRODUCT_COMPONENT_NAME}\"" 
        DisplayTrace "    _PRODUCT_VERSION                : \"${_PRODUCT_VERSION}\"" 
        DisplayTrace "    _PRODUCT_INSTALLATION_DIRECTORY : \"${_PRODUCT_INSTALLATION_DIRECTORY}\"" 
        DisplayTrace "    _PRODUCT_VERSION_NEW            : \"${_PRODUCT_VERSION_NEW}\"" 
        

        # Fichier temporaire vide cree avec les memes droits et proprietaire que
        # l'original
        _FILE_TEMP="${_PRODUCT_DATABASE_FILE}.$$"
        ${CP} -p "${_PRODUCT_DATABASE_FILE}" "${_FILE_TEMP}"
        > "${_FILE_TEMP}"
        
        _PRODUCT_NUMBER_VERSION=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f1`
        _PRODUCT_NUMBER_RELEASE=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f2`
        _PRODUCT_NUMBER_MODIFICATION=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f3`
        _PRODUCT_NUMBER_FIXPAK=`echo ${_PRODUCT_VERSION} | ${CUT} -d'.' -f4`

        DisplayTrace ""
        DisplayTrace "    _PRODUCT_NUMBER_VERSION      : ${_PRODUCT_NUMBER_VERSION}"
        DisplayTrace "    _PRODUCT_NUMBER_RELEASE      : ${_PRODUCT_NUMBER_RELEASE}"
        DisplayTrace "    _PRODUCT_NUMBER_MODIFICATION : ${_PRODUCT_NUMBER_MODIFICATION}"
        DisplayTrace "    _PRODUCT_NUMBER_FIXPAK       : ${_PRODUCT_NUMBER_FIXPAK}"
        DisplayTrace ""
        

        _PRODUCT_NUMBER_VERSION_NEW=`echo ${_PRODUCT_VERSION_NEW} | ${CUT} -d'.' -f1`
        _PRODUCT_NUMBER_RELEASE_NEW=`echo ${_PRODUCT_VERSION_NEW} | ${CUT} -d'.' -f2`
        _PRODUCT_NUMBER_MODIFICATION_NEW=`echo ${_PRODUCT_VERSION_NEW} | ${CUT} -d'.' -f3`
        _PRODUCT_NUMBER_FIXPAK_NEW=`echo ${_PRODUCT_VERSION_NEW} | ${CUT} -d'.' -f4`

        DisplayTrace ""
        DisplayTrace "    _PRODUCT_NUMBER_VERSION_NEW      : ${_PRODUCT_NUMBER_VERSION_NEW}"
        DisplayTrace "    _PRODUCT_NUMBER_RELEASE_NEW      : ${_PRODUCT_NUMBER_RELEASE_NEW}"
        DisplayTrace "    _PRODUCT_NUMBER_MODIFICATION_NEW : ${_PRODUCT_NUMBER_MODIFICATION_NEW}"
        DisplayTrace "    _PRODUCT_NUMBER_FIXPAK_NEW       : ${_PRODUCT_NUMBER_FIXPAK_NEW}"
        DisplayTrace ""

        
        _STRING_REG_EXPR="^${_PRODUCT_NAME}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_COMPONENT_NAME}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}[^|]*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_VERSION}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_RELEASE}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_MODIFICATION}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_FIXPAK}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}[^|]*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}[^|]*"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}${_PRODUCT_INSTALLATION_DIRECTORY}"
        _STRING_REG_EXPR="${_STRING_REG_EXPR}${FS_PRODUCT_DATABASE}"

        
        DisplayTrace "    _STRING_REG_EXPR   : \"${_STRING_REG_EXPR}\" "
     
        
        _EXISTING_PRODUCT_ENTRY=`${GREP} "${_STRING_REG_EXPR}" "${_PRODUCT_DATABASE_FILE}"`
        RETURN_CODE=$?
        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            DisplayTrace "    Updating product"
            
            # Retrieval of field from existing entry which was not given as 
            # calling argument
            _PRODUCT_DESCRIPTION=`echo "${_EXISTING_PRODUCT_ENTRY}" | ${CUT} -d"${FS_PRODUCT_DATABASE}" -f3` 
            
            DisplayTrace "    _PRODUCT_DESCRIPTION            : \"${_PRODUCT_DESCRIPTION}\"" 
            
            
            _CURRENT_DATE=`date +"%d/%m/%Y"`
            _CURRENT_TIME=`date +"%H:%M"`

            _UPDATED_PRODUCT_ENTRY="${_PRODUCT_NAME}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}${_PRODUCT_COMPONENT_NAME}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}${_PRODUCT_DESCRIPTION}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_VERSION_NEW}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_RELEASE_NEW}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_MODIFICATION_NEW}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}${_PRODUCT_NUMBER_FIXPAK_NEW}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}${_CURRENT_DATE}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}${_CURRENT_TIME}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}${_PRODUCT_INSTALLATION_DIRECTORY}"
            _UPDATED_PRODUCT_ENTRY="${_UPDATED_PRODUCT_ENTRY}${FS_PRODUCT_DATABASE}"
            
            DisplayTrace "    _UPDATED_PRODUCT_ENTRY   : \"${_UPDATED_PRODUCT_ENTRY}\" "
            
            # Removal of existing entry
            ${GREP} -v "${_STRING_REG_EXPR}" "${_PRODUCT_DATABASE_FILE}" > "${_FILE_TEMP}"
            RETURN_CODE=$?

            if [ "${RETURN_CODE}" -eq 0 ]
            then
            
                # Addition of replacement entry
                echo "${_UPDATED_PRODUCT_ENTRY}" >> "${_FILE_TEMP}"
                RETURN_CODE=$?
            
                if [ "${RETURN_CODE}" -ne 0 ]
                then
                    DisplayError "Error while adding replacement entry"
                    RETURN_CODE=4
                fi            
            
            
                if [ "${RETURN_CODE}" -eq 0 ]
                then
            
                    ${MV} "${_FILE_TEMP}" "${_PRODUCT_DATABASE_FILE}"
                    RETURN_CODE=$?
                
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error while replacing product database file"
                        RETURN_CODE=4
                    fi            
                fi   
            else  

                DisplayError "Error while removing existing entry"
                RETURN_CODE=4
            fi

        else
            DisplayError "Product not found in product database file"
            RETURN_CODE=2
        fi    

        ${RM} -f "${_FILE_TEMP}"
        
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# IdentifyPlatform
###########################################################################

# 
# Identification of operating system and hardware platform
#
#
# Global variables set : 
# OS_NAME      : result from uname -s
# OS_VERSION   : result from uname -r ### A MODIFIER ###
# PLATFORM     : product platform identification name (combines operating system and
#                hardware platform
# ARCHITECTURE : result from uname -p / uname -m
#


IdentifyPlatform ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> IdentifyPlatform"
    
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    

        OS_NAME=`${UNAME} -s`
        # OS_VERSION=`${UNAME} -r`

        DisplayTrace "    OS_NAME    : ${OS_NAME}"
        # DisplayTrace "    OS_VERSION : ${OS_VERSION}"

        case "${OS_NAME}" in
        
        "AIX")
            ARCHITECTURE=`${UNAME} -p`
            SYSTEM_VERSION="`${UNAME} -v`.`${UNAME} -r`"
            
            case "${ARCHITECTURE}" in
            
            "powerpc")
                DisplayTrace "    PowerPC"
                PLATFORM="aix"
                ;;

            *)
                PLATFORM=""
                :
                ;;

            esac

            ;;


        "SunOS")
            
            ARCHITECTURE=`${UNAME} -p`
            SYSTEM_VERSION="`${UNAME} -r`"
            
            case "${ARCHITECTURE}" in
            
            "i386")
                DisplayTrace "    Intel x86"
                PLATFORM="sun_x86"
                ;;

            "sparc")
                DisplayTrace "    Sun SPARC"
                PLATFORM="sun"
                ;;

            *)
                PLATFORM="sun"
                ;;

            esac

            ;;


        "Linux")
            ARCHITECTURE=`${UNAME} -m`
            SYSTEM_VERSION="`${UNAME} -r`"
            
            case "${ARCHITECTURE}" in
            
            "i386"|"i686")
                DisplayTrace "    Intel x86"
                PLATFORM="linux"
                ;;

            "x86_64")
                DisplayTrace "    Intel 64/AMD 64"
                PLATFORM="linux"
                ;;

            "ppc64")
                DisplayTrace "    PowerPC 64"
                PLATFORM="plinux"
                ;;

            "ppc64le")
                DisplayTrace "    PowerPC 64"
                PLATFORM="plinuxle"
                ;;

            "s390x")
                DisplayTrace "    s390x"
                PLATFORM="zlinux"
                ;;

            "ia64")
                DisplayTrace "    ia64"
                PLATFORM="ialinux"
                ;;

            *)
                PLATFORM=""
                ;;

            esac

            ;;

        "HP-UX")
            ARCHITECTURE=`${UNAME} -m`
            SYSTEM_VERSION="`${UNAME} -r`"
            
            case "${ARCHITECTURE}" in
                
            "ia64")
                DisplayTrace "    Intel Itanium 64"
                PLATFORM="hpia"
                ;;

            *)
                DisplayTrace "    PA RISC"
                PLATFORM="hp"
                ;;

            esac
            ;;

        *)
            ARCHITECTURE=``
            DisplayTrace "    Unknown architecture"
            DisplayTrace "    Unknown platform"
            PLATFORM=""
            ;;

        esac


    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# IdentifyLocale
###########################################################################

# 
# Identification of operating system and hardware platform
#
#
# Gobal variables set : 
# LOCALE      : XXX
# CHARMAP   : XXX
# ARCHITECTURE : result from uname -p / uname -m
#
#
#
#

IdentifyLocale ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> IdentifyLocale"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    

        # LOCALE="${LANG}"
        LOCALE="${LANG=}"
        CHARMAP=`\locale charmap`

        DisplayTrace "    LOCALE  : ${LOCALE}" 
        DisplayTrace "    CHARMAP : ${CHARMAP}" 

        FILE_CHARMAPS_PLATFORM="${DIR_SUPPORT}/charmaps_${PLATFORM}.txt"
###        FILE_LANGUAGE_MAPPINGS="${DIR_SUPPORT}/locales_languages.txt"
        
        # Checking charmap (codeset) from locale with the charmap mapping table
        # specific to the platform
        
        
        if [ -f "${FILE_CHARMAPS_PLATFORM}" ]
        then
            ENTRY_CHARMAPS=`${GREP} '^'"${CHARMAP}"':' "${FILE_CHARMAPS_PLATFORM}"`

            if [ $? -eq 0 ]
            then
                # Charmap supported 
                DisplayTrace "    Supported charmap (encoding)"

                LABEL_CHARMAP=`echo "${ENTRY_CHARMAPS}" | ${CUT} -d ':' -f2`

                # Assumed that a charmap label does not contain any spaces 
                LABEL_CHARMAP=`echo "${LABEL_CHARMAP}" | ${TR} -d ' '`

                # Checking language to use from locale

                echo ""

                case "${LANG=}" in
                "C"|"POSIX")
                    LANGUAGE="en"
                    DisplayTrace "    Language set to ${LANGUAGE}"

                    TERRITORY=""

                    DisplayTrace "    LANGUAGE           : \"${LANGUAGE}\" "
                    DisplayTrace "    TERRITORY          : \"${TERRITORY}\" "
                    ;;

                "")
                    DisplayTrace "    Unidentified language"
                    LANGUAGE="en"
                    DisplayTrace "    Language set to ${LANGUAGE}"

                    TERRITORY=""

                    DisplayTrace "    LANGUAGE           : \"${LANGUAGE}\" "
                    DisplayTrace "    TERRITORY          : \"${TERRITORY}\" "
                    ;;

                *)
###                 LANG_PREFIX=`echo "${LANG}" | ${CUT} -c1-2`
###                 LANG_PREFIX=`ConvertToLowerCase "${LANG_PREFIX}"`
### 
### 
###                 DisplayTrace "    LANG_PREFIX : \"${LANG_PREFIX}\" "
                
###                 # Looking for supported language by installer
###                 
###                 LINE_FOUND=`${GREP} '^'"${LANG_PREFIX}:" "${FILE_LANGUAGE_MAPPINGS}"`
###                 if [ $? -eq 0 ]
###                 then
###                     DisplayTrace "    Supported language"
###                     
###                     LANGUAGE=`echo "${LINE_FOUND}" | ${CUT} -d":" -f2`
###                 else
###                     DisplayTrace "    Unidentified or unsupported language"
###                     LANGUAGE="en"
###                     DisplayTrace "    Language set to ${LANGUAGE}"
###                     
###                     
###                     # A faire peut-etre : textes en anglais dans l'encodage etranger 
###                     # possible ?
###                     
###                     LABEL_CHARMAP="ASCII"
###                     DisplayTrace "    Charmap (encoding) reset to ASCII"
###                     DisplayTrace "    LABEL_CHARMAP : \"${LABEL_CHARMAP}\" "
###                 fi    
                
                
                    # Extraction of language/territory component from locale name

                    echo "${LANG}" | ${GREP} '\.' >/dev/null 2>&1
                    if [ $? -eq 0 ]
                    then
                        LANGUAGE_TERRITORY=`echo "${LANG}" | ${CUT} -d'.' -f1`
                    else
                        echo "${LANG}" | ${GREP} '@' >/dev/null 2>&1
                        if [ $? -eq 0 ]
                        then
                            LANGUAGE_TERRITORY=`echo "${LANG}" | ${CUT} -d'@' -f1`
                        else    
                            LANGUAGE_TERRITORY="${LANG}"
                        fi
                    fi
                    DisplayTrace "    LANGUAGE_TERRITORY : \"${LANGUAGE_TERRITORY}\" "


                    echo "${LANGUAGE_TERRITORY}" | ${GREP} '_' >/dev/null 2>&1
                    if [ $? -eq 0 ]
                    then
                        # "language_territory" form

                        LANGUAGE=`echo "${LANGUAGE_TERRITORY}" | ${CUT} -d'_' -f1`
                        TERRITORY=`echo "${LANGUAGE_TERRITORY}" | ${CUT} -d'_' -f2`
                    else
                        # "othername" form
                        # Arbitrary language name extracted from the first two characters
                        # These two characters will be used to match the language
                        # 
                        # "japanese" --> "ja" (like "ja_JP") 
                        # Problem with some languages:
                        # "swedish" --> "sw" (and not "sv" from "sv_SV" !)
                        # Use of ${FILE_LANGUAGE_MAPPINGS} file: to do

                        LANGUAGE=`echo "${LANGUAGE_TERRITORY}" | ${CUT} -c1-2`
                        TERRITORY=""
                    fi

                    # Conversion to lowercase in order to access installer message
                    # files that have uniform filenames
                    DisplayTrace "    LANGUAGE           : \"${LANGUAGE}\" "
                    LANGUAGE=`ConvertToLowerCase "${LANGUAGE}"`

                    DisplayTrace "    LANGUAGE           : \"${LANGUAGE}\" "
                    DisplayTrace "    TERRITORY          : \"${TERRITORY}\" "
                    ;;

                esac


            else    
                # Unsupported charmap (encoding)
                DisplayTrace "    Unsupported charmap (encoding)"
                DisplayTrace "    Using ASCII charmap (encoding)"

                LABEL_CHARMAP="ASCII"

                LANGUAGE="en"
                DisplayTrace "     Language set to ${LANGUAGE}"

                TERRITORY=""

                DisplayTrace "    LANGUAGE           : \"${LANGUAGE}\" "
                DisplayTrace "    TERRITORY          : \"${TERRITORY}\" "
            fi
        else
            # Charmap file not found
            DisplayTrace "    Charmap file not found"
            DisplayTrace "    Using ASCII charmap (encoding)"

            LABEL_CHARMAP="ASCII"

            LANGUAGE="en"
            DisplayTrace "     Language set to ${LANGUAGE}"

            TERRITORY=""

            DisplayTrace "    LANGUAGE           : \"${LANGUAGE}\" "
            DisplayTrace "    TERRITORY          : \"${TERRITORY}\" "
        fi

        DisplayTrace ""
        DisplayTrace "    LABEL_CHARMAP : \"${LABEL_CHARMAP}\" "
        DisplayTrace "    LANGUAGE      : \"${LANGUAGE}\" "


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            # Initialization of message directory to use
            DIR_LOCALIZED_MESSAGES="${DIR_MESSAGES}/${LABEL_CHARMAP}"

            if [ ! -d "${DIR_LOCALIZED_MESSAGES}" ]
            then
                # This case can occur for languages for which there is a translated
                # license text file, but not a translated messages file
                DisplayTrace "    Directory ${DIR_LOCALIZED_MESSAGES} not found"
            fi
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            FILE_MESSAGES="${DIR_LOCALIZED_MESSAGES}/${LABEL_CHARMAP}.${LANGUAGE}_${TERRITORY}.message.txt"

            if [ ! -f "${FILE_MESSAGES}" ]
            then
                # If fully localized (language and territory) message file not
                # found, fallback to localized messages (same charmap)
                DisplayTrace "    File ${FILE_MESSAGES} not found"

                FILE_MESSAGES="${DIR_LOCALIZED_MESSAGES}/${LABEL_CHARMAP}.${LANGUAGE}.message.txt"

                if [ ! -f "${FILE_MESSAGES}" ]
                then
                    # If localized (language) message file not found, fallback 
                    # to english messages (same charmap)
                    DisplayTrace "    File ${FILE_MESSAGES} not found"
                    
                    # Do not change "LANGUAGE" variable in order to use it 
                    # intact in ShowLicense () (Message file missing but license file 
                    # may be available)
                    _LANGUAGE="en"
                    
                    # DisplayTrace "    Language set to \"${LANGUAGE}\" "
                    # DisplayTrace "    LANGUAGE      : \"${LANGUAGE}\" "

                    DisplayTrace "    Using language \"${_LANGUAGE}\" "

                    FILE_MESSAGES="${DIR_LOCALIZED_MESSAGES}/${LABEL_CHARMAP}.${_LANGUAGE}.message.txt"

                    if [ ! -f "${FILE_MESSAGES}" ]
                    then
                        # If english message file not found, fallback 
                        # to default english messages
                        DisplayTrace "    File ${FILE_MESSAGES} not found"
                        
                        FILE_MESSAGES="${DIR_MESSAGES}/msgs.txt"

                        if [ ! -f "${FILE_MESSAGES}" ]
                        then
                            DisplayTrace "    File ${FILE_MESSAGES} not found"

                            DisplayTraceAndError "File ${FILE_MESSAGES} not found"
                            RETURN_CODE=1
                        else
                            DisplayTrace "    File ${FILE_MESSAGES} found"
                        fi
                    else
                        DisplayTrace "    File ${FILE_MESSAGES} found"
                    fi
                else
                    DisplayTrace "    File ${FILE_MESSAGES} found"
                fi
            else
                DisplayTrace "    File ${FILE_MESSAGES} found"
            fi
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            DisplayTrace "    DIR_LOCALIZED_MESSAGES : \"${DIR_LOCALIZED_MESSAGES}\" "    
            DisplayTrace "    FILE_MESSAGES          : \"${FILE_MESSAGES}\" "    
        fi
    fi
    
    return ${RETURN_CODE}
}


##############################################################################
# DisplayMessage
##############################################################################

# Affichage message localise avec parametres (sans positionnement dans le message)

# Pas de controle possible du nombre de parametres d'un message donne
#

DisplayMessage ()
{

    # Controle agruments d'appel a faire
    # echo "1 : \"$1\" "
    # echo "2 : \"$2\" "
    # echo "3 : \"$3\" "
    # echo "4 : \"$4\" "
    # echo ""
    
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> DisplayMessage"
    
    

    if [ $# -le 1 ]
    then
        echo "Missing arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        MESSAGE_ID="$1"
        shift
        MESSAGE_DEFAULT="$1"
        shift


        MESSAGE=`${GREP} "^${MESSAGE_ID}" "${FILE_MESSAGES}" | ${CUT} -d: -f2-`
        if [ -z "${MESSAGE}" ] 
        then
            MESSAGE="${MESSAGE_DEFAULT}"
        fi

        # echo "MESSAGE original : \"${MESSAGE}\" "
        # echo ""
        
        # On some shells, @ is unset (and not empty) so an error 
        # "@: parameter not set" is raised.
        if [ $# -eq 0 ]
        then
            ${PRINTF} "${MESSAGE}\n"
        else
            ${PRINTF} "${MESSAGE}\n" "$@"
        fi
        
    fi
    
    return ${RETURN_CODE}
}


##############################################################################
# GetMessage
##############################################################################

# Recuperation message localise (sans affichage du message ni sans substitution
# des parametres)

# Pas de controle possible du nombre de parametres d'un message donne
#

GetMessage ()
{

    # Controle agruments d'appel a faire
    # echo "1 : \"$1\" "
    # echo "2 : \"$2\" "
    # echo "3 : \"$3\" "
    # echo "4 : \"$4\" "
    # echo ""
    
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> GetMessage"
    

    if [ $# -le 1 ]
    then
        echo "Missing arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        MESSAGE_ID="$1"
        shift
        MESSAGE_DEFAULT="$1"
        shift


        MESSAGE=`${GREP} "^${MESSAGE_ID}" "${FILE_MESSAGES}" | ${CUT} -d: -f2-`
        if [ -z "${MESSAGE}" ] 
        then
            MESSAGE="${MESSAGE_DEFAULT}"
        fi

        # echo "MESSAGE original : \"${MESSAGE}\" "
        # echo ""

        echo "${MESSAGE}"
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# VerifyActivationContext
###########################################################################

# 
# Verification of license activation context
#

VerifyActivationContext ()
{
    RETURN_CODE=0
    RETURN_STRING=""
    
    
    DisplayTrace ""
    DisplayTrace ">>> VerifyActivationContext"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else
    
        case "${SUB_CONTEXT}" in
        "COMPLETE")
            ;;
            
        "FIXPAK")
            DisplayMessage "INZP061E" "License activation can be enabled for a complete installation only.\n"
            DisplayTraceAndError "    License activation not possible for a Fixpak installation"
            RETURN_CODE=1
            ;;
            
        *)
            DisplayError "Invalid value for PRODUCT_INSTALLER_TYPE : \"${PRODUCT_INSTALLER_TYPE}\" "
            ;;
        esac


        if [ "${RETURN_CODE}" -eq 0 ]
        then
        
            case `ConvertToUpperCase "${PRODUCT_LICENCE_TYPE}"` in
            "FULL"|"WORKGROUP")
                ;;

            "TRY_AND_BUY")
                DisplayMessage "INZP062E" "License activation cannot be enabled with a Try And Buy version.\n"
                DisplayTraceAndError "    License activation not possible with a Try And Buy package"
                RETURN_CODE=1
                ;;

            *)
                DisplayError "Invalid value for PRODUCT_LICENCE_TYPE : \"${PRODUCT_LICENCE_TYPE}\" "
                ;;
            esac
        
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
        
            if [ -d "${DIR_INSTALLATION}" ]
            then
                # Installation directory already existing: it must contain a T&B version
                
                CheckEmptyDirectory "${DIR_INSTALLATION}" 
                if [ $? -eq 0 ]
                then
                    # Empty directory : activation impossible
                    RETURN_CODE=1
                else
                    CheckExistingInstallation
                    RETURN_CODE=$?
                    
                    if [ "${RETURN_CODE}" -eq 0 ]
                    then
                    
                        case `ConvertToUpperCase "${EXISTING_PRODUCT_LICENCE_TYPE}"` in
                        "FULL"|"WORKGROUP")
                            RETURN_CODE=1
                            ;;

                        "TRY_AND_BUY")
                            ;;

                        *)
                            DisplayError "Invalid value for PRODUCT_LICENCE_TYPE : \"${PRODUCT_LICENCE_TYPE}\" "
                            ;;
                        esac
                        
                    fi
                fi
            else
                # Directory unexistent : activation impossible
                RETURN_CODE=1
            fi

            if [ "${RETURN_CODE}" -ne 0 ]
            then
                DisplayMessage "INZP063E" "License activation must be enabled against a directory where a Try And Buy version of the same level is already installed.\n"
                DisplayTraceAndError "    Installation directory not suitable for a license activation"
                RETURN_CODE=1
            fi
            
        fi        
        
    fi
    
    
    return ${RETURN_CODE}
}


###########################################################################
# ShowProductInformation
###########################################################################

# 
# Display of product information
#
# Display of:
# - Product description and licence
# - Installation type
#

ShowProductInformation ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> ShowProductInformation"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    


        case "${CONTEXT}" in
        "inst")
            DisplayMessage "INZP030I" "This script will install %s on your computer" "${PRODUCT_DESCRIPTION}"
            echo ""
            _TEXT_=`GetMessage "INZP034I" "Installation"`
            ;;

        "uninst")
            DisplayMessage "INZP032I" "This script will uninstall %s on your computer" "${PRODUCT_DESCRIPTION}"
            echo ""
            _TEXT_=`GetMessage "INZP035I" "Uninstallation"`
            ;;

        *)
            echo "Unknown context \"${CONTEXT}\" " 1>&2
            RETURN_CODE=1
            ;;
        esac

        echo ""
        echo "-------------------------------------------------------------------------------"
        echo ""
        
        case `ConvertToUpperCase "${PRODUCT_LICENCE_TYPE}"` in
        "FULL"|"TRY_AND_BUY"|"WORKGROUP")
            echo "${PRODUCT_NAME} ${PRODUCT_VERSION} (${PRODUCT_LICENCE_TYPE})"
            ;;
            
        "NONE")
            echo "${PRODUCT_NAME} ${PRODUCT_VERSION}"
            ;;
            
        *)
            DisplayError "Invalid value for PRODUCT_LICENCE_TYPE : \"${PRODUCT_LICENCE_TYPE}\" "
            ;;
        esac
        
        case `ConvertToUpperCase "${PRODUCT_INSTALLER_TYPE}"` in
        "COMPLETE")
            _TEXT_3=`GetMessage "INZP037I" "${PRODUCT_INSTALLER_TYPE}"`
            ;;
            
        "FIXPAK")
            _TEXT_3=`GetMessage "INZP038I" "${PRODUCT_INSTALLER_TYPE}"`
            ;;
            
        *)
            DisplayError "Invalid value for PRODUCT_INSTALLER_TYPE : \"${PRODUCT_INSTALLER_TYPE}\" "
            ;;
        esac
        
        DisplayMessage "INZP039I" "%s installation" "${_TEXT_3}"
        echo ""
        echo "IBM Corp."
        echo "http://www.ibm.com/software/data/db2imstools"
        
        echo ""
        
        if [ "${CONTEXT}" = "inst" ]
        then
            if [ "${SUB_CONTEXT}" = "COMPLETE" ]
            then
                DisplayMessage "INZP036I" "Installation directory: %s\nThis is the directory in which the product is going to be installed." \
                               "${DIR_INSTALLATION}"
            else
                DisplayMessage "INZP050I" "Installation directory: %s\nThis is the directory in which the product is going to be updated. You must specify the installation directory of the existing copy." \
                               "${DIR_INSTALLATION}"
            fi
        else
            DisplayMessage "INZP051I" "Installation directory: %s\nThis is the directory from which the product is going to be removed." \
                           "${DIR_INSTALLATION}"
        fi
        echo ""

        if [ "${CONTEXT}" = "inst" ]
        then
            if [ "${LICENSE_ACTIVATION}" = "yes" ]
            then
                DisplayMessage "INZP060I" "License activation enabled on %s installation directory.\n" \
                               "${DIR_INSTALLATION}"
            fi
        fi
        
        if [ "${SILENT_INSTALLATION}" = "no" ]
        then
            # Technique for getting updated variables values when the "while" 
            # loop exits
            exec 9<&0 

            _ANSWER="XX"

            while [ "${_ANSWER}" != "1" -a "${_ANSWER}" != "2" ]
            do
                DisplayMessage "INZP006I" "Enter 1 for next, 2 to cancel"
                read  _ANSWER

                case "${_ANSWER}" in
                "1")
                    echo ""
                    :
                    ;;

                "2")
                    echo ""
                    DisplayMessage "INZP031I" "%s cancelled by user request" "${_TEXT_}"
                    DisplayTrace "    Installation/uninstallation cancelled by user request" 
                    RETURN_CODE=1
                    ;;

                *)
                    DisplayTrace "Unknown answer"
                    ;;
                esac
            done

            exec 0<&9 9<&-
        fi

    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# ShowLicense
###########################################################################

# 
# Display and validation of license
#
#
#
# >>> Attention : licences differentes Full, T&B, WGE, ....
# Les fichiers de licences varieront selon le type de package installe (fait lors
# du packaging
#
#

ShowLicense ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> ShowLicense"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        case `ConvertToUpperCase "${PRODUCT_LICENCE_TYPE}"` in
        "FULL")
            _LICENSE_FILE_SUFFIX="full"
            ;;
            
        "TRY_AND_BUY")
            _LICENSE_FILE_SUFFIX="tab"
            ;;
            
        "WORKGROUP")
            _LICENSE_FILE_SUFFIX="wge"
            ;;
            
        *)
            DisplayError "Invalid value for PRODUCT_LICENCE_TYPE : \"${PRODUCT_LICENCE_TYPE}\" "
            ;;
        esac


        if [ "${RETURN_CODE}" -eq 0 ]
        then
           # Initialization of license directory to use
           DIR_LOCALIZED_LICENSES="${DIR_LICENSES}/${LABEL_CHARMAP}"
        fi
        

        if [ "${RETURN_CODE}" -eq 0 ]
        then
            FILE_LICENSE="${DIR_LOCALIZED_LICENSES}/${LABEL_CHARMAP}.${LANGUAGE}_${TERRITORY}.license.${_LICENSE_FILE_SUFFIX}.txt"

            if [ ! -f "${FILE_LICENSE}" ]
            then
                # If fully localized (language and territory) license file
                # not found, fallback to localized license (same charmap)
                DisplayTrace "    File ${FILE_LICENSE} not found"

                FILE_LICENSE="${DIR_LOCALIZED_LICENSES}/${LABEL_CHARMAP}.${LANGUAGE}.license.${_LICENSE_FILE_SUFFIX}.txt"

                if [ ! -f "${FILE_LICENSE}" ]
                then
                    # If localized (language) license file not found,  
                    # fallback to english license (ASCII charmap)

                    DisplayTrace "    File ${FILE_LICENSE} not found"
                    _LANGUAGE="en"
                    _LABEL_CHARMAP="ASCII"
                    DisplayTrace "    Looking for license file with language \"${_LANGUAGE}\" and charmap \"${_LABEL_CHARMAP}\" "

                    FILE_LICENSE="${DIR_LICENSES}/${_LABEL_CHARMAP}/${_LABEL_CHARMAP}.${_LANGUAGE}.license.${_LICENSE_FILE_SUFFIX}.txt"

                    if [ ! -f "${FILE_LICENSE}" ]
                    then
                        DisplayTraceAndError "File ${FILE_LICENSE} not found"
                        DisplayMessage "INZP019E" "License file not found"
                        RETURN_CODE=1
                    else
                        DisplayTrace "    File ${FILE_LICENSE} found"
                    fi
                else
                    DisplayTrace "    File ${FILE_LICENSE} found"
                fi
            else
                DisplayTrace "    File ${FILE_LICENSE} found"
            fi
        fi
        
        
        if [ "${RETURN_CODE}" -eq 0 ]
        then 
            # Technique for getting updated variables values when the "while" 
            # loop exits
            exec 9<&0 
            
            _ANSWER="XX"
            
            while [ "${_ANSWER}" != "1" -a "${_ANSWER}" != "2" ]
            do
                ${MORE} -d "${FILE_LICENSE}"
                DisplayMessage "INZP004I" "Enter 1 to accept the agreement, 2 to decline it"
                read  _ANSWER

                case "${_ANSWER}" in
                "1")
                    :
                    ;;
                    
                "2")
                    DisplayMessage "INZP005I" "You have chosen to decline the license agreement"
                    DisplayTrace "    Installation cancelled by user request (license decline)"
                    RETURN_CODE=1
                    ;;
                
                *)
                    DisplayTrace "Unknown answer"
                    ;;
                
                esac

            done
            exec 0<&9 9<&-

            # DisplayTrace "_ANSWER : \"${_ANSWER}\" "
        fi
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CalculatePackageSize
###########################################################################

# 
# Calculation of size of files to be installed
#
#
#
#
#


CalculatePackageSize ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> CalculatePackageSize"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    


###         for _FILE_NAME in ${LIST_ARCHIVES_FILES}
###         do
###             _FILE_SIZE=`${LS} -l "${_FILE_NAME}" | ${AWK} '{print $5}'`
###             
###             DisplayTrace "File : ${_FILE_NAME}"
###             DisplayTrace "Size : ${_FILE_SIZE}"
###         
###         done

        # Risque depassement capacite ligne si nombre de fichiers important
        # Peut-etre depassement addition 
        # Autres methodes addition : 
        # - Avec expr 
        # - Avec operateurs ksh
        
        SIZE_COMPONENTS_FILES=`${LS} -l ${LIST_COMPONENTS_FILES} | ${AWK} 'BEGIN {SUM=0} {SUM=SUM+$5} END {printf "%ld" , SUM}'`
        SIZE_COMPONENTS_FILES=`${EXPR} ${SIZE_COMPONENTS_FILES} "/" 1024`
        DisplayTrace "    SIZE_COMPONENTS_FILES : ${SIZE_COMPONENTS_FILES}"


        SIZE_INSTALLER_FILES=`${LS} -l ${FILE_INSTALLER_ARCHIVE} | ${AWK} 'BEGIN {SUM=0} {SUM=SUM+$5} END {printf "%ld" , SUM}'`
        SIZE_INSTALLER_FILES=`${EXPR} ${SIZE_INSTALLER_FILES} "/" 1024`
        DisplayTrace "    SIZE_INSTALLER_FILES  : ${SIZE_INSTALLER_FILES}"


        SIZE_UNINSTALLER_FILES=`${LS} -l ${FILE_UNINSTALLER_ARCHIVE} | ${AWK} 'BEGIN {SUM=0} {SUM=SUM+$5} END {printf "%ld" , SUM}'`
        SIZE_UNINSTALLER_FILES=`${EXPR} ${SIZE_UNINSTALLER_FILES} "/" 1024`
        DisplayTrace "    SIZE_UNINSTALLER_FILES  : ${SIZE_UNINSTALLER_FILES}"

        # Size required in installation directory :
        # - Size components (feature) archive(s)
        # - Size files extracted from installer archive 
        # - Size files extracted from components (feature) archive(s)
        #
        # Simple calculation of total size of extracted files from an archive : size of  
        # the archive that contains them
        
        SIZE_TOTAL_REQUIRED=`${EXPR} ${SIZE_COMPONENTS_FILES} "+" ${SIZE_INSTALLER_FILES}`
        SIZE_TOTAL_REQUIRED=`${EXPR} ${SIZE_TOTAL_REQUIRED} "+" ${SIZE_UNINSTALLER_FILES}`
        SIZE_TOTAL_REQUIRED=`${EXPR} ${SIZE_TOTAL_REQUIRED} "+" ${SIZE_COMPONENTS_FILES}`
        SIZE_TOTAL_REQUIRED_DISPLAY=`echo "${SIZE_TOTAL_REQUIRED}" | ${SED} -e ':l' -e 's/\(.*[^,]\)\([0-9]\{3\}\)/\1,\2/' -e 't l' | ${SED} -e 's/,/ /g'`

        # DisplayTrace "    SIZE_TOTAL_REQUIRED : ${SIZE_TOTAL_REQUIRED} (${SIZE_TOTAL_REQUIRED_DISPLAY} bytes)"
        DisplayTrace "    SIZE_TOTAL_REQUIRED : ${SIZE_TOTAL_REQUIRED} (${SIZE_TOTAL_REQUIRED_DISPLAY} Kbytes)"
    fi

    return ${RETURN_CODE}
}


###########################################################################
# CalculateFreeDiskSpace
###########################################################################

# 
# ### Calculation of free disk space (filesystem) in bytes
# Calculation of free disk space (filesystem) in Kbytes
#
# Argument 1 : directory in the filesystem to check
#
# The free disk space is from the filesystem where the directory is located
#
#


CalculateFreeDiskSpace ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> CalculateFreeDiskSpace"
    

    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        _DIRECTORY=$1
        
        
        if [ -d "${_DIRECTORY}" ]
        then
            # The removal of "\n" is made in order to take into account a df output of 2 lines or more.
            # The df output is in two lines when the file system name is very long.
            SIZE_FREE_DISK=`${DF} "${DF_OPTIONS}" "${_DIRECTORY}" | ${SED} -e '1d' | ${TR} '\n' ' ' | ${AWK} '{print $4}'`
        
            DisplayTrace "    SIZE_FREE_DISK : ${SIZE_FREE_DISK} KB"

            # SIZE_FREE_DISK=`${EXPR} ${SIZE_FREE_DISK} "*" 1024`
            SIZE_FREE_DISK_DISPLAY=`echo "${SIZE_FREE_DISK}" | ${SED} -e ':l' -e 's/\(.*[^,]\)\([0-9]\{3\}\)/\1,\2/' -e 't l' | ${SED} -e 's/,/ /g'`
            
            # DisplayTrace "    SIZE_FREE_DISK : ${SIZE_FREE_DISK} (${SIZE_FREE_DISK_DISPLAY} bytes)"
            DisplayTrace "    SIZE_FREE_DISK : ${SIZE_FREE_DISK} (${SIZE_FREE_DISK_DISPLAY} Kbytes)"
        else
            DisplayError "Directory ${_DIRECTORY} not found"
            RETURN_CODE=1
        fi

        unset _DIRECTORY
    fi

    return ${RETURN_CODE}
   
}


###########################################################################
# CheckFreeDiskSpace
###########################################################################

# 
# Check if there is enough free disk space in the filesystem where the installation
# directory is located
#
# Argument 1 : number of bytes required
# Argument 2 : number of free (available) bytes
#
#
#


CheckFreeDiskSpace ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> CheckFreeDiskSpace"
    

    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        _QUANTITY_REQUIRED=$1
        _QUANTITY_FREE=$2
    
        # Controle valeur numerique a faire 

        _QUANTITY_REQUIRED_DISPLAY=`echo "${_QUANTITY_REQUIRED}" | ${SED} -e ':l' -e 's/\(.*[^,]\)\([0-9]\{3\}\)/\1,\2/' -e 't l' | ${SED} -e 's/,/ /g'`
        _QUANTITY_FREE_DISPLAY=`echo "${_QUANTITY_FREE}" | ${SED} -e ':l' -e 's/\(.*[^,]\)\([0-9]\{3\}\)/\1,\2/' -e 't l' | ${SED} -e 's/,/ /g'`
        DisplayTrace "    _QUANTITY_REQUIRED_DISPLAY : ${_QUANTITY_REQUIRED_DISPLAY}"
        DisplayTrace "    _QUANTITY_FREE_DISPLAY     : ${_QUANTITY_FREE_DISPLAY}"

    
        if [ ${_QUANTITY_REQUIRED} -ge ${_QUANTITY_FREE} ]
        then
            DisplayMessage "INZP040E" "Unsufficient free disk space in file system"
            # DisplayMessage "INZP041E" "Required space  : %s bytes" "${_QUANTITY_REQUIRED_DISPLAY}"
            # DisplayMessage "INZP042E" "Available space : %s bytes" "${_QUANTITY_FREE_DISPLAY}"
            DisplayMessage "INZP041E" "Required space  : %s Kbytes" "${_QUANTITY_REQUIRED_DISPLAY}"
            DisplayMessage "INZP042E" "Available space : %s Kbytes" "${_QUANTITY_FREE_DISPLAY}"

            DisplayTraceAndError "Unsufficient free disk space in file system "
            # DisplayTraceAndError "Required space  : ${_QUANTITY_REQUIRED_DISPLAY} bytes"
            # DisplayTraceAndError "Available space : ${_QUANTITY_FREE_DISPLAY} bytes"
            DisplayTraceAndError "Required space  : ${_QUANTITY_REQUIRED_DISPLAY} Kbytes"
            DisplayTraceAndError "Available space : ${_QUANTITY_FREE_DISPLAY} Kbytes"
            
            RETURN_CODE=7
        else
            DisplayTrace "    Sufficient free disk space"
        fi
        
        unset _QUANTITY_REQUIRED
        unset _QUANTITY_FREE
        unset _QUANTITY_REQUIRED_DISPLAY
        unset _QUANTITY_FREE_DISPLAY
    fi

    return ${RETURN_CODE}
}


###########################################################################
# InitComponentFilesVariables
###########################################################################

# 
# Initialisation of component/feature variables
# 
# Installation : from product/component name, determines features to install
#
# Uninstallation : from features present, determines product/component to
# uninstall
# 
#

InitComponentFilesVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitComponentFilesVariables"


    DisplayTrace ""

    COMPONENT_FEATURES_MAPPING=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}setup" "component_features_mapping"` 
    RETURN_CODE=$?


    DisplayTrace "    COMPONENT_FEATURES_MAPPING : ${COMPONENT_FEATURES_MAPPING}"


    if [ ${RETURN_CODE} -eq 0 ]
    then
        LIST_COMPONENTS_MAPPING=`echo "${COMPONENT_FEATURES_MAPPING}" | ${TR} '|' '\n'`

        DisplayTrace "    LIST_COMPONENTS_MAPPING : ${LIST_COMPONENTS_MAPPING}"
    fi


    case "${CONTEXT}" in

    "inst")    
        # Find features names to install from component product name

        if [ ${RETURN_CODE} -eq 0 ]
        then
            # List of features to be installed
            LIST_FEATURES_INSTALL=`echo "${LIST_COMPONENTS_MAPPING}" | ${GREP} "${PRODUCT_COMPONENT_NAME}" \
                                   | ${CUT} -d ':' -f2- | ${TR} ',' ' '`

            DisplayTrace "    List of features to install :"
            DisplayTrace "    LIST_FEATURES_INSTALL : ${LIST_FEATURES_INSTALL}"


            # List of remaining features (not to be installed)
            # (Used to create variables file)
            LIST_FEATURES_NO_INSTALL=`echo "${LIST_COMPONENTS_MAPPING}" | ${GREP} -v "${PRODUCT_COMPONENT_NAME}" \
                                   | ${CUT} -d ':' -f2- | ${TR} ',' ' ' | ${TR} '\n' ' '`


            # In order to find remaining features (i.e. features that won't be installed) :
            # - Retrieve list of remaining components
            # - Get list of features from theses components
            # - Purge list of features of the features that are selected fo install
            #   The purge is necessary because components may include common features and are not exclusive 
            #   
            # Lists have to be prepared : no duplicate values, sorted, and no empty lines

            echo "${LIST_FEATURES_INSTALL}" | ${TR} ' ' '\n' | ${SORT} -u > "${DIR_LAUNCH}/liste_install"
            echo "${LIST_FEATURES_NO_INSTALL}" | ${TR} ' ' '\n' | ${SORT} -u | ${SED} -e '/^$/d' > "${DIR_LAUNCH}/liste_no_install.tmp"

            # A FAIRE : Utiliser noms de fichiers temporaires (avec suppression apres utilisation)

            ${COMM} -13 "${DIR_LAUNCH}/liste_install" "${DIR_LAUNCH}/liste_no_install.tmp" > "${DIR_LAUNCH}/liste_no_install"
            LIST_FEATURES_NO_INSTALL=`${SED} -e 's/^ //' -e 's/ $//' "${DIR_LAUNCH}/liste_no_install" | ${TR} '\n' ' ' ` 

            \rm "${DIR_LAUNCH}/liste_install"
            \rm "${DIR_LAUNCH}/liste_no_install.tmp"
            \rm "${DIR_LAUNCH}/liste_no_install"

            DisplayTrace "    LIST_FEATURES_INSTALL    : \"${LIST_FEATURES_INSTALL}\" "
            DisplayTrace "    LIST_FEATURES_NO_INSTALL : \"${LIST_FEATURES_NO_INSTALL}\" "
        fi


        # Check if corresponding archive files exist

        FILE_ARCHIVES=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}archive_files" "file_archives"` 

        DisplayTrace "    FILE_ARCHIVES : ${FILE_ARCHIVES}"

        # Verification presence fichiers d'archive (uniquement pour les features a installer)
        if [ ${RETURN_CODE} -eq 0 ]
        then
            # List of package archives to be installed (core/network or plugin)
            LIST_COMPONENTS_FILES=""
            
            for NAME in ${LIST_FEATURES_INSTALL}
            do
                FILE=`GetFieldSublist ${FILE_ARCHIVES} '|' ':' ${NAME} 2`

                DisplayTrace "    Feature : \"$NAME\" "
                DisplayTrace "    FILE    : \"${FILE}\" "

                CheckReadableFile ${FILE}
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -ne 0 ]
                then
                    break
                fi

                LIST_COMPONENTS_FILES="${LIST_COMPONENTS_FILES} ${FILE}"
            done

            DisplayTrace "    LIST_COMPONENTS_FILES : \"${LIST_COMPONENTS_FILES}\" "
        fi    
        ;;

        
    "uninst")    
        if [ ${RETURN_CODE} -eq 0 ]
        then
            # Find package component name from features list
            # The component name is not stored in the installation settings file.
            
            # List of features to be uninstalled : one feature by line, sorted, no empty lines
            echo "${LIST_FEATURES_UNINSTALL}" | ${TR} ' ' '\n' | ${SED} -e '/^ *$/d' | ${SORT} -u > "${DIR_SETUP}/list_uninstall"
            
            PRODUCT_COMPONENT_NAME=""
            for _NAME in ${LIST_COMPONENTS_MAPPING}
            do
                DisplayTrace "    _NAME : ${_NAME}"
                NAME_COMPONENT=`echo "${_NAME}" | ${CUT} -d: -f1`
                NAME_VALUE=`echo "${_NAME}" | ${CUT} -d: -f2-`
                
                DisplayTrace "    NAME_COMPONENT : \"${NAME_COMPONENT}\" "
                DisplayTrace "    NAME_VALUE     : \"${NAME_VALUE}\" "

                # List of features for the component, one feature by line, sorted
                echo "${NAME_VALUE}" | ${TR} ',' '\n' | ${SORT} -u > "${DIR_SETUP}/list_component"

                # >>> Mettre commande "cmp" dans fichier .INI <<<<<<<<

                \cmp -s "${DIR_SETUP}/list_uninstall" "${DIR_SETUP}/list_component" 
                if [ $? -eq 0 ]
                then
                    DisplayTrace "    Component found : \"${NAME_COMPONENT}\" "
                    PRODUCT_COMPONENT_NAME="${NAME_COMPONENT}"
                    break
                fi
            done

            DisplayTrace "    PRODUCT_COMPONENT_NAME : \"${PRODUCT_COMPONENT_NAME}\" "
            
            if [ "${PRODUCT_COMPONENT_NAME}" = "" ]
            then
                DisplayTraceAndError "    Product component name cannot be determined"
                RETURN_CODE=1
            fi
            
            ${RM} "${DIR_SETUP}/list_uninstall"
            ${RM} "${DIR_SETUP}/list_component"
        fi
        ;;

    *)
        echo "Unknown context \"${CONTEXT}\" " 1>&2
        RETURN_CODE=2    
        ;;
    esac

    return ${RETURN_CODE}
}


###########################################################################
# InitInstallerVariables
###########################################################################

# 
# 
# Loading (from .INI file) of data for installer script (${LPRODUCT}_inst.sh)
# 
# Some variables will be used when creating InstallShield variables file
# 
# 
#

InitInstallerVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitInstallerVariables"


    DIR_IS_LOG=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}setup" "dir_is_log"` 
    DisplayTrace "    DIR_IS_LOG     : ${DIR_IS_LOG}"


    DIR_IS_TEMP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}setup" "dir_is_temp"` 
    DisplayTrace "    DIR_IS_TEMP    : ${DIR_IS_TEMP}"


    DIR_IS_INSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}setup" "dir_is_install"` 
    DisplayTrace "    DIR_IS_INSTALL : ${DIR_IS_INSTALL}"

    DEFAULT_INSTALL_PREFIX_DIR="${DIR_IS_INSTALL}"


    BATCH_COMMAND=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}setup" "batch_command"` 
    DisplayTrace "    BATCH_COMMAND  : ${BATCH_COMMAND}"


    FILE_INSTALLER_ARCHIVE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}setup" "file_installer_archive"` 
    DisplayTrace "    FILE_INSTALLER_ARCHIVE : ${FILE_INSTALLER_ARCHIVE}"


    FILE_UNINSTALLER_ARCHIVE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}setup" "file_uninstaller_archive"` 
    DisplayTrace "    FILE_UNINSTALLER_ARCHIVE : ${FILE_UNINSTALLER_ARCHIVE}"

    return ${RETURN_CODE}
}


###########################################################################
# CopyInstallationFiles
###########################################################################

# 
# Copy of necessary files for the installation
#
# Argument 1 : 
# Argument 2 : 
#
#
#

CopyInstallationFiles ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> CopyInstallationFiles"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        DIR_CURRENT=`\pwd`
        
        \cd "${DIR_INSTALLATION}"
        RETURN_CODE=$?

        if [ "${RETURN_CODE}" -eq 0 ]
        then
            # Copy of installer files by unarchiving them directly in destination
            # directory

            DisplayTrace "    Extraction of installer files in ${DIR_INSTALLATION} directory"
            DisplayTrace "    FILE_INSTALLER_ARCHIVE : \"${FILE_INSTALLER_ARCHIVE}\" "

            if [ "${LICENSE_ACTIVATION}" = "no" ]
            then
                ACTIVATION_FILES=""
            else
                ACTIVATION_FILES="cfg swidtag"
            fi
            
            if [ "${PLATFORM}" = "hp" -o "${PLATFORM}" = "hpia" ]
            then
                ${TAR} xvf "${DIR_SETUP}/${FILE_INSTALLER_ARCHIVE}" ${ACTIVATION_FILES} >> "${FILE_TRACE}" 2>&1
            else    
                ${TAR} xvf "${DIR_SETUP}/${FILE_INSTALLER_ARCHIVE}" ${ACTIVATION_FILES} >> "${FILE_TRACE}" 2>> "${FILE_TRACE_ERROR}"
            fi
            RETURN_CODE=$?
            
            if [ "${LICENSE_ACTIVATION}" = "no" ]
            then
                if [ "${RETURN_CODE}" -ne 0 ]
                then
                    DisplayError "Error while extracting archive file \"${FILE_INSTALLER_ARCHIVE}\" "
                    DisplayMessage "INZP026E" "Unable to install in the %s directory." "${DIR_INSTALLATION}"
                    RETURN_CODE=1
                fi
            else
                if [ "${RETURN_CODE}" -ne 0 ]
                then
                    DisplayError "Error while extracting archive file \"${FILE_INSTALLER_ARCHIVE}\" "
                    DisplayMessage "INZP065E" "An error occurred during license activation.\n\nPlease check the file %s for details.\n" \
                                   "${DIR_INSTALLATION}/install/`basename ${FILE_TRACE_ERROR}`"
                    RETURN_CODE=1
                else
                    DisplayMessage "INZP064I" "License activation completed with success.\n"
                fi
            fi
        fi
        
        \cd "${DIR_CURRENT}"

        if [ "${LICENSE_ACTIVATION}" = "no" ]
        then
            if [ "${RETURN_CODE}" -eq 0 ]
            then
                \cd "${DIR_SETUP}"
                RETURN_CODE=$?
            fi


            if [ "${RETURN_CODE}" -eq 0 ]
            then
                # Copy of archive files (one by feature)
                DisplayTrace "    Copying components archive files to destination directory"

                for FILE in ${LIST_COMPONENTS_FILES}
                do
                    DisplayTrace "    Copying file ${FILE}"
                    ${CP} -p "${FILE}" "${DIR_INSTALLATION}/install" 
                    RETURN_CODE=$?
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error copying file \"${FILE}\" to \"${DIR_INSTALLATION}/install\" directory"
                        break
                    fi            
                done
            fi
        fi
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# FormatISPlatformName
###########################################################################

# 
# Formatting of platform name from product internal name into InstallShield (Java)
# format
#
# Argument 1 : 
# Argument 2 : 
#
#
#

FormatISPlatformName ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> FormatISPlatformName"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        case "${PLATFORM}" in
        "aix")
            IS_PLATFORM="AIX"
            IS_ARCHITECTURE="ppc"
            ;;

        "sun")
            IS_PLATFORM="SunOS"
            IS_ARCHITECTURE="sparc"
            ;;

        "sun_x86")
            IS_PLATFORM="SunOS"
            IS_ARCHITECTURE="i386"
            ;;

        "linux")
            IS_PLATFORM="Linux"
            IS_ARCHITECTURE="x86"
            ;;

        "ialinux")
            IS_PLATFORM="Linux"
            IS_ARCHITECTURE="ia64"
            ;;

        "plinux")
            IS_PLATFORM="Linux"
            IS_ARCHITECTURE="ppc64"
            ;;

        "plinuxle")
            IS_PLATFORM="Linux"
            IS_ARCHITECTURE="ppc64le"
            ;;

        "zlinux")
            IS_PLATFORM="Linux"
            IS_ARCHITECTURE="s390x"
            ;;

        "hp")
            IS_PLATFORM="HP-UX"
            IS_ARCHITECTURE="PA_RISC"
            ;;
        
        "hpia")
            IS_PLATFORM="HP-UX"
            IS_ARCHITECTURE="IA64N"
            ;;

        *)
            DisplayTraceAnfError "Unknown platform"
            IS_PLATFORM=""
            IS_ARCHITECTURE=""
            ;;

        esac
        
        DisplayTrace "    IS_PLATFORM     : \"${IS_PLATFORM}\" "
        DisplayTrace "    IS_ARCHITECTURE : \"${IS_ARCHITECTURE}\" "
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CreateVariablesFile
###########################################################################

# 
# Creation of variables file (is_variables/is_variables_uninstall)
#
#
#
# The variables file is transmitted to install script.
# With InstallShield packaging, this file is generated by the Installshield
# installer
#
#
# >>> Cas FixPak a faire
#


CreateVariablesFile ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> CreateVariablesFile"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            # Formatting of platform/architecture names into InstallShield 
            # compatible values
            
            FormatISPlatformName
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi


        # Determine feature list to put in this file 
        LIST_IS_FEATURES=""
        
        case "${CONTEXT}" in
        "inst")
            for NAME in ${LIST_FEATURES_INSTALL}
            do
                _TEMP_STRING="${NAME}:true:false"
                if [ "${LIST_IS_FEATURES}" = "" ]
                then
                    LIST_IS_FEATURES="${_TEMP_STRING}"
                else
                    LIST_IS_FEATURES="${LIST_IS_FEATURES}|${_TEMP_STRING}"
                fi
            done


            for NAME in ${LIST_FEATURES_NO_INSTALL}
            do
                _TEMP_STRING="${NAME}:false:false"
                if [ "${LIST_IS_FEATURES}" = "" ]
                then
                    LIST_IS_FEATURES="${_TEMP_STRING}"
                else
                    LIST_IS_FEATURES="${LIST_IS_FEATURES}|${_TEMP_STRING}"
                fi
            done
            ;;

        "uninst")
            for NAME in ${LIST_FEATURES_UNINSTALL}
            do
                _TEMP_STRING="${NAME}:true:true"
                if [ "${LIST_IS_FEATURES}" = "" ]
                then
                    LIST_IS_FEATURES="${_TEMP_STRING}"
                else
                    LIST_IS_FEATURES="${LIST_IS_FEATURES}|${_TEMP_STRING}"
                fi
            done

            for NAME in ${LIST_FEATURES_NO_UNINSTALL}
            do
                _TEMP_STRING="${NAME}:true:false"
                if [ "${LIST_IS_FEATURES}" = "" ]
                then
                    LIST_IS_FEATURES="${_TEMP_STRING}"
                else
                    LIST_IS_FEATURES="${LIST_IS_FEATURES}|${_TEMP_STRING}"
                fi
            done
            ;;

        *)
            echo "Unknown context \"${CONTEXT}\" " 1>&2
            RETURN_CODE=1
            ;;
        esac

        DisplayTrace "    LIST_IS_FEATURES : \"${LIST_IS_FEATURES}\" "
        
        
        # Determine filename to use
        
        # Cas FP ???
        # FixPak : filenames are the same
        case "${CONTEXT}" in
        "inst")
            FILE_IS_VARIABLES="is_variables"
            FILE_FULL_IS_VARIABLES="${DIR_INSTALLATION}/install/${FILE_IS_VARIABLES}"
            _MESSAGE="installer"
            ;;

        "uninst")
            FILE_IS_VARIABLES="is_variables_uninstall"
            # File is created in install directory (instead of uninstall directory - arbitrary choice)
            FILE_FULL_IS_VARIABLES="${DIR_INSTALLATION}/install/${FILE_IS_VARIABLES}"
            
            _MESSAGE="uninstaller"
            ;;

        *)
            echo "Unknown context \"${CONTEXT}\" " 1>&2
            RETURN_CODE=1
            ;;
        esac
        
        DisplayTrace "    FILE_IS_VARIABLES : \"${FILE_IS_VARIABLES}\" "


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            # Creation of file

            DisplayTrace "    Creation of file \"${FILE_IS_VARIABLES}\" "

            > "${FILE_FULL_IS_VARIABLES}"

            echo "#"                                       >> "${FILE_FULL_IS_VARIABLES}"
            echo "# @(#) Generated by ${_MESSAGE} script"  >> "${FILE_FULL_IS_VARIABLES}"
            echo "#"                                       >> "${FILE_FULL_IS_VARIABLES}" 
            echo ""                                        >> "${FILE_FULL_IS_VARIABLES}"
            echo "FEATURES=\"${LIST_IS_FEATURES}\""        >> "${FILE_FULL_IS_VARIABLES}"
            echo ""                                        >> "${FILE_FULL_IS_VARIABLES}"
            echo "LOCATION_PATH=${DIR_INSTALLATION}"       >> "${FILE_FULL_IS_VARIABLES}"
            echo "LOG=${DIR_IS_LOG}"                       >> "${FILE_FULL_IS_VARIABLES}"
            echo ""                                        >> "${FILE_FULL_IS_VARIABLES}"
            echo "PLATFORM=${IS_PLATFORM}"                 >> "${FILE_FULL_IS_VARIABLES}"
            echo "ARCHITECTURE=${IS_ARCHITECTURE}"         >> "${FILE_FULL_IS_VARIABLES}"
            echo "VERSION=${SYSTEM_VERSION}"               >> "${FILE_FULL_IS_VARIABLES}"
            echo ""                                        >> "${FILE_FULL_IS_VARIABLES}"
            echo "TEMPORARY_PATH=${DIR_IS_TEMP}"           >> "${FILE_FULL_IS_VARIABLES}"
            
            
            DisplayTrace ""
            DisplayTrace "    Contents of file \"${FILE_IS_VARIABLES}\" "
            DisplayTrace "    --------------------------------------------"
            
            ${CAT} "${FILE_FULL_IS_VARIABLES}" | while read _LINE
            do
                DisplayTrace "    ${_LINE}"
            done    
            DisplayTrace "    --------------------------------------------"
            DisplayTrace ""          
        fi
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# UpdateProductDatabase
###########################################################################

# 
# Add or remove entry in the prodcut database file
#
# Fonction obsolete a priori.
# Fonction remplacee par UpdateProductInformation ()
#
# Attention : l'installation complete du plugin in s'ajoute a une installation deja presente
# mais d'un autre "composant"
#
#
#


UpdateProductDatabase ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> UpdateProductDatabase"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    

        case "${CONTEXT}" in

        "inst")    
            case "${SUB_CONTEXT}" in
        
            "COMPLETE")
                AddProduct "${FILE_PRODUCT_DATABASE_DATA}" "${PRODUCT_NAME}" "${PRODUCT_COMPONENT_NAME}" \
                           "${PRODUCT_VERSION}" \
                           "${PRODUCT_DESCRIPTION}" \
                           "${DIR_INSTALLATION}"
                RETURN_CODE=$?
                DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"

                # Add informations to product settings file

                # >>> On suppose que les variables n'existent pas deja !!!!
                
                # File containing informations for uninstallation use
                FILE_INSTALL_SETTINGS=${DIR_INSTALLATION}/install/${LPRODUCT}_inst_settings.cfg

                AddSettingsVariable "PRODUCT_NAME" "${PRODUCT_NAME}" 
                # Pas de PRODUCT_COMPONENT_NAME : entre en conflit si deux components installes : perte unicite de la variable
                # AddSettingsVariable "PRODUCT_COMPONENT_NAME" "${PRODUCT_COMPONENT_NAME}"
                AddSettingsVariable "PRODUCT_VERSION" "${PRODUCT_VERSION}"
                AddSettingsVariable "PRODUCT_DESCRIPTION" "${PRODUCT_DESCRIPTION}"
                AddSettingsVariable "PRODUCT_OPERATING_SYSTEM" "${PRODUCT_OPERATING_SYSTEM}"
                AddSettingsVariable "PRODUCT_ARCHITECTURE" "${PRODUCT_ARCHITECTURE}"
                AddSettingsVariable "PRODUCT_INSTALLER_TYPE" "${PRODUCT_INSTALLER_TYPE}"
                AddSettingsVariable "PRODUCT_LICENCE_TYPE" "${PRODUCT_LICENCE_TYPE}" 
                ;;
            
            "FIXPAK")
                :
                ;;
        
            *)
                DisplayError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac

            ;;

    
        "uninst")    
            case "${SUB_CONTEXT}" in
        
            "COMPLETE")
###                 # Pas d'indication du numero de version actuellement
###                 RemoveProduct "${FILE_PRODUCT_DATABASE_DATA}" "${UPRODUCT}" "${PRODUCT_COMPONENT_NAME}" \
###                            '.*' \
###                            "${DIR_INSTALLATION}"
###                 RETURN_CODE=$?
###                 DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
                :
                ;;
            
            "FIXPAK")
                :
                ;;
        
            *)
                DisplayError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac

            ;;

        *)
            :
            ;;
        esac

    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# UpdateProductInformation
###########################################################################

# 
# Update product information 
#
# Replaces UpdateProductDatabase ()
#
#
#
# >>> Ajout des infos product dans le fichier ${LPRODUCT}_inst_settings.cfg inutile 
#     puisqu'il y a a present le fichier product_information.sig ????!!!
#
#
# At uninstallation time, there is no need to update any information because 
# there is no global product database (as it was the case with InstallShield)
#

UpdateProductInformation ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> UpdateProductInformation"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    

        case "${CONTEXT}" in
        "inst")    
            case "${SUB_CONTEXT}" in
        
            "COMPLETE")
                if [ "${LICENSE_ACTIVATION}" = "no" ]
                then
                    if [ "${RETURN_CODE}" -eq 0 ]
                    then 
                        # Copy of product signature file
                        DisplayTrace "    Copy of product signature file"
                        ${CP} -p "${FILE_PRODUCT_INFORMATION}" "${DIR_INSTALLATION}/install"
                        RETURN_CODE=$?

                        if [ "${RETURN_CODE}" -ne 0 ]
                        then
                            DisplayTraceAndError "Error while copying file \"${FILE_PRODUCT_INFORMATION}\" "
                        fi
                    fi

                    if [ "${RETURN_CODE}" -eq 0 ]
                    then 
                        DisplayTrace "    Change ownership of product signature file"
                        ${CHOWN} root:0 "${DIR_INSTALLATION}/install/`\basename ${FILE_PRODUCT_INFORMATION}`"
                        RETURN_CODE=$?

                        if [ "${RETURN_CODE}" -ne 0 ]
                        then
                            DisplayTraceAndError "Error while changing ownership of file \"${FILE_PRODUCT_INFORMATION}\" "
                        fi
                    fi

                    if [ "${RETURN_CODE}" -eq 0 ]
                    then 
                        # Add informations to PRODUCT settings file

                        # >>> On suppose que les variables n'existent pas deja !!!!

                        # File containing informations for uninstallation use
                        FILE_INSTALL_SETTINGS=${DIR_INSTALLATION}/install/${LPRODUCT}_inst_settings.cfg

                        AddSettingsVariable "PRODUCT_NAME" "${PRODUCT_NAME}" 
                        # Pas de PRODUCT_COMPONENT_NAME : entre en conflit si deux components installes : perte unicite de la variable
                        # AddSettingsVariable "PRODUCT_COMPONENT_NAME" "${PRODUCT_COMPONENT_NAME}"
                        AddSettingsVariable "PRODUCT_VERSION" "${PRODUCT_VERSION}"
                        AddSettingsVariable "PRODUCT_VERSION_REQUIRED" "${PRODUCT_VERSION_REQUIRED}"
                        AddSettingsVariable "PRODUCT_DESCRIPTION" "${PRODUCT_DESCRIPTION}"
                        AddSettingsVariable "PRODUCT_OPERATING_SYSTEM" "${PRODUCT_OPERATING_SYSTEM}"
                        AddSettingsVariable "PRODUCT_ARCHITECTURE" "${PRODUCT_ARCHITECTURE}"
                        AddSettingsVariable "PRODUCT_INSTALLER_TYPE" "${PRODUCT_INSTALLER_TYPE}"
                        AddSettingsVariable "PRODUCT_LICENCE_TYPE" "${PRODUCT_LICENCE_TYPE}" 
                    fi
                else
                    if [ "${RETURN_CODE}" -eq 0 ]
                    then
                        # Update of product signature file (of the updated PRODUCT license type)

                        ReplaceConfigVariable "${DIR_INSTALLATION}/install/`\basename ${FILE_PRODUCT_INFORMATION}`" "PRODUCT_LICENCE_TYPE" "${PRODUCT_LICENCE_TYPE}" 
                        RETURN_CODE=$?

                        if [ "${RETURN_CODE}" -ne 0 ]
                        then
                            DisplayTraceAndError "Error while updating file \""${DIR_INSTALLATION}/install/`\basename ${FILE_PRODUCT_INFORMATION}`"\" "
                        fi
                    fi
                    
                    if [ "${RETURN_CODE}" -eq 0 ]
                    then
                        # Update of PRODUCT settings file (of the updated PRODUCT license type)

                        ReplaceConfigVariable "${DIR_INSTALLATION}/install/${LPRODUCT}_inst_settings.cfg" "PRODUCT_LICENCE_TYPE" "${PRODUCT_LICENCE_TYPE}" 
                        RETURN_CODE=$?

                        if [ "${RETURN_CODE}" -ne 0 ]
                        then
                            DisplayTraceAndError "Error while updating file \""${DIR_INSTALLATION}/install/${LPRODUCT}_inst_settings.cfg"\" "
                        fi
                    fi
                fi
                ;;
            
            "FIXPAK")
                # A present time only the version is updated
                # Updated files: 
                # - Product information file
                # - Settings file
            
                if [ "${RETURN_CODE}" -eq 0 ]
                then 
                    # Update of product signature file (of the updated PRODUCT)
                    # (and not copy the complete file)
                    # Other data to change in the file?

                    ReplaceConfigVariable "${DIR_INSTALLATION}/install/product_information.sig" "PRODUCT_VERSION" "${PRODUCT_VERSION}" 
                    RETURN_CODE=$?
                    
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayTraceAndError "Error while updating file \""${DIR_INSTALLATION}/install/product_information.sig"\" "
                    fi
                fi
                
                if [ "${RETURN_CODE}" -eq 0 ]
                then 
                    # Update of PRODUCT settings file (of the updated PRODUCT)

                    ReplaceConfigVariable "${DIR_INSTALLATION}/install/${LPRODUCT}_inst_settings.cfg" "PRODUCT_VERSION" "${PRODUCT_VERSION}" 
                    RETURN_CODE=$?
                    
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayTraceAndError "Error while updating file \""${DIR_INSTALLATION}/install/${LPRODUCT}_inst_settings.cfg"\" "
                    fi
                    
                    # This variable is added if it is not already there
                    FILE_INSTALL_SETTINGS=${DIR_INSTALLATION}/install/${LPRODUCT}_inst_settings.cfg
                    
                    ${GREP} "PRODUCT_VERSION_REQUIRED" "${FILE_INSTALL_SETTINGS}" >/dev/null 2>&1
                    if [ $? -ne 0 ]
                    then
                        AddSettingsVariable "PRODUCT_VERSION_REQUIRED" "${PRODUCT_VERSION_REQUIRED}"
                    fi
                fi                
                ;;
        
            *)
                DisplayError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac

            ;;

    
        "uninst")
            # There is no need to update any information because there is no global product 
            # database (as it was the case with InstallShield)
            
            case "${SUB_CONTEXT}" in
            "COMPLETE")
                # # Pas d'indication du numero de version actuellement
                # RemoveProduct "${FILE_PRODUCT_DATABASE_DATA}" "${UPRODUCT}" "${PRODUCT_COMPONENT_NAME}" \
                #            '.*' \
                #            "${DIR_INSTALLATION}"
                # RETURN_CODE=$?
                # DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
                :
                ;;
            
            "FIXPAK")
                # Rien de particulier : non applicable
                :
                ;;
        
            *)
                DisplayError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac
            ;;

        *)
            echo "Unknown context \"${CONTEXT}\" " 1>&2
            RETURN_CODE=1
            ;;
        esac
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# RunInstallScript
###########################################################################

# 
# Execution of install script ${LPRODUCT}_inst.sh
#
#
#
#


RunInstallScript ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> RunInstallScript"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        # Copy of license files to install sudirectory
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            case "${SUB_CONTEXT}" in
            "COMPLETE")
                if [ "${LICENSE_ACTIVATION}" = "yes" ]
                then
                    ${RM} -Rf "${DIR_INSTALLATION}/install/license"
                fi
                ${CP} -pr "license" "${DIR_INSTALLATION}/install"
                RETURN_CODE=$?

                if [ "${RETURN_CODE}" -ne 0 ]
                then
                    DisplayError "Error while copying license files to \"${DIR_INSTALLATION}/install\" "
                fi
                ;;

            "FIXPAK")
                ;;
            
            *)
                DisplayError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac
        fi        

        
        # Copy of trace files to install sudirectory
        # Peut-etre copie tout a fait a la fin de l'installation ???

        if [ "${RETURN_CODE}" -eq 0 ]
        then
            case "${SUB_CONTEXT}" in
            "COMPLETE")
                ${CP} -p "${FILE_TRACE}" "${DIR_INSTALLATION}/install"
                RETURN_CODE=$?

                if [ "${RETURN_CODE}" -ne 0 ]
                then
                    DisplayError "Error while copying file \"${FILE_TRACE}\" to \"${DIR_INSTALLATION}/install\" "
                fi
                ;;

            "FIXPAK")
                # Concatenation to existing trace file
                _BASENAME_FILE_TRACE=`\basename ${FILE_TRACE}` 
                
                echo "" >> "${DIR_INSTALLATION}/install/${_BASENAME_FILE_TRACE}"
                echo "" >> "${DIR_INSTALLATION}/install/${_BASENAME_FILE_TRACE}"
                echo "================================================================================" >> "${DIR_INSTALLATION}/install/${_BASENAME_FILE_TRACE}"
                ${CAT} "${FILE_TRACE}" >> "${DIR_INSTALLATION}/install/${_BASENAME_FILE_TRACE}"
                RETURN_CODE=$?
                
                if [ "${RETURN_CODE}" -ne 0 ]
                then
                    DisplayError "Error while concatenating file \"${FILE_TRACE}\" to \"${DIR_INSTALLATION}/install/${_BASENAME_FILE_TRACE}\" "
                fi
                ;;
            
            *)
                DisplayError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac
        fi        

        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            ${CP} -p "${FILE_TRACE_ERROR}" "${DIR_INSTALLATION}/install"
            RETURN_CODE=$?
            
            if [ "${RETURN_CODE}" -ne 0 ]
            then
                DisplayError "Error while copying file \"${FILE_TRACE_ERROR}\" to \"${DIR_INSTALLATION}/install\" "
            fi
        fi        


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            ${CP} -p "${FILE_TRACE_WARNING}" "${DIR_INSTALLATION}/install"
            RETURN_CODE=$?
            
            if [ "${RETURN_CODE}" -ne 0 ]
            then
                DisplayError "Error while copying file \"${FILE_TRACE_WARNING}\" to \"${DIR_INSTALLATION}/install\" "
            fi
        fi        


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            DisplayTrace "    Running \"${DIR_INSTALLATION}/install/${LPRODUCT}_inst.sh\" script..."

            # FILE_TRACE and FILE_TRACE_ERROR are full path names
            
            case "${SUB_CONTEXT}" in
            "COMPLETE")
                if [ "${LICENSE_ACTIVATION}" = "no" ]
                then
                    "${DIR_INSTALLATION}/install/${LPRODUCT}_inst.sh" \
                        INST \
                        "${DIR_INSTALLATION}/install/${FILE_IS_VARIABLES}" \
                        "${DIR_INSTALLATION}/install/`\basename ${FILE_TRACE}`" \
                        "${DIR_INSTALLATION}/install/`\basename ${FILE_TRACE_WARNING}`" \
                        2>> "${DIR_INSTALLATION}/install/`\basename ${FILE_TRACE_ERROR}`"
                    INSTALL_SCRIPT_RETURN_CODE=$?
                else
                    INSTALL_SCRIPT_RETURN_CODE=0
                fi
                ;;

            "FIXPAK")
                # Call to ${LPRODUCT}_inst.sh script with supplementary argument "FIXPACK"
                "${DIR_INSTALLATION}/install/${LPRODUCT}_inst.sh" \
                    INST \
                    "${DIR_INSTALLATION}/install/${FILE_IS_VARIABLES}" \
                    "${DIR_INSTALLATION}/install/`\basename ${FILE_TRACE}`" \
                    "${DIR_INSTALLATION}/install/`\basename ${FILE_TRACE_WARNING}`" \
                    "FIXPACK" \
                    2>> "${DIR_INSTALLATION}/install/`\basename ${FILE_TRACE_ERROR}`"
                INSTALL_SCRIPT_RETURN_CODE=$?
                ;;

            *)
                DisplayTraceAndError "Unknown subcontext \"${SUB_CONTEXT}\" " 
                echo "Unknown subcontext \"${SUB_CONTEXT}\" " 1>&2
                RETURN_CODE=2            
                ;;
            esac
        fi
            
            
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            ERROR_SCRIPT=0
            if [ "${INSTALL_SCRIPT_RETURN_CODE}" -ne 0 ]
            then 
                ERROR_SCRIPT=1
            fi
            
            # Check if error messages file is not empty
            
            if [ -s "${DIR_INSTALLATION}/install/`basename ${FILE_TRACE_ERROR}`" ]
            then 
                ERROR_SCRIPT=1
            fi

            if [ "${ERROR_SCRIPT}" -eq 0 ]
            then
                if [ -s "${DIR_INSTALLATION}/install/`basename ${FILE_TRACE_WARNING}`" ]
                then 
                    DisplayMessage "INZP017I" "Installation completed with warnings.\n\nWarning messages are available in the file %s.\n" \
                                   "${DIR_INSTALLATION}/install/`basename ${FILE_TRACE_WARNING}`"
                else
                    DisplayMessage "INZP010I" "Installation completed with success.\n"
                fi

                case "${SUB_CONTEXT}" in
                "COMPLETE")
                    if [ "${LICENSE_ACTIVATION}" = "no" ]
                    then
                        DisplayMessage "INZP011I" ""
                    fi
                    ;;

                "FIXPAK")
                    :
                    ;;

                *)
                    DisplayTraceAndError "Unknown subcontext \"${SUB_CONTEXT}\" " 
                    echo "Unknown subcontext \"${SUB_CONTEXT}\" " 1>&2
                    RETURN_CODE=2            
                    ;;

                esac
            else
                DisplayMessage "INZP012E" "An error occurred during installation.\n\nError messages are available in the file %s.\nSummary information is available in the file %s.\n" \
                               "${DIR_INSTALLATION}/install/`basename ${FILE_TRACE_ERROR}`" \
                               "${DIR_INSTALLATION}/install/install_summary"
            fi            
        fi
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CopyUninstallerFiles
###########################################################################

# 
# Copy of the uninstaller files
#
# Argument 1 : 
# Argument 2 : 
#
#
#

CopyUninstallerFiles ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> CopyUninstallerFiles"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    
        DIR_CURRENT=`\pwd`
        
        \cd "${DIR_SETUP}"
        RETURN_CODE=$?


        DIR_UNINSTALLER="${DIR_INSTALLATION}/uninstall"

        DisplayTrace "    Creation of uninstaller directory : ${DIR_UNINSTALLER}"

        ${MKDIR} "${DIR_UNINSTALLER}" 2>/dev/null

        DisplayTrace "    Copy of files to uninstaller directory"

        case "${SUB_CONTEXT}" in
        "COMPLETE")
            \cd ${DIR_INSTALLATION}
            
            DisplayTrace "    Extraction of uninstaller files in ${DIR_INSTALLATION} directory"
            DisplayTrace "    FILE_UNINSTALLER_ARCHIVE : \"${FILE_UNINSTALLER_ARCHIVE}\" "
            
            # Specific redirection fo HP-UX tar command
            if [ "${PLATFORM}" = "hp" -o "${PLATFORM}" = "hpia" ]
            then
                ${TAR} xvf "${DIR_SETUP}/${FILE_UNINSTALLER_ARCHIVE}"  >> "${FILE_TRACE}" 2>&1
            else
                ${TAR} xvf "${DIR_SETUP}/${FILE_UNINSTALLER_ARCHIVE}"  >> "${FILE_TRACE}" 2>> "${FILE_TRACE_ERROR}"
            fi
            
            if [ $? -ne 0 ]
            then
                DisplayError "Error while extracting archive file \"${DIR_SETUP}/${FILE_UNINSTALLER_ARCHIVE}\" "
                RETURN_CODE=1
            fi

            # Copy of uninstaller setup files
            if [ "${RETURN_CODE}" -eq 0 ]
            then
                LIST_UNINSTALLER_FILES="uninstall_${LPRODUCT}.sh installer_setup.sh installer_common.sh"
            
                for _FILE in ${LIST_UNINSTALLER_FILES}
                do
                    ${CP} -p "${DIR_SETUP}/${_FILE}" "${DIR_UNINSTALLER}"
                    RETURN_CODE=$?
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error while copying file \"${_FILE}\" to \"${DIR_UNINSTALLER}\" "
                        break                
                    fi
                    
                    ${CHOWN} root:0 "${DIR_UNINSTALLER}/${_FILE}"
                    RETURN_CODE=$?
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error while changing ownership of file \"${_FILE}\" in \"${DIR_UNINSTALLER}\" "
                        break                
                    fi
                done
            fi
            

            # Copy of uninstaller setup directories
            if [ "${RETURN_CODE}" -eq 0 ]
            then
                LIST_UNINSTALLER_DIRECTORIES="messages support"
            
                for _DIRECTORY in ${LIST_UNINSTALLER_DIRECTORIES}
                do
                    ${CP} -pr "${DIR_SETUP}/${_DIRECTORY}" "${DIR_UNINSTALLER}"
                    RETURN_CODE=$?
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error while copying directory \"${_DIRECTORY}\" to \"${DIR_UNINSTALLER}\" "
                        break                
                    fi
                    
                    ${CHOWN} -R root:0 "${DIR_UNINSTALLER}/${_DIRECTORY}"
                    RETURN_CODE=$?
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error while changing ownership of tree \"${_DIRECTORY}\" in \"${DIR_UNINSTALLER}\" "
                        break                
                    fi
                done
            fi
            ;;

        "FIXPAK")
            # For now, same code as "COMPLETE"
            
            \cd ${DIR_INSTALLATION}
            
            DisplayTrace "    Extraction of uninstaller files in ${DIR_INSTALLATION} directory"
            DisplayTrace "    FILE_UNINSTALLER_ARCHIVE : \"${FILE_UNINSTALLER_ARCHIVE}\" "
            
            # Specific redirection fo HP-UX tar command
            if [ "${PLATFORM}" = "hp" -o "${PLATFORM}" = "hpia" ]
            then
                ${TAR} xvf "${DIR_SETUP}/${FILE_UNINSTALLER_ARCHIVE}"  >> "${FILE_TRACE}" 2>&1
            else
                ${TAR} xvf "${DIR_SETUP}/${FILE_UNINSTALLER_ARCHIVE}"  >> "${FILE_TRACE}" 2>> "${FILE_TRACE_ERROR}"
            fi
            
            if [ $? -ne 0 ]
            then
                DisplayError "Error while extracting archive file \"${DIR_SETUP}/${FILE_UNINSTALLER_ARCHIVE}\" "
                RETURN_CODE=1
            fi

            # Copy of uninstaller setup files
            if [ "${RETURN_CODE}" -eq 0 ]
            then
                LIST_UNINSTALLER_FILES="uninstall_${LPRODUCT}.sh installer_setup.sh installer_common.sh"
            
                for _FILE in ${LIST_UNINSTALLER_FILES}
                do
                    ${CP} -p "${DIR_SETUP}/${_FILE}" "${DIR_UNINSTALLER}"
                    RETURN_CODE=$?
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error while copying file \"${_FILE}\" to \"${DIR_UNINSTALLER}\" "
                        break                
                    fi
                    
                    ${CHOWN} root:0 "${DIR_UNINSTALLER}/${_FILE}"
                    RETURN_CODE=$?
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error while changing ownership of file \"${_FILE}\" in \"${DIR_UNINSTALLER}\" "
                        break                
                    fi
                done
            fi
            

            # Copy of uninstaller setup directories
            if [ "${RETURN_CODE}" -eq 0 ]
            then
                LIST_UNINSTALLER_DIRECTORIES="messages support"
            
                for _DIRECTORY in ${LIST_UNINSTALLER_DIRECTORIES}
                do
                    ${CP} -pr "${DIR_SETUP}/${_DIRECTORY}" "${DIR_UNINSTALLER}"
                    RETURN_CODE=$?
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error while copying directory \"${_DIRECTORY}\" to \"${DIR_UNINSTALLER}\" "
                        break                
                    fi
                    
                    ${CHOWN} -R root:0 "${DIR_UNINSTALLER}/${_DIRECTORY}"
                    RETURN_CODE=$?
                    if [ "${RETURN_CODE}" -ne 0 ]
                    then
                        DisplayError "Error while changing ownership of tree \"${_DIRECTORY}\" in \"${DIR_UNINSTALLER}\" "
                        break                
                    fi
                done
            fi
            
            ;;


        *)
            DisplayError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
            ;;
        esac

    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# VerifyLicense
###########################################################################

#
# Verification validite licence: en mode T&B, cela permet de generer le
# fichier de licence temporaire, car l'installation etant executee en tant
# que root, cette operation est donc possible 
#
# 
# 

VerifyLicense ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> VerifyLicense"

    LANG=C LC_ALL=C "${DIR_INSTALLATION}"/"${BATCH_COMMAND}" --verify-license 2>/dev/null
    RETURN_CODE=$?
    
    if [ "${RETURN_CODE}" -ne 0 ]
    then
        if [ "${RETURN_CODE}" -eq 4 ]
        then
            DisplayTrace "    The license verified is a T&B one"
            RETURN_CODE=0
        else
            LANG=C LC_ALL=C "${DIR_INSTALLATION}"/"${BATCH_COMMAND}" --verify-license 2> ${FILE_TRACE_ERROR}
            DisplayError "Error while verifying license"
        fi
    fi

    return ${RETURN_CODE}
}


###########################################################################
# RunUninstallScript
###########################################################################

# 
# Execution of install script ${LPRODUCT}_inst.sh
#
#
# >>> Cas desinstallation a faire, ...
# Fusionner avec RunInstallScript () ?
#
#

RunUninstallScript ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> RunUninstallScript"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else    

        # Copy of trace files to specific subdirectory
        # Peut-etre copie tout a fait a la fin de l'installation ???
        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            ${CP} -p "${FILE_TRACE}" "/tmp"
            RETURN_CODE=$?
            
            if [ "${RETURN_CODE}" -ne 0 ]
            then
                DisplayError "Error while copying file \"${FILE_TRACE}\" to \"/tmp\" "
            fi
        fi        
        
        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            ${CP} -p "${FILE_TRACE_ERROR}" "/tmp"
            RETURN_CODE=$?
            
            if [ "${RETURN_CODE}" -ne 0 ]
            then
                DisplayError "Error while copying file \"${FILE_TRACE_ERROR}\" to \"/tmp\" "
            fi
        fi        


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            ${CP} -p "${FILE_TRACE_WARNING}" "/tmp"
            RETURN_CODE=$?
            
            if [ "${RETURN_CODE}" -ne 0 ]
            then
                DisplayError "Error while copying file \"${FILE_TRACE_WARNING}\" to \"/tmp\" "
            fi
        fi        



        if [ "${RETURN_CODE}" -eq 0 ]
        then
            # Arbitrary choice to run ${LPRODUCT}_inst.sh from uninstall directory
            ### DisplayTrace "    Running \"${DIR_INSTALLATION}/install/${LPRODUCT}_inst.sh\" script..."
            DisplayTrace "    Running \"${DIR_INSTALLATION}/uninstall/${LPRODUCT}_inst.sh\" script..."

            # FILE_TRACE and FILE_TRACE_ERROR are full path names
            
            # For now, trace files have the same name with COMPLETE or FIXPAK installation
            
            # Utiliser FILE_FULL_IS_VARIABLES de CreateVariablesFile () ???
            "${DIR_INSTALLATION}/uninstall/${LPRODUCT}_inst.sh" \
                UNINST \
                "${DIR_INSTALLATION}/install/${FILE_IS_VARIABLES}" \
                "/tmp/`\basename ${FILE_TRACE}`" \
                "/tmp/`\basename ${FILE_TRACE_WARNING}`" \
                2>> "/tmp/`\basename ${FILE_TRACE_ERROR}`"

            UNINSTALL_SCRIPT_RETURN_CODE=$?
        fi
            
            
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            ERROR_SCRIPT=0
            if [ "${UNINSTALL_SCRIPT_RETURN_CODE}" -ne 0 ]
            then 
                ERROR_SCRIPT=1
            fi
            

            if [ -s "/tmp/`basename ${FILE_TRACE_ERROR}`" ]
            then 
                ERROR_SCRIPT=1
            fi

            if [ "${ERROR_SCRIPT}" -eq 0 ]
            then
                if [ -s "/tmp/`basename ${FILE_TRACE_WARNING}`" ]
                then 
                    DisplayMessage "INZP018I" "Uninstallation completed with warnings.\n\nPlease check the file %s for details.\n" \
                                   "${DIR_IS_TEMP}/`\basename ${FILE_TRACE_WARNING}`"
                else
                    DisplayMessage "INZP015I" "Uninstallation completed with success.\n"
                fi
            else
                DisplayMessage "INZP016E" "An error occurred during uninstallation.\n\nPlease check the file %s for details.\n" \
                               "${DIR_IS_TEMP}/`\basename ${FILE_TRACE_ERROR}`"
            fi            
        fi
        
        
        DisplayTrace "    Messages will now be traced in "${DIR_IS_TEMP}/`\basename ${FILE_TRACE}`" "
        DisplayTrace "    and in "${DIR_IS_TEMP}/`\basename ${FILE_TRACE_ERROR}`""
        DisplayTrace "    and in "${DIR_IS_TEMP}/`\basename ${FILE_TRACE_WARNING}`""
        
        # Use of the same trace files as uninstall script from now on
        FILE_TRACE="${DIR_IS_TEMP}/`\basename ${FILE_TRACE}`"
        FILE_TRACE_ERROR="${DIR_IS_TEMP}/`\basename ${FILE_TRACE_ERROR}`"
        FILE_TRACE_WARNING="${DIR_IS_TEMP}/`\basename ${FILE_TRACE_WARNING}`"
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# InstallProduct
###########################################################################

# 
#
# Installation of the product
#
#
#

InstallProduct ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> InstallProduct"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            case "${SUB_CONTEXT}" in
            "COMPLETE")
                # Processing with user-specified installation directory if any
                if [ "${DIR_INSTALLATION}" = "" ]
                then
                    # If unspecified, default directory from product information file 
                    DisplayTrace "    Installation in the default directory from product information file"
                    DIR_INSTALLATION="${PRODUCT_DEFAULT_INSTALLATION_DIR}"
                else
                    if [ "${DIR_INSTALLATION}" = "/"  ]
                    then
                        # If specified with /, default directory from product information file 
                        DisplayTrace "    Installation on the root directory with no subdirectory is not permitted"
                        DisplayTrace "    Installation in the default directory from product information file"
                        DIR_INSTALLATION="${PRODUCT_DEFAULT_INSTALLATION_DIR}"
                    fi
                fi

                DisplayTrace "    DIR_INSTALLATION  : ${DIR_INSTALLATION}"
                # echo ""
                # echo "Installation directory : ${DIR_INSTALLATION}"
                ;;
                
            "FIXPAK")
                # Processing with user-specified installation directory if any
                if [ "${DIR_INSTALLATION}" = "" ]
                then
                    # If unspecified, default directory from product information file 
                    DIR_INSTALLATION="${PRODUCT_DEFAULT_INSTALLATION_DIR}"
                fi
                    
                DisplayTrace "    DIR_INSTALLATION  : ${DIR_INSTALLATION}"
                :
                ;;

            *)
                DisplayTraceAndError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac
        fi

        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            # Display of product information

            ShowProductInformation
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi
        
        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            if [ "${LICENSE_ACTIVATION}" = "yes" ]
            then
                # Verify activation context

                VerifyActivationContext
                RETURN_CODE=$?
                DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
            fi
        fi
        
        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            case "${SUB_CONTEXT}" in
            "COMPLETE")
                if [ "${LICENSE_ACTIVATION}" = "no" ]
                then
                    if [ -d "${DIR_INSTALLATION}" ]
                    then
                        # Installation directory already existing

                        CheckEmptyDirectory "${DIR_INSTALLATION}" 
                        if [ $? -eq 0 ]
                        then
                            # Empty directory : installation allowed
                            DisplayTrace "    Directory ${DIR_INSTALLATION} empty"
                            DisplayTrace "    Product not already installed"
                        else
                            # Directory not empty (anything in it) : installation forbidden
                            DisplayTraceAndError "    Directory ${DIR_INSTALLATION} not empty"
                            DisplayTraceAndError "    Product may already be installed in ${DIR_INSTALLATION}"
                            DisplayMessage "INZP020E" "Installation directory is not empty.\nProduct may already be installed."
                            RETURN_CODE=1
                        fi
                    else
                        # Directory unexistent : installation allowed
                        DisplayTrace "    Directory ${DIR_INSTALLATION} non existent"
                        DisplayTrace "    Product not already installed"
                    fi
                fi
                ;;

            "FIXPAK")
                CheckExistingInstallation
                RETURN_CODE=$?
                ;;

            *)
                DisplayError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac
        fi
        
        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            case "${SUB_CONTEXT}" in
            "COMPLETE")
                if [ "${SILENT_INSTALLATION}" = "no" ]
                then
                    # Display of product's license

                    ShowLicense
                    RETURN_CODE=$?
                    DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
                fi
                ;;
                
            "FIXPAK")
                # No display of product's license
                ;;
                
            *)
                DisplayTraceAndError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac
        fi


        ### DEBUG ###
###         DisplayTrace "### DEBUG ###"
###         DisplayTrace "Suppression du repertoire \"${DIR_INSTALLATION}\" "
###         echo "### DEBUG ###   Suppression du repertoire \"${DIR_INSTALLATION}\" "
###         
###         set -u
###         \rm -Rf "${DIR_INSTALLATION}"
###         DisplayTrace "### FIN DEBUG ###"
        ### FIN DEBUG ###


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            case "${SUB_CONTEXT}" in
            "COMPLETE")
                if [ "${LICENSE_ACTIVATION}" = "no" ]
                then
                    DisplayTrace "    Creation of directory ${DIR_INSTALLATION}"
                    ${MKDIR} -p "${DIR_INSTALLATION}" 2>/dev/null
                    if [ $? -ne 0 ]
                    then
                        DisplayError "Cannot create install directory \"${DIR_INSTALLATION}\" "
                        DisplayMessage "INZP025E" "Unable to create the %s directory." "${DIR_INSTALLATION}"
                        RETURN_CODE=1
                    fi
                fi
                ;;
                
            "FIXPAK")
                # No creation of directory
                ;;
            
            *)
                DisplayTraceAndError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac
        fi

        if [ "${RETURN_CODE}" -eq 0 ]
        then
            if [ "${LICENSE_ACTIVATION}" = "no" ]
            then
                # Calculation of package (product) size

                CalculatePackageSize 
                RETURN_CODE=$?
                DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
            fi
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            if [ "${LICENSE_ACTIVATION}" = "no" ]
            then
                # Calculation of available disk space
                CalculateFreeDiskSpace "${DIR_INSTALLATION}"
                RETURN_CODE=$?
                DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
            fi
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            if [ "${LICENSE_ACTIVATION}" = "no" ]
            then
                # Check if enough disk space

                CheckFreeDiskSpace "${SIZE_TOTAL_REQUIRED}" "${SIZE_FREE_DISK}"
                RETURN_CODE=$?
                DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
            fi
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            echo ""
            CopyInstallationFiles
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE  : ${RETURN_CODE}"
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            if [ "${LICENSE_ACTIVATION}" = "no" ]
            then
                echo ""
                CreateVariablesFile
                RETURN_CODE=$?
                DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
            fi
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            RunInstallScript
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            UpdateProductInformation
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            if [ "${LICENSE_ACTIVATION}" = "no" ]
            then
                CopyUninstallerFiles
                RETURN_CODE=$?
                DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
            fi
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            DeleteTemporaryFiles
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi
        
        
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            case "${SUB_CONTEXT}" in
            "COMPLETE")
                if [ "${LICENSE_VERIFICATION}" = "yes" ]
                then
                    VerifyLicense
                    RETURN_CODE=$?
                    DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
                fi
                ;;
                
            "FIXPAK")
                # No verification of license
                ;;
            
            *)
                DisplayTraceAndError "Invalid value for SUB_CONTEXT : \"${SUB_CONTEXT}\" "
                ;;
            esac
        fi
    fi
    
### DEBUG ###
# DisplayTrace "Sortie script ### DEBUG ###"
# RETURN_CODE=1
### FIN DEBUG ###

    return ${RETURN_CODE}
}


###########################################################################
# UninstallProduct
###########################################################################

# 
#
#
# Argument 1 : 
# Argument 2 : 
#
# Cas FP a faire : peut-etre pas a gerer cas desinstallation toujours totale ?
#

UninstallProduct ()
{
    RETURN_CODE=0
    RETURN_STRING=""

    
    DisplayTrace ""
    DisplayTrace ">>> UninstallProduct"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=6
    else  
        if [ "${RETURN_CODE}" -eq 0 ]
        then
            # Show product information

            ShowProductInformation 
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi

        if [ "${RETURN_CODE}" -eq 0 ]
        then
            CreateVariablesFile
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            RunUninstallScript
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            UpdateProductInformation
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi


        if [ "${RETURN_CODE}" -eq 0 ]
        then
            RemoveInstallDir
            RETURN_CODE=$?
            DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
        fi
    fi

    return ${RETURN_CODE}
}


###########################################################################
# ReportEndingErrors
###########################################################################

#
# Compte-rendu en cas d'erreur

ReportEndingErrors ()
{
    DisplayTrace ""
    DisplayTrace ">>> ReportEndingErrors"

    
    # Check if error messages file is not empty

    if [ -s "${FILE_TRACE_ERROR}" ]
    then 
        DisplayMessage "INZP028E" "An error occurred during final steps.\n\nError messages are available in the file %s.\n" \
                       "${FILE_TRACE_ERROR}"
    else
        if [ -s "${FILE_TRACE_WARNING}" ]
        then 
            DisplayMessage "INZP029I" "Final steps completed with warnings.\n\nWarning messages are available in the file %s.\n" \
                           "${FILE_TRACE_WARNING}"
        fi
    fi            
}


###########################################################################
# InitSetupVariables
###########################################################################

#
# Initialisation variables
#
# Amelioration : gestion des versions differentes pour une meme plateforme

InitSetupVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitSetupVariables"

    
    # A mettre en action dans une version future du script
    USER_MASK="022"
    
    \umask ${USER_MASK}
    

    DisplayTrace "    UMASK : ${USER_MASK}"


    # Repertoire pour les fichiers temporaires

    if [ "${CONTEXT}" = "inst" ]
    then
        DIR_TEMP="${DIR_LAUNCH}/temp"
    else
        DIR_TEMP="${DIR_SETUP}/temp"
    fi
    
    \mkdir ${DIR_TEMP} 2>/dev/null
    # RETURN_CODE=$?
    
    
    # Directories of files used par installer setup
    DIR_SUPPORT="${DIR_SETUP}/support"
    DIR_MESSAGES="${DIR_SETUP}/messages"
    DIR_LICENSES="${DIR_SETUP}/license"
    DIR_CONFIG="${DIR_SETUP}/config"
    
    CheckExistingDirectory "${DIR_SUPPORT}"
    if [ "${RETURN_CODE}" -eq 0 ]
    then
        CheckExistingDirectory "${DIR_MESSAGES}"
    fi
    
    if [ "${RETURN_CODE}" -eq 0 ]
    then
        if [ "${CONTEXT}" = "inst" ]
        then
            CheckExistingDirectory "${DIR_CONFIG}"
        fi
    fi

        
   
    
    PREFIX_CONFIG_FILE="tmp_section_"


    ### DEBUG ###
    ${RM}  "${DIR_TEMP}/${PREFIX_CONFIG_FILE}"* 2>/dev/null
    ### FIN DEBUG ###
    
    
    
    if [ "${RETURN_CODE}" -eq 0 ]
    then
        if [ "${CONTEXT}" = "uninst" ]
        then
             DIR_INSTALLATION=`dirname "${DIR_SETUP}"`
             DisplayTrace "    DIR_INSTALLATION  : ${DIR_INSTALLATION}"
        fi
    fi    
    
    
    if [ "${RETURN_CODE}" -eq 0 ]
    then
        # Platform identification (operating system and architecture)

        DisplayTrace "    Platform identification (operating system and architecture)"

        IdentifyPlatform
        RETURN_CODE=$?
        DisplayTrace "    RETURN_CODE    : ${RETURN_CODE}"
        DisplayTrace "    ARCHITECTURE   : ${ARCHITECTURE}"
        DisplayTrace "    PLATFORM       : ${PLATFORM}"
        DisplayTrace "    SYSTEM_VERSION : ${SYSTEM_VERSION}"
        DisplayTrace ""

    fi


    if [ "${RETURN_CODE}" -eq 0 ]
    then
        # Locale identification (encoding and language)

        DisplayTrace ""
        DisplayTrace "    Locale identification (encoding and language)"

        IdentifyLocale
        RETURN_CODE=$?
        DisplayTrace "    RETURN_CODE  : ${RETURN_CODE}"
        DisplayTrace ""
    fi
    
    
    if [ "${RETURN_CODE}" -eq 0 ]
    then
        # Product information checking
        
        case "${CONTEXT}" in
        "inst")
            FILE_PRODUCT_INFORMATION="${DIR_CONFIG}/product_information.sig"
            if [ "${RETURN_CODE}" -eq 0 ]
            then
                ReadProductInformationFile "${FILE_PRODUCT_INFORMATION}"
            fi

            if [ "${RETURN_CODE}" -eq 0 ]
            then
                CheckProductInformation
            fi
            
            if [ "${RETURN_CODE}" -eq 0 ]
            then
                if [ "${SUB_CONTEXT}" = "COMPLETE" ]
                then
                    CheckExistingDirectory "${DIR_LICENSES}"
                fi
            fi
            ;;
            
        "uninst")
        
            # >>> Lecture informations plutot du fichier product_information.sig ????
        
        
            # File storing informations for uninstallation use
            FILE_INSTALL_SETTINGS=${DIR_INSTALLATION}/install/${LPRODUCT}_inst_settings.cfg

            PRODUCT_NAME=`GetSettingsVariable "PRODUCT_NAME"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "Parameter \"PRODUCT_NAME\" not found in ${FILE_INSTALL_SETTINGS}"
            fi

            # Pas de PRODUCT_COMPONENT_NAME dans le fichier

            PRODUCT_VERSION=`GetSettingsVariable "PRODUCT_VERSION"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "Parameter \"PRODUCT_VERSION\" not found in ${FILE_INSTALL_SETTINGS}"
            fi

            PRODUCT_DESCRIPTION=`GetSettingsVariable "PRODUCT_DESCRIPTION"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "Parameter \"PRODUCT_DESCRIPTION\" not found in ${FILE_INSTALL_SETTINGS}"
            fi

            PRODUCT_OPERATING_SYSTEM=`GetSettingsVariable "PRODUCT_OPERATING_SYSTEM"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "Parameter \"PRODUCT_OPERATING_SYSTEM\" not found in ${FILE_INSTALL_SETTINGS}"
            fi

            PRODUCT_ARCHITECTURE=`GetSettingsVariable "PRODUCT_ARCHITECTURE"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "Parameter \"PRODUCT_ARCHITECTURE\" not found in ${FILE_INSTALL_SETTINGS}"
            fi

            PRODUCT_INSTALLER_TYPE=`GetSettingsVariable "PRODUCT_INSTALLER_TYPE"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "Parameter \"PRODUCT_INSTALLER_TYPE}\" not found in ${FILE_INSTALL_SETTINGS}"
            fi

            PRODUCT_LICENCE_TYPE=`GetSettingsVariable "PRODUCT_LICENCE_TYPE"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "Parameter \"PRODUCT_LICENCE_TYPE\" not found in ${FILE_INSTALL_SETTINGS}"
            fi

            # Arbitrary
            SUB_CONTEXT="COMPLETE"
            ;;
    
        *)
            echo "Unknown context \"${CONTEXT}\" " 1>&2
            RETURN_CODE=1
            ;;
        esac
    
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Initialization of configuration filename to use
        # Configuration filename has a name which is platform specific 
        # and installation type specific (complete/fixpak)
        
        case "${SUB_CONTEXT}" in
        "FIXPAK")
            PLATFORM_CONFIG_FILE="install_fp_${PLATFORM}.ini"
            ;;
        
        "COMPLETE")
            PLATFORM_CONFIG_FILE="install_${PLATFORM}.ini"
            ;;
        
        *)
            DisplayTraceAndError "Unknown subcontext \"${SUB_CONTEXT}\" " 
            echo "Unknown subcontext \"${SUB_CONTEXT}\" " 1>&2
            RETURN_CODE=2            
            ;;
        esac
    fi
    
  
###     # Liste complete des features (a inst/desinstaller ou non)
###     LIST_FEATURES=`echo ${FEATURES} | \tr -s '|' '\n' | \cut -d':' -f1`
### 
### 
###     DisplayTrace "    LIST_FEATURES     : \"${LIST_FEATURES}\" "    

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Scattering of configuration file into sevaral files
        # (one by section)
        PrepareINIFile "${DIR_SETUP}"/"${PLATFORM_CONFIG_FILE}" "${PREFIX_CONFIG_FILE}"
        RETURN_CODE=$?
    fi
    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        \cd "${DIR_SETUP}"
        RETURN_CODE=$?
        if [ ${RETURN_CODE} -ne 0 ]
        then
            DisplayTraceAndError "Error while changing to directory \"${DIR_SETUP}\" "
            RETURN_CODE=$?
        fi
    fi
    
    
     
    # C'est normalement un argument d'appel ? (ou repertoire courant ?)
    # Repertoire d'ou s'execute le script d'installation / repertoire 
    # pour extraire l'archive des fichiers a installer
    ### DIRECTORY_TEMP='/home/a_ng/PRODUCT/ScriptsInstallation/ArborescenceTest/tmp'
    
    
    DIR_ARCHIVE="${DIR_TEMP}/Files"
    
    
    # Creation de ${LPRODUCT}_inst_settings.cfg ici ???
    
    ### # Fichier memorisant certaines informations en vue de la desinstallation 
    ### FILE_INSTALL_SETTINGS=${LOCATION_PATH}/install/${LPRODUCT}_inst_settings.cfg
###     
###     case "${CONTEXT}" in
### 
###     "inst")    
###         case "${SUB_CONTEXT}" in
### 
###         "FIXPACK")
###             if [ ${RETURN_CODE} -eq 0 ]
###             then
###                 CheckReadableFile ${FILE_INSTALL_SETTINGS}
###                 RETURN_CODE=$?
###             fi    
###             ;;
### 
###         "")
###             # Vidage au prealable ?
### 
###             echo "#"                            >> ${FILE_INSTALL_SETTINGS}
###             echo "# @(#) PRODUCT installation settings file"  >> ${FILE_INSTALL_SETTINGS}
###             echo "#"                            >> ${FILE_INSTALL_SETTINGS}
###             echo ""                             >> ${FILE_INSTALL_SETTINGS}
###             echo ""                             >> ${FILE_INSTALL_SETTINGS}
###             ;;
### 
###         *)
###             echo "Unknown sub-context \"${SUB_CONTEXT}\" " 
###             echo "Unknown sub-context \"${SUB_CONTEXT}\" " 1>&2
###             RETURN_CODE=2            
###             ;;
###             
###         esac
###         
###         ;;
###         
###     "uninst")    
### 
###         if [ ${RETURN_CODE} -eq 0 ]
###         then
###             CheckReadableFile ${FILE_INSTALL_SETTINGS}
###             RETURN_CODE=$?
###         fi    
###         ;;
###       
###         
###     *)    
###         echo "Unknown context \"${CONTEXT}\" " 1>&2
###         RETURN_CODE=2    
###         ;;
###         
###     esac
        

    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitCommandsVariables
        RETURN_CODE=$?
    fi

     
    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitSystemFilesVariables
        RETURN_CODE=$?
    fi

        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitOtherSystemVariables
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        if [ "${CONTEXT}" = "uninst" ]
        then
            RetrieveInstallationSettings
            RETURN_CODE=$?
        fi
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitComponentFilesVariables
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitInstallerVariables
        RETURN_CODE=$?
    fi

    return ${RETURN_CODE}
}


###########################################################################
# RetrieveInstallationSettings
###########################################################################

#
# Retrieve settings from installation settings file
#
# 

RetrieveInstallationSettings ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> RetrieveInstallationSettings"


    if [ "${RETURN_CODE}" -eq 0 ]
    then
        # Recuperation des informations INSTALL_FEATURE_CORE, 
        # INSTALL_FEATURE_NETWORK, INSTALL_FEATURE_PLUGIN
        # Il se peut qu'elles n'aient pas ete memorisees (PRODUCT 3.1 Full, WGE, 
        # T&B n'incluaient pas encore ces informations).
        # Dans ce cas on les cree.
        #
        # Peut-etre effets inattendus a verifier/prevoir.

        # Test presence des informations base uniquement sur une seule variable au lieu des trois

        ${GREP} "INSTALL_FEATURE_CORE" "${FILE_INSTALL_SETTINGS}" >/dev/null 2>&1
        if [ $? -eq 0 ]
        then

            LIST_FEATURES_UNINSTALL=""
            LIST_FEATURES_NO_UNINSTALL=""
            
            # Retrieval of INSTALL_FEATURE_CORE information

            DisplayTrace "    Retrieval of parameter \"INSTALL_FEATURE_CORE\" from installation settings file"

            INSTALL_FEATURE_CORE=`GetSettingsVariable "INSTALL_FEATURE_CORE"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "    Parameter \"INSTALL_FEATURE_CORE}\" not found in ${FILE_INSTALL_SETTINGS}" 
            else
                DisplayTrace "    INSTALL_FEATURE_CORE  : \"${INSTALL_FEATURE_CORE}\" "
                if [ "${INSTALL_FEATURE_CORE}" -eq 1 ]
                then
                    LIST_FEATURES_UNINSTALL="${LIST_FEATURES_UNINSTALL} core"
                else
                    LIST_FEATURES_NO_UNINSTALL="${LIST_FEATURES_NO_UNINSTALL} core"
                fi
            fi


            # Retrieval of INSTALL_FEATURE_NETWORK information

            DisplayTrace "    Retrieval of parameter \"INSTALL_FEATURE_NETWORK\" from installation settings file"

            INSTALL_FEATURE_NETWORK=`GetSettingsVariable "INSTALL_FEATURE_NETWORK"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "    Parameter \"INSTALL_FEATURE_NETWORK}\" not found in ${FILE_INSTALL_SETTINGS}"
            else
                DisplayTrace "    INSTALL_FEATURE_NETWORK : \"${INSTALL_FEATURE_NETWORK}\" "
                if [ "${INSTALL_FEATURE_NETWORK}" -eq 1 ]
                then
                    LIST_FEATURES_UNINSTALL="${LIST_FEATURES_UNINSTALL} network"
                else
                    LIST_FEATURES_NO_UNINSTALL="${LIST_FEATURES_NO_UNINSTALL} network"
                fi
            fi
        
            
            # Retrieval of INSTALL_FEATURE_PROCEDURE information

            DisplayTrace "    Retrieval of parameter \"INSTALL_FEATURE_PROCEDURE\" from installation settings file"

            INSTALL_FEATURE_PROCEDURE=`GetSettingsVariable "INSTALL_FEATURE_PROCEDURE"`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                INSTALL_FEATURE_PROCEDURE=0
                DisplayTrace "    Parameter \"INSTALL_FEATURE_PROCEDURE}\" not found in ${FILE_INSTALL_SETTINGS}"
            else
                DisplayTrace "    INSTALL_FEATURE_PROCEDURE  : \"${INSTALL_FEATURE_PROCEDURE}\" "
                if [ "${INSTALL_FEATURE_PROCEDURE}" -eq 1 ]
                then
                    LIST_FEATURES_UNINSTALL="${LIST_FEATURES_UNINSTALL} procedure"
                else
                    LIST_FEATURES_NO_UNINSTALL="${LIST_FEATURES_NO_UNINSTALL} procedure"
                fi
            fi
            
            DisplayTrace "    LIST_FEATURES_UNINSTALL : \"${LIST_FEATURES_UNINSTALL}\" "
            DisplayTrace "    LIST_FEATURES_NO_UNINSTALL : \"${LIST_FEATURES_NO_UNINSTALL}\" "
            
        fi
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# RemoveInstallDir
###########################################################################

#
# Removal of installation directory
#
# 

RemoveInstallDir ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> RemoveInstallDir"


    if [ "${RETURN_CODE}" -eq 0 ]
    then
        if [ ! -d ${DIR_INSTALLATION} ]
        then
            DisplayTraceAndError "Directory \"${DIR_INSTALLATION}\" not found."
            RETURN_CODE=1
        fi
    fi
    

    if [ "${RETURN_CODE}" -eq 0 ]
    then
        DisplayTrace "    Contents of directory \"${DIR_INSTALLATION}\":"
        ${LS} -1 "${DIR_INSTALLATION}"   >> "${FILE_TRACE}"
        DisplayTrace ""
    fi
    
    for _NAME in `${LS} ${DIR_INSTALLATION}`
    do

        if [ -d "${DIR_INSTALLATION}/${_NAME}" ]
        then
            # Controles de securite pour eviter des resultats catastrophiques
            # (suppression repertoires sous la racine)

            if [ -z "${DIR_INSTALLATION}" ]
            then
                DisplayTrace "Error : DIR_INSTALLATION empty !!!"
                RETURN_CODE=1
                break
            fi

            if [ -z "${_NAME}" ]
            then
                DisplayTrace "Error : directory name  empty !!!"
                RETURN_CODE=1
                break
            fi

            if [ "${DIR_INSTALLATION}/${_NAME}" = "/${_NAME}" ]
            then
                DisplayTrace "Error : complete path to directory to delete is \"/${_NAME}\" !!!" 
                RETURN_CODE=1
                break
            fi

            if [ "${DIR_INSTALLATION}/${_NAME}" = "//${_NAME}" ]
            then
                DisplayTrace "Error : complete path to directory to delete is \"//${_NAME}\" !!!" 
                RETURN_CODE=1
                break
            fi


            case ${_NAME} in

            "${DIR_SETUP}")
                DisplayTrace "    No deletion of directory \"${DIR_INSTALLATION}/${_NAME}\" yet."
                DisplayTrace ""
                ;;

            *)    
                DisplayTrace "    Deletion of directory \"${DIR_INSTALLATION}/${_NAME}\" " 
                DisplayTrace ""
                DisplayTrace "    Contents:"
                (\cd ${DIR_INSTALLATION}/${_NAME} ; ${LS} -la) >> "${FILE_TRACE}"
                DisplayTrace ""        
\cd /tmp
                ${RM} -Rf "${DIR_INSTALLATION}/${_NAME}"
                if [ $? -ne 0 ]
                then
                    DisplayTrace "Error removing directory \"${DIR_INSTALLATION}/${_NAME}\" "
                fi    
                ;;

            esac            

        else
            # if [ -f "${DIR_INSTALLATION}/${NAME}" ]
            # then

                DisplayTrace "    Deletion of file \"${DIR_INSTALLATION}/${_NAME}\" " 

                ${RM} -f "${DIR_INSTALLATION}/${_NAME}"
                if [ $? -ne 0 ]
                then
                    DisplayTrace "Error removing file \"${DIR_INSTALLATION}/${_NAME}\" "
                fi    
            # fi
        fi
    done


    \cd /tmp

    if [ "${RETURN_CODE}" -eq 0 ]
    then
        ${RM} -Rf "${DIR_INSTALLATION}/${DIR_SETUP}"
        if [ $? -ne 0 ]
        then
            DisplayTrace "Error removing directory \"${DIR_INSTALLATION}/${DIR_SETUP}\" "
        fi    
    fi


    DisplayTrace "    Deletion of directory \"${DIR_INSTALLATION}\" " 
    ${RMDIR} "${DIR_INSTALLATION}"
    if [ $? -ne 0 ]
    then
        DisplayTrace "Error removing directory"
    fi 
    
    return ${RETURN_CODE}
}


###########################################################################
# main
###########################################################################

# 
# -h        Help
# -d path   Installation directory
# 
# Argument 1 : context "INST" or "UNINST"
# 
# 
# 

RETURN_CODE=0



SCRIPT_NAME=`\basename $0`




# Full pathname of the directory where the current script is located

DIR_LAUNCH=`\pwd`
DIR_SETUP=`\dirname $0`
DIR_SETUP=`\cd "${DIR_SETUP}" ; \pwd`
\cd "${DIR_LAUNCH}"




# Command line parsing

### DIR_INSTALLATION=""
### 
### while getopts :hd: OPTION
### do
###     case "${OPTION}" in
###     h)
###         DisplaySyntax
###         exit 0
###         ;;
###         
###     d)
###         DIR_INSTALLATION="${OPTARG}"
###         # echo "Argument option ${OPTARG}"
###         ;;
###         
###     ?)
###         echo "Unknown Option"
###         :
###         ;;   
###     esac 
### done
### 
### 
### 
### # echo "OPTIND : $OPTIND"
### # echo $*
### # echo "Nombre d'arguments : $#"
### 
### shift `expr "${OPTIND}" - 1`
### # echo $*
### # echo "Nombre d'arguments (apres shift) : $#"


# Check parameters


# Check for no less than 6 parameters 
if [ $# -lt 6 ]
then
        echo "Missing parameters ($#)" 1>&2
        exit 1
fi



CONTEXT="$1"

# Loading of common functions

if [ "${RETURN_CODE}" -eq 0 ]
then
    . "${DIR_SETUP}/installer_common.sh"
fi


# Controle sur le contexte d'utilisation

CONTEXT=`ConvertToLowerCase "${CONTEXT}"`
if [ ${RETURN_CODE} -eq 0 ]
then
    case "${CONTEXT}" in
    "inst")
        FILE_TRACE="${DIR_LAUNCH}/$2"
        FILE_TRACE_ERROR="${DIR_LAUNCH}/$3"
        FILE_TRACE_WARNING="${DIR_LAUNCH}/$4"
        DisplayTrace "Context             : Installation"
        ;;

    "uninst")
        FILE_TRACE="${DIR_SETUP}/$2"
        FILE_TRACE_ERROR="${DIR_SETUP}/$3"
        FILE_TRACE_WARNING="${DIR_SETUP}/$4"
        DisplayTrace "Context             : Uninstallation"
        ;;

    *)
        echo "Unknown context \"${CONTEXT}\" " 1>&2
        RETURN_CODE=1
        ;;

    esac
fi


SILENT_INSTALLATION="$5"

UPRODUCT=`echo $6 | tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'`
LPRODUCT=`echo $6 | tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz'`


# On ne conserve que le fichier de traces standard des tentatives precedentes
\rm "${FILE_TRACE_ERROR}" 2>/dev/null
\rm "${FILE_TRACE_WARNING}" 2>/dev/null

# Creation of empty trace error file in case no error occurs
> "${FILE_TRACE_ERROR}" 
> "${FILE_TRACE_WARNING}" 


DisplayTrace ""
DisplayTrace "${UPRODUCT} trace file"
DisplayTrace ""


DisplayTrace "Script name         : ${SCRIPT_NAME}"
DisplayTrace "Number of arguments : $#"
DisplayTrace "Arguments           : $*"
DisplayTrace "Current directory   : `pwd`"
DisplayTrace "Current date        : `\date`"
DisplayTrace ""


# Controle sur le mode d'installation

SILENT_INSTALLATION=`ConvertToLowerCase "${SILENT_INSTALLATION}"`
if [ ${RETURN_CODE} -eq 0 ]
then
    case "${SILENT_INSTALLATION}" in
    "yes")
        DisplayTrace "Installation mode   : Silent"
        ;;

    "no")
        DisplayTrace "Installation mode   : Interactive"
        ;;

    *)
        echo "Unknown installation mode \"${SILENT_INSTALLATION}\" " 1>&2
        RETURN_CODE=1
        ;;

    esac
fi


# Check for additional parameter in case of installation

if [ "${CONTEXT}" = "inst" ]
then
    # Additional parameter 
    if [ $# -eq 9 ]
    then
        LICENSE_ACTIVATION="$7"
        LICENSE_ACTIVATION=`ConvertToLowerCase "${LICENSE_ACTIVATION}"`
        if [ ${RETURN_CODE} -eq 0 ]
        then
            case "${LICENSE_ACTIVATION}" in
            "yes")
                DisplayTrace "License activation: Yes"
                ;;

            "no")
                DisplayTrace "License activation: No"
                ;;

            *)
                echo "Unknown license activation mode \"${LICENSE_ACTIVATION}\" " 1>&2
                RETURN_CODE=1
                ;;

            esac
        fi
        
        LICENSE_VERIFICATION="$8"
        LICENSE_VERIFICATION=`ConvertToLowerCase "${LICENSE_VERIFICATION}"`
        if [ ${RETURN_CODE} -eq 0 ]
        then
            case "${LICENSE_VERIFICATION}" in
            "yes")
                DisplayTrace "License verification: Yes"
                ;;

            "no")
                DisplayTrace "License verification: No"
                ;;

            *)
                echo "Unknown license verification mode \"${LICENSE_VERIFICATION}\" " 1>&2
                RETURN_CODE=1
                ;;

            esac
        fi
        
        DIR_INSTALLATION="$9"
        DisplayTrace "    DIR_INSTALLATION : \"${DIR_INSTALLATION}\" "
    else
        echo "Missing parameter ($#)" 1>&2
        exit 1
    fi
fi

DisplayTrace "    DIR_SETUP : \"${DIR_SETUP}\" "
DisplayTrace "    DIR_LAUNCH : \"${DIR_LAUNCH}\" "




# >>> Separateur de liste (a declarer ailleurs)
FS_PRODUCT_DATABASE='|'


if [ "${RETURN_CODE}" -eq 0 ]
then
    InitDefaultVariables
fi



# Check user's identity: must be "root"

if [ ${RETURN_CODE} -eq 0 ]
then
    USER_ID=`GetUserID` 
    
    if [ "${USER_ID}" -eq 0 ]
    then
        DisplayTrace ""
        DisplayTrace "    Current UID is \"root\"."
    else
        DisplayTrace ""
        DisplayTrace "    Current UID is not \"root\" : ${USER_ID}"
        DisplayTrace "    Aborting. "
        
        echo "You are not the \"root\" user." 1>&2
        echo "You must be the \"root\" user in order to run this script." 1>&2
        echo "Aborting. " 1>&2

        RETURN_CODE=1
    fi    
fi


if [ "${RETURN_CODE}" -eq 0 ]
then
    InitSetupVariables
fi


if [ "${RETURN_CODE}" -eq 0 ]
then
    case "${CONTEXT}" in

    "inst")    
        InstallProduct
        RETURN_CODE=$?
        ;;

    "uninst")    
        UninstallProduct
        RETURN_CODE=$?
        ;;
        
    *)
        echo "Unknown context \"${CONTEXT}\" " 1>&2
        RETURN_CODE=2    
        ;;    
        
    esac
fi


if [ "${RETURN_CODE}" -ne 0 ]
then
    ReportEndingErrors
fi


DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
DisplayTrace ""
DisplayTrace ""
DisplayTrace ""



exit ${RETURN_CODE}






