#! /bin/sh

#
# @(#) Script d'installation de HPU
# 


### DEBUG ###
# set -x
set -u
### FIN DEBUG ###



###########################################################################
# DisplayTrace
###########################################################################

# 
# Affichage message de trace
# 
# 
# Argument 1 : texte du message
# 
# 

DisplayTrace ()
{
    # RETURN_CODE=0
    
    echo "$1"  >> "${FILE_TRACE}"
    
    # return ${RETURN_CODE}
}


###########################################################################
# DisplayError
###########################################################################

# 
# Affichage message sur stderr
# 
# 
# Argument 1 : texte du message
# 
# 
# 

DisplayError ()
{
    # RETURN_CODE=0
    
    echo "$1" 1>&2
    
    # return ${RETURN_CODE}
}


###########################################################################
# DisplayTraceAndError
###########################################################################

# 
# Affichage de trace (dans fichier de trace) et d'erreur (sur stderr)
# 
# 
# Argument 1 : texte du message
# 
# 
# 

DisplayTraceAndError ()
{
    # RETURN_CODE=0
    
    echo "$1" >> "${FILE_TRACE}"
    echo "$1" 1>&2
    
    # return ${RETURN_CODE}
}


###########################################################################
# DisplayWarning
###########################################################################

# 
# Displays message to warning trace file
# 
# 
# Argument 1 : texte du message
# 
# 
# 

DisplayWarning ()
{
    # RETURN_CODE=0
    
    echo "$1" >> "${FILE_TRACE_WARNING}"
    
    # return ${RETURN_CODE}
}


###########################################################################
# DisplayScreen
###########################################################################

# 
# Affichage message de trace
# 
# 
# Argument 1 : texte du message
# 
# 

DisplayScreen ()
{
    RETURN_CODE=0
    
    ${PRINTF} "$1"
    
    # return ${RETURN_CODE}
}


###########################################################################
# DisplayTextFileTrace
###########################################################################

# 
# Affichage du contenu d'un fichier texte dans le fichier de trace
# 
# 
# Argument 1 : nom du fichier
# 
# 

DisplayTextFileTrace ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> DisplayTextFileTrace"
    
    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        if [ -f "$1" ]
        then
            ${CAT} "$1" >> "${FILE_TRACE}"
        else
            DisplayError "File \"$1\" not found"
            RETURN_CODE=2
        fi
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# ActivateTraceRedirection
###########################################################################

# 
# Activation of redirection of stdout output to trace file 
# 
# 
# No RETURN_CODE value set.
# 
# 

ActivateTraceRedirection ()
{
    DisplayTrace ""
    DisplayTrace ">>> ActivateTraceRedirection"


    # Holding of stdout into file descriptor 7
    exec 7>&1

    # Redirection of stdout to trace file 
    exec >> "${FILE_TRACE}"
}


###########################################################################
# DeactivateTraceRedirection
###########################################################################

# 
# Deactivation of redirection of stdout output to trace file 
# 
# 
# No RETURN_CODE value set.
# 

DeactivateTraceRedirection ()
{
    DisplayTrace ""
    DisplayTrace ">>> DeactivateTraceRedirection"


    # Restoration of stdout 
    exec >&7

    # Closing of file descriptor 7
    exec 7>&-
}


###########################################################################
# GetDB2InstanceBitWidth
###########################################################################

# 
# Recherche de la largeur de bits (32 ou 64) de l'instance DB2 par defaut
# 
# 
# Argument 1 : repertoire de connexion de l'instance DB2
# 
#
#
# La methode utilisee est celle de la fonction InstanceBitWidth () du script 
# db2iutil (situe dans le repertoire <installation DB2>/instance).
# Recherche dans le fichier <repertoire instance/sqllib/ctrl/.instuse de la 
# ligne 
# BitWidth=XXX
#
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# 
# 
# 
# 
# 

GetDB2InstanceBitWidth ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> GetDB2InstanceBitWidth"
    

    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        if [ -z "$1" ]
        then
            DisplayError "Empty argument"
            echo '\07Error'
            RETURN_CODE=2
        else


            FILE_INTERNAL_INSTANCE=$1/sqllib/ctrl/.instuse

            CheckReadableFile ${FILE_INTERNAL_INSTANCE} 1>/dev/null 2>&1
            RETURN_CODE=$?

            if [ ${RETURN_CODE} -eq 0 ]
            then
                LINE_BITWIDTH=`${GREP} "^BitWidth" ${FILE_INTERNAL_INSTANCE}`
                RETURN_CODE=$?
            fi    
            
            if [ ${RETURN_CODE} -eq 0 ]
            then
                echo "${LINE_BITWIDTH}" | ${CUT} -d "=" -f2
            fi

        fi

    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# EvaluateConditionsINST
###########################################################################

# 
# Evaluation des conditions associees a une section (installation)
# 
# 
# Argument 1 : nom de section
# 
# 
# Code de retour : 
# 
# 0 : la condition est satisfaite (resultat evaluation de l'expression : "vrai")
# 1 : la condition n'est pas satisfaite (resultat evaluation de l'expression : "faux")
# 
# 
# 
# A faire : mettre "\07Error" dans une constante globale
# 
# Utilisation de /usr/bin/test ou test inegre au shell ?
# 
# 

EvaluateConditionsINST ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> EvaluateConditionsINST"


    CONDITION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "condition"` 
    RETURN_CODE=$?
    
    if [ "${RETURN_CODE}" -eq 0 ]
    then
        DisplayTrace "    A least one condition present" 
    else    
        if [ "${RETURN_CODE}" -eq 2 ]
        then
            # Aucune condition presente : equivalent condition toujours vraie
            # L'action est toujours executee
            DisplayTrace "    No condition present : action always executed" 
            RETURN_CODE=0
            return ${RETURN_CODE}
        else    
            # Cas improbable
            RETURN_CODE=1
            return ${RETURN_CODE}
        fi    
    fi
    
    
    # Recherche du coordinateur 
    
    CONDITION_COORDINATOR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "condition_coordinator"` 
    RETURN_CODE=$?

    if [ "${RETURN_CODE}" -eq 0 ]
    then
     
        DisplayTrace "    CONDITION_COORDINATOR : \"${CONDITION_COORDINATOR}\"" 
        DisplayTrace ""

        if [ "${CONDITION_COORDINATOR}" != "" ]
        then
            CONDITION_COORDINATOR=`echo "${CONDITION_COORDINATOR}" | ${TR} 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz'`
        fi


        case "${CONDITION_COORDINATOR}" in

        "and"|"or")
            ;;

        *)
            DisplayError "Error on condition_coordinator parameter value : \"${CONDITION_COORDINATOR}\" " 
            return 1
            ;;

        esac

    else    
        if [ "${RETURN_CODE}" -eq 2 ]
        then
            DisplayTrace "Missing condition_coordinator parameter"
            DisplayError "Missing condition_coordinator parameter"
            
            return ${RETURN_CODE}
        else    
            # Cas improbable
            RETURN_CODE=1
            return ${RETURN_CODE}
        fi    
    fi

    

    # Astuce afin que GLOBAL_CODE_EVALUATION reste modifie en sortie de la 
    # boucle "while"
    exec 9<&0 < "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}"
    
    # Resultat global de l'evaluation des conditions
    GLOBAL_CODE_EVALUATION=""
    
    # Lecture du fichier contenant la section sans utiliser de pipe car besoin de conserver
    # la valeur de variable modifiees une fois sorti de la boucle.
    # Normalement on supprime la premiere ligne avec un "sed -e '1d'" mais ce n'est pas 
    # indispensable.
    while read LINE 
    do
        if [ "${LINE}" != "" ]
        then
            IDENTIFIER=`echo ${LINE} | ${CUT} -d'=' -f1`
            VALUE=`echo ${LINE} | ${CUT} -d'=' -f2-`


            case "${IDENTIFIER}" in 
            "condition")

                DisplayTrace "    LINE  : \"${LINE}\"" 
                DisplayTrace "    VALUE : \"${VALUE}\"" 
                
                
                if [ "${VALUE}" = "" ]
                then
                    # Condition vide : equivalent a "vrai"
                    DisplayTrace "    Empty condition." 
                    
                    CODE_EVALUATION=0
                else
                    # Evaluation avec la commande "test" du shell
                    
                    RESULT=`\eval test "${VALUE}"`
                    CODE_EVALUATION=$?
                fi
                    
                DisplayTrace "    CODE_EVALUATION : ${CODE_EVALUATION}"
                
                if [ "${CODE_EVALUATION}" -eq 0 ]
                then
                    DisplayTrace "    Evaluation result is \"true\"."

                    case "${CONDITION_COORDINATOR}" in

                    "and")
                        GLOBAL_CODE_EVALUATION="true"
                        ;;

                    "or")
                        case "${GLOBAL_CODE_EVALUATION}" in
                        "true")
                            # Inchange
                            GLOBAL_CODE_EVALUATION="true"
                            
                            # Le resultat global est "vrai" avec arret de l'evaluation
                            # (plus besoin de continuer)
                            DisplayTrace "    Evaluation of conditions interrupted"

                            break 
                            ;;
                            
                       "false")
                            # Changement du resultat global
                            GLOBAL_CODE_EVALUATION="true"
                            
                            # Le resultat global est "vrai" avec arret de l'evaluation
                            # (plus besoin de continuer)
                            DisplayTrace "    Evaluation of conditions interrupted"
                            
                            break                             
                            ;;
                            
                       "")
                            # GLOBAL_CODE_EVALUATION n'avait pas encore ete initialise
                            GLOBAL_CODE_EVALUATION="true"

                            break 
                            ;;
                                                        
                       *)
                            echo "Error" 1>&2

                            exec 0<&9 9<&-
                            
                            return 1
                            ;;

                        esac                        
                        ;;

                    esac

                else

                    DisplayTrace "    Evaluation result is \"false\"."
                    
                    case "${CONDITION_COORDINATOR}" in

                    "and")
                        # Le resultat global est "faux" avec arret de l'evaluation
                        # (plus besoin de continuer)
                        GLOBAL_CODE_EVALUATION="false"
                        DisplayTrace "    Evaluation of conditions interrupted"
                        break 
                        ;;

                    "or")
                        if [ "${GLOBAL_CODE_EVALUATION}" = "" ]
                        then
                            # GLOBAL_CODE_EVALUATION n'avait pas encore ete initialise
                            GLOBAL_CODE_EVALUATION="false"
                        fi
                        ;;

                    esac
                fi
                
                DisplayTrace "    Current evaluation result"
                DisplayTrace "    GLOBAL_CODE_EVALUATION : \"${GLOBAL_CODE_EVALUATION}\"" 
                DisplayTrace ""
                ;;

                
            *)
                ### DisplayTrace "    Line discarded"

                continue
                ;;
           esac 
        fi


    
    # done < "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" 
    done
       
    exec 0<&9 9<&-
        
    DisplayTrace "    Final evaluation result"
    DisplayTrace "    GLOBAL_CODE_EVALUATION : \"${GLOBAL_CODE_EVALUATION}\"" 
        
    case "${GLOBAL_CODE_EVALUATION}" in
    "true")
        DisplayTrace ""
        DisplayTrace "    Action will be executed"
        RETURN_CODE=0
        ;;

    "false")
        DisplayTrace ""
        DisplayTrace "    Action won't be executed"
        RETURN_CODE=1
        ;;

    *)
        DisplayTrace ""
        DisplayTrace "    Action won't be executed"
        RETURN_CODE=1
        ;;

    esac                        
        
    return ${RETURN_CODE}
}


###########################################################################
# EvaluateConditionsUNINST
###########################################################################

# 
# Evaluation des conditions associees a une section (desinstallation)
# 
# 
# Argument 1 : nom de section
# 
# 
# 
# Code de retour : 
# 
# 0 : la condition est satisfaite (resultat evaluation de l'expression : "vrai")
# 1 : la condition n'est pas satisfaite (resultat evaluation de l'expression : "faux")
# 
# A faire : mettre "\07Error" dans une constante globale
# 
# Utilisation de /usr/bin/test ou test inegre au shell ?
# 
# 

EvaluateConditionsUNINST ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> EvaluateConditionsUNINST"



    CONDITION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_condition"` 
    RETURN_CODE=$?
    
    if [ "${RETURN_CODE}" -eq 0 ]
    then
        DisplayTrace "    A least one condition present" 
    else    
        if [ "${RETURN_CODE}" -eq 2 ]
        then
            # Aucune condition presente : equivalent condition toujours vraie
            # L'action est toujours executee
            DisplayTrace "    No condition present : action always executed" 
            RETURN_CODE=0
            return ${RETURN_CODE}
        else    
            # Cas improbable
            RETURN_CODE=1
            return ${RETURN_CODE}
        fi    
    fi
    
    
    # Recherche du coordinateur 
    
    CONDITION_COORDINATOR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_condition_coordinator"` 
    RETURN_CODE=$?

    if [ "${RETURN_CODE}" -eq 0 ]
    then
     
        DisplayTrace "    CONDITION_COORDINATOR : \"${CONDITION_COORDINATOR}\"" 
        DisplayTrace ""

        if [ "${CONDITION_COORDINATOR}" != "" ]
        then
            CONDITION_COORDINATOR=`echo "${CONDITION_COORDINATOR}" | ${TR} 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz'`
        fi


        case "${CONDITION_COORDINATOR}" in

        "and"|"or")
            ;;

        *)
            DisplayError "Error on condition_coordinator parameter value : \"${CONDITION_COORDINATOR}\" "
            return 1
            ;;

        esac

    else    
        if [ "${RETURN_CODE}" -eq 2 ]
        then
            DisplayTrace "Missing condition_coordinator parameter"
            DisplayError "Missing condition_coordinator parameter" 
            
            return ${RETURN_CODE}
        else    
            # Cas improbable
            RETURN_CODE=1
            return ${RETURN_CODE}
        fi    
    fi

    

    # Astuce afin que GLOBAL_CODE_EVALUATION reste modifie en sortie de la 
    # boucle "while"
    exec 9<&0 < "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}"
    
    # Resultat global de l'evaluation des conditions
    GLOBAL_CODE_EVALUATION=""
    
    # Lecture du fichier contenant la section sans utiliser de pipe car besoin de conserver
    # la valeur de variable modifiees une fois sorti de la boucle.
    # Normalement on supprime la premiere ligne avec un "sed -e '1d'" mais ce n'est pas 
    # indispensable.
    while read LINE 
    do
        if [ "${LINE}" != "" ]
        then

            ### DisplayTrace "    LINE : \"${LINE}\"" 


            IDENTIFIER=`echo ${LINE} | ${CUT} -d'=' -f1`
            VALUE=`echo ${LINE} | ${CUT} -d'=' -f2-`

            case "${IDENTIFIER}" in 
            "_condition")

                DisplayTrace "    LINE  : \"${LINE}\"" 
                DisplayTrace "    VALUE : \"${VALUE}\"" 
                
                
                if [ "${VALUE}" = "" ]
                then
                    # Condition vide : equivalent a "vrai"
                    DisplayTrace "    Empty condition." 
                    
                    CODE_EVALUATION=0
                else
                    # Evaluation avec la commande "test" du shell
                    
                    RESULT=`\eval test "${VALUE}"`
                    CODE_EVALUATION=$?
                fi
                    
                DisplayTrace "    CODE_EVALUATION : ${CODE_EVALUATION}"
                
                if [ "${CODE_EVALUATION}" -eq 0 ]
                then
                    DisplayTrace "    Evaluation result is \"true\"."

                    case "${CONDITION_COORDINATOR}" in

                    "and")
                        GLOBAL_CODE_EVALUATION="true"
                        ;;

                    "or")
                        case "${GLOBAL_CODE_EVALUATION}" in
                        "true")
                            # Inchange
                            GLOBAL_CODE_EVALUATION="true"
                            
                            # Le resultat global est "vrai" avec arret de l'evaluation
                            # (plus besoin de continuer)
                            DisplayTrace "    Evaluation of conditions interrupted"
                            break 
                            ;;
                            
                       "false")
                            # Changement du resultat global
                            GLOBAL_CODE_EVALUATION="true"
                            
                            # Le resultat global est "vrai" avec arret de l'evaluation
                            # (plus besoin de continuer)
                            DisplayTrace "    Evaluation of conditions interrupted"
                            break                             
                            ;;
                            
                       "")
                            # GLOBAL_CODE_EVALUATION n'avait pas encore ete initialise
                            GLOBAL_CODE_EVALUATION="true"
                            break 
                            ;;
                            
                                                        
                       *)
                            DisplayError "Error"
                            exec 0<&9 9<&-
                            return 1
                            ;;

                        esac                        
                        ;;

                    esac

                else

                    DisplayTrace "    Evaluation result is \"false\"."
                    
                    case "${CONDITION_COORDINATOR}" in

                    "and")
                        # Le resultat global est "faux" avec arret de l'evaluation
                        # (plus besoin de continuer)
                        GLOBAL_CODE_EVALUATION="false"
                        DisplayTrace "    Evaluation of conditions interrupted"
                        break 
                        ;;

                    "or")
                        if [ "${GLOBAL_CODE_EVALUATION}" = "" ]
                        then
                            # GLOBAL_CODE_EVALUATION n'avait pas encore ete initialise
                            GLOBAL_CODE_EVALUATION="false"
                        fi
                        ;;

                    esac
                fi
                
                DisplayTrace "    Current evaluation result"
                DisplayTrace "    GLOBAL_CODE_EVALUATION : \"${GLOBAL_CODE_EVALUATION}\"" 
                DisplayTrace ""
                ;;

                
            *)
                ### DisplayTrace "    Line discarded"

                continue
                ;;
           esac 
        fi


    
    # done < "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" 
    done
       
    exec 0<&9 9<&-
        
    DisplayTrace "    Final evaluation result"
    DisplayTrace "    GLOBAL_CODE_EVALUATION : \"${GLOBAL_CODE_EVALUATION}\"" 
        
    case "${GLOBAL_CODE_EVALUATION}" in
    "true")
        DisplayTrace ""
        DisplayTrace "    Action will be executed"
        RETURN_CODE=0
        ;;

    "false")
        DisplayTrace ""
        DisplayTrace "    Action won't be executed"
        RETURN_CODE=1
        ;;

    *)
        DisplayTrace ""
        DisplayTrace "    Action won't be executed"
        RETURN_CODE=1
        ;;

    esac                        
        
    return ${RETURN_CODE}
}


###########################################################################
# AddServiceInetd
###########################################################################

# 
# Mise-a-jour du service (demon) dans une configuration avec demon inetd
# (mode installation)
# 
# Aucun argument 
# 
# 
# Code de retour : 
# 
# 
# 
# 
# Attention :
# Utilisation des variables deja definies par la fonction AddInetServiceINST () 
# (action ADD_INET_SERVICE).
# Fonction appelee uniquement par AddInetServiceINST ().
# 
# 
# 

AddServiceInetd ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> AddServiceInetd"


    if [ -f "${FILE_INET_SERVICES}" ]
    then
        # Controle presence ligne 
        # (Entre les crochets : espace et tabulation)
        ${GREP} '^'"${INET_SERVICE_NAME}"'[ 	]' "${FILE_INET_SERVICES}" >/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            # (Entre les crochets : espace et tabulation)
            MESSAGE=`${GREP} '^'"${INET_SERVICE_NAME}"'[ 	]' "${FILE_INET_SERVICES}" `
            DisplayTrace "    Entry already found for service ${INET_SERVICE_NAME} in ${FILE_INET_SERVICES}" 
            DisplayTrace "    ${MESSAGE}"

            DisplaySummary "Entry already found for service ${INET_SERVICE_NAME} in ${FILE_INET_SERVICES}" 
            DisplaySummary "${MESSAGE}"

            # Fichier temporaire vide cree avec les memes droits et proprietaire que
            # l'original
            FILE_TEMP="${FILE_INET_SERVICES}.$$"
            ${CP} -p "${FILE_INET_SERVICES}" "${FILE_TEMP}"
            > "${FILE_TEMP}"

            ${CP} -p "${FILE_INET_SERVICES}" "${FILE_INET_SERVICES}.${SUFFIX}"

            DisplayTrace "    Removal of existing entry"
            DisplaySummary "Removal of existing entry"

            # Suppression entree existante
            ${GREP} -v '^'"${INET_SERVICE_NAME}"'[ 	]' "${FILE_INET_SERVICES}" > "${FILE_TEMP}"
            ${CP} -p "${FILE_TEMP}" "${FILE_INET_SERVICES}" 
            ${RM} "${FILE_TEMP}"
        else
            ${CP} -p "${FILE_INET_SERVICES}" "${FILE_INET_SERVICES}.${SUFFIX}"
        fi    

        echo "${INET_SERVICE_NAME} ${INET_SERVICE_ENDPOINT_TYPE} ${INET_SERVICE_PROTOCOL} ${INET_SERVICE_WAIT_STATUS} ${INET_SERVICE_UID} ${INET_SERVICE_SERVER_PROGRAM} ${INET_SERVICE_SERVER_ARGUMENT}" >> "${FILE_INET_SERVICES}"

        DisplayTrace "    File ${FILE_INET_SERVICES} updated"
        DisplaySummary "File ${FILE_INET_SERVICES} updated"
    else
        # Messages indicatifs : pas d'erreur
        MESSAGE="File ${FILE_INET_SERVICES} not found"
        DisplayTrace "${MESSAGE}"
        DisplaySummary "${MESSAGE}"
        
        MESSAGE="Line that would be added in ${FILE_INET_SERVICES}:"
        DisplayTrace "${MESSAGE}"
        DisplaySummary "${MESSAGE}"
        
        MESSAGE="${INET_SERVICE_NAME} ${INET_SERVICE_ENDPOINT_TYPE} ${INET_SERVICE_PROTOCOL} ${INET_SERVICE_WAIT_STATUS} ${INET_SERVICE_UID} ${INET_SERVICE_SERVER_PROGRAM} ${INET_SERVICE_SERVER_ARGUMENT}"
        DisplayTrace "${MESSAGE}"
        DisplaySummary "${MESSAGE}"

        RETURN_CODE=1
    fi

    return ${RETURN_CODE}
}


###########################################################################
# AddServiceXinetd
###########################################################################

# 
# Mise-a-jour du service (demon) dans une configuration avec demon xinetd
# (mode installation)
# 
# Aucun argument 
# 
# 
# Code de retour : 
# 
# 
# 
# 
# Attention :
# Utilisation des variables deja definies par la fonction AddInetServiceINST () 
# (action ADD_INET_SERVICE).
# Fonction appelee uniquement par AddInetServiceINST ().
# 
# 
# 

AddServiceXinetd ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> AddServiceXinetd"


    if [ -f "${FILE_INET_SERVICES}" ]
    then
        # Recherche du repertoire pour xinetd
        # Le repertoire ne est indique par le parametre "includedir" dans xinetd.conf
        ${GREP} '^'"includedir" "${FILE_INET_SERVICES}" >/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            XINETD_DIR=`${GREP} '^'"includedir" "${FILE_INET_SERVICES}" | ${AWK} '{print $2}'`

            DisplayTrace "    XINETD_DIR : ${XINETD_DIR}"

            # A faire : tester existence repertoire
            if [ ! -d "${XINETD_DIR}" ]
            then
                DisplayTrace "Directory \"${XINETD_DIR}\" not found"
                DisplayTrace "Creating directory \"${XINETD_DIR}\" "

                ${MKDIR} "${XINETD_DIR}"
                if [ $? -ne 0 ]
                then
                    DisplayTrace "Error while creating directory \"${XINETD_DIR}\" "
                    RETURN_CODE=2
                    # Erreur non fatale ? 
                fi    
            fi

            if [ ${RETURN_CODE} -eq 0 ]
            then
                FILE_XINETD_CONFIGURATION="${XINETD_DIR}/${INET_SERVICE_NAME}"

                if [ -f "${FILE_XINETD_CONFIGURATION}" ]
                then
                    DisplayTrace "    File ${FILE_XINETD_CONFIGURATION} already found"
                    DisplayTrace "    The file will be removed"

                    DisplaySummary "File ${FILE_XINETD_CONFIGURATION} already found"
                    DisplaySummary "The file will be removed"
                fi

                > "${FILE_XINETD_CONFIGURATION}"


                # Creation du fichier de configuration du service pour xinetd
                # Suppression a faire ?????

                echo "#"                            >> "${FILE_XINETD_CONFIGURATION}"
                echo "# @(#) ${INET_SERVICE_NAME}"  >> "${FILE_XINETD_CONFIGURATION}"
                echo "#"                            >> "${FILE_XINETD_CONFIGURATION}"

                echo ""                             >> "${FILE_XINETD_CONFIGURATION}"
                echo ""                             >> "${FILE_XINETD_CONFIGURATION}"
                echo "service ${INET_SERVICE_NAME}" >> "${FILE_XINETD_CONFIGURATION}"
                echo "{"                            >> "${FILE_XINETD_CONFIGURATION}"
                echo "    disable         = no"     >> "${FILE_XINETD_CONFIGURATION}"
                echo "    flags           = REUSE"  >> "${FILE_XINETD_CONFIGURATION}"
                echo "    socket_type     = ${INET_SERVICE_ENDPOINT_TYPE}" >> "${FILE_XINETD_CONFIGURATION}"  
                echo "    protocol        = ${INET_SERVICE_PROTOCOL}"      >> "${FILE_XINETD_CONFIGURATION}"

                # La valeur pour le parametre "wait" est "yes/no" au lieu de "wait/nowait".
                case ${INET_SERVICE_WAIT_STATUS} in
                "wait") 
                    echo "    wait            = yes" >> "${FILE_XINETD_CONFIGURATION}"
                    ;;

                "nowait") 
                    echo "    wait            = no" >> "${FILE_XINETD_CONFIGURATION}"
                    ;;

                *)
                    DisplayError "Unknown value for \"INET_SERVICE_WAIT_STATUS\" : \"${INET_SERVICE_WAIT_STATUS}\" "
                    RETURN_CODE=1
                    ;;
                esac 

                echo "    user            = ${INET_SERVICE_UID}"            >> "${FILE_XINETD_CONFIGURATION}"
                echo "    server          = ${INET_SERVICE_SERVER_PROGRAM}" >> "${FILE_XINETD_CONFIGURATION}"

                ARGUMENT=`echo ${INET_SERVICE_SERVER_ARGUMENT} | ${CUT} -d' ' -f 2-`
                echo "    server_args     = ${ARGUMENT} " >> "${FILE_XINETD_CONFIGURATION}"


                echo "    log_on_failure += USERID HOST "             >> "${FILE_XINETD_CONFIGURATION}"
                echo "    log_on_success += USERID PID HOST DURATION" >> "${FILE_XINETD_CONFIGURATION}"
                echo "}"                            >> "${FILE_XINETD_CONFIGURATION}"
                echo ""                             >> "${FILE_XINETD_CONFIGURATION}"


                DisplayTrace "    File ${FILE_XINETD_CONFIGURATION} created"
                DisplaySummary "File ${FILE_XINETD_CONFIGURATION} created"
            fi

        else
            DisplayError "Keyword \"includedir\" not found in ${FILE_INET_SERVICES}"
            RETURN_CODE=1
        fi                   

    else
        # Messages indicatifs : pas d'erreur
        MESSAGE="File ${FILE_INET_SERVICES} not found"
        DisplayTrace "${MESSAGE}"
        DisplaySummary "${MESSAGE}"
        
        RETURN_CODE=1
    fi

    return ${RETURN_CODE}
}


###########################################################################
# AddServiceInetdSMF
###########################################################################

# 
# Mise-a-jour du service (demon) dans une configuration avec demon inetd
# sous Solaris 10 (Service Management Facility) (mode installation)
# 
# Aucun argument 
# 
# 
# A present une erreur au niveau de l'ajout du service ne doit pas etre fatale.
# Les commandes Solaris 10 ont leur stderr redirige pour eviter une erreur fatale.
# 
# Code de retour : 
# 
# 0 : aucune erreur
# 1 : une erreur est survenue
# 
# 
# 
# Attention :
# Utilisation des variables deja definies par la fonction AddInetServiceINST () 
# (action ADD_INET_SERVICE).
# Fonction appelee uniquement par AddInetServiceINST ().
# 
# 
# 

AddServiceInetdSMF ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> AddServiceInetdSMF"


    ### A FAIRE ### Detecter si le service est deja present

    DisplayTrace ""
    DisplayTrace "    Solaris 10 version service specific processing"

    FILE_INET_SMF="${LOCATION_PATH}/install/db2hpu_inetd.conf"

    DisplayTrace ""
    DisplayTrace "    Creation of dedicated service configuration file \"${FILE_INET_SMF}\" "


    # Creation d'un fichier analogue a inetd.conf mais ne contenant que 
    # le service db2hpudm3

    echo "#"                            >> "${FILE_INET_SMF}"
    echo "# @(#) ${INET_SERVICE_NAME}"  >> "${FILE_INET_SMF}"
    echo "#"                            >> "${FILE_INET_SMF}"

    echo ""                             >> "${FILE_INET_SMF}"
    echo ""                             >> "${FILE_INET_SMF}"

    echo "${INET_SERVICE_NAME} ${INET_SERVICE_ENDPOINT_TYPE} ${INET_SERVICE_PROTOCOL} ${INET_SERVICE_WAIT_STATUS} ${INET_SERVICE_UID} ${INET_SERVICE_SERVER_PROGRAM} ${INET_SERVICE_SERVER_ARGUMENT}" >> "${FILE_INET_SMF}"


    DisplayTrace ""
    DisplayTrace "    Contents of file \"${FILE_INET_SMF}\" :"
    DisplayTrace "    --------------------------------------------"
    DisplayTextFileTrace "${FILE_INET_SMF}"
    DisplayTrace "    --------------------------------------------"
    DisplayTrace ""


    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Creation du service a partir du fichier specifique
        # Avertisssement : toutes les entrees de inetd.conf sont converties
        DisplayTrace ""
        DisplayTrace "    Conversion of dedicated service configuration file into service" 

        ActivateTraceRedirection

        # >>> Modifier option -o !!!???
        # /usr/sbin/inetconv -f -i ${FILE_INET_SMF} -o /tmp
        /usr/sbin/inetconv -f -i "${FILE_INET_SMF}" 2>&1
        RETURN_CODE=$?

        DeactivateTraceRedirection
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace ""
        DisplayTrace "    Service state"

        ActivateTraceRedirection

        /usr/sbin/inetadm | ${GREP} '/'${INET_SERVICE_NAME}'/' 2>&1
        RETURN_CODE=$?

        DeactivateTraceRedirection
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace ""
        DisplayTrace "    Service detailed state"

        ActivateTraceRedirection

        \svcs -l '*'${INET_SERVICE_NAME}'*' 2>&1
        RETURN_CODE=$?

        DeactivateTraceRedirection
    fi

    return ${RETURN_CODE}
}


###########################################################################
# ExecuteInetdConfigReloadCommand
###########################################################################

# 
# Rechargement de la configuration Inet par excution d'une commande
# Fonction commune
# 
# Argument 1 : nom de plXXXateforme
# 
#
# Attention :
# Utilisation des variables deja definies par l'action ADD_INET_SERVICE
# Fonction appelee uniquement par AddInetServiceINST () et AddInetServiceUNINST ()
# 

ExecuteInetdConfigReloadCommand ()
{
    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> ExecuteInetdConfigReloadCommand"


    if [ $# -ne 0 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        # Relecture du fichier de configuration du demon

        PID_INETD=`GetPIDFromName  "${DAEMON_INETD}"`
        RETURN_CODE=$?
        
        if [ ${RETURN_CODE} -eq 0 ]
        then
            # Evaluation de la ligne de commande a utiliser
        
            # Les variables DAEMON_INETD_RECONFIG_COMMAND et 
            # DAEMON_INETD_RECONFIG_PARAMETERS ne sont evaluees qu'ici car elle
            # peuvent utiliser des variables (PID_INETD, DAEMON_INETD, ...) qui
            # n'existent que maintenant

            DisplayTrace ""
            DisplayTrace "    Before evaluation : "
            DisplayTrace "    DAEMON_INETD_RECONFIG_COMMAND    : ${DAEMON_INETD_RECONFIG_COMMAND}"
            DisplayTrace "    DAEMON_INETD_RECONFIG_PARAMETERS : ${DAEMON_INETD_RECONFIG_PARAMETERS}"

            DAEMON_INETD_RECONFIG_COMMAND=`eval echo "${DAEMON_INETD_RECONFIG_COMMAND}"`
            DAEMON_INETD_RECONFIG_PARAMETERS=`eval echo "${DAEMON_INETD_RECONFIG_PARAMETERS}"`

            DisplayTrace ""
            DisplayTrace "    After evaluation : "
            DisplayTrace "    DAEMON_INETD_RECONFIG_COMMAND    : ${DAEMON_INETD_RECONFIG_COMMAND}"
            DisplayTrace "    DAEMON_INETD_RECONFIG_PARAMETERS : ${DAEMON_INETD_RECONFIG_PARAMETERS}"

            if [ "${DAEMON_INETD}" = "inetd" -o -f "${DAEMON_INETD_RECONFIG_COMMAND}" ]
            then
                CheckExecutableFile ${DAEMON_INETD_RECONFIG_COMMAND}
                RETURN_CODE=$?
            else
                DisplayTrace ""
                DisplayTrace "    Reconfiguration command ${DAEMON_INETD_RECONFIG_COMMAND} not found" 
                DisplayTrace "    Trying the fallback command" 
                DAEMON_INETD_RECONFIG_COMMAND="service ${DAEMON_INETD}"
            fi
        else
            DisplayTrace "Problem identifying inet daemon process"
        fi


        # Si une erreur est survenue, erreur consideree comme non fatale
        # et pas de message dans la trace d'erreur
        
        if [ ${RETURN_CODE} -eq 0 ]
        then
            DisplayTrace ""
            DisplayTrace "    Executing command :" 
            DisplayTrace "    ${DAEMON_INETD_RECONFIG_COMMAND} ${DAEMON_INETD_RECONFIG_PARAMETERS}"
        
            ActivateTraceRedirection
        
            ${DAEMON_INETD_RECONFIG_COMMAND} ${DAEMON_INETD_RECONFIG_PARAMETERS}
            
            DeactivateTraceRedirection

            # Tester code de retour ?
            
            DisplayTrace "    ${DAEMON_INETD} daemon configuration file reloaded" 
            
            DisplaySummary "${DAEMON_INETD} daemon configuration file reloaded" 
            DisplaySummary "by using ${DAEMON_INETD_RECONFIG_COMMAND} ${DAEMON_INETD_RECONFIG_PARAMETERS}" 
        else
            DisplayWarning "Problem accessing ${DAEMON_INETD} daemon for configuration file reloading"
            DisplayWarning "Please restart ${DAEMON_INETD} daemon manually"
            
            DisplaySummary "Problem accessing ${DAEMON_INETD} daemon for configuration file reloading"
            DisplaySummary "Please restart ${DAEMON_INETD} daemon manually"
            
            RETURN_CODE=0
        fi    
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# FormatPlatformName
###########################################################################

# 
# Reformatage du nom de plateforme transmis par InstallShield
# 
# 
# Argument 1 : nom de plateforme
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# Aucun affichage parasite doit etre fait dans cette fonction.
# 
# Pour Solaris, le nom peut varier entre "Solaris" et "SunOS" selon la version du 
# JVM utilise.
# 
# Pour Linux, differenciation selon l'architecture pour "linux", "zlinux", "plinux", "plinuxle" et "ialinux"
# 
# 
# Attention : ARCHITECTURE non converti en minuscules
# 

FormatPlatformName ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> FormatPlatformName"
    

    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else  
        NAME=`\echo $1 | \tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz'`
        case "${NAME}" in
        
        "linux"|"Linux")

            case "${ARCHITECTURE}" in
            
            "x86"|"i386")
                echo "linux"
                ;;
            
            "s390"|"s390x") 
                echo "zlinux"
                ;;

            "ppc64") 
                echo "plinux"
                ;;

            "ppc64le") 
                echo "plinuxle"
                ;;

            "ia64")
                echo "ialinux"
                ;;

            *) 
                echo '\07Error'
                RETURN_CODE=1  
                ;;       
            esac         

            ;;

        "solaris"|"sunos")

            case "${ARCHITECTURE}" in
            
            "i386")
                echo "sun_x86"
                ;;

            "sparc")
                echo "sun"
                ;;

            *)
                echo '\07Error'
                RETURN_CODE=1  
                ;;       
            esac         

            ;;
            
        "aix")
            echo "aix"
            ;;
            
        "hp-ux")
            # echo "hp"
            case "${ARCHITECTURE}" in
            
            "PA_RISC")
                echo "hp"
                ;;
            
            "IA64N") 
                echo "hpia"
                ;;

            *) 
                echo '\07Error'
                RETURN_CODE=1  
                ;;       
            esac         
            ;;
            
        *)
            echo '\07Error'
            RETURN_CODE=1        
            ;;
       esac
        
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# InitActionFilesVariables
###########################################################################

# 
# Initialisation des variables correspondant aux fichiers des actions
# 
# Etiquette de la section : "action_files"
#
#
#

InitActionFilesVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitActionFilesVariables"
    
    
    # Fichiers des actions pour les differents features (unload, network, plugin) 
    # lors de l'installation et la desinstallation  
        
        
    FILE_ACTIONS_INSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_files" "file_actions_install"` 

    DisplayTrace "    FILE_ACTIONS_INSTALL : ${FILE_ACTIONS_INSTALL}"


    FILE_ACTIONS_UNINSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_files" "file_actions_uninstall"` 

    DisplayTrace "    FILE_ACTIONS_UNINSTALL : ${FILE_ACTIONS_UNINSTALL}"




     

    
    
    # Fichiers prologue et epilogue pour l'installer
    # A faire : penser aux chemins complets !!!
    
    FILE_ACTIONS_PROLOG_INSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_installers_files" "file_actions_prolog_install"` 

    DisplayTrace "    FILE_ACTIONS_PROLOG_INSTALL : ${FILE_ACTIONS_PROLOG_INSTALL}"


    FILE_ACTIONS_EPILOG_INSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_installers_files" "file_actions_epilog_install"` 

    DisplayTrace "    FILE_ACTIONS_EPILOG_INSTALL : ${FILE_ACTIONS_EPILOG_INSTALL}"



    # Fichiers prologue et epilogue pour l'uninstaller
    # A faire : penser aux chemins complets !!!
    
    FILE_ACTIONS_PROLOG_UNINSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_installers_files" "file_actions_prolog_uninstall"` 

    DisplayTrace "    FILE_ACTIONS_PROLOG_UNINSTALL : ${FILE_ACTIONS_PROLOG_UNINSTALL}"

    
    FILE_ACTIONS_EPILOG_UNINSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_installers_files" "file_actions_epilog_uninstall"` 

    DisplayTrace "    FILE_ACTIONS_EPILOG_UNINSTALL : ${FILE_ACTIONS_EPILOG_UNINSTALL}"

    
    case "${CONTEXT}" in
    "inst")
        if [ ${RETURN_CODE} -eq 0 ]
        then
            CheckReadableFile "${FILE_ACTIONS_PROLOG_INSTALL}"
            RETURN_CODE=$?
        fi    

        if [ ${RETURN_CODE} -eq 0 ]
        then
            CheckReadableFile "${FILE_ACTIONS_EPILOG_INSTALL}"
            RETURN_CODE=$?
        fi    
        ;;

    "uninst")
        if [ ${RETURN_CODE} -eq 0 ]
        then
            CheckReadableFile "${FILE_ACTIONS_PROLOG_UNINSTALL}"
            RETURN_CODE=$?
        fi    

        if [ ${RETURN_CODE} -eq 0 ]
        then
            CheckReadableFile "${FILE_ACTIONS_EPILOG_UNINSTALL}"
            RETURN_CODE=$?
        fi    
        ;;
    esac
    
    return ${RETURN_CODE}
}


###########################################################################
# InitArchiveFilesVariables
###########################################################################

# 
# Initialisation des variables correspondant aux fichiers archives (tar)
# 
# Etiquette de la section : "archive_files"
#
# Uniquement si on est en mode installation
# Necessite la liste des features a installer (cree par PrepareActionsLists ())
#

InitArchiveFilesVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitArchiveFilesVariables"

    
    
    if [ "${CONTEXT}" = "inst" ]
    then
        FILE_ARCHIVES=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}archive_files" "file_archives"` 

        DisplayTrace "    FILE_ARCHIVES : ${FILE_ARCHIVES}"

        # Verification presence fichiers d'archive (uniquement pour les features a installer)
        if [ ${RETURN_CODE} -eq 0 ]
        then
            for NAME in ${LIST_FEATURES_INSTALL}
            do
                FILE=`GetFieldSublist ${FILE_ARCHIVES} '|' ':' ${NAME} 2`

                DisplayTrace "    Feature : \"$NAME\" "
                DisplayTrace "    FILE    : \"${FILE}\" "

                CheckReadableFile ${FILE}
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -ne 0 ]
                then
                    break
                fi            
            done
        fi    
    fi    
        
    return ${RETURN_CODE}
}


###########################################################################
# InitScriptFilesVariables
###########################################################################

# 
# Initialisation des variables correspondant aux fichiers scripts
# 
# Etiquette de la section : "script_files"
#
#
#

InitScriptFilesVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitScriptFilesVariables"


    SCRIPT_INIT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}script_files" "script_init"` 
    
    DisplayTrace "    Before evaluation :"
    DisplayTrace "    SCRIPT_INIT : ${SCRIPT_INIT}"

    SCRIPT_INIT=`\eval echo "${SCRIPT_INIT}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation :"
    DisplayTrace "    SCRIPT_INIT : ${SCRIPT_INIT}"
    
    return ${RETURN_CODE}
}


###########################################################################
# PrepareActionsLists
###########################################################################

#
# Preparation des listes d'actions a executer
#
# 
#

PrepareActionsLists ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> PrepareActionsLists"


    # Utilisation du fichier d'actions correspondant en fonction du contexte
    # >>> A faire : FILE_ACTIONS a passer en parametre ????
    
    case "${CONTEXT}" in

    "inst")    
        LIST_FEATURES_INSTALL=""
        LIST_FEATURES_NO_INSTALL=""
        
        for NAME in ${LIST_FEATURES}
        do
            # Exploitation uniquement de la propriete InstallShield "Active" (champ 2 de la sous-liste)
            PROPERTY_INSTALL=`GetFieldSublist ${FEATURES} '|' ':' ${NAME} 2`
            

            DisplayTrace "    Feature \"$NAME\" "
            DisplayTrace "    PROPERTY_INSTALL     : \"${PROPERTY_INSTALL}\" "

            case "${PROPERTY_INSTALL}" in
            "true")
                LIST_FEATURES_INSTALL="${LIST_FEATURES_INSTALL}${NAME} "
                ;;
                
            "false")
                LIST_FEATURES_NO_INSTALL="${LIST_FEATURES_NO_INSTALL}${NAME} "
                ;;

            *)
                DisplayTraceAndError "Unexpected value for PROPERTY_INSTALL : \"${PROPERTY_INSTALL}\" "
                RETURN_CODE=1
                ;;
            esac
        done
        

        DisplayTrace "    List of features to install :"
        DisplayTrace "    LIST_FEATURES_INSTALL : \"${LIST_FEATURES_INSTALL}\" "
        DisplayTrace "    List of features not to be installed :"
        DisplayTrace "    LIST_FEATURES_NO_INSTALL : \"${LIST_FEATURES_NO_INSTALL}\" "
        ;;


    "uninst")    
        LIST_FEATURES_UNINSTALL=""
        
        for NAME in ${LIST_FEATURES}
        do
            # Exploitation des proprietes InstallShield "Active" (champ 2 de la sous-liste)
            # et "ActiveForUninstall" (champ 3 de la sous-liste)
            # On ne desinstalle que ce qui est marque comme deja installe
            # Il y a exploitation des deux variables provenant d'InstallShield
            # afin de determine ce qui est rellement installe (methode en fait non
            # completement fiable) 
            PROPERTY_INSTALL=`GetFieldSublist ${FEATURES} '|' ':' ${NAME} 2`
            PROPERTY_UNINSTALL=`GetFieldSublist ${FEATURES} '|' ':' ${NAME} 3`
            

            DisplayTrace "    Feature \"$NAME\" :  "
            DisplayTrace "    PROPERTY_INSTALL     : \"${PROPERTY_INSTALL}\" "
            DisplayTrace "    PROPERTY_UNINSTALL   : \"${PROPERTY_UNINSTALL}\" "
            DisplayTrace ""

        
            if [ "${PROPERTY_UNINSTALL}" = "true" ]
            then
                if [ "${PROPERTY_INSTALL}" = "true" ]
                then
                    LIST_FEATURES_UNINSTALL="${LIST_FEATURES_UNINSTALL}${NAME} "
                fi
            fi    
        done

        # Inversion de la liste
        LIST_FEATURES_UNINSTALL=`ReverseString "${LIST_FEATURES_UNINSTALL}"` 
        
        DisplayTrace "    List of features to uninstall :"
        DisplayTrace "    LIST_FEATURES_UNINSTALL : \"${LIST_FEATURES_UNINSTALL}\" "
        ;;
        
    *)
        DisplayError "Unknown context \"${CONTEXT}\" "
        RETURN_CODE=2    
        ;;
    esac


    # Creation des fichiers des actions de desinstallation 

    if [ ${RETURN_CODE} -eq 0 ]
    then    
    
        if [ "${CONTEXT}" = "uninst" ]
        then
        
            # Creation des fichiers des actions de desinstallation 
         
            # Creation du fichier des actions de desinstallation : inversion
            # de l'ordre des lignes du fichier d'installation
            
            for NAME in ${LIST_FEATURES}
            do
                FILE_INSTALL=`GetFieldSublist ${FILE_ACTIONS_INSTALL} '|' ':' ${NAME} 2`
                FILE_UNINSTALL=`GetFieldSublist ${FILE_ACTIONS_UNINSTALL} '|' ':' ${NAME} 2`
                
                DisplayTrace "    Feature : \"$NAME\" "
                DisplayTrace "    FILE_INSTALL     : \"${FILE_INSTALL}\" "
                DisplayTrace "    FILE_UNINSTALL   : \"${FILE_UNINSTALL}\" "
           
                ReverseFile "${FILE_INSTALL}" "${FILE_UNINSTALL}"
            done

        fi
    fi    





    # Creation de la liste des fichiers d'actions pour l'installation/desinstallation

    case "${CONTEXT}" in

    "inst")    

        LIST_FILES_ACTIONS_INSTALL=""
        
        for NAME in ${LIST_FEATURES_INSTALL}
        do
            FILE_INSTALL=`GetFieldSublist ${FILE_ACTIONS_INSTALL} '|' ':' ${NAME} 2`
            LIST_FILES_ACTIONS_INSTALL="${LIST_FILES_ACTIONS_INSTALL}${FILE_INSTALL} "
        done

        DisplayTrace "    List of action files to execute for install :"
        DisplayTrace "    LIST_FILES_ACTIONS_INSTALL : \"${LIST_FILES_ACTIONS_INSTALL}\" "
        ;;

    "uninst")    

        LIST_FILES_ACTIONS_UNINSTALL=""
        
        for NAME in ${LIST_FEATURES_UNINSTALL}
        do
            FILE_UNINSTALL=`GetFieldSublist ${FILE_ACTIONS_UNINSTALL} '|' ':' ${NAME} 2`
            LIST_FILES_ACTIONS_UNINSTALL="${LIST_FILES_ACTIONS_UNINSTALL}${FILE_UNINSTALL} "
        done

        DisplayTrace "    List of action files to execute for uninstall :"
        DisplayTrace "    LIST_FILES_ACTIONS_UNINSTALL : \"${LIST_FILES_ACTIONS_UNINSTALL}\" "
        ;;

    *)
        DisplayError "Unknown context \"${CONTEXT}\" "
        RETURN_CODE=2    
        ;;
    esac


    # Si desinstallation, recherche d'elements communs

    ###
    ### A faire dans version future
    ###    
    ### Manque liste/fichier indiquant ce qui est deja installe   
    ### Pour faire la difference avec ce que l'onv eut desinstaller   

    return ${RETURN_CODE}
}


###########################################################################
# InitVariables
###########################################################################

#
# Initialisation variables
#
# Amelioration : gestion des versions differentes pour une meme plateforme

InitVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitVariables"

    
    # A mettre en action dans une version future du script
    USER_MASK="022"
    
    \umask ${USER_MASK}
    
    DisplayTrace "    UMASK : ${USER_MASK}"
        
    # Le fichier de configuration porte un nom specifique a chaque plateforme 
    # afin d'eviter toute confusion
    # Le nom de la plateforme est transmis par InstallShield et est "converti"
    # afin de correspondre aux noms de plateformes HPU ("sun" au lieu de 
    # "solaris", etc.)
    PLATFORM_CONVERTED=`FormatPlatformName ${PLATFORM}`
    RETURN_CODE=$?
    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace ""
        DisplayTrace "    PLATFORM_CONVERTED : \"${PLATFORM_CONVERTED}\""

        DisplaySummary ""
        DisplaySummary "Platform code name : \"${PLATFORM_CONVERTED}\""


        PLATFORM=${PLATFORM_CONVERTED}
        
        case "${SUB_CONTEXT}" in
        
        "FIXPACK")
            PLATFORM_CONFIG_FILE="install_fp_${PLATFORM}.ini"
            ;;
        
        "")
            PLATFORM_CONFIG_FILE="install_${PLATFORM}.ini"
            ;;
        
        *)
            DisplayTrace "Unknown context \"${CONTEXT}\" " 
            DisplayError "Unknown context \"${CONTEXT}\" "
            RETURN_CODE=2            
            ;;
            
        esac
        
        PREFIX_CONFIG_FILE="tmp_section_"
        
    else    
    
        DisplayError "Unidentified platform : \"${PLATFORM}\" "
        DisplayTrace "Unidentified platform : \"${PLATFORM}\" " 

    fi
    
    
    # Repertoire pour les fichiers temporaires

    DIR_TEMP="${LOCATION_PATH}/temp"
    
    \mkdir "${DIR_TEMP}"
    # RETURN_CODE=$?
    
    
    # Liste complete des features (a inst/desinstaller ou non)
    LIST_FEATURES=`echo ${FEATURES} | \tr -s '|' '\n' | \cut -d':' -f1`


    DisplayTrace "    LIST_FEATURES     : \"${LIST_FEATURES}\" "    

    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Decomposition fichier de configuration selon les sections
        # qu'il contient
        PrepareINIFile "${PLATFORM_CONFIG_FILE}" "${PREFIX_CONFIG_FILE}"
        RETURN_CODE=$?
    fi
    
    
    FILE_ARCHIVE="archive.tar"

     
    # C'est normalement un argument d'appel ? (ou repertoire courant ?)
    # Repertoire d'ou s'execute le script d'installation / repertoire 
    # pour extraire l'archive des fichiers a installer
    ### DIRECTORY_TEMP='/home/a_ng/HPU/ScriptsInstallation/ArborescenceTest/tmp'
    
    
    DIR_ARCHIVE="${DIR_TEMP}/Files"
    
    
    # Fichier memorisant certaines informations en vue de la desinstallation 
    FILE_INSTALL_SETTINGS="${LOCATION_PATH}/install/hpu_inst_settings.cfg"
    
    case "${CONTEXT}" in

    "inst")    
        case "${SUB_CONTEXT}" in

        "FIXPACK")
            if [ ${RETURN_CODE} -eq 0 ]
            then
                CheckReadableFile "${FILE_INSTALL_SETTINGS}"
                RETURN_CODE=$?
            fi    
            ;;

        "")
            > "${FILE_INSTALL_SETTINGS}"

            echo "#"                            >> "${FILE_INSTALL_SETTINGS}"
            echo "# @(#) HPU installation settings file"  >> "${FILE_INSTALL_SETTINGS}"
            echo "#"                            >> "${FILE_INSTALL_SETTINGS}"
            echo ""                             >> "${FILE_INSTALL_SETTINGS}"
            echo ""                             >> "${FILE_INSTALL_SETTINGS}"
            ;;

        *)
            DisplayTrace "Unknown sub-context \"${SUB_CONTEXT}\" " 
            DisplayError "Unknown sub-context \"${SUB_CONTEXT}\" "
            RETURN_CODE=2            
            ;;
            
        esac
        ;;
        
    "uninst")    

        if [ ${RETURN_CODE} -eq 0 ]
        then
            CheckReadableFile "${FILE_INSTALL_SETTINGS}"
            RETURN_CODE=$?
        fi    
        ;;
      
        
    *)    
        DisplayError "Unknown context \"${CONTEXT}\" "
        RETURN_CODE=2    
        ;;
        
    esac
        

    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitCommandsVariables
        RETURN_CODE=$?
    fi
     
    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitSystemFilesVariables
        RETURN_CODE=$?
    fi

        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitActionFilesVariables
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitOtherSystemVariables
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitScriptFilesVariables
        RETURN_CODE=$?
    fi


    # Plus de plug-in a present.
    # STANDALONE_INSTALLATION vaut 1.
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        case "${CONTEXT}" in 
        "inst")

            case "${SUB_CONTEXT}" in

            "FIXPACK")
                # Recuperation de l'information STANDALONE_INSTALLATION

                DisplayTrace "    Retrieval of parameter \"STANDALONE_INSTALLATION}\" from installation settings file"

                STANDALONE_INSTALLATION=`${GREP} "STANDALONE_INSTALLATION" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
                if [ $? -ne 0 ]
                then
                    # Information non memorisee
                    DisplayTrace "Parameter \"STANDALONE_INSTALLATION\" not found in ${FILE_INSTALL_SETTINGS}"
                    DisplayError "Parameter \"STANDALONE_INSTALLATION\" not found in ${FILE_INSTALL_SETTINGS}"
                    RETURN_CODE=1
                else
                    DisplayTrace "    STANDALONE_INSTALLATION : \"${STANDALONE_INSTALLATION}\" "
                fi
                ;;

            "")
                STANDALONE_INSTALLATION=1

                DisplayTrace "    STANDALONE_INSTALLATION     : \"${STANDALONE_INSTALLATION}\" "            
                ;;


            *)
                DisplayTrace "Unknown sub-context \"${SUB_CONTEXT}\" " 
                DisplayError "Unknown sub-context \"${SUB_CONTEXT}\" "
                RETURN_CODE=2            
                ;;

            esac
            ;;
            
        "uninst")

            # Recuperation de l'information STANDALONE_INSTALLATION

            DisplayTrace "    Retrieval of parameter \"STANDALONE_INSTALLATION}\" from installation settings file"

            STANDALONE_INSTALLATION=`${GREP} "STANDALONE_INSTALLATION" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
            if [ $? -ne 0 ]
            then
                # Information non memorisee
                DisplayTrace "Parameter \"STANDALONE_INSTALLATION\" not found in ${FILE_INSTALL_SETTINGS}"
                DisplayError "Parameter \"STANDALONE_INSTALLATION\" not found in ${FILE_INSTALL_SETTINGS}" 
                RETURN_CODE=1
            else
                DisplayTrace "    STANDALONE_INSTALLATION : \"${STANDALONE_INSTALLATION}\" "
            fi
            ;;
        
        *)
            DisplayError "Unknown context \"${CONTEXT}\" "
            RETURN_CODE=2    
            ;;
            
        esac            
    fi

    return ${RETURN_CODE}
}


###########################################################################
# AddLineFileINST
###########################################################################

#
# Ajout d'une ligne a la fin d'un fichier texte  
# (installation)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "add_line_file"
#
# Pb portabilite de la commande eval  ?
# 

AddLineFileINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> AddLineFileINST"


    FILE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "file"` 
    LINE_TO_ADD=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_to_add"` 


    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    FILE        : \"${FILE}\"" 
    DisplayTrace "    LINE_TO_ADD : \"${LINE_TO_ADD}\"" 

    FILE=`\eval echo "${FILE}"`
    LINE_TO_ADD=`\eval echo "${LINE_TO_ADD}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    FILE        : \"${FILE}\"" 
    DisplayTrace "    LINE_TO_ADD : \"${LINE_TO_ADD}\"" 


    if [ ${RETURN_CODE} -eq 0 ]
    then
        # CheckReadableFile ${FILE}
        CheckWriteableFile "${FILE}"
        RETURN_CODE=$?
    fi

    # Pour l'instant pas de sauvegarde du fichier modifie
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        \echo "${LINE_TO_ADD}" >> "${FILE}"  
        RETURN_CODE=$?
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace "    Line added successfully"
    fi

    return ${RETURN_CODE}
}


###########################################################################
# AddLineFileUNINST
###########################################################################

#
# Ajout d'une ligne a la fin d'un fichier texte  
# (desinstallation)
#
# >>> Actuellement rien n'est a faire pour la desinstallation
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "add_line_file"
#
# Pb portabilite de la commande eval  ?
# 

AddLineFileUNINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> AddLineFileUNINST"


    FILE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "file"` 
    LINE_TO_ADD=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_to_add"` 

    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    FILE        : \"${FILE}\"" 
    DisplayTrace "    LINE_TO_ADD : \"${LINE_TO_ADD}\"" 

    FILE=`eval echo "${FILE}"`
    LINE_TO_ADD=`eval echo "${LINE_TO_ADD}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    FILE        : \"${FILE}\"" 
    DisplayTrace "    LINE_TO_ADD : \"${LINE_TO_ADD}\"" 

    DisplayTrace "    No uninstall treatment to do"

    return ${RETURN_CODE}
}


###########################################################################
# AddLineFile
###########################################################################

#
# Mise a jour d'une variable dans un fichier texte (fichier script shell)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "add_line_file"
#
# Pb portabilite de la commande eval  ?
# 

AddLineFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> AddLineFile"

    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)" 
        RETURN_CODE=1
    else    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 

        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"

        
        # Controle de compatibilite de la section avec la commande

        case "${SECTION_TYPE}" in
        
        "add_line_file")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    AddLineFileINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    AddLineFileUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
            
        "no_action")
        
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action" 
            RETURN_CODE=1
            ;;
            
        esac    
        
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# AddServicesINST
###########################################################################

# Ajout du service dans le fichier /etc/services (installation)
#
# Argument 1 : nom de section 
#
# Type de section concernee : "add_services"
#
#

AddServicesINST ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> AddServicesINST"
    
    
    # Lecture infos relatives au service a ajouter

    
    SERVICE_NAME=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" service_name` 
    DisplayTrace "    SERVICE_NAME : ${SERVICE_NAME}"
    

    SERVICE_PORT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" service_port` 
    DisplayTrace "    SERVICE_PORT : ${SERVICE_PORT}"


    SERVICE_PROTOCOL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" service_protocol` 
    DisplayTrace "    SERVICE_PROTOCOL : ${SERVICE_PROTOCOL}"


    SERVICE_COMMENT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" service_comment` 
    DisplayTrace "    SERVICE_COMMENT : ${SERVICE_COMMENT}"


    CONFIG_FILE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" config_file` 
    DisplayTrace "    CONFIG_FILE     : ${CONFIG_FILE}"

    
    IDENTIFIER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "variable"` 
    DisplayTrace "    IDENTIFIER      : \"${IDENTIFIER}\"" 
    
    
    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    CONFIG_FILE  : \"${CONFIG_FILE}\"" 

    CONFIG_FILE=`\eval echo "${CONFIG_FILE}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    CONFIG_FILE  : \"${CONFIG_FILE}\"" 
    

    # Suffixe pour la sauvegarde du fichier original 
    SUFFIX=`date +"%d%m%y_%H%M%S"`

    FILE_TEMP="daemon_lines.$$"
    FILE_SERVICES_NEW="${FILE_SERVICES}.new.$$"


###     # Controle presence ligne 
###     # (Entre les crochets : espace et tabulation)
###     ${GREP} '^'"${SERVICE_DAEMON}"'[ 	]' "${FILE_SERVICES}" >${FILE_TEMP} 2>/dev/null
###     if [ $? -eq 0 ]
###     then
###         DisplayTrace "    Entry already found for daemon ${SERVICE_DAEMON} in ${FILE_SERVICES} :"
###         ${CAT} "${FILE_TEMP}"
### 
###         DisplayTrace "    Removing existing entries for daemon ${SERVICE_DAEMON} in ${FILE_SERVICES}"
###         ${CP} -p ${FILE_SERVICES} ${FILE_SERVICES}.${SUFFIX}
### 
###         # Nouveau fichier cree avec les memes droits et proprietaire que
###         # l'original
###         ${CP} -p "${FILE_SERVICES}" "${FILE_SERVICES_NEW}"
###         > "${FILE_SERVICES_NEW}"
### 
###         # (Entre les crochets : espace et tabulation)
###         ${GREP} -v '^'"${SERVICE_DAEMON}"'[ 	]' "${FILE_SERVICES}" >  ${FILE_SERVICES_NEW}
###         ${MV} ${FILE_SERVICES_NEW} ${FILE_SERVICES}
###     fi
###             
###     ${RM} "${FILE_TEMP}"

    # Controle presence nom du service a ajouter 
    # (Entre les crochets : espace et tabulation)
    
    # Si le service est deja present il est laisse intact : un autre nom de 
    # de service est genere et recherche a nouveau dans le ficheir des services
    #
    # Vocabulaire utilise :"service daemon" est equivalent a "service name"
    
    COUNT=0
    PREFIX_SERVICE_NAME=${SERVICE_NAME}
    
    ${GREP} '^'"${SERVICE_NAME}"'[ 	]' "${FILE_SERVICES}" >"${FILE_TEMP}" 2>/dev/null
    while [ $? -eq 0 ]
    do
        DisplayTrace "    Entry already found for daemon/service ${SERVICE_NAME} in ${FILE_SERVICES} :"
        DisplayTextFileTrace "${FILE_TEMP}"

        COUNT=`${EXPR} ${COUNT} + 1`
        SERVICE_NAME="${PREFIX_SERVICE_NAME}_${COUNT}"
        
        DisplayTrace "    New service name to check : \"${SERVICE_NAME}\" "

        # (Entre les crochets : espace et tabulation)
        ${GREP} '^'"${SERVICE_NAME}"'[ 	]' "${FILE_SERVICES}" >"${FILE_TEMP}" 2>/dev/null
    done

    ${RM} "${FILE_TEMP}" 

    DisplayTrace ""
    DisplayTrace "    Service name chosen : \"${SERVICE_NAME}\" "
    
    DisplaySummary ""
    DisplaySummary "Service name chosen : \"${SERVICE_NAME}\" "

    HPU_SERVICE_NAME=${SERVICE_NAME}
    CheckSettingsVariable "HPU_SERVICE_NAME"
    if [ $? -ne 0 ]
    then
        AddSettingsVariable "HPU_SERVICE_NAME" "${HPU_SERVICE_NAME}"
    else
        DisplayTraceAndError "Information \"HPU_SERVICE_NAME\" already found in settings file"
        RETURN_CODE=1
    fi
    
            
    DisplayTrace ""
    DisplaySummary ""
            
    # Recherche si numero de port deja utilise    
    ${GREP}  '^[^#]*'"${SERVICE_PORT}/" ${FILE_SERVICES} >/dev/null 2>&1 
    while [ $? -eq 0 ]
    do
        DisplayTrace "    Port number ${SERVICE_PORT} already used in ${FILE_SERVICES}"
        
        DisplaySummary "Port number ${SERVICE_PORT} already used in ${FILE_SERVICES}"
        
        SERVICE_PORT=`expr ${SERVICE_PORT} + 1`
        DisplayTrace "    Trying port number ${SERVICE_PORT}"
        DisplayTrace ""
        ${GREP} '^[^#]*'"${SERVICE_PORT}/" ${FILE_SERVICES} >/dev/null 2>&1 
    done
    
    DisplayTrace ""
    DisplayTrace "    Port number chosen: ${SERVICE_PORT}"
    
    DisplaySummary ""
    DisplaySummary "Port number chosen: ${SERVICE_PORT}"
    
    
    HPU_SERVICE_PORT=${SERVICE_PORT}
    CheckSettingsVariable "HPU_SERVICE_PORT"
    if [ $? -ne 0 ]
    then
        AddSettingsVariable "HPU_SERVICE_PORT" "${HPU_SERVICE_PORT}"
    else
        DisplayTraceAndError "Information \"HPU_SERVICE_PORT\" already found in settings file"
        RETURN_CODE=1
    fi
    
    
    # Une copie de sauvegarde peut deja avoir ete faite precedemment (cas de la
    # suppression des lignes existantes)
    if [ ! -f "${FILE_SERVICES}.${SUFFIX}" ]
    then
        ${CP} -p "${FILE_SERVICES}" "${FILE_SERVICES}.${SUFFIX}"
    fi
    

    DisplayTrace "    Line added to ${FILE_SERVICES}: "
    DisplayTrace "    ${SERVICE_NAME}  ${SERVICE_PORT}/${SERVICE_PROTOCOL}    ${SERVICE_COMMENT}" 
    
    echo "${SERVICE_NAME}  ${SERVICE_PORT}/${SERVICE_PROTOCOL}    ${SERVICE_COMMENT}" >> "${FILE_SERVICES}" 
    

    # Traitement effectue separement maintenant
    # CONFIG_FILE et IDENTIFIER sont obsoletes a present
    
    ###     # Ajout du parametre precisant le nom de service ("netservice") dans le
    ###     # fichier de configuration
    ###     # On suppose que ce parametre ne s'y trouve pas deja
    ### 
    ###     echo "${IDENTIFIER}=${SERVICE_DAEMON}" >> ${CONFIG_FILE}
    ### 
    ###     DisplayTrace ""
    ###     DisplayTrace  "    Line added to ${CONFIG_FILE} file"
    ###     DisplayTrace  "    ${IDENTIFIER}=${SERVICE_DAEMON}"
    
    return ${RETURN_CODE}
}


###########################################################################
# AddServicesUNINST
###########################################################################

#
# Suppression du service dans le fichier /etc/services (desinstallation)
#
# Argument 1 : nom de section 
#
# Type de section concernee : "add_services"
#
#

AddServicesUNINST ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> AddServicesUNINST"


    # Lecture infos relatives au service a ajouter
    
    SERVICE_NAME=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" service_name` 
    DisplayTrace "    SERVICE_NAME     : ${SERVICE_NAME}"
    

    SERVICE_PORT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" service_port` 
    DisplayTrace "    SERVICE_PORT     : ${SERVICE_PORT}"


    SERVICE_PROTOCOL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" service_protocol` 
    DisplayTrace "    SERVICE_PROTOCOL : ${SERVICE_PROTOCOL}"


    SERVICE_COMMENT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" service_comment` 
    DisplayTrace "    SERVICE_COMMENT  : ${SERVICE_COMMENT}"


    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    SERVICE_NAME  : \"${SERVICE_NAME}\" " 
    DisplayTrace "    SERVICE_PORT  : \"${SERVICE_PORT}\" "

    SERVICE_NAME=`\eval echo "${SERVICE_NAME}"`
    SERVICE_PORT=`\eval echo "${SERVICE_PORT}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    SERVICE_NAME  : \"${SERVICE_NAME}\" " 
    DisplayTrace "    SERVICE_PORT  : \"${SERVICE_PORT}\" "


    # Fichier temporaire vide cree avec les memes droits et proprietaire que
    # l'original
    FILE_TEMP="${FILE_SERVICES}.$$"
    ${CP} -p "${FILE_SERVICES}" "${FILE_TEMP}"
    > "${FILE_TEMP}"

    # Suffixe pour la sauvegarde du fichier original 
    SUFFIX=`date +"%d%m%y_%H%M%S"`

    # Controle presence ligne
    # (Entre les crochets : espace et tabulation)
    

    ### ${GREP} '^'"${SERVICE_DAEMON}"'[ 	]' "${FILE_SERVICES}" >/dev/null 2>&1
    ### if [ $? -eq 0 ]
    ### then
    ###     # Pas de controle sur le numero de port
    ###     # (Entre les crochets : espace et tabulation)
    ### 
    ###     ${SED}  -e '/^'"${SERVICE_DAEMON}"'[ 	]/d' "${FILE_SERVICES}" > ${FILE_TEMP}
    ###     
    ###     ${MV} ${FILE_SERVICES} ${FILE_SERVICES}.${SUFFIX}
    ###     ${MV} ${FILE_TEMP} ${FILE_SERVICES}
    ### 
    ###     DisplayTrace "    Removal of \"${SERVICE_DAEMON}\" service"
    ### else
    ###     # Aucune erreur si ligne absente 
    ###     DisplayTrace "    Entry not found for daemon ${SERVICE_DAEMON} in ${FILE_SERVICES}"
    ###     ${RM} ${FILE_TEMP}
    ### fi


    ${GREP} '^'"${SERVICE_NAME}"'[ 	]' "${FILE_SERVICES}" >/dev/null 2>&1
    if [ $? -eq 0 ]
    then
        # Pas de controle sur le numero de port
        # (Entre les crochets : espace et tabulation)

        ${SED}  -e '/^'"${SERVICE_NAME}"'[ 	]/d' "${FILE_SERVICES}" > "${FILE_TEMP}"
        
        ${MV} "${FILE_SERVICES}" "${FILE_SERVICES}.${SUFFIX}"
        ${MV} "${FILE_TEMP}" "${FILE_SERVICES}"

        DisplayTrace "    Removal of \"${SERVICE_NAME}\" service"
    else
        # Aucune erreur si ligne absente 
        DisplayTrace "    Entry not found for daemon ${SERVICE_NAME} in ${FILE_SERVICES}"
        ${RM} ${FILE_TEMP}
    fi

    return ${RETURN_CODE}
}


###########################################################################
# AddServices
###########################################################################

#
# Ajout du service dans le fichier /etc/services
#
# Argument 1 : nom de section 
#
# Type de section concernee : "add_services"
#
#

AddServices ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> AddServices"

    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 
            

        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"

    
        # Controle de compatibilite de la section avec la commande
    
        case "${SECTION_TYPE}" in
        
        "add_services")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    AddServicesINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    AddServicesUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
            
        "no_action")
        
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
    fi        
    
    return ${RETURN_CODE}
}


###########################################################################
# AddStartupINST
###########################################################################

#
# Ajout du lancement du demon service dans le fichier /etc/inittab (installation)
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "add_startup"
#

AddStartupINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> AddStartupINST"

    
    # Lecture infos relatives au service a ajouter

    INIT_ENTRY_ID=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" init_entry_id` 
    INIT_RUN_LEVELS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" init_run_levels` 
    INIT_ACTION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" init_action` 
    INIT_PROCESS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" init_process` 
    INIT_COMMENT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" init_comment`

    DisplayTrace "    INIT_ENTRY_ID   : \"${INIT_ENTRY_ID}\"" 
    DisplayTrace "    INIT_RUN_LEVELS : \"${INIT_RUN_LEVELS}\"" 
    DisplayTrace "    INIT_ACTION     : \"${INIT_ACTION}\"" 
    DisplayTrace "    INIT_PROCESS    : \"${INIT_PROCESS}\"" 
    DisplayTrace "    INIT_COMMENT    : \"${INIT_COMMENT}\"" 

    INIT_ENTRY_ID=`\eval echo "${INIT_ENTRY_ID}"`
    INIT_RUN_LEVELS=`\eval echo "${INIT_RUN_LEVELS}"`
    INIT_ACTION=`\eval echo "${INIT_ACTION}"`
    INIT_PROCESS=`\eval echo "${INIT_PROCESS}"`


    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    INIT_ENTRY_ID   : \"${INIT_ENTRY_ID}\"" 
    DisplayTrace "    INIT_RUN_LEVELS : \"${INIT_RUN_LEVELS}\"" 
    DisplayTrace "    INIT_ACTION     : \"${INIT_ACTION}\"" 
    DisplayTrace "    INIT_PROCESS    : \"${INIT_PROCESS}\"" 

    
    # Suffixe pour la sauvegarde du fichier original 
    SUFFIX=`date +"%d%m%y_%H%M%S"`
    
    # Controle presence ligne 
    ${GREP} '^'"${INIT_ENTRY_ID}:${INIT_RUN_LEVELS}:" "${FILE_STARTUP}" > /dev/null
    CODE_RETOUR=$?
    
    if [ ${CODE_RETOUR} -eq 0 ]
    then
        DisplayError "Entry already found for label ${INIT_ENTRY_ID} in ${FILE_STARTUP}"
        RETURN_CODE=1
    else
    
       ${CP} -p ${FILE_STARTUP} ${FILE_STARTUP}.${SUFFIX}
        echo "${INIT_ENTRY_ID}:${INIT_RUN_LEVELS}:${INIT_ACTION}:${INIT_PROCESS}  ${INIT_COMMENT}" >> "${FILE_STARTUP}"
    
    fi
    
    return ${RETURN_CODE}
}

###########################################################################
# AddStartupUNINST
###########################################################################

#
# Suppression du lancement du demon service dans le fichier /etc/inittab (desinstallation)
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "add_startup"
#

AddStartupUNINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> AddStartupUNINST"
    
    
    # Lecture infos relatives au service a ajouter

    INIT_ENTRY_ID=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" init_entry_id` 
    INIT_RUN_LEVELS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" init_run_levels` 
    INIT_ACTION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" init_action` 
    INIT_PROCESS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" init_process` 
    INIT_COMMENT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" init_comment`


    DisplayTrace "    INIT_ENTRY_ID   : \"${INIT_ENTRY_ID}\"" 
    DisplayTrace "    INIT_RUN_LEVELS : \"${INIT_RUN_LEVELS}\"" 
    DisplayTrace "    INIT_ACTION     : \"${INIT_ACTION}\"" 
    DisplayTrace "    INIT_PROCESS    : \"${INIT_PROCESS}\"" 
    DisplayTrace "    INIT_COMMENT    : \"${INIT_COMMENT}\"" 


    INIT_ENTRY_ID=`\eval echo "${INIT_ENTRY_ID}"`
    INIT_RUN_LEVELS=`\eval echo "${INIT_RUN_LEVELS}"`
    INIT_ACTION=`\eval echo "${INIT_ACTION}"`
    INIT_PROCESS=`\eval echo "${INIT_PROCESS}"`


    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    INIT_ENTRY_ID   : \"${INIT_ENTRY_ID}\"" 
    DisplayTrace "    INIT_RUN_LEVELS : \"${INIT_RUN_LEVELS}\"" 
    DisplayTrace "    INIT_ACTION     : \"${INIT_ACTION}\"" 
    DisplayTrace "    INIT_PROCESS    : \"${INIT_PROCESS}\"" 

    
    # Fichier temporaire vide cree avec les memes droits et proprietaire que
    # l'original
    FILE_TEMP="${FILE_STARTUP}.$$"
    ${CP} -p "${FILE_STARTUP}" "${FILE_TEMP}"
    > "${FILE_TEMP}"
        
    # Suffixe pour la sauvegarde du fichier original 
    SUFFIX=`date +"%d%m%y_%H%M%S"`
            
    # Controle presence ligne 
    ${GREP} '^'"${INIT_ENTRY_ID}:${INIT_RUN_LEVELS}:" "${FILE_STARTUP}" > /dev/null
    CODE_RETOUR=$?
    
    if [ ${CODE_RETOUR} -eq 0 ]
    then
        ${SED} -e "/^${INIT_ENTRY_ID}:${INIT_RUN_LEVELS}:/d" "${FILE_STARTUP}" > "${FILE_TEMP}"
        
        ${MV} "${FILE_STARTUP}" "${FILE_STARTUP}.${SUFFIX}"
        ${MV} "${FILE_TEMP}" "${FILE_STARTUP}"
           
    else
        DisplayError "Entry not found for label ${INIT_ENTRY_ID} in ${FILE_STARTUP}"
        ${RM} "${FILE_TEMP}"
        RETURN_CODE=1
    fi
        
    return ${RETURN_CODE}
}

###########################################################################
# AddStartup
###########################################################################

#
# Ajout du lancement du demon service dans le fichier /etc/inittab
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "add_startup"
#
#
#

AddStartup ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> AddStartup"
    
    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 
        

        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"

        
        # Controle de compatibilite de la section avec la commande
        
        case "${SECTION_TYPE}" in
        "add_startup")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    AddStartupINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    AddStartupUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
            
        "no_action")
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
        
    fi

    return ${RETURN_CODE}
}


###########################################################################
# ExecuteCommandConditionINST
###########################################################################

#
# Execution d'une commande UNIX conditionnee par un feature (installation)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "execute_command_condition"
#
# 

ExecuteCommandConditionINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ExecuteCommandConditionINST"


    CONDITION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "condition"`
    COMMAND=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "command"` 
    OPTIONS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "options"` 
    ARGUMENTS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "arguments"` 


    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    CONDITION : \"${CONDITION}\"" 
    DisplayTrace "    COMMAND   : \"${COMMAND}\"" 
    DisplayTrace "    OPTIONS   : \"${OPTIONS}\"" 
    DisplayTrace "    ARGUMENTS : \"${ARGUMENTS}\"" 

    # Evaluation (substitution des variables eventuelles)
    COMMAND=`eval echo "${COMMAND}"`
    ARGUMENTS=`eval echo "${ARGUMENTS}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    COMMAND   : \"${COMMAND}\"" 
    DisplayTrace "    ARGUMENTS : \"${ARGUMENTS}\"" 


    NAME=`GetFieldList ${CONDITION} ":" 1`
    VALUE=`GetFieldList ${CONDITION} ":" 2`
    # Recuperation de la valeur de la propriete activeForUninstall pour le 
    # feature indique
    PROPERTY_UNINSTALL=`GetFieldSublist ${FEATURES} '|' ':' ${NAME} 3`
    

    DisplayTrace "    NAME               : \"${NAME}\"" 
    DisplayTrace "    VALUE              : \"${VALUE}\"" 
    DisplayTrace "    PROPERTY_UNINSTALL : \"${PROPERTY_UNINSTALL}\" "


    # Execution commande s'il y en une
    if [ "${COMMAND}" != "" ]
    then

        if [ "${VALUE}" = "${PROPERTY_UNINSTALL}" ]
        then

            DisplayTrace ""
            DisplayTrace "    Start of output of executed command"
            DisplayTrace ""

            \eval "${COMMAND}" "${OPTIONS}" "${ARGUMENTS}" 
            RETURN_CODE_COMMAND=$?
            
            DisplayTrace ""
            DisplayTrace "    End of output of executed command"  
                      
            if [ ${RETURN_CODE_COMMAND} -ne 0 ]
            then
                DisplayError "Error executing command \"${COMMAND}\" \"${OPTIONS}\" \"${ARGUMENTS}\" "
                DisplayError "(return code ${RETURN_CODE_COMMAND})"
                RETURN_CODE=2
            fi   
        fi     
    else
        DisplayTrace "No command to execute"
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# ExecuteCommandConditionUNINST
###########################################################################

#
# Execution d'une commande UNIX conditionnee par un feature (desinstallation)
#
# Dans certains cas, aucune commande a executer.
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "execute_command_condition"
#
# 

ExecuteCommandConditionUNINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ExecuteCommandConditionUNINST"


    CONDITION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_condition"`
    COMMAND=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_command"` 
    OPTIONS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_options"` 
    ARGUMENTS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_arguments"` 


    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    CONDITION : \"${CONDITION}\"" 
    DisplayTrace "    COMMAND   : \"${COMMAND}\"" 
    DisplayTrace "    OPTIONS   : \"${OPTIONS}\"" 
    DisplayTrace "    ARGUMENTS : \"${ARGUMENTS}\"" 

    # Evaluation (substitution des variables eventuelles)
    COMMAND=`\eval echo "${COMMAND}"`
    ARGUMENTS=`\eval echo "${ARGUMENTS}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    COMMAND   : \"${COMMAND}\"" 
    DisplayTrace "    ARGUMENTS : \"${ARGUMENTS}\"" 


    NAME=`GetFieldList ${CONDITION} ":" 1`
    VALUE=`GetFieldList ${CONDITION} ":" 2`
    # Recuperation de la valeur de la propriete activeForUninstall pour le 
    # feature indique
    PROPERTY_UNINSTALL=`GetFieldSublist ${FEATURES} '|' ':' ${NAME} 3`


    DisplayTrace "    NAME               : \"${NAME}\"" 
    DisplayTrace "    VALUE              : \"${VALUE}\"" 
    DisplayTrace "    PROPERTY_UNINSTALL : \"${PROPERTY_UNINSTALL}\" "

 
    # Execution commande s'il y en une
    if [ "${COMMAND}" != "" ]
    then

        if [ "${VALUE}" = "${PROPERTY_UNINSTALL}" ]
        then

            DisplayTrace ""
            DisplayTrace "    Start of output of executed command"
            DisplayTrace ""
        
            \eval "${COMMAND}" "${OPTIONS}" "${ARGUMENTS}" 
            RETURN_CODE_COMMAND=$?
            
            DisplayTrace ""
            DisplayTrace "    End of output of executed command"
            
            if [ ${RETURN_CODE_COMMAND} -ne 0 ]
            then
                DisplayError "Error executing command \"${COMMAND}\" \"${OPTIONS}\" \"${ARGUMENTS}\" "
                DisplayError "(return code ${RETURN_CODE_COMMAND})"
                RETURN_CODE=2
            fi   
        fi    
    else
        DisplayTrace "No command to execute"
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# ExecuteCommandCondition
###########################################################################

#
# Execution d'une commande UNIX sous la condition de presence d'un feature
#
# Attention !
# Tres specifique pour l'instant : uniquement pour detecter si un feature
# est present (ie a ete installe) lors de la desinstallation.
# La condition est tres specifique : la valeur est testee avec la propriete 
# activeForUninstall du feature sous InstallShield.
#
#
# Action amenee a disparaitre car devient obsolete du fait de la gestion des
# conditions pour les autres actions.
# Seule cette action ne gere pas les conditions.
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "execute_command_condition"
#
# 

ExecuteCommandCondition ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ExecuteCommandCondition"

    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 

        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"

        
        # Controle de compatibilite de la section avec la commande
        
        case "${SECTION_TYPE}" in
        "execute_command_condition")
        
            case "${CONTEXT}" in 
            "inst")
                ExecuteCommandConditionINST $@
                RETURN_CODE=$?
                ;;  

            "uninst")
                ExecuteCommandConditionUNINST $@
                RETURN_CODE=$?
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
        "no_action")
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
    
    fi
    
    return ${RETURN_CODE}
}



###########################################################################
# UpdateEnvironmentINST
###########################################################################

#
# Ajout d'une variable initialisee dans l'environnement (installation)
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_environment"
#
#
# Pour l'instant, pas de remplacement de la valeur d'une variable deja presente
#
#
# L'environnement est l'environnement global : /etc/profile 
# La variable est ajoutee dans un paragraphe delimite de la forme suivante :
#
# 
# # Begin HPU Environment Variable Section
# # Do not edit this section manually
# VARIABLE=valeur
# export VARIABLE
# # End HPU Environment Variable Section
#
#
# Si le paragraphe n'existe pas il est cree.
# Si la variable est deja declaree dans le paragraphe, sa valeur est remplacee.
# Si la variable n'est pas deja declaree dans le paragraphe, sa declaration y
# est ajoutee.
#
# Il n'y a qu'un seul paragraphe et celui-ci peut declarer plusieurs variables.
#
#
# A faire : mettre les etiquettes de debut et de fin de paragraphe dans le fichier
#            .INI 
# A faire : utiliser des variables pour les etiquettes de debut et de fin de 
#           paragraphe 
#


UpdateEnvironmentINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> UpdateEnvironmentINST"

    
    IDENTIFIER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "variable"` 
    VALUE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "value"` 

    DisplayTrace "    IDENTIFIER  : \"${IDENTIFIER}\"" 
    DisplayTrace "    VALUE       : \"${VALUE}\"" 


    VALUE=`\eval echo "${VALUE}"`


    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    VALUE       : \"${VALUE}\"" 


    # Fichier temporaire vide cree avec les memes droits et proprietaire que
    # l'original
    # >>> A faire ? Ne pas mettre le fichier temporaire au meme endroit que l'original
    # car le FILE_PROFILE est un chemin complet
    FILE_TEMP="${FILE_PROFILE}.$$"
    ${CP} -p "${FILE_PROFILE}" "${FILE_TEMP}"
    > "${FILE_TEMP}"

    # Fichier pour script "sed"
    FILE_SED_TEMP="sed_script.$$"
    > "${FILE_SED_TEMP}"
    
    # Suffixe pour la sauvegarde du fichier original 
    SUFFIX=`date +"%d%m%y_%H%M%S"`

    # Recherche du paragraphe
    ${AWK} '/# Begin HPU/ , /# End HPU/ {print}'  "${FILE_PROFILE}" > "${FILE_TEMP}"

    if [ -s "${FILE_TEMP}" ]
    then
        DisplayTrace "    Paragraph found"
    
        # Si paragraphe deja existant 
        # Recherche si la variable est deja declaree
        ${GREP} '^'"${IDENTIFIER}=" "${FILE_TEMP}"
        CODE_RETOUR=$?

        if [ ${CODE_RETOUR} -eq 0 ]
        then
            DisplayTrace "    Variable already declared"

        
            ${CP} -p "${FILE_PROFILE}" "${FILE_PROFILE}.${SUFFIX}"
                   
            # Variable deja presente
            ReplaceVariable "${FILE_PROFILE}" "${IDENTIFIER}" "${VALUE}" 
            
            ${RM} "${FILE_SED_TEMP}"
            ${RM} "${FILE_TEMP}" 
            
        else
            DisplayTrace "    Undeclared variable"

            # Ajouter la declaration de la variable dans le paragraphe
            # On travaille directement a partir du fichier original car on sait 
            # que le paragraphe s'y trouve deja

            # Les directives "sed" sont dans un fichier a cause de l'utilisation de 
            # variables shell a substituer
            echo '/^# End HPU /i\' >> "${FILE_SED_TEMP}"
            echo "${IDENTIFIER}=${VALUE}\\" >> "${FILE_SED_TEMP}" 
            echo "export ${IDENTIFIER}" >> "${FILE_SED_TEMP}" 

            # Attention reutilisation du meme fichier temporaire
            
            ${SED} -f "${FILE_SED_TEMP}" "${FILE_PROFILE}" > "${FILE_TEMP}"
            
            ${MV} "${FILE_PROFILE}" "${FILE_PROFILE}.${SUFFIX}"
            ${MV} "${FILE_TEMP}" "${FILE_PROFILE}"
            
            ${RM} "${FILE_SED_TEMP}"
        fi

    else
        DisplayTrace "    Paragraph not found"
    
        # Paragraphe inexistant
        # Le paragraphe est ajoute en fin 
        
        ${CP} -p "${FILE_PROFILE}" "${FILE_PROFILE}.${SUFFIX}"
        
        echo "" >> "${FILE_PROFILE}"
        echo "# Begin HPU Environment Variable Section" >> "${FILE_PROFILE}"
        echo "# Do not edit this section manually" >> "${FILE_PROFILE}"
        echo "${IDENTIFIER}=${VALUE}" >> "${FILE_PROFILE}" 
        echo "export ${IDENTIFIER}" >> "${FILE_PROFILE}"
        echo "# End HPU Environment Variable Section" >> "${FILE_PROFILE}"
        echo "" >> "${FILE_PROFILE}"

        ${RM} "${FILE_SED_TEMP}"
        ${RM} "${FILE_TEMP}" 
    fi

    return ${RETURN_CODE}
}


###########################################################################
# UpdateEnvironmentUNINST
###########################################################################

#
# Supression d'une variable initialisee dans l'environnement (desinstallation)
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_environment"
#
# La partie a retirer doit etre delimitee par "# Begin HPU" et "# End HPU".
#

UpdateEnvironmentUNINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> UpdateEnvironmentUNINST"

    
    IDENTIFIER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "variable"` 
    VALUE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "value"` 


    DisplayTrace "    IDENTIFIER  : \"${IDENTIFIER}\"" 
    DisplayTrace "    VALUE       : \"${VALUE}\"" 

    VALUE=`eval echo "${VALUE}"`

    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    VALUE       : \"${VALUE}\"" 


    # Fichier temporaire vide cree avec les memes droits et proprietaire que
    # l'original
    # >>> A faire ? Ne pas mettre le fichier temporaire au meme endroit que l'original
    # car le FILE_PROFILE est un chemin complet
    FILE_TEMP="${FILE_PROFILE}.$$"
    ${CP} -p "${FILE_PROFILE}" "${FILE_TEMP}"
    > "${FILE_TEMP}"


    FILE_TEMP_2="${FILE_PROFILE}.$$_2"
    ${CP} -p "${FILE_PROFILE}" "${FILE_TEMP_2}"
    > "${FILE_TEMP_2}"

    # Suffixe pour la sauvegarde du fichier original 
    SUFFIX=`date +"%d%m%y_%H%M%S"`
    
    # Recherche du paragraphe
    ${AWK} '/# Begin HPU/ , /# End HPU/ {print}'  "${FILE_PROFILE}" > "${FILE_TEMP}"

    if [ -s "${FILE_TEMP}" ]
    then
        DisplayTrace "    Paragraph found "
    
        # Si paragraphe deja existant 
        # Recherche si la variable y est declaree
        ${GREP} '^'"${IDENTIFIER}=" "${FILE_TEMP}" >/dev/null 2>&1
        CODE_RETOUR=$?

        if [ ${CODE_RETOUR} -eq 0 ]
        then
            DisplayTrace "    Variable found"
        
            # Variable presente

            # Tentative de suppression des lignes dans le paragraphe
            # pour determiner ce qui reste apres 
            ${SED} -e "/^${IDENTIFIER}=/,/^export ${IDENTIFIER}/d" "${FILE_TEMP}" > "${FILE_TEMP_2}"
            NB_LINES=`${WC} -l "${FILE_TEMP_2}" | ${AWK} '{print $1}' | ${TR} -d ' '`
            # echo "NB_LINES : ${NB_LINES}"

            if [ ${NB_LINES} -eq 3 ]
            then
                DisplayTrace "    Paragraph removed"
                
                # Il n'y a plus que les commentaires
                # Suppression de tout le paragraphe
                # Attention reutilisation du meme fichier temporaire
                ${SED} -e '/# Begin HPU/,/# End HPU/d' "${FILE_PROFILE}" > "${FILE_TEMP}"
                
                ${MV} "${FILE_PROFILE}" "${FILE_PROFILE}.${SUFFIX}"
                ${MV} "${FILE_TEMP}" "${FILE_PROFILE}"
                
                ${RM} -f "${FILE_TEMP_2}"
                
            else
                DisplayTrace "    Variable removed"
                
                # Il y a d'autres variables declarees dans le paragraphe
                # Suppression uniquement des lignes de la variable
               
                # Pour faciliter le traitement (ameliorable) : suppression du paragraphe complet
                # puis ajout du paragraphe modifie en fin 
                # Le paragraphe est donc deplace en fin de fichier d'environnement !
                
                ${SED} -e '/# Begin HPU/,/# End HPU/d' "${FILE_PROFILE}" > "${FILE_TEMP}"
                ${CAT} "${FILE_TEMP_2}" >> "${FILE_TEMP}"
                
                ${MV} "${FILE_PROFILE}" "${FILE_PROFILE}.${SUFFIX}"
                ${MV} "${FILE_TEMP}" "${FILE_PROFILE}"
                
                ${RM} -f "${FILE_TEMP_2}"
            fi

        else
            DisplayTrace "    Variable not found !"

            ${RM} -f "${FILE_TEMP}" "${FILE_TEMP_2}"
       fi
    else
        DisplayTrace "    Paragraph not found"
    fi   
    
    return ${RETURN_CODE}
}


###########################################################################
# UpdateEnvironment
###########################################################################

#
# Ajout dans l'environnement d'une variable initialisee 
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_environment"
#
#
# Pour l'instant, pas de remplacement de la valeur d'une variable deja presente
#

UpdateEnvironment ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> UpdateEnvironment"

    
    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 
        
        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"

        
        # Controle de compatibilite de la section avec la commande
        
        case "${SECTION_TYPE}" in
        
        "update_environment")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    UpdateEnvironmentINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    UpdateEnvironmentUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
        "no_action")
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
    fi

    return ${RETURN_CODE}
}


###########################################################################
# UpdateFileINST
###########################################################################

#
# Mise a jour d'une variable dans un fichier texte (fichier script shell) 
# (installation)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "update_file"
#
# Pb portabilite de la commande eval  ?
# 

UpdateFileINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> UpdateFileINST"


    FILE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "file"` 
    IDENTIFIER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "variable"` 
    VALUE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "value"` 

    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    FILE  : \"${FILE}\"" 
    DisplayTrace "    IDENTIFIER  : \"${IDENTIFIER}\"" 
    DisplayTrace "    VALUE       : \"${VALUE}\"" 

    FILE=`\eval echo "${FILE}"`
    VALUE=`\eval echo "${VALUE}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    FILE  : \"${FILE}\"" 
    DisplayTrace "    VALUE : \"${VALUE}\"" 


    ReplaceVariable "${FILE}" "${IDENTIFIER}" "${VALUE}" 

    return ${RETURN_CODE}
}


###########################################################################
# UpdateFileUNINST
###########################################################################

#
# Mise a jour d'une variable dans un fichier texte (fichier script shell) 
# (desinstallation)
#
# >>> Actuellement rien n'est a faire pour la desinstallation
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "update_file"
#
# Pb portabilite de la commande eval  ?
# 

UpdateFileUNINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> UpdateFileUNINST"


    FILE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "file"` 
    IDENTIFIER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "variable"` 
    VALUE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "value"` 


    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    FILE       : \"${FILE}\"" 
    DisplayTrace "    IDENTIFIER : \"${IDENTIFIER}\"" 
    DisplayTrace "    VALUE      : \"${VALUE}\"" 

    FILE=`eval echo "${FILE}"`
    VALUE=`eval echo "${VALUE}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    FILE       : \"${FILE}\"" 
    DisplayTrace "    VALUE      : \"${VALUE}\"" 

    DisplayTrace "    No uninstall treatment to do"

    return ${RETURN_CODE}
}


###########################################################################
# UpdateFile
###########################################################################

#
# Mise a jour d'une variable dans un fichier texte (fichier script shell)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "update_file"
#
# Pb portabilite de la commande eval  ?
# 

UpdateFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> UpdateFile"

    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 

        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"

        
        # Controle de compatibilite de la section avec la commande

        case "${SECTION_TYPE}" in
        
        "update_file")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    UpdateFileINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    UpdateFileUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
        "no_action")
        
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
        
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# ExecuteCommandINST
###########################################################################

#
# Execution d'une commande externe (installation)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "execute_command"
#
# 

ExecuteCommandINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ExecuteCommandINST"


    COMMAND=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "command"` 
    OPTIONS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "options"` 
    ARGUMENTS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "arguments"` 
    STDOUT_REDIRECTION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "stdout_redirection"` 

    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    COMMAND   : \"${COMMAND}\"" 
    DisplayTrace "    OPTIONS   : \"${OPTIONS}\"" 
    DisplayTrace "    ARGUMENTS : \"${ARGUMENTS}\"" 
    DisplayTrace "    STDOUT_REDIRECTION : \"${STDOUT_REDIRECTION}\"" 


    # Evaluation (substitution des variables eventuelles)
    COMMAND=`eval echo "${COMMAND}"`
    ARGUMENTS=`eval echo "${ARGUMENTS}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    COMMAND   : \"${COMMAND}\"" 
    DisplayTrace "    ARGUMENTS : \"${ARGUMENTS}\"" 

    
    if [ ${RETURN_CODE} -eq 0 ]
    then

        # Execution commande s'il y en une
        if [ "${COMMAND}" != "" ]
        then
            STDOUT_REDIRECTION=`ConvertToLowerCase "${STDOUT_REDIRECTION}"`
            case "${STDOUT_REDIRECTION}" in
            "yes"|"no")
                :
                ;;

            *)
                DisplayTraceAndError "Unexpected STDOUT_REDIRECTION value: \"${STDOUT_REDIRECTION}\" " 
                RETURN_CODE=1
                ;;
            esac

            if [ ${RETURN_CODE} -eq  0 ]
            then
                DisplayTrace ""
                DisplayTrace "    Start of output of executed command"
                DisplayTrace ""

                if [ "${STDOUT_REDIRECTION}" = "yes" ]
                then
                    ActivateTraceRedirection
                fi

                \eval "${COMMAND}" "${OPTIONS}" "${ARGUMENTS}" 
                RETURN_CODE_COMMAND=$?

                if [ "${STDOUT_REDIRECTION}" = "yes" ]
                then
                    DeactivateTraceRedirection
                fi

                DisplayTrace ""
                DisplayTrace "    End of output of executed command"

                if [ ${RETURN_CODE_COMMAND} -ne 0 ]
                then
                    DisplayError "Error executing command \"${COMMAND}\" \"${OPTIONS}\" \"${ARGUMENTS}\" "
                    DisplayError "(return code ${RETURN_CODE_COMMAND})"
                    RETURN_CODE=2
                fi  
            fi

        else
            DisplayTrace "    No command to execute"
        fi    
    
    fi

    return ${RETURN_CODE}
}


###########################################################################
# ExecuteCommandUNINST
###########################################################################

#
# Execution d'une commande externe (desinstallation)
#
# Dans certains cas, aucune commande a executer.
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "execute_command"
#
# 

ExecuteCommandUNINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ExecuteCommandUNINST"


    COMMAND=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_command"` 
    OPTIONS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_options"` 
    ARGUMENTS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_arguments"` 
    STDOUT_REDIRECTION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_stdout_redirection"` 


    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    COMMAND   : \"${COMMAND}\"" 
    DisplayTrace "    OPTIONS   : \"${OPTIONS}\"" 
    DisplayTrace "    ARGUMENTS : \"${ARGUMENTS}\"" 
    DisplayTrace "    STDOUT_REDIRECTION : \"${STDOUT_REDIRECTION}\" " 


    # Evaluation (substitution des variables eventuelles)
    COMMAND=`\eval echo "${COMMAND}"`
    ARGUMENTS=`\eval echo "${ARGUMENTS}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    COMMAND   : \"${COMMAND}\"" 
    DisplayTrace "    ARGUMENTS : \"${ARGUMENTS}\"" 

    
    if [ ${RETURN_CODE} -eq 0 ]
    then


    # Execution commande s'il y en une
    if [ "${COMMAND}" != "" ]
    then
        STDOUT_REDIRECTION=`ConvertToLowerCase "${STDOUT_REDIRECTION}"`
        case "${STDOUT_REDIRECTION}" in
        "yes"|"no")
            :
            ;;

        *)
            DisplayTraceAndError "Unexpected STDOUT_REDIRECTION value: \"${STDOUT_REDIRECTION}\" " 
            RETURN_CODE=1
            ;;
        esac
        
        if [ ${RETURN_CODE} -eq  0 ]
        then
            DisplayTrace ""
            DisplayTrace "    Start of output of executed command"
            DisplayTrace ""

            if [ "${STDOUT_REDIRECTION}" = "yes" ]
            then
                ActivateTraceRedirection
            fi

            \eval "${COMMAND}" "${OPTIONS}" "${ARGUMENTS}" 
            RETURN_CODE_COMMAND=$?

            if [ "${STDOUT_REDIRECTION}" = "yes" ]
            then
                DeactivateTraceRedirection
            fi

            DisplayTrace ""
            DisplayTrace "    End of output of executed command"

            if [ ${RETURN_CODE_COMMAND} -ne 0 ]
            then
                DisplayError "Error executing command \"${COMMAND}\" \"${OPTIONS}\" \"${ARGUMENTS}\" "
                DisplayError "(return code ${RETURN_CODE_COMMAND})"
                RETURN_CODE=2
            fi
        
        fi
            
    else
        DisplayTrace "    No command to execute"
    fi

    fi

    return ${RETURN_CODE}
}


###########################################################################
# ExecuteCommand
###########################################################################

#
# Execution d'une commande externe
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "execute_command"
#
# 

ExecuteCommand ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ExecuteCommand"

    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 
        
        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"
        
        # Controle de compatibilite de la section avec la commande
        
        case "${SECTION_TYPE}" in
        
        "execute_command")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    ExecuteCommandINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    ExecuteCommandUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
        "no_action")
        
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
    fi

    return ${RETURN_CODE}
}


###########################################################################
# AddInetServiceINST
###########################################################################

# Ajout d'un serveur a lancer par inetd (dans inetd.conf) (installation)
#
# Argument 1 : nom de section 
#
# Type de section concernee : "add_inet_service"
#
# Si une erreur se produit elle n'est pas fatale a present.
#
#
# Appel de la fonction interne hpuDetermineDaemonBinaryName ().
# De ce fait, la fonction doit etre lancee apres l'installation des executables 
# (repertoire bin).
#
#
# Attention : code specifique pour Solaris 10 (detection uniquement si 
# "Solaris 10")
#

AddInetServiceINST ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> AddInetServiceINST"
    
    
    # Determination du nom du binaire HPU a lancer par inet (32 bit ou 64 bit)
    # Doit etre faite avant l'evaluation des parametres de l'action car definit 
    # une variable shell
    hpuDetermineDaemonBinaryName
    RETURN_CODE=$?
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Lecture infos relatives au service a ajouter

        DAEMON_LOG_DIRECTORY=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" daemon_log_directory` 
        INET_SERVICE_NAME=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_name` 
        INET_SERVICE_ENDPOINT_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_endpoint_type` 
        INET_SERVICE_PROTOCOL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_protocol` 
        INET_SERVICE_WAIT_STATUS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_wait_status` 
        INET_SERVICE_UID=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_uid` 
        INET_SERVICE_SERVER_PROGRAM=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_server_program` 
        INET_SERVICE_SERVER_ARGUMENT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_server_argument` 

        DisplayTrace ""
        DisplayTrace "    Before evaluation of action arguments : "
        DisplayTrace "    DAEMON_LOG_DIRECTORY         : ${DAEMON_LOG_DIRECTORY}"
        DisplayTrace "    INET_SERVICE_NAME            : ${INET_SERVICE_NAME}"
        DisplayTrace "    INET_SERVICE_ENDPOINT_TYPE   : ${INET_SERVICE_ENDPOINT_TYPE}"
        DisplayTrace "    INET_SERVICE_PROTOCOL        : ${INET_SERVICE_PROTOCOL}"
        DisplayTrace "    INET_SERVICE_WAIT_STATUS     : ${INET_SERVICE_WAIT_STATUS}"
        DisplayTrace "    INET_SERVICE_UID             : ${INET_SERVICE_UID}"
        DisplayTrace "    INET_SERVICE_SERVER_PROGRAM  : ${INET_SERVICE_SERVER_PROGRAM}"
        DisplayTrace "    INET_SERVICE_SERVER_ARGUMENT : ${INET_SERVICE_SERVER_ARGUMENT}"


        # Evaluation (substitution des variables eventuelles)
        INET_SERVICE_NAME=`eval echo "${INET_SERVICE_NAME}"`
        DAEMON_LOG_DIRECTORY=`eval echo "${DAEMON_LOG_DIRECTORY}"`
        INET_SERVICE_SERVER_PROGRAM=`eval echo "${INET_SERVICE_SERVER_PROGRAM}"`
        INET_SERVICE_SERVER_ARGUMENT=`eval echo "${INET_SERVICE_SERVER_ARGUMENT}"`


        DisplayTrace ""
        DisplayTrace "    After evaluation of action arguments : "
        DisplayTrace "    INET_SERVICE_NAME            : ${INET_SERVICE_NAME}"
        DisplayTrace "    DAEMON_LOG_DIRECTORY         : ${DAEMON_LOG_DIRECTORY}"
        DisplayTrace "    INET_SERVICE_SERVER_PROGRAM  : ${INET_SERVICE_SERVER_PROGRAM}"
        DisplayTrace "    INET_SERVICE_SERVER_ARGUMENT : ${INET_SERVICE_SERVER_ARGUMENT}"
    fi

    
    # Creation du repertoire pour les logs du demon HPU (option --log)
    if [ ${RETURN_CODE} -eq 0 ]
    then
        if [ -d "${DAEMON_LOG_DIRECTORY}" ]
        then
            DisplayTrace "    Directory for daemon log files already existing : ${DAEMON_LOG_DIRECTORY}"
        else
            DisplayTrace "    Creating directory for daemon log files : ${DAEMON_LOG_DIRECTORY}"
            ${MKDIR} -p "${DAEMON_LOG_DIRECTORY}"
            RETURN_CODE=$?
            if [ ${RETURN_CODE} -ne 0 ]
            then
                ### A FAIRE ### Erreur fatale ?
                DisplayError "Error while creating HPU daemon log directory."
                DisplayTrace "Error while creating HPU daemon log directory."
            fi
        fi
    fi
    

    if [ ${RETURN_CODE} -eq 0 ]
    then

        # Suffixe pour la sauvegarde du fichier original 
        SUFFIX=`date +"%d%m%y_%H%M%S"`

        DisplayTrace ""

        case "${PLATFORM}" in

        "linux"|"zlinux"|"plinux"|"plinuxle"|"ialinux")

            # Plateformes Linux, zLinux, pLinux, pLinuxle, Linux IA64

            # Recherche du demon inetd utilise (inetd, xinetd)

            FILE_INET_SERVICES=/etc/xinetd.conf
            if [ -f "${FILE_INET_SERVICES}" ]
            then
                DAEMON_INETD=xinetd
            else
                FILE_INET_SERVICES=/etc/inetd.conf
                if [ -f "${FILE_INET_SERVICES}" ]
                then
                    DAEMON_INETD=inetd
                else
                    DAEMON_INETD=unknown
                fi
            fi
            
            DisplayTrace "    inetd daemon found : \"${DAEMON_INETD}\" "
            DisplaySummary "inetd daemon found : \"${DAEMON_INETD}\" "

            case "${DAEMON_INETD}" in

            "inetd")
                DAEMON_INETD_RECONFIG_SIGNAL=HUP

                AddServiceInetd
                RETURN_CODE=$?
                ;;

            "xinetd")
                DAEMON_INETD_RECONFIG_SIGNAL=USR1

                AddServiceXinetd
                RETURN_CODE=$?
                ;;

            "unknown")
                DisplayWarning "HPU daemon not configured: no inetd/xinetd configuration file found !"
                DisplayTrace "Unknown inetd daemon \"${DAEMON_INETD}\" !"
                ;;

            esac    
            ;;

        *)
            # Plateformes UNIX autres que Linux 
            ### A FAIRE ### Tenir compte versions ulterieures

            if [ "${PLATFORM}" = "sun" -o "${PLATFORM}" = "sun_x86" ]
            then
                if [ "${VERSION}" = "5.10" ]
                then
                    # Creation et activation service pour Solaris 10

                    AddServiceInetdSMF
                    RETURN_CODE=$?
                else
                    # Creation et activation service pour UNIX autres que Solaris 10

                    AddServiceInetd
                    RETURN_CODE=$?
                fi
            else
                # Creation et activation service pour UNIX autres que Solaris 10

                AddServiceInetd
                RETURN_CODE=$?
            fi
            ;;

        esac
    fi
        
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Relecture du fichier de configuration du demon

        case "${PLATFORM}" in
    
        "sun")
            # Rechargement configuration non necessaire pour Solaris 10
            if [ "${VERSION}" != "5.10" ]
            then
                ExecuteInetdConfigReloadCommand
                RETURN_CODE=$?
            fi         
            ;;
    
        "sun_x86")
            # Rechargement configuration non necessaire pour Solaris 10
            if [ "${VERSION}" != "5.10" ]
            then
                ExecuteInetdConfigReloadCommand
                RETURN_CODE=$?
            fi         
            ;;
            
        *)
            ExecuteInetdConfigReloadCommand
            RETURN_CODE=$?
            ;;
        esac
    fi
    
    if [ ${RETURN_CODE} -ne 0 ]
    then
        # S'il y a eu une erreur, elle n'est pas fatale pour l'installation
        DisplayTrace "Update of inet configuration did not succeed."
        DisplaySummary "Update of inet configuration did not succeed."
        RETURN_CODE=0
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# AddInetServiceUNINST
###########################################################################

# Suppression du serveur a lancer par inetd (dans inetd.conf) (desinstallation)
#
# Argument 1 : nom de section 
#
# Type de section concernee : "add_inet_service"
#
# Si une erreur se produit elle n'est pas fatale a present.
# Les commandes Solaris 10 ont leur stderr redirige pour eviter une erreur fatale.
#

AddInetServiceUNINST ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> AddInetServiceUNINST"

    
    # Recuperation de l'information HPU_DAEMON_NAME
    # Doit etre faite avant l'evaluation des parametres de l'action car definit 
    # une variable shell

    DisplayTrace "    Retrieval of parameter \"INSTALL_FEATURE_NETWORK\" from installation settings file"

    HPU_DAEMON_NAME=`${GREP} "HPU_DAEMON_NAME" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
    if [ $? -ne 0 ]
    then
        # Information non memorisee
        DisplayError "Parameter \"HPU_DAEMON_NAME}\" not found in ${FILE_INSTALL_SETTINGS}"
        RETURN_CODE=1
    fi
    
    DisplayTrace "    HPU_DAEMON_NAME : \"${HPU_DAEMON_NAME}\" "
    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Lecture infos relatives au service a ajouter

        DAEMON_LOG_DIRECTORY=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" daemon_log_directory` 
        INET_SERVICE_NAME=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_name` 
        INET_SERVICE_ENDPOINT_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_endpoint_type` 
        INET_SERVICE_PROTOCOL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_protocol` 
        INET_SERVICE_WAIT_STATUS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_wait_status` 
        INET_SERVICE_UID=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_uid` 
        INET_SERVICE_SERVER_PROGRAM=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_server_program` 
        INET_SERVICE_SERVER_ARGUMENT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" inet_service_server_argument` 

        DisplayTrace ""
        DisplayTrace "    Before evaluation of action arguments : "
        DisplayTrace "    DAEMON_LOG_DIRECTORY         : ${DAEMON_LOG_DIRECTORY}"
        DisplayTrace "    INET_SERVICE_NAME            : ${INET_SERVICE_NAME}"
        DisplayTrace "    INET_SERVICE_ENDPOINT_TYPE   : ${INET_SERVICE_ENDPOINT_TYPE}"
        DisplayTrace "    INET_SERVICE_PROTOCOL        : ${INET_SERVICE_PROTOCOL}"
        DisplayTrace "    INET_SERVICE_WAIT_STATUS     : ${INET_SERVICE_WAIT_STATUS}"
        DisplayTrace "    INET_SERVICE_UID             : ${INET_SERVICE_UID}"
        DisplayTrace "    INET_SERVICE_SERVER_PROGRAM  : ${INET_SERVICE_SERVER_PROGRAM}"
        DisplayTrace "    INET_SERVICE_SERVER_ARGUMENT : ${INET_SERVICE_SERVER_ARGUMENT}"


        # Evaluation (substitution des variables eventuelles)
        INET_SERVICE_NAME=`eval echo "${INET_SERVICE_NAME}"`
        DAEMON_LOG_DIRECTORY=`eval echo "${DAEMON_LOG_DIRECTORY}"`
        INET_SERVICE_SERVER_PROGRAM=`eval echo "${INET_SERVICE_SERVER_PROGRAM}"`
        INET_SERVICE_SERVER_ARGUMENT=`eval echo "${INET_SERVICE_SERVER_ARGUMENT}"`


        DisplayTrace ""
        DisplayTrace "    After evaluation of action arguments : "
        DisplayTrace "    INET_SERVICE_NAME            : ${INET_SERVICE_NAME}"
        DisplayTrace "    DAEMON_LOG_DIRECTORY         : ${DAEMON_LOG_DIRECTORY}"
        DisplayTrace "    INET_SERVICE_SERVER_PROGRAM  : ${INET_SERVICE_SERVER_PROGRAM}"
        DisplayTrace "    INET_SERVICE_SERVER_ARGUMENT : ${INET_SERVICE_SERVER_ARGUMENT}"
    fi


    # FILE_INET_SERVICES peut changer pour certaines plateformes (detection plus loin)
    ###     # Fichier temporaire vide cree avec les memes droits et proprietaire que
    ###     # l'original
    ###     FILE_TEMP="${FILE_INET_SERVICES}.$$"
    ###     ${CP} -p "${FILE_INET_SERVICES}" "${FILE_TEMP}"
    ###     > "${FILE_TEMP}"


    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Suffixe pour la sauvegarde du fichier original 
        SUFFIX=`date +"%d%m%y_%H%M%S"`

        DisplayTrace ""

        case "${PLATFORM}" in

        "linux"|"zlinux"|"plinux"|"plinuxle"|"ialinux")
            # Plateformes Linux 

            # Recherche du demon inetd utilise (inetd, xinetd)

            FILE_INET_SERVICES=/etc/xinetd.conf
            if [ -f "${FILE_INET_SERVICES}" ]
            then
                DAEMON_INETD=xinetd
            else
                FILE_INET_SERVICES=/etc/inetd.conf
                if [ -f "${FILE_INET_SERVICES}" ]
                then
                    DAEMON_INETD=inetd
                else
                    DAEMON_INETD=unknown
                fi
            fi
            
            DisplayTrace "    inetd daemon found : \"${DAEMON_INETD}\" "

            case "${DAEMON_INETD}" in

            "inetd")
                DAEMON_INETD_RECONFIG_SIGNAL=HUP

                if [ -f "${FILE_INET_SERVICES}" ]
                then
                    # Fichier temporaire vide cree avec les memes droits et proprietaire que
                    # l'original
                    FILE_TEMP="${FILE_INET_SERVICES}.$$"
                    ${CP} -p "${FILE_INET_SERVICES}" "${FILE_TEMP}"
                    > "${FILE_TEMP}"

                    # Controle presence ligne 
                    # (Entre les crochets : espace et tabulation)
                    # ${GREP} '^'"${INET_SERVICE_NAME}" "${FILE_INET_SERVICES}" >/dev/null 2>&1
                    ${GREP} '^'"${INET_SERVICE_NAME}"'[ 	]' "${FILE_INET_SERVICES}" >/dev/null 2>&1
                    if [ $? -eq 0 ]
                    then
                        # ${SED}  -e '/^'"${INET_SERVICE_NAME}"'/d' "${FILE_INET_SERVICES}" > ${FILE_TEMP}
                        ${SED}  -e '/^'"${INET_SERVICE_NAME}"'[ 	]/d' "${FILE_INET_SERVICES}" > "${FILE_TEMP}"

                        ${MV} "${FILE_INET_SERVICES}" "${FILE_INET_SERVICES}.${SUFFIX}"
                        ${MV} "${FILE_TEMP}" "${FILE_INET_SERVICES}"

                        DisplayTrace "     File ${FILE_INET_SERVICES} updated"
                    else
                        DisplayTrace "    Entry not found for service ${INET_SERVICE_NAME} in ${FILE_INET_SERVICES}"
                        ${RM} "${FILE_TEMP}"
                    fi
                else
                    DisplayTrace="File ${FILE_INET_SERVICES} not found"
                    RETURN_CODE=1
                fi    
                ;;

            "xinetd")
                DAEMON_INETD_RECONFIG_SIGNAL=USR1

                if [ -f "${FILE_INET_SERVICES}" ]
                then
                    # Suppression du fichier de configuration du service pour xinetd

                    XINETD_DIR=`${GREP} '^'"includedir" "${FILE_INET_SERVICES}" | ${AWK} '{print $2}'`

                    DisplayTrace "    XINETD_DIR : ${XINETD_DIR}"

                    # A faire : tester existence repertoire

                    FILE_XINETD_CONFIGURATION="${XINETD_DIR}/${INET_SERVICE_NAME}"

                    if [ ! -f "${FILE_XINETD_CONFIGURATION}" ]
                    then
                        DisplayTrace "    File ${FILE_XINETD_CONFIGURATION} not found"
                    fi

                    ${RM} -f "${FILE_XINETD_CONFIGURATION}"

                    DisplayTrace "    File ${FILE_XINETD_CONFIGURATION} deleted"

                else
                    DisplayTrace "File ${FILE_INET_SERVICES} not found"
                    RETURN_CODE=1
                fi    
                ;;


            "unknown")
                DisplayWarning "HPU daemon not configured: no inetd/xinetd configuration file found !"
                DisplayTrace "Unknown inetd daemon \"${DAEMON_INETD}\" !"
                ;;

            esac   
            ;;

        *)
            # Plateformes UNIX autres que Linux 
            ### A FAIRE ### Tenir compte versions ulterieures

            if [ "${PLATFORM}" = "sun" -a "${VERSION}" = "5.10" -o "${PLATFORM}" = "sun_x86" -a "${VERSION}" = "5.10" ]
            then
                # Desactivation service pour Solaris 10

                DisplayTrace ""
                DisplayTrace "    Solaris 10 version service specific processing"

                \inetadm | ${GREP} '/'${INET_SERVICE_NAME}'/' >/dev/null 2>&1
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -ne 0 ]
                then
                    # Service non trouve : pas d'erreur 
                    DisplayTrace "    Service \"${INET_SERVICE_NAME}\" not found at SVCS level"
                    # RETURN_CODE=0
                else
                    # Recuperation FMRI du service
                    FMRI_SERVICE=`\inetadm | ${GREP} '/'${INET_SERVICE_NAME}'/' | ${AWK} '{print $3}'`

                    DisplayTrace ""
                    DisplayTrace "    FMRI_SERVICE : ${FMRI_SERVICE}"

                    if [ ${RETURN_CODE} -eq 0 ]
                    then
                        # Desactivation du service
                        # La desactivation est n'est pas suivie directement de la suppression 
                        # afin que la desactivation effective puisse avoir lieu
                        # Il y a donc des affichages de statut auparavant
                        DisplayTrace ""
                        DisplayTrace "    Deactivation of service"

                        ActivateTraceRedirection

                        \inetadm -d "${FMRI_SERVICE}" 2>&1
                        RETURN_CODE=$?
                        
                        DeactivateTraceRedirection
                    fi

                    if [ ${RETURN_CODE} -eq 0 ]
                    then
                        DisplayTrace ""
                        DisplayTrace "    Service state"
                        
                        ActivateTraceRedirection
                        
                        \inetadm | ${GREP} '/'${INET_SERVICE_NAME}'/' 2>&1
                        RETURN_CODE=$?
                        
                        DeactivateTraceRedirection
                     fi


                    if [ ${RETURN_CODE} -eq 0 ]
                    then
                        # Service status

                        DisplayTrace ""
                        DisplayTrace "    Service detailed service"
                        
                        ActivateTraceRedirection

                        \inetadm -l "${FMRI_SERVICE}" 2>&1
                        RETURN_CODE=$?

                        DeactivateTraceRedirection
                    fi


                    if [ ${RETURN_CODE} -eq 0 ]
                    then
                        # Suppression du service

                        DisplayTrace ""
                        DisplayTrace "    Deletion of service"

                        ActivateTraceRedirection
                        
                        \svccfg -v delete "${FMRI_SERVICE}" 2>&1
                        RETURN_CODE=$?
                        
                        DeactivateTraceRedirection
                     fi
                fi
            else
                # Suppression service pour UNIX autres que Solaris 10

                if [ -f "${FILE_INET_SERVICES}" ]
                then
                    # Fichier temporaire vide cree avec les memes droits et proprietaire que
                    # l'original
                    FILE_TEMP="${FILE_INET_SERVICES}.$$"
                    ${CP} -p "${FILE_INET_SERVICES}" "${FILE_TEMP}"
                    > "${FILE_TEMP}"

                    # Controle presence ligne 
                    # (Entre les crochets : espace et tabulation)
                    # ${GREP} '^'"${INET_SERVICE_NAME}" "${FILE_INET_SERVICES}" >/dev/null 2>&1
                    ${GREP} '^'"${INET_SERVICE_NAME}"'[ 	]' "${FILE_INET_SERVICES}" >/dev/null 2>&1
                    if [ $? -eq 0 ]
                    then
                        # ${SED} -e '/^'"${INET_SERVICE_NAME}"'/d' "${FILE_INET_SERVICES}" > ${FILE_TEMP}
                        ${SED} -e '/^'"${INET_SERVICE_NAME}"'[ 	]/d' "${FILE_INET_SERVICES}" > "${FILE_TEMP}"

                        ${MV} "${FILE_INET_SERVICES}" "${FILE_INET_SERVICES}.${SUFFIX}"
                        ${MV} "${FILE_TEMP}" "${FILE_INET_SERVICES}"

                        DisplayTrace "    File ${FILE_INET_SERVICES} updated"

                    else
                        DisplayTrace "    Entry not found for service ${INET_SERVICE_NAME} in ${FILE_INET_SERVICES}"
                        ${RM} "${FILE_TEMP}"
                    fi
                else
                    DisplayTrace="File ${FILE_INET_SERVICES} not found"
                    RETURN_CODE=1
                fi    
                    
            fi
            ;;

        esac        
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Relecture du fichier de configuration du demon

        case "${PLATFORM}" in
    
        "sun")
            # Rechargement configuration non necessaire pour Solaris 10
            if [ "${VERSION}" != "5.10" ]
            then
                ExecuteInetdConfigReloadCommand
                RETURN_CODE=$?
            fi
            ;;

        "sun_x86")
            # Rechargement configuration non necessaire pour Solaris 10
            if [ "${VERSION}" != "5.10" ]
            then
                ExecuteInetdConfigReloadCommand
                RETURN_CODE=$?
            fi
            ;;
            
        *)
            ExecuteInetdConfigReloadCommand
            RETURN_CODE=$?
            ;;
        esac
    fi


    if [ ${RETURN_CODE} -ne 0 ]
    then
        # S'il y a eu une erreur, elle n'est pas fatale pour la desinstallation
        DisplayTrace "Update of inet configuration did not succeed."
        RETURN_CODE=0
    fi

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Suppression du fichier de log du demon
        ${RM} -f "${DAEMON_LOG_DIRECTORY}/${INET_SERVICE_NAME}.log"
    fi

    return ${RETURN_CODE}
}    


###########################################################################
# AddInetService
###########################################################################

#
# Ajout d'un serveur a lancer par inetd (dans inetd.conf)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "add_inet_server"
#
# 

AddInetService ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> AddInetService"

    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 
        
        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"
        
        # Controle de compatibilite de la section avec la commande
        
        case "${SECTION_TYPE}" in
        
        "add_inet_service")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    AddInetServiceINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    AddInetServiceUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
            
        "no_action")
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# ExtractArchiveFiles
###########################################################################

#
# Extraction des fichiers d'archive contenant les fichiers a installer
# si installation demandee
# 
# 
# 
# 

ExtractArchiveFiles ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> ExtractArchiveFiles"
    
    if [ "${CONTEXT}" = "inst" ]
    then
        DIR_CURRENT=`\pwd`
    
        ${MKDIR} "${DIR_ARCHIVE}"
        RETURN_CODE=$?

        if [ ${RETURN_CODE} -ne 0 ]
        then
            DisplayError "Error while creating directory \"${DIR_ARCHIVE}\" "
        else      
        
            # Extraction des fichiers de l'archive

            \cd "${DIR_ARCHIVE}"

            for NAME in ${LIST_FEATURES_INSTALL}
            do
                FILE=`GetFieldSublist ${FILE_ARCHIVES} '|' ':' ${NAME} 2`
                
                DisplayTrace ""
                DisplayTrace "    FILE : \"${FILE}\" "

                ActivateTraceRedirection
                
                if [ "${PLATFORM}" = "hp" -o "${PLATFORM}" = "hpia" ]
                then
                    # Redirection car sous HP-UX la liste des fichiers va dans stderr
                    ${TAR} xvf "${DIR_CURRENT}/${FILE}" 2>&1
                    
                else
                    ${TAR} xvf "${DIR_CURRENT}/${FILE}"
                fi
                
                if [ $? -ne 0 ]
                then
                    DisplayError "Error while extracting archive file \"${FILE}"
                    RETURN_CODE=1
                fi

                DeactivateTraceRedirection

                if [ ${RETURN_CODE} -eq 0 ]
                then
                    # Suppression de l'archive au fur et a mesure
                    ${RM} "${DIR_CURRENT}/${FILE}"
                fi
            done
            
            \cd "${DIR_CURRENT}"
        fi    
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# ExtractArchives
###########################################################################

# 
# Extraction des fichiers d'archive contenant les fichiers a installer
# si installation demandee
# 
# 
# 

ExtractArchives ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> ExtractArchives"
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitArchiveFilesVariables
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        ExtractArchiveFiles
        RETURN_CODE=$?
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# InstallDirectoriesINST
###########################################################################

# 
# Installation repertoires (installation)
#
# Actuellement : pas de filtres (metacaracteres) supportes 
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "install_directories"
#
#
# A faire : controle validite de SUBDIR_INCLUSION
#
#

InstallDirectoriesINST ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> InstallDirectoriesINST"


    INSTALL_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "install_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 
    SUBDIR_INCLUSION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "include_subdir"` 
    OWNER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "owner"`


    DisplayTrace "    LOCATION_PATH    : ${LOCATION_PATH}"
    DisplayTrace "    INSTALL_DIR      : ${INSTALL_DIR}"
    DisplayTrace "    SOURCE_DIR       : ${SOURCE_DIR}"
    DisplayTrace "    SUBDIR_INCLUSION : ${SUBDIR_INCLUSION}"
    DisplayTrace "    OWNER            : ${OWNER}"
    
    case "${SUBDIR_INCLUSION}" in
    "y")
        # Si le repertoire existe deja (pour une raison ou une autre, ne pas 
        # indiquer ce repertoire en destination sinon il est recree en dessous
        if [ -d ${LOCATION_PATH}/${INSTALL_DIR} ]
        then
            ${CP} -pr "${DIR_ARCHIVE}/${SOURCE_DIR}" "${LOCATION_PATH}"
        else
            ${CP} -pr "${DIR_ARCHIVE}/${SOURCE_DIR}" "${LOCATION_PATH}/${INSTALL_DIR}"
        fi
        ;;

    "n")
        ${MKDIR} "${LOCATION_PATH}/${INSTALL_DIR}"

        # Copie des fichiers seuls sans les repertoires

        # La commande "cp" basique suivante est remplacee par la commande "ls" ... 
        # afin de gerer un tres grand nombre de fichiers avec "xargs"

        # Pas de guillemets si metacaractere utilise
        # ${CP} -p ${DIR_ARCHIVE}/${SOURCE_DIR}/* "${LOCATION_PATH}/${INSTALL_DIR}"

        ${LS} -l "${DIR_ARCHIVE}/${SOURCE_DIR}" | ${GREP} -v '^d' | ${SED} -e '1d' | ${AWK} '{print $NF}' | \
           ${XARGS} -i ${CP} -p "${DIR_ARCHIVE}/${SOURCE_DIR}/"\{\}  "${LOCATION_PATH}/${INSTALL_DIR}"
        ;;

    *)
        DisplayError "Incorrect value for \"include_subdir\" parameter in section : \"${SUBDIR_INCLUSION}\" " 
        RETURN_CODE=2
        ;;
    esac

    if [ "${OWNER}" != "" ]
    then
        ${CHOWN} -R ${OWNER} "${LOCATION_PATH}/${INSTALL_DIR}"
    fi            

    return ${RETURN_CODE}
}


###########################################################################
# InstallDirectoriesUNINST
###########################################################################

# 
# Suppression repertoires (desinstallation)
#
# Il y a suppression du repertoire et de sons contenu de maniere recursive
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "install_directories"
#
#
#
#

InstallDirectoriesUNINST ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> InstallDirectoriesUNINST"


    INSTALL_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "install_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 
    SUBDIR_INCLUSION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "include_subdir"` 
    OWNER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "owner"`

    DisplayTrace "    LOCATION_PATH    : ${LOCATION_PATH}"
    DisplayTrace "    INSTALL_DIR      : ${INSTALL_DIR}"
    DisplayTrace "    SOURCE_DIR       : ${SOURCE_DIR}"
    DisplayTrace "    SUBDIR_INCLUSION : ${SUBDIR_INCLUSION}"
    DisplayTrace "    OWNER            : ${OWNER}"

    ${RM} -Rf "${LOCATION_PATH}/${INSTALL_DIR}"

    return ${RETURN_CODE}
}


###########################################################################
# InstallDirectories
###########################################################################

# 
# Installation repertoires
#
# Actuellement : pas de filtres (metacaracteres) supportes 
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "install_directories"
#
#
# A faire : controle validite de SUBDIR_INCLUSION
#
#

InstallDirectories ()

{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> InstallDirectories"

    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 

        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"
        
        # Controle de compatibilite de la section avec la commande
        
        case "${SECTION_TYPE}" in
        
        "install_directories")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    InstallDirectoriesINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    InstallDirectoriesUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
        "no_action")
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
    fi

    return ${RETURN_CODE}
}


###########################################################################
# InstallFilesINST
###########################################################################

# 
# Installation fichiers (installation)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "install_files"
#
# 
# Si le fichier existe deja dans le repertoire destination, il n'est pas copie
#
# >>> Modifier la gestion des IDENT_PRESENT quand ils sont deja presents
#     dans le fichier de parametres d'installation (cas du FP)
#

InstallFilesINST ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> InstallFilesINST"

    
    INSTALL_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "install_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 
    INSTALL_OPTION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "install_option"` 

    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "

    DisplayTrace "    INSTALL_DIR    : ${INSTALL_DIR}"
    DisplayTrace "    SOURCE_DIR     : ${SOURCE_DIR}"
    DisplayTrace "    INSTALL_OPTION : ${INSTALL_OPTION}"

    # Analyse parametre optionnel "search_directories"
    
    SEARCH_DIRECTORIES=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "search_directories"` 
    RETURN_CODE=$?
    
    if [ "${RETURN_CODE}" -eq 0 ]
    then
        DisplayTrace "    Search directories present" 
        DisplayTrace "    SEARCH_DIRECTORIES : ${SEARCH_DIRECTORIES}"
        
        if [ "${INSTALL_OPTION}" != "if_not_found" ]
        then
            DisplayTrace "    Warning:"
            DisplayTrace "    \"search_directories\" taken into account only with \"install_option\" value of \"if_not_found\" "
        fi        
    else    
        if [ "${RETURN_CODE}" -eq 2 ]
        then
            # Parametre absent
            # Le parametre est facultatif donc pas d'erreur
            DisplayTrace "    No \"search_directories\" parameter present"
            SEARCH_DIRECTORIES=""
            RETURN_CODE=0
        else    
            # Cas improbable
            RETURN_CODE=1
            return ${RETURN_CODE}
        fi    
    fi
    
    
    # Analyse parametre optionnel "error_processing"
    
    # Valeur par defaut de 1 pour les indicateurs de gestion d'erreur
    R_E_CANNOT_WRITE_DESTINATION=1
    
    
    ERROR_PROCESSING=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "error_processing"` 
    RETURN_CODE=$?
    
    if [ "${RETURN_CODE}" -eq 0 ]
    then
        DisplayTrace ""
        DisplayTrace "    Error processing parameter present" 
        DisplayTrace "    ERROR_PROCESSING : ${ERROR_PROCESSING}"
        
        if [ "${ERROR_PROCESSING}" != "" ]
        then
        
            NB_ERROR_PROCESSING_FLAGS=`NumberElementsList ${ERROR_PROCESSING} '+'`
            COUNTER=${NB_ERROR_PROCESSING_FLAGS}
            DisplayTrace "    COUNTER : ${COUNTER}"
            
            while [ ${COUNTER} -ne 0 ]
            do
                FLAG_COUPLE=`GetFieldList ${ERROR_PROCESSING} '+' ${COUNTER}`
        
                DisplayTrace "    FLAG_COUPLE : ${FLAG_COUPLE}"
                
                FLAG_NAME=`GetFieldList ${FLAG_COUPLE} ':' 1`
                FLAG_VALUE=`GetFieldList ${FLAG_COUPLE} ':' 2`
                
                DisplayTrace "    FLAG_NAME  : ${FLAG_NAME}"
                DisplayTrace "    FLAG_VALUE : ${FLAG_VALUE}"
                
                case "${FLAG_NAME}" in
                
                cannot_write_destination)
                        
                    case "${FLAG_VALUE}" in
                    no_error)
                        # No raise of error
                        R_E_CANNOT_WRITE_DESTINATION=0
                        ;;
                        
                    error_on_first)    
                        # Raise of first error
                        R_E_CANNOT_WRITE_DESTINATION=1
                        ;;
                        
                   error_on_last)    
                        # Raise of last error
                        R_E_CANNOT_WRITE_DESTINATION=2
                        ;;
                
                    *)
                        DisplayTrace "    Unknown value for ${FLAG_NAME}  : ${FLAG_VALUE}"
                        ;;
                    esac                    
                    DisplayTrace "    R_E_CANNOT_WRITE_DESTINATION : ${R_E_CANNOT_WRITE_DESTINATION}"
                    ;;
                    
                *)
                    DisplayTrace "    Unknown flag name : ${FLAG_NAME}"
                    ;;
                    
                esac 
                
                COUNTER=`expr ${COUNTER} "-" 1`
            done
        fi
        
    else    
        if [ "${RETURN_CODE}" -eq 2 ]
        then
            # Parametre absent
            # Le parametre est facultatif donc pas d'erreur
            DisplayTrace "    No \"error_processing\" parameter present"
            ERROR_PROCESSING=""
            RETURN_CODE=0
        else    
            # Cas improbable
            RETURN_CODE=1
            return ${RETURN_CODE}
        fi    
    fi
    
    DisplayTrace "    R_E_CANNOT_WRITE_DESTINATION : ${R_E_CANNOT_WRITE_DESTINATION}"
    

    INSTALL_DIR=`\eval echo "${INSTALL_DIR}"`
    SOURCE_DIR=`\eval echo "${SOURCE_DIR}"`
    
    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    INSTALL_DIR    : ${INSTALL_DIR}"
    DisplayTrace "    SOURCE_DIR     : ${SOURCE_DIR}"
    
    DisplayTrace ""
    

    NUMBER_DESTINATION_DIRECTORIES=`NumberElementsList "${INSTALL_DIR}" ":"`

    DisplayTrace "    NUMBER_DESTINATION_DIRECTORIES :  \"${NUMBER_DESTINATION_DIRECTORIES}\" " 
    
    
    # Traitement pour chaque fichier a installer
    ${CAT} "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" | ${SED} -e '1d' | while read LINE 
    do
        if [ "${LINE}" != "" ]
        then
            # DisplayTrace "    LINE : \"${LINE}\"" 

            IDENTIFIER=`echo ${LINE} | ${CUT} -d'=' -f1`
            VALUE=`echo ${LINE} | ${CUT} -d'=' -f2`

            if [ "${IDENTIFIER}" != "file" ]
            then
                # DisplayTrace "    Line discarded"

                continue
            else
                DisplayTrace ""
                DisplayTrace "    -------------------------------"
                DisplayTrace "    LINE : \"${LINE}\"" 
            fi

            FILE=`echo ${VALUE} | ${CUT} -d':' -f1`
            PERM=`echo ${VALUE} | ${CUT} -d':' -f2`
            OWNER=`echo ${VALUE} | ${CUT} -d':' -f3`
            GROUP=`echo ${VALUE} | ${CUT} -d':' -f4`
            IDENT_PRESENCE=`echo ${VALUE} | ${CUT} -d':' -f5`
            IDENT_PATH=`echo "${VALUE}" | ${CUT} -d':' -f6`

            DisplayTrace ""
            DisplayTrace "    FILE           : \"${FILE}\"" 
            DisplayTrace "    PERM           : \"${PERM}\"" 
            DisplayTrace "    OWNER          : \"${OWNER}\"" 
            DisplayTrace "    GROUP          : \"${GROUP}\"" 
            DisplayTrace "    IDENT_PRESENCE : \"${IDENT_PRESENCE}\"" 
            DisplayTrace "    IDENT_PATH     : \"${IDENT_PATH}\"" 
            
            
            # IDENT_PATH est necessaire a partir du moment ou INSTALL_DIR 
            # contient plus d'un repertoire
            if [ "${NUMBER_DESTINATION_DIRECTORIES}" -gt 1 -a "${IDENT_PATH}" = "" ]
            then
                DisplayError "Missing IDENT_PATH value in \"${LINE}\" " 
                DisplayTrace "Missing IDENT_PATH value in \"${LINE}\" " 
                RETURN_CODE=1
                break
            fi    


            # Si la ligne ne comporte aucun separateur de champ, 
            # le "cut" renvoie le nom de fichier dans tous les champs
            if [ "${FILE}" = "${GROUP}" ]
            then
                DisplayError "Incorrect value format in line \"${LINE}\" "
                RETURN_CODE=1
                break
            fi


###             ${CP} -p "${DIR_ARCHIVE}/${SOURCE_DIR}/${FILE}" "${LOCATION_PATH}/${INSTALL_DIR}"
###             if [ $? -ne 0 ]
###             then
###                 DisplayError "Error while copying file \"${FILE}\" "
###                 return 1
###             fi   


            # Recherche dans les repertoires
            if [ "${RETURN_CODE}" -eq 0 ]
            then
                if [ "${INSTALL_OPTION}" = "if_not_found" ]
                then
                    if [ "${SEARCH_DIRECTORIES}" != "" ]
                    then
                        # On suppose que les repertoires de SEARCH_DIRECTORIES ne 
                        # contiennent pas d'espaces

                        # Indicateur si la recherche dans les repertoires a ete fructueuse
                        # ou non
                        FILE_FOUND=0

                        SearchFileInDirectories "${FILE}" "${SEARCH_DIRECTORIES}"
                        case $? in
                        0)
                            FILE_FOUND=1
                            ;;

                        1)
                            FILE_FOUND=0
                            RETURN_CODE=1  
                            ;;

                        2)
                            FILE_FOUND=0
                            ;;

                        *)
                            DisplayTrace "Invalid return value for SearchFileInDirectories ()" 
                            DisplayError "Invalid return value for SearchFileInDirectories ()"
                            RETURN_CODE=1
                            ;;
                        esac   
                    else    
                        FILE_FOUND=0
                    fi
                    
                    DisplayTrace "    FILE_FOUND : ${FILE_FOUND}"
                    
                else     
                    # Pas de recherche 
                    DisplayTrace "    No directory search for file"
                fi
            fi

    
            # Indicateur que l'installation du fichier s'est bien effectuee
            # (Indicateur sous forme de fichier : variable pas facilement utilisable 
            # car boucles imbriquees)
            FILE_INSTALL_FILE_SUCCESS="${TEMPORARY_PATH}/install_file_success.tmp"
            
            ${RM} -f "${FILE_INSTALL_FILE_SUCCESS}"

            
            # Installation dans un des repertoires destination
            # 
            
            LIST_DESTINATION_DIRECTORIES=`echo "${INSTALL_DIR}" | ${TR} ':' '\n'`
            
            DisplayTrace ""
            DisplayTrace "    LIST_DESTINATION_DIRECTORIES   :  \"${LIST_DESTINATION_DIRECTORIES}\" " 
                        
            # Traitement pour chaque repertoire destination
            
            echo "${LIST_DESTINATION_DIRECTORIES}" | while read DESTINATION_DIR 
            do
                DisplayTrace ""
                DisplayTrace "    DESTINATION_DIR : ${DESTINATION_DIR}"
            
                # Controle si chemin absolu
                echo "${DESTINATION_DIR}" | ${GREP} '^/' >/dev/null 2>&1
                if [ $? -eq 0 ]
                then
                    # Chemin absolu
                    # DESTINATION_DIR=${DESTINATION_DIR}
                    DisplayTrace "    DESTINATION_DIR is an absolute pathname"
                else
                    # Chemin relatif a LOCATION_PATH
                    DESTINATION_DIR=${LOCATION_PATH}/${DESTINATION_DIR}
                    DisplayTrace "    DESTINATION_DIR is a relative pathname"
                fi
 
                DisplayTrace "    DESTINATION_DIR : ${DESTINATION_DIR}"


                # Determination si l'installation (copie) du fichier est effective ou non
                
                EFFECTIVE_COPY=0

                case "${INSTALL_OPTION}" in

                "always")
                    EFFECTIVE_COPY=1
                    ;;

                "if_not_present")  
                   if [ -f "${DESTINATION_DIR}/${FILE}" ]
                   then
                        DisplayTrace "    File \"${FILE}\" already found in ${DESTINATION_DIR}."
                        EFFECTIVE_COPY=0
                    else
                        DisplayTrace "    File \"${FILE}\" not found in ${DESTINATION_DIR}."
                        EFFECTIVE_COPY=1
                    fi
                    ;;

                "never")
                    EFFECTIVE_COPY=0
                    ;;

                "if_not_found")  
                    case "${FILE_FOUND}" in
                    0)
                        EFFECTIVE_COPY=1
                        ;;

                    1)
                        EFFECTIVE_COPY=0
                        ;;

                    *)
                        DisplayTrace "Invalid return value for FILE_FOUND : \"${FILE_FOUND}\" " 
                        DisplayError "Invalid return value for FILE_FOUND : \"${FILE_FOUND}\" " 
                        RETURN_CODE=1
                        ;;
                    esac   
                    ;;                

                *)
                    EFFECTIVE_COPY=0
                    DisplayTrace "    Incorrect value for INSTALL_OPTION : ${INSTALL_OPTION}"
                    DisplayError "Incorrect value for INSTALL_OPTION : ${INSTALL_OPTION}"
                    ;;

                esac

                DisplayTrace "    EFFECTIVE_COPY :  \"${EFFECTIVE_COPY}\" " 

                # Memorisation presence du fichier avec IDENT_PRESENCE
                # Si le fichier existe deja dans le repertoire destination,  
                # memorisation uniquement si IDENT_PRESENCE non encore memorise
                # (car le fichier peut avoir ete deja installe par une action 
                # identique precedente)
                
                if [ "${IDENT_PRESENCE}" != "" ]
                then
                    if [ -f "${DESTINATION_DIR}/${FILE}" ]
                    then
                        DisplayTrace "    File \"${FILE}\" already found in ${DESTINATION_DIR}."
                        VALUE_IDENT_PRESENCE=1
    
                        CheckSettingsVariable "${IDENT_PRESENCE}"
                        if [ $? -ne 0 ]
                        then
                            AddSettingsVariable "${IDENT_PRESENCE}" "${VALUE_IDENT_PRESENCE}"
                        fi   
                    fi
                fi


                # Memorisation repertoire avec IDENT_PATH

                if [ "${IDENT_PATH}" != "" ]
                then
                    if [ -f "${DESTINATION_DIR}/${FILE}" ]
                    then
                        # Memorisation repertoire ou se trouve le fichier
                        CheckSettingsVariable "${IDENT_PATH}"
                        if [ $? -ne 0 ]
                        then
                            AddSettingsVariable "${IDENT_PATH}" "${DESTINATION_DIR}"
                        fi    
                    fi
                fi




                # Installation (par copie) du fichier

                DisplayTrace ""

                case "${EFFECTIVE_COPY}" in
                0)
                    DisplayTrace "    File not copied."
                    > "${FILE_INSTALL_FILE_SUCCESS}"
                    break
                    ;;

                1)
                    # Traitement repertoire au prealable
                    
                    # Pas d'utilisation de la fonction CheckExistingDirectory () car un 
                    # message d'erreur est affiche si inexistant
                    if [ ! -d "${DESTINATION_DIR}" ]
                    then
                        DisplayTrace "    Creation of directory \"${DESTINATION_DIR}\" " 
                        ${MKDIR} -p "${DESTINATION_DIR}" 2> "${DIR_TEMP}/error_output"
                        if [ $? -ne 0 ]
                        then
                            if [ ${R_E_CANNOT_WRITE_DESTINATION} -eq 1 ]
                            then
                                ${CAT} "${DIR_TEMP}/error_output" 1>&2
                                DisplayError "Error while copying file \"${FILE}\" to \"${DESTINATION_DIR}\" directory"
                                DisplayTrace "Error while copying file \"${FILE}\" to directory \"${DESTINATION_DIR}\" "
                                break 
                                # return 1
                            else
                                DisplayTextFileTrace "${DIR_TEMP}/error_output"
                                DisplayTrace "    File \"${FILE}\" cannot be copied to directory \"${DESTINATION_DIR}\" "
                                DisplaySummary "    File \"${FILE}\" cannot be copied to directory \"${DESTINATION_DIR}\" "

                                DisplayTrace "    Cannot create directory \"${DESTINATION_DIR}\" " 
                                # Passer au repertoire suivant ###
                                DisplayTrace "    Skipping to next install directory" 
                                DisplayTrace ""

                                continue
                            fi
                        fi
                    fi


                    if [ -w "${DESTINATION_DIR}" ]
                    then
                        # Repertoire accessible en ecriture
                        DisplayTrace "    Directory \"${DESTINATION_DIR}\" writable" 
                    else
                        # Repertoire inaccessible en ecriture
                        if [ ${R_E_CANNOT_WRITE_DESTINATION} -eq 1 ]
                        then
                            DisplayError "Directory \"${DESTINATION_DIR}\" not writable" 
                            DisplayTrace "Directory \"${DESTINATION_DIR}\" not writable" 
                            break
                        else
                            DisplayTrace "    Directory \"${DESTINATION_DIR}\" not writable" 
                            # ????
                            # Passer au repertoire suivant ###
                            DisplayTrace "    Skipping to next install directory" 
                            DisplayTrace ""

                            continue
                        fi
                    fi    
                
                
                    # Copie du fichier
                
                    ${CP} -p "${DIR_ARCHIVE}/${SOURCE_DIR}/${FILE}" "${DESTINATION_DIR}" 2> "${DIR_TEMP}/error_output"
                    if [ $? -ne 0 ]
                    then

                        ${RM} -f "${DESTINATION_DIR}/${FILE}" >/dev/null 2>&1
                        if [ ${R_E_CANNOT_WRITE_DESTINATION} -eq 1 ]
                        then
                            ${CAT} "${DIR_TEMP}/error_output" 1>&2
                            DisplayError "Error while copying file \"${FILE}\" to \"${DESTINATION_DIR}\" directory"
                            DisplayTrace "Error while copying file \"${FILE}\" to directory \"${DESTINATION_DIR}\" "
                            break
                            # return 1
                        else
                            DisplayTextFileTrace "${DIR_TEMP}/error_output"
                            DisplayTrace "    File \"${FILE}\" cannot be copied to directory \"${DESTINATION_DIR}\" "
                            DisplaySummary "    File \"${FILE}\" cannot be copied to directory \"${DESTINATION_DIR}\" "

                            # Passage au repertoire suivant
                            continue
                        fi
                    else
                        DisplayTrace "    File copied."

                        # Memorisation absence fichier initiale IDENT_PRESENCE
                        ### gerer FP (variable deja presente) ###
                        # >>> ### Section a finaliser ### 
                        
                        if [ "${IDENT_PRESENCE}" != "" ]
                        then
                            VALUE_IDENT_PRESENCE=0

                            CheckSettingsVariable "${IDENT_PRESENCE}"
                            if [ $? -ne 0 ]
                            then
                                AddSettingsVariable "${IDENT_PRESENCE}" "${VALUE_IDENT_PRESENCE}"
                            fi   
                        fi



                        # Memorisation repertoire avec IDENT_PATH
                        ### gerer FP (variable deja presente) ###
                        # >>> ### Section a finaliser ### 

                        if [ "${IDENT_PATH}" != "" ]
                        then
                            CheckSettingsVariable "${IDENT_PATH}"
                            if [ $? -ne 0 ]
                            then
                                # Memorisation repertoire ou se trouve le fichier
                                AddSettingsVariable "${IDENT_PATH}" "${DESTINATION_DIR}"
                            fi   
                        fi



                        if [ "${OWNER}" != "" ]
                        then
                            SetOwnerFile ${OWNER} "${DESTINATION_DIR}/${FILE}"
                        fi

                        if [ "${GROUP}" != "" ]
                        then
                            SetGroupFile ${GROUP} "${DESTINATION_DIR}/${FILE}"
                        fi

                        if [ "${PERM}" != "" ]
                        then
                            SetPermissionsFile ${PERM} "${DESTINATION_DIR}/${FILE}"
                        fi
                        
                        > "${FILE_INSTALL_FILE_SUCCESS}"
                        DisplayTrace "    Successful installation of file"
                        break 
                    fi   
                    ;;

                *)
                    DisplayTrace "Unexpected value for EFFECTIVE_COPY : ${EFFECTIVE_COPY}"
                    DisplayError "Unexpected value for EFFECTIVE_COPY : ${EFFECTIVE_COPY}"
                    ;;

                esac
            
            done  # Traitement pour chaque repertoire destination
            

            if [ -f "${FILE_INSTALL_FILE_SUCCESS}" ]
            then
                DisplayTrace "    File \"${FILE}\" has been processed successfully"
                ${RM} -f "${FILE_INSTALL_FILE_SUCCESS}"
            else    
                case "${R_E_CANNOT_WRITE_DESTINATION}" in
                0)    
                    DisplayTrace "    File \"${FILE}\" has not been processed successfully"
                    ;;  
                              
                1|2) 
                    DisplayError "File \"${FILE}\" has not been processed successfully"
                    DisplayTrace "File \"${FILE}\" has not been processed successfully"
                    RETURN_CODE=1
                    break
                    ;;            
                    
                *)
                    DisplayError "Unexpected value for R_E_CANNOT_WRITE_DESTINATION \"${R_E_CANNOT_WRITE_DESTINATION}\" "
                    DisplayTrace "Unexpected value for R_E_CANNOT_WRITE_DESTINATION \"${R_E_CANNOT_WRITE_DESTINATION}\" "
                    ;;            
                esac

            fi

        fi
    done  # Traitement pour chaque fichier

    return ${RETURN_CODE}
}


###########################################################################
# InstallFilesUNINST
###########################################################################

#
# Desinstallation fichiers (desinstallation)
#
# Test existence fichier destination a faire !
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "install_files"
#
# Pas de suppression du repertoire ?
#
# ### A FAIRE ### Ne pas supprimer le fichier s'il etait deja present lors de l'installation
#
#
#

InstallFilesUNINST ()
{
    RETURN_CODE=0

    
    DisplayTrace ""
    DisplayTrace ">>> InstallFilesUNINST"
    
    
    INSTALL_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "install_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 
    UNINSTALL_OPTION=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "_uninstall_option"` 

    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    


    DisplayTrace "    INSTALL_DIR      : ${INSTALL_DIR}"
    DisplayTrace "    SOURCE_DIR       : ${SOURCE_DIR}"
    DisplayTrace "    UNINSTALL_OPTION : ${UNINSTALL_OPTION}"

    INSTALL_DIR=`\eval echo "${INSTALL_DIR}"`
    SOURCE_DIR=`\eval echo "${SOURCE_DIR}"`
    
    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    INSTALL_DIR      : ${INSTALL_DIR}"
    DisplayTrace "    SOURCE_DIR       : ${SOURCE_DIR}"
    
    DisplayTrace ""
    
    # Controle si chemin absolu
    echo "${INSTALL_DIR}" | ${GREP} '^/' >/dev/null 2>&1
    if [ $? -eq 0 ]
    then
        # Chemin absolu
        DESTINATION_DIR=${INSTALL_DIR}
        DisplayTrace "    INSTALL_DIR is an absolute pathname"
    else
        # Chemin relatif aLOCATION_PATH
        DESTINATION_DIR=${LOCATION_PATH}/${INSTALL_DIR}
        DisplayTrace "    INSTALL_DIR is a relative pathname"
    fi
    
    
    ${CAT} "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" | ${SED} -e '1d' | while read LINE 
    do
        if [ "${LINE}" != "" ]
        then
            # DisplayTrace "    LINE : \"${LINE}\"" 

            IDENTIFIER=`echo ${LINE} | ${CUT} -d'=' -f1`
            VALUE=`echo ${LINE} | ${CUT} -d'=' -f2`

            if [ "${IDENTIFIER}" != "file" ]
            then
                # DisplayTrace "    Line discarded"

                continue
            else

                DisplayTrace "    LINE : \"${LINE}\"" 
            fi

            FILE=`echo ${VALUE} | ${CUT} -d':' -f1`
            PERM=`echo ${VALUE} | ${CUT} -d':' -f2`
            OWNER=`echo ${VALUE} | ${CUT} -d':' -f3`
            GROUP=`echo ${VALUE} | ${CUT} -d':' -f4`
            IDENT_PRESENCE=`echo ${VALUE} | ${CUT} -d':' -f5`

            DisplayTrace ""
            DisplayTrace "    FILE           : \"${FILE}\"" 
            DisplayTrace "    PERM           : \"${PERM}\"" 
            DisplayTrace "    OWNER          : \"${OWNER}\"" 
            DisplayTrace "    GROUP          : \"${GROUP}\"" 
            DisplayTrace "    IDENT_PRESENCE : \"${IDENT_PRESENCE}\"" 


            # Si la ligne ne comporte aucun separateur de champ, 
            # le "cut" renvoie le nom de fichier dans tous les champs
            if [ "${FILE}" = "${GROUP}" ]
            then
                DisplayError "Incorrect value format in line \"${LINE}\" "
                RETURN_CODE=1
                break
            fi


            if [ "${IDENT_PRESENCE}" != "" ]
            then
                # Recuperation de l'information sur l'existence du fichier avant 
                # l'installation HPU
                VALUE_PRESENCE=`${GREP} "${IDENT_PRESENCE}" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2-`
                if [ $? -ne 0 ]
                then
                    # Information non memorisee (pas d'erreur)
                    DisplayTrace "    Parameter \"${IDENT_PRESENCE}\" not found in ${FILE_INSTALL_SETTINGS}"
                    VALUE_PRESENCE=2
                fi


                EFFECTIVE_REMOVAL=0

                case "${UNINSTALL_OPTION}" in

                "always")
                    EFFECTIVE_REMOVAL=1
                    ;;

                "if_not_previously_present")  
                   case "${VALUE_PRESENCE}" in 
                   0)
                        EFFECTIVE_REMOVAL=1
                        ;;

                   1)
                        EFFECTIVE_REMOVAL=0
                        ;; 

                   2)
                        # Information non memorisee : suppression du fichier
                        # inconditionelle
                        EFFECTIVE_REMOVAL=1
                        ;; 

                   *) 
                        DisplayTrace "Unexpected value for VALUE_PRESENCE : ${VALUE_PRESENCE}"
                        DisplayError "Unexpected value for VALUE_PRESENCE : ${VALUE_PRESENCE}"
                        ;;
                   esac
                   ;;     

                "never")
                    EFFECTIVE_REMOVAL=0
                    ;;

                *)
                    EFFECTIVE_REMOVAL=0
                    DisplayTrace "    Incorrect value for UNINSTALL_OPTION : ${UNINSTALL_OPTION}"
                    DisplayError "Incorrect value for UNINSTALL_OPTION : ${UNINSTALL_OPTION}"
                    ;;

                esac
            else
                # Pas d'identificateur defini --> pas de memorisation
                # Par defaut : suppression    
                EFFECTIVE_REMOVAL=1
            fi    

            DisplayTrace ""
            
            case "${EFFECTIVE_REMOVAL}" in
            0)
                DisplayTrace "    Not removed."
                ;;

            1)
                ${RM} -f "${DESTINATION_DIR}/${FILE}"
                if [ $? -ne 0 ]
                then
                    DisplayError "Error while deleting file \"${FILE}\" "
                    # return 1
                else
                    DisplayTrace "    Removed."
                fi    
                ;;
                
            *)
                DisplayTrace "Unexpected value for EFFECTIVE_REMOVAL : ${EFFECTIVE_REMOVAL}"
                DisplayError "Unexpected value for EFFECTIVE_REMOVAL : ${EFFECTIVE_REMOVAL}"
                ;;
            esac

        fi
    done


    # Suppression du repertoire si vide
    if CheckEmptyDir "${DESTINATION_DIR}"
    then
        # Si, lors de l'installation plusieurs niveaux de repertoires ont ete crees,
        # seulement le dernier niveau est supprime maintenant
        DisplayTrace "    Deletion of directory \"${DESTINATION_DIR}\" " 
           
        ${RMDIR} "${DESTINATION_DIR}"
    else
        # Variable positionnee par CheckEmptyDir ()
        RETURN_CODE=0    
    fi        

    return ${RETURN_CODE}
}


###########################################################################
# InstallFiles
###########################################################################

#
# Installation fichiers
#
# Test existence fichier destination a faire !
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "install_files"
#
# 
#

InstallFiles ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> InstallFiles"
    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 
        
        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"
        
        
        # Controle de compatibilite de la section avec la commande

        case "${SECTION_TYPE}" in
        
        "install_files")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    InstallFilesINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    InstallFilesUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" "
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
            
        "no_action")
        
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
    
    fi

    return ${RETURN_CODE}
}


###########################################################################
# UpdateGroupLinesFileINST
###########################################################################

#
# Mise a jour d'un groupe de lignes dans un fichier texte (installation)
#
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "update_group_lines_file"
#
#
# 

UpdateGroupLinesFileINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> UpdateGroupLinesFileINST"


    FILE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "file"` 
    LINE_GROUP_BEGIN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_group_begin"`
    LINE_GROUP_END=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_group_end"`
    KEEP_LINE_GROUP_BEGIN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "keep_line_group_begin"`
    KEEP_LINE_GROUP_END=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "keep_line_group_end"`
    LINE_GROUP_REPLACEMENT_BEGIN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_group_replacement_begin"`
    LINE_GROUP_REPLACEMENT_END=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_group_replacement_end"`


    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    FILE                         : \"${FILE}\"" 
    DisplayTrace "    LINE_GROUP_BEGIN             : \"${LINE_GROUP_BEGIN}\"" 
    DisplayTrace "    LINE_GROUP_END               : \"${LINE_GROUP_END}\"" 
    DisplayTrace "    KEEP_LINE_GROUP_BEGIN        : \"${KEEP_LINE_GROUP_BEGIN}\"" 
    DisplayTrace "    KEEP_LINE_GROUP_END          : \"${KEEP_LINE_GROUP_END}\"" 
    DisplayTrace "    LINE_GROUP_REPLACEMENT_BEGIN : \"${LINE_GROUP_REPLACEMENT_BEGIN}\"" 
    DisplayTrace "    LINE_GROUP_REPLACEMENT_END   : \"${LINE_GROUP_REPLACEMENT_END}\"" 


    FILE=`\eval echo "${FILE}"`
###    VALUE=`\eval echo "${VALUE}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    FILE                         : \"${FILE}\"" 

    # Fichier temporaire vide cree avec les memes droits et proprietaire que
    # l'original
    # >>> A faire ? Ne pas mettre le fichier temporaire au meme endroit que l'original
    # car le FILE_PROFILE est un chemin complet
    FILE_RESULT="${FILE}.$$"
    ${CP} -p "${FILE}" "${FILE_RESULT}"
    > "${FILE_RESULT}"

    # Fichier contenant le groupe de lignes destine a remplacer l'actuel
    FILE_SECTION="groupe_nouveau"


    # Fichiers temporaires
    FILE_AUX="${FILE}.aux.$$"
    FILE_TEMP="${FILE}.temp.$$"
    FILE_GROUP_LINES="${FILE}.group_lines.$$"
    
    
    # Constitution chaine de programme AWK au prealable pour eviter tout pb avec les dieses 
    # et les apostrophes
    AWK_TEXT="/${LINE_GROUP_BEGIN}/,/${LINE_GROUP_END}/ {print}"

    DisplayTrace "    AWK_TEXT   : \"${AWK_TEXT}\"" 

    # Recherche du paragraphe
    ${AWK} -v LINE_BEGIN="${LINE_GROUP_BEGIN}" \
           -v LINE_END="${LINE_GROUP_END}" \
           "${AWK_TEXT}"  "${FILE}"    > "${FILE_TEMP}"

    if [ -s "${FILE_TEMP}" ]
    then
        DisplayTrace ""
        DisplayTrace "    Group of lines found:"
        DisplayTrace "    Beginning of lines found ------------------------"
        DisplayTextFileTrace "${FILE_TEMP}"
        DisplayTrace "    End of lines found ------------------------------"

        #
        # Constitution nouveau groupe de lignes
        # 

        > "${FILE_GROUP_LINES}"

        # Traitement ligne de debut du groupe

        case "${KEEP_LINE_GROUP_BEGIN}" in
        "yes")
            echo "${LINE_GROUP_BEGIN}" >> "${FILE_GROUP_LINES}"
            ;;

        "no")
            echo "${LINE_GROUP_REPLACEMENT_BEGIN}" >> "${FILE_GROUP_LINES}"
            ;;

        *)
            DisplayTrace "Error on KEEP_LINE_GROUP_BEGIN value: \"${KEEP_LINE_GROUP_BEGIN}\" " 
            DisplayError "Error on KEEP_LINE_GROUP_BEGIN value: \"${KEEP_LINE_GROUP_BEGIN}\" "
            RETURN_CODE=1
            ;;
        esac    

        # Traitement autres lignes du groupe (entre ligne de debut et ligne de fin)

        ${CAT} "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" | ${SED} -e '1d' | while read LINE 
        do
            if [ "${LINE}" != "" ]
            then
                ### DisplayTrace "    LINE : \"${LINE}\"" 

                IDENTIFIER=`echo ${LINE} | cut -d'=' -f1`
                # On prend tout le RESTE de la ligne (car il peut y avoir 
                # d'autres "=") 
                VALUE=`echo ${LINE} | cut -d'=' -f2-`

                ### DisplayTrace "    IDENTIFIER : \"${IDENTIFIER}\"" 
                ### DisplayTrace "    VALUE : \"${VALUE}\"" 

                if [ "${IDENTIFIER}" != "line_group_replacement" ]
                then
                    ### DisplayTrace "    Line discarded"
                    continue
                else
                    DisplayTrace "    LINE : \"${LINE}\"" 
                fi


                LINE_GROUP_REPLACEMENT="${VALUE}" 

                # Les lignes sont evaluees
                
                DisplayTrace ""
                DisplayTrace "    Before evaluation : "
                DisplayTrace "    LINE_GROUP_REPLACEMENT  : \"${LINE_GROUP_REPLACEMENT}\"" 

                LINE_GROUP_REPLACEMENT=`\eval echo "${LINE_GROUP_REPLACEMENT}"`

                DisplayTrace ""
                DisplayTrace "    After evaluation : "
                DisplayTrace "    LINE_GROUP_REPLACEMENT  : \"${LINE_GROUP_REPLACEMENT}\"" 

                echo "${LINE_GROUP_REPLACEMENT}" >> "${FILE_GROUP_LINES}"
            fi
        done


        # Traitement ligne de fin du groupe

        case "${KEEP_LINE_GROUP_END}" in
        "yes")
            echo "${LINE_GROUP_END}" >> "${FILE_GROUP_LINES}"
            ;;

        "no")
            echo "${LINE_GROUP_REPLACEMENT_END}" >> "${FILE_GROUP_LINES}"
            ;;

        *)
            DisplayTrace "Error on KEEP_LINE_GROUP_END value: \"${KEEP_LINE_GROUP_END}\" " 
            DisplayError "Error on KEEP_LINE_GROUP_END value: \"${KEEP_LINE_GROUP_END}\" "
            RETURN_CODE=1
            ;;
        esac    


        DisplayTrace ""
        DisplayTrace "    Replacement group of lines: "
        DisplayTrace "    Beginning of replacement lines found -------------------------"
        DisplayTextFileTrace "${FILE_GROUP_LINES}"
        DisplayTrace "    End of replacement lines found -------------------------------"

        #
        # Remplacement effectif du groupe de lignes
        # 

        # Identification de la ligne a partir de laquelle debute le groupe de lignes

        LINE_START_NUMERO=`${GREP} -n '^'"${LINE_GROUP_BEGIN}" "${FILE}" | ${CUT} -d':' -f1`

        DisplayTrace ""
        DisplayTrace "    LINE_START_NUMERO : ${LINE_START_NUMERO}"

        # Suppression du groupe de lignes a remplacer
        
        STRING_BEGIN=`MaskRegularExpressionCharacters "${LINE_GROUP_BEGIN}"`
        STRING_END=`MaskRegularExpressionCharacters "${LINE_GROUP_END}"`

        ${SED} -e '/^'"${STRING_BEGIN}"'/,/^'"${STRING_END}"'/d' "${FILE}" > "${FILE_AUX}"
        

        if [ "${LINE_START_NUMERO}" -eq 1 ]
        then
            # Si numero le groupe de ligne a modifier debute a la ligne 1,  maniere de faire
            # differente

            ${CAT} "${FILE_GROUP_LINES}" "${FILE_AUX}" > "${FILE_RESULT}"
        else
            # La methode utilisee se sert du numero de ligne qui precede
            LINE_START_NUMERO=`expr ${LINE_START_NUMERO} - 1`

            ${SED} -e "${LINE_START_NUMERO}r ${FILE_GROUP_LINES}" "${FILE_AUX}" > "${FILE_RESULT}"
        fi
        
        ${MV} "${FILE_RESULT}" "${FILE}"
        
    else
        # Le groupe de lignes n'a pas ete trouve dans le fichier
        # Erreur non fatale
        
        DisplayTrace "No group of lines beginning with"
        DisplayTrace "\"${LINE_GROUP_BEGIN}\"" 
        DisplayTrace "and ending with "
        DisplayTrace " \"${LINE_GROUP_END}\"" 
        DisplayTrace "found in \"${FILE}\" file."
    fi

    ${RM} -f "${FILE_AUX}"
    ${RM} -f "${FILE_TEMP}"
    ${RM} -f "${FILE_GROUP_LINES}"

    return ${RETURN_CODE}
}


###########################################################################
# UpdateGroupLinesFileUNINST
###########################################################################

#
# Mise a jour d'un groupe de lignes dans un fichier texte (desinstallation)
#
# >>> Actuellement rien n'est a faire pour la desinstallation
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "update_group_lines_file"
#
#
# 

UpdateGroupLinesFileUNINST ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> UpdateGroupLinesFileUNINST"


    FILE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "file"` 
    LINE_GROUP_BEGIN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_group_begin"`
    LINE_GROUP_END=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_group_end"`
    KEEP_LINE_GROUP_BEGIN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "keep_line_group_begin"`
    KEEP_LINE_GROUP_END=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "keep_line_group_end"`
    LINE_GROUP_REPLACEMENT_BEGIN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_group_replacement_begin"`
    LINE_GROUP_REPLACEMENT_END=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "line_group_replacement_end"`


    DisplayTrace ""
    DisplayTrace "    Before evaluation of action arguments : "
    DisplayTrace "    FILE                         : \"${FILE}\"" 
    DisplayTrace "    LINE_GROUP_BEGIN             : \"${LINE_GROUP_BEGIN}\"" 
    DisplayTrace "    LINE_GROUP_END               : \"${LINE_GROUP_END}\"" 
    DisplayTrace "    KEEP_LINE_GROUP_BEGIN        : \"${KEEP_LINE_GROUP_BEGIN}\"" 
    DisplayTrace "    KEEP_LINE_GROUP_END          : \"${KEEP_LINE_GROUP_END}\"" 
    DisplayTrace "    LINE_GROUP_REPLACEMENT_BEGIN : \"${LINE_GROUP_REPLACEMENT_BEGIN}\"" 
    DisplayTrace "    LINE_GROUP_REPLACEMENT_END   : \"${LINE_GROUP_REPLACEMENT_END}\"" 


    FILE=`\eval echo "${FILE}"`
###    VALUE=`\eval echo "${VALUE}"`

    DisplayTrace ""
    DisplayTrace "    After evaluation of action arguments : "
    DisplayTrace "    FILE                         : \"${FILE}\"" 

    DisplayTrace ""
    DisplayTrace "    No uninstall treatment to do"

    return ${RETURN_CODE}
}


###########################################################################
# UpdateGroupLinesFile
###########################################################################

#
# Mise a jour d'un groupe de lignes dans un fichier texte
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
#
# Type de section concernee : "update_group_lines_file"
#
# 
# 

UpdateGroupLinesFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> UpdateGroupLinesFile"

    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 

        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"

        # Controle de compatibilite de la section avec la commande
        
        case "${SECTION_TYPE}" in
        
        "update_group_lines_file")
        
            case "${CONTEXT}" in 
            "inst")
                EvaluateConditionsINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
            
                    UpdateGroupLinesFileINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            "uninst")
                EvaluateConditionsUNINST $@
                RETURN_CODE=$?

                if [ ${RETURN_CODE} -eq 0 ] 
                then
                    UpdateGroupLinesFileUNINST $@
                    RETURN_CODE=$?
                else
                    # Une condition d'evaluation differente de 0 ne fait pas executer 
                    # l'action, mais ce n'est pas une erreur d'execution du script
                    # d'installation
                    RETURN_CODE=0
                fi    
                ;;  

            *)
                DisplayError "Unknown context \"${CONTEXT}\" " 
                RETURN_CODE=2
                ;;  
            esac
            ;;
            
        "no_action")
            # Section a ne pas traiter
            
            DisplayTrace "    Section not processed."
            DisplayTrace "    No action executed."
            ;;
            
        *)    
            DisplayError "Incompatible section type \"${SECTION_TYPE}\" with action"
            RETURN_CODE=1
            ;;
            
        esac    
        
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# DeleteTemporaryFiles
###########################################################################

#
# Suppression des fichiers temporaires 
#
# 
# 
# A faire : ou mettre les fichiers de trace apres desinstallation ?????
# Dans /tmp ???
# 
# 

DeleteTemporaryFiles ()
{

    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> DeleteTemporaryFiles"
    # DisplayTrace "FILE_ACTIONS_UNINSTALL : \"${FILE_ACTIONS_UNINSTALL}\" "


    # Fichiers du repertoire temp
    ${RM} -R "${DIR_TEMP}"

    if [ "${CONTEXT}" = "uninst" ]
    then
        
        # Fichiers du repertoire install
        # Ces deux fichiers sont crees par les installer et desinstaller de 
        # InstallShield
        # A faire : modifier car un des noms est deja en parametre d'appel 
        ${RM} "${LOCATION_PATH}/install/is_variables"
        ${RM} "${LOCATION_PATH}/install/is_variables_uninstall"

        # Fichiers des actions de desinstallation de chaque feature
        # (Ils ont tous ete systematiquement crees)
        for NAME in ${LIST_FEATURES}
        do
            FILE_UNINSTALL=`GetFieldSublist ${FILE_ACTIONS_UNINSTALL} '|' ':' ${NAME} 2`
            ${RM} "${FILE_UNINSTALL}"
        done
    fi

    return ${RETURN_CODE}
}


###########################################################################
# DeleteTraceFiles
###########################################################################

#
# Suppression des fichiers de trace 
#
# 
# Les fichiers de traces relatifs au traitement en cours (installation ou 
# desinstallation) ont leur contenu vide car ils sont en cours d'utilisation par
# InstallShield.
# 
# On supprime les fichiers resultant d'une installation/desinstallation precedente
# dans le cas ou ils n'avaient pas ete supprimes.
# 
# 

DeleteTraceFiles ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> DeleteTraceFiles"

     
    case "${CONTEXT}" in
    "inst")
        # ${RM} ${LOCATION_PATH}/install/trace_uninstall >/dev/null 2>&1
        # ${RM} ${LOCATION_PATH}/install/trace_error_uninstall >/dev/null 2>&1
        
        ${RM} "${TEMPORARY_PATH}/trace_uninstall" >/dev/null 2>&1
        ${RM} "${TEMPORARY_PATH}/trace_error_uninstall" >/dev/null 2>&1
        ;;

    "uninst")
        ${RM} "${LOCATION_PATH}/install/trace_install" >/dev/null 2>&1
        ${RM} "${LOCATION_PATH}/install/trace_error_install" >/dev/null 2>&1
        ;;
        
    *)
        DisplayError "Unknown context \"${CONTEXT}\" "
        RETURN_CODE=2
        ;;  
        
    esac
    
    return ${RETURN_CODE}
}


###########################################################################
# ExecuteActions
###########################################################################

#
# Execution des actions selon le mode d'utilisation 
# (installation ou desinstallation)
#
#
# 
# 
# 

ExecuteActions ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> ExecuteActions"
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        case "${CONTEXT}" in
        "inst")
            ExecuteActionsInstallation
            ;;

        "uninst")
            ExecuteActionsUninstallation
            ;;

        *)
            DisplayError "Unknown context \"${CONTEXT}\" "
            RETURN_CODE=1
            ;;

        esac
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# ExecuteActionsInstallation
###########################################################################

#
# Execution des directives pour une installation
#
# 
# 
# - Execution des actions du fichier prologue d'installation
# - Execution des actions des fichiers des "features"
# - Execution des actions du fichier epilogue d'installation
# 

ExecuteActionsInstallation ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> ExecuteActionsInstallation"


    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace ""
        DisplayTrace "    Executing install prolog file"
        ExecuteActionsFile "${FILE_ACTIONS_PROLOG_INSTALL}"
        RETURN_CODE=$?
    fi   

    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace "    Executing install file(s)"
        for FILE_ACTIONS in ${LIST_FILES_ACTIONS_INSTALL}
        do
            ExecuteActionsFile "${FILE_ACTIONS}"
            RETURN_CODE=$?
            
            if [ ${RETURN_CODE} -ne 0 ]
            then
                DisplayTrace "One action in \"${FILE_ACTIONS}\" returned an error"
                DisplayTrace "Execution of install file(s) aborted"
                
                DisplayError "One action in \"${FILE_ACTIONS}\" returned an error"
                DisplayError "Execution of install file(s) aborted" 
                
                break
            fi
        done    
    fi   
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace ""
        DisplayTrace "    Executing install epilog file"
        ExecuteActionsFile "${FILE_ACTIONS_EPILOG_INSTALL}"
        RETURN_CODE=$?
    fi   

    DisplayScreen "\n\n"

    return ${RETURN_CODE}
}


###########################################################################
# ExecuteActionsUninstallation
###########################################################################

#
# Execution des directives pour une desinstallation
#
# 
# 
# - Execution des actions du fichier prologue de desinstallation
# - Execution des actions des fichiers des "features"
# - Execution des actions du fichier epilogue de desinstallation
# 
# 

ExecuteActionsUninstallation ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> ExecuteActionsUninstallation"


    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace "    Executing uninstall prolog file"
        ExecuteActionsFile "${FILE_ACTIONS_PROLOG_UNINSTALL}"
        RETURN_CODE=$?
    fi   

        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace "    Executing uninstall file(s)"
        for FILE_ACTIONS in ${LIST_FILES_ACTIONS_UNINSTALL}
        do
            ExecuteActionsFile "${FILE_ACTIONS}"
            RETURN_CODE=$?
            
            if [ ${RETURN_CODE} -ne 0 ]
            then
                DisplayTrace "One action in \"${FILE_ACTIONS}\" returned an error"
                DisplayTrace "Execution of install file(s) aborted"
                
                DisplayError "One action in \"${FILE_ACTIONS}\" returned an error"
                DisplayError "Execution of install file(s) aborted"
                
                break
            fi
        done    
    fi   
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        DisplayTrace "    Executing uninstall epilog file"
        ExecuteActionsFile "${FILE_ACTIONS_EPILOG_UNINSTALL}"
        RETURN_CODE=$?
    fi   

    DisplayScreen "\n\n"

    return ${RETURN_CODE}
}


###########################################################################
# ExecuteActionsFile
###########################################################################

#
# Execution des directives contenues dans un fichier d'actions
#
# 
# Argument 1 : fichier des actions a executer
# 
# 
# Attention : les noms des commandes ne doivent pas debuter par une meme
# portion de texte sinon une commande risque de ne jamais etre reconnue
# sauf si l'ordre est correct dans les entrees dans le "case".
# 

ExecuteActionsFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ExecuteActionsFile"


    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        
        CheckReadableFile "${1}"
        RETURN_CODE=$?
    fi
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        FILE_ACTIONS=$1
        
        DisplayTrace ""
        DisplayTrace "    Executing actions from file :"
        DisplayTrace "    FILE_ACTIONS : \"${FILE_ACTIONS}\" "


        ${CAT} "${FILE_ACTIONS}" | while read LINE
        do
            DisplayScreen "."
        
            # DisplayTrace "LINE : \"${LINE}\" "
            DisplayTrace ""

            case ${LINE} in

            "ADD_INET_SERVICE"*)
            
                DisplayTrace "ADD_INET_SERVICE action ================================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" "
                    RETURN_CODE=1
                    break
                else
                    AddInetService "$2"
                    RETURN_CODE=$?
                fi             
                ;;


            "ADD_LINE_FILE"*)
            
                DisplayTrace "ADD_LINE_FILE action ====================================================="

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" " 
                    RETURN_CODE=1
                    break
                else
                    AddLineFile "$2"
                    RETURN_CODE=$?
                fi 
                ;;


            "ADD_SERVICES"*)

                DisplayTrace "ADD_SERVICES action ====================================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" "
                    RETURN_CODE=1
                    break
                else
                    AddServices "$2"
                    RETURN_CODE=$?
                fi             
                ;;


            "ADD_STARTUP"*)

                DisplayTrace "ADD_STARTUP action =====================================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" "
                    RETURN_CODE=1
                    break
                else
                    AddStartup "$2"
                    RETURN_CODE=$?
                fi             
                ;;


            "EXECUTE_COMMAND_CONDITION"*)

                DisplayTrace "EXECUTE_COMMAND_CONDITION action =======================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" "
                    RETURN_CODE=1
                    break
                else
                    ExecuteCommandCondition "$2"
                    RETURN_CODE=$?
                fi                
                ;;


            "EXECUTE_COMMAND"*)

                DisplayTrace "EXECUTE_COMMAND action =================================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" "
                    RETURN_CODE=1
                    break
                else
                    ExecuteCommand "$2"
                    RETURN_CODE=$?
                fi                
                ;;


            "UPDATE_ENVIRONMENT"*)

                DisplayTrace "UPDATE_ENVIRONMENT action ==============================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" "
                    RETURN_CODE=1
                    break
                else
                    UpdateEnvironment "$2"
                    RETURN_CODE=$?
                fi 
                ;;


            "INSTALL_FILES"*)

                DisplayTrace "INSTALL_FILES action ===================================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" "
                    RETURN_CODE=1
                    break
                else
                    InstallFiles "$2"
                    RETURN_CODE=$?
                fi 
                ;;

            "INSTALL_DIRECTORIES"*)

                DisplayTrace "INSTALL_DIRECTORIES action =============================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" "
                    RETURN_CODE=1
                    break
                else
                    InstallDirectories "$2"
                    RETURN_CODE=$?
                fi 

                ;;


            "UPDATE_FILE"*)

                DisplayTrace "UPDATE_FILE action ====================================================="

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action \"$1\" "
                    RETURN_CODE=1
                    break
                else
                    UpdateFile "$2"
                    RETURN_CODE=$?
                fi 
                ;;


            "UPDATE_GROUP_LINES_FILE"*)

                DisplayTrace "UPDATE_GROUP_LINES_FILE action ========================================="

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    DisplayError "Missing argument for action command: \"${LINE}\" "
                    RETURN_CODE=1
                    break
                else
                    UpdateGroupLinesFile "$2"
                    RETURN_CODE=$?
                fi 

                ;;


            "")
                # echo "  Empty action \"${LINE}\" " 1>&2
                ;;   


            "#"*)
                # Commentaire
                
                ;;   


            *)
                DisplayTrace ""
                DisplayTrace "  Unknown action \"${LINE}\" " 
                DisplayTrace ""

                DisplayError "  Unknown action \"${LINE}\" "
                ;;   

            esac


           if [ ${RETURN_CODE} -ne 0 ]
           then   
                # Attention ! 
                # Le variable RETURN_CODE n'est pas modifiee en sortie car le 
                # "while" est execute dans un shell fils
                # Sa valeur en erreur n'est donc pas remontee a l'appelant
                 
                DisplayTrace ""
                DisplayTrace "    Error detected following action execution of \"${LINE}\"" 
                DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
                
                DisplayError ""
                DisplayError "Error detected following action execution of \"${LINE}\" " 
                DisplayError "RETURN_CODE : ${RETURN_CODE}"
                
                break
           fi       

        done
    fi

    return ${RETURN_CODE}
}


###########################################################################
# main
###########################################################################

# 
# Programme principal (point d'entree du script)
#
# A faire : 
# Gestion fichier journal (log)
# Actuellement les fichiers journaux sont geres par InstallShield 
# dans le "bean" executant le script d'installation
#
# Argument 1 : contexte d'utilisation ("inst" ou "uninst")
# Argument 2 : chemin complet vers le fichier contenant les variables InstallShield
# Argument 3 : sous-contexte d'utilisation ("FIXPACK") (argument facultatif)
#
#

RETURN_CODE=0




# Controles arguments d'appel

if [ $# -gt 5 ]
then
    echo "Incorrect number or arguments" 1>&2
    exit 1
fi

if [ $# -lt 4 ]
then
    echo "Incorrect number or arguments" 1>&2
    exit 1
fi


if [ $# -eq 5 ]
then
    CONTEXT="$1"
    FILE_IS_VARIABLES="$2"
    FILE_TRACE="$3"
    FILE_TRACE_WARNING="$4"
    SUB_CONTEXT="$5"
fi


if [ $# -eq 4 ]
then
    CONTEXT="$1"
    FILE_IS_VARIABLES="$2"
    FILE_TRACE="$3"
    FILE_TRACE_WARNING="$4"
    SUB_CONTEXT=""
fi


DisplayTrace "HPU trace file"
DisplayTrace ""



DisplayTrace "Number of arguments : $#"
DisplayTrace "Arguments           : $*"
DisplayTrace "Current directory   : `pwd`"


# Fichier de trace ne doit pas etre remise a zero car contient deja la trace de
# l'installer setup
DisplayTrace "    FILE_TRACE : \"${FILE_TRACE}\" "



DisplayTrace "SubContext          : ${SUB_CONTEXT}"


# Controle sur le contexte d'utilisation

CONTEXT=`echo "${CONTEXT}" | \tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz'`
if [ ${RETURN_CODE} -eq 0 ]
then
    case "${CONTEXT}" in
    "inst")
        DisplayTrace "Context             : Installation"
        ;;

    "uninst")
        DisplayTrace "Context             : Uninstallation"
        ;;

    *)
        echo "Unknown context \"${CONTEXT}\" " 1>&2
        RETURN_CODE=1
        ;;

    esac
fi

FILE_COMMON_FUNCTIONS="installer_common.sh"
if [ -f "${FILE_COMMON_FUNCTIONS}" ]
then
    . ./"${FILE_COMMON_FUNCTIONS}"
else
    echo "File \"${FILE_COMMON_FUNCTIONS}\" not found" 1>&2
    exit 1
fi


# Controle sur le fichier des variables InstallShield

if [ ${RETURN_CODE} -eq 0 ]
then
    CheckReadableFile "${FILE_IS_VARIABLES}"
    RETURN_CODE=$?
fi


# Initialisation des variables shell incluses dans le fichier provenant 
# d'InstallShield
# Ajouter controle si variables existantes apres...
if [ ${RETURN_CODE} -eq 0 ]
then
    . "${FILE_IS_VARIABLES}"

    DisplayTrace ""
    DisplayTrace "    Data transmitted from InstallShield : " 
    DisplayTrace "    (file \"${FILE_IS_VARIABLES}\")"
    DisplayTrace ""
    DisplayTrace "    FEATURES         : \"${FEATURES}\" "
    DisplayTrace "    LOCATION_PATH    : \"${LOCATION_PATH}\" "
    DisplayTrace "    LOG              : \"${LOG}\" "
    DisplayTrace "    PLATFORM         : \"${PLATFORM}\" "
    DisplayTrace "    ARCHITECTURE     : \"${ARCHITECTURE}\" "
    DisplayTrace "    VERSION          : \"${VERSION}\" "
    
    DisplayTrace "    TEMPORARY_PATH   : \"${TEMPORARY_PATH}\" "


    # Infos recapitulatives    
    DisplaySummary "HPU install summary file"
    DisplaySummary ""
    DisplaySummary "HPU installation directory    : ${LOCATION_PATH}"
    DisplaySummary ""
fi


if [ ${RETURN_CODE} -eq 0 ]
then
    InitDefaultVariables
    RETURN_CODE=$?
fi
    

if [ ${RETURN_CODE} -eq 0 ]
then
    case "${CONTEXT}" in
    "inst")
        \cd ${LOCATION_PATH}/install
        ;;

    "uninst")
        \cd ${LOCATION_PATH}/uninstall
        ;;
    esac
    RETURN_CODE=$?
fi



### DEBUG ###
# Suppression fichiers temporaires resultant de l'execution precedente
# \rm -f ${LOCATION_PATH}/temp/tmp_section_* 
# \rm -f ${LOCATION_PATH}/temp/tmp_install_* 
# \rm -Rf ${LOCATION_PATH}/temp/Files
### FIN DEBUG ###


if [ ${RETURN_CODE} -eq 0 ]
then
    InitVariables
    RETURN_CODE=$?
fi
    

if [ ${RETURN_CODE} -eq 0 ]
then
    CheckEnvironment
    RETURN_CODE=$?
fi   


if [ ${RETURN_CODE} -eq 0 ]
then
    PrepareActionsLists
    RETURN_CODE=$?
fi



if [ ${RETURN_CODE} -eq 0 ]
then
    ExtractArchives
    RETURN_CODE=$?
fi



if [ ${RETURN_CODE} -eq 0 ]
then
    DisplayTrace ""
    DisplayTrace "    Loading of additional module : ${SCRIPT_INIT}"
    DisplayTrace "    SCRIPT_INIT : ${SCRIPT_INIT}"
    
    . "${SCRIPT_INIT}"

    ### # Substitution minimale de certaines variables dans le cas ou l'utilisateur
    ### # a demande la recuperation d'une configuration existante
    ### if [ "${USE_EXISTING_SETTINGS}" = "1" ]
    ### then
    ###     hpuSubstituteEssentialSettings "${EXISTING_INSTALLATION_DIRECTORY}"
    ###     RETURN_CODE=$?
    ### fi 
    
    ###     if [ ${RETURN_CODE} -eq 0 ]
    ###     then
    ###         if [ "${CONTEXT}" = "inst" ]
    ###         then
    ###             if [ "${STANDALONE_INSTALLATION}" != "1" ]
    ###             then
    ###                 # Determination du repertoire d'installation de DB2 a partir de 
    ###                 # DEFAULT_HOME
    ### 
    ###                 hpuDetermineDB2InstallDirectory
    ###                 RETURN_CODE=$?
    ###             fi    
    ###         fi    
    ###     fi
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Initialisation variables globales pour les actions
        hpuInitActionGlobalVariables
    
        RETURN_CODE=$?
    fi    
fi



# Execution des actions selon le mode d'utilisation

if [ ${RETURN_CODE} -eq 0 ]
then
    ExecuteActions
    RETURN_CODE=$?
fi


 
# Suppression des fichiers temporaires (sections, ...)

DeleteTemporaryFiles

DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"


# Suppression des fichiers de trace

DeleteTraceFiles   

DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"


exit ${RETURN_CODE}

