#! /bin/sh

#
# @(#) Common functions
# 
# 
# List of functions: 
# grep '^[^( ]* ()' installer_common.sh
# 
# 
# 

### DEBUG ###
# set -x
set -u
### FIN DEBUG ###




### ###########################################################################
### # DisplayTrace
### ###########################################################################
### 
### # 
### # Affichage message de trace (sur sortie standard)
### # 
### # 
### # Argument 1 : texte du message
### # 
### # 
### 
### DisplayTrace ()
### {
###     # RETURN_CODE=0
###     
###     echo "$1"
###     
###     # return ${RETURN_CODE}
### }


###########################################################################
# DisplaySummary
###########################################################################

# 
# Affichage message dans fichier compte-rendu
# 
# 
# Argument 1 : texte du message
# 
# 
# Parametrage du nom du fichier !
# 

DisplaySummary ()
{
    # RETURN_CODE=0
    
    echo "$1" >> "${LOCATION_PATH}/install/install_summary"
    
    # return ${RETURN_CODE}
}


### ###########################################################################
### # DisplayError
### ###########################################################################
### 
### # 
### # Affichage message sur stderr
### # 
### # 
### # Argument 1 : texte du message
### # 
### # 
### # 
### 
### DisplayError ()
### {
###     # RETURN_CODE=0
###     
###     echo "$1" 1>&2
###     
###     # return ${RETURN_CODE}
### }


### ###########################################################################
### # DisplayTraceAndError
### ###########################################################################
### 
### # 
### # Affichage de trace (sur sortie standard) et d'erreur (sur stderr)
### # 
### # 
### # Argument 1 : texte du message
### # 
### # 
### # 
### 
### DisplayTraceAndError ()
### {
###     # RETURN_CODE=0
###     
###     echo "$1"
###     echo "$1" 1>&2
###     
###     # return ${RETURN_CODE}
### }


###########################################################################
# RemoveLeadingZeroes
###########################################################################

# 
# Suppression des zeros non significatifs d'une chaine numerique
# 
# 
# Si la chaine est vide, le resultat est 0
# Si la chaine ne contient que des zeros, le resultat est 0
# Il n'y a pas de controle de validite de la chaine donnee en argument (presence 
# ou non de lettres, ...
# 
# 
# Argument 1 : chaine a traiter
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
#
# 
# 
# 

RemoveLeadingZeroes ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> RemoveLeadingZeroes"


    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    

        if [ -z "$1" ]
        then
            echo "0"
            RETURN_CODE=0
        else
            # Suppression zeros non significatifs
            TEMP_STRING=`echo "$1" | ${SED} -e 's/^0*//'`
            if [ "${TEMP_STRING}" = "" ]
            then
                echo "0"
                RETURN_CODE=0
            else
                echo "${TEMP_STRING}"
                RETURN_CODE=0
            fi
        fi
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# ConvertToUpperCase
###########################################################################

# 
# Conversion of string to uppercase characters
# 
# 
# Argument 1 : text to convert
# 
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# 
# 

ConvertToUpperCase ()
{
    RETURN_CODE=0
    
    # DisplayTrace ""
    # DisplayTrace ">>> ConvertToUpperCase"
    
    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        # echo "$1" | \tr '[a-z]' '[A-Z]'
        echo "$1" | \tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# ConvertToLowerCase
###########################################################################

# 
# Conversion of string to lowercase characters
# 
# 
# Argument 1 : text to convert
# 
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# 
# 

ConvertToLowerCase ()
{
    RETURN_CODE=0
    
    # DisplayTrace ""
    # DisplayTrace ">>> ConvertToLowerCase"
    
    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        # echo "$1" | \tr '[a-z]' '[A-Z]'
        echo "$1" | \tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz'
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckSettingsVariable >>> OK
###########################################################################

# 
# Controle si une variable existe deja dans le fichier de parametres 
# d'installation
# 
# Nom du fichier de parametres d'installation : contenu dans la variable 
# globale FILE_INSTALL_SETTINGS.
# 
# Argument 1 : nom de variable a rechercher
#
#
# 
# Une variable est recherchee dans le fichier sous la forme
# NOM_VARIABLE=
#
# Existence du fichier de parametres d'installation non effectuee.
#
# Code de retour : 
# 
# 0 : variable trouvee
# 1 : variable non trouvee
# 
# 
# 

CheckSettingsVariable ()
{
    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> CheckSettingsVariable"
    

    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        IDENT_VARIABLE="$1" 

        DisplayTrace "    IDENT_VARIABLE : \"${IDENT_VARIABLE}\"" 

        ${GREP} '^'"${IDENT_VARIABLE}=" "${FILE_INSTALL_SETTINGS}" >/dev/null 2>&1
        if [ $? -ne 0 ]
        then
            DisplayTrace "    \"${IDENT_VARIABLE}\" not found in installation settings"
            RETURN_CODE=1
        else
            DisplayTrace "    \"${IDENT_VARIABLE}\" already present in installation settings"
            VALUE_VARIABLE=`${GREP} '^'"${IDENT_VARIABLE}=" "${FILE_INSTALL_SETTINGS}" | ${CUT} -d'=' -f2- `
            DisplayTrace "    with value \"${VALUE_VARIABLE}\" "
            RETURN_CODE=0
        fi   
    fi

    return ${RETURN_CODE}
}


###########################################################################
# AddSettingsVariable >>> OK
###########################################################################

# 
# Ajoute une variable dans le fichier de parametres d'installation 
#
# Nom du fichier de parametres d'installation : contenu dans la variable 
# globale FILE_INSTALL_SETTINGS.
# Attention : la variable ne doit pas etre deja presente (pas de controle)
# 
# Argument 1 : nom de la variable a ajouter
# Argument 2 : valeur de la variable a ajouter
#
#
# 
# Une variable est ecrite dans le fichier sous la forme
# NOM_VARIABLE=VALEUR_VARIABLE
#
# Existence du fichier de parametres d'installation non effectuee.
#
#
# Code de retour : 
# 
# 0 : succes
# 1 : erreur
# 
# 

AddSettingsVariable ()
{
    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> AddSettingsVariable"
    

    if [ $# -ne 2 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        IDENT_VARIABLE="$1" 
        VALUE_VARIABLE="$2" 

        DisplayTrace "    IDENT_VARIABLE : \"${IDENT_VARIABLE}\"" 
        DisplayTrace "    VALUE_VARIABLE : \"${VALUE_VARIABLE}\"" 

        echo "${IDENT_VARIABLE}=${VALUE_VARIABLE}" >> "${FILE_INSTALL_SETTINGS}"
        if [ $? -ne 0 ]
        then
            # Message sur erreur a faire
            DisplayTrace "Error while writing \"${IDENT_VARIABLE}\" in installation settings"
            RETURN_CODE=1
        else
            DisplayTrace "    \"${IDENT_VARIABLE}\" written in installation settings"
            DisplayTrace "    with value \"${VALUE_VARIABLE}\" "
            RETURN_CODE=0
        fi   
    fi

    return ${RETURN_CODE}
}


###########################################################################
# GetSettingsVariable >>> OK
###########################################################################

# 
# Obtention de la valeur d'une variable contenue dans le fichier de parametres 
# d'installation
# 
# Nom du fichier de parametres d'installation : contenu dans la variable 
# globale FILE_INSTALL_SETTINGS.
# 
# 
# Argument 1 : nom de variable a rechercher
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# Aucun affichage parasite sur stdout ne doit etre fait dans cette fonction.
#
# 
# Une variable est recherchee dans le fichier sous la forme
# NOM_VARIABLE=
#
# Existence du fichier de parametres d'installation non effectuee.
#
# Code de retour : 
# 
# 0 : variable trouvee
# 1 : erreur sur les arguments d'appel
# 2 : variable non trouvee
# 
# 
# 

GetSettingsVariable ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> GetSettingsVariable"
    

    if [ $# -ne 1 ]
    then
        echo '\07Error'
        RETURN_CODE=1
    else    
        IDENT_VARIABLE="$1" 


        LINE=`${GREP} '^'"${IDENT_VARIABLE}=" "${FILE_INSTALL_SETTINGS}" `
        if [ $? -eq 0 ]
        then
            echo "${LINE}" | ${CUT}  -d'=' -f2-
            RETURN_CODE=0
        else
            echo '\07Error'
            RETURN_CODE=2
        fi   
    fi

    return ${RETURN_CODE}
}





###########################################################################
# SearchFileInDirectories >>> OK
###########################################################################

# 
# Recherche d'un fichier dans un ensemble de repertoires
# 
# 
# Argument 1 : nom du fichier a rechercher
# Argument 2 : liste des repertoires dans lesquels le fichier doit etre recherche
#              Les repertoires sont separes par des ":"  
#
#
# La recherche est non recursive.
# 
# Un repertoire de recherche inexistant ne provoque pas d'erreur.
# 
#
# Code de retour : 
# 
# 0 : le fichier a ete trouve
# 1 : erreur sur le nombre d'arguments
# 2 : le fichier n'a pas ete trouve
# 
# DIR_FOUND : repertoire ou a ete trouve le fichier
#                   ou chaine vide sinon      
# 
# >>> Risque de problemes si les repertoires contiennent des espaces
#     (Modifier en transformant la liste en une colonne et traitement avec "while read..."
#      au lieu de "for" (attention shell fils)
# 

SearchFileInDirectories ()
{
    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> SearchFileInDirectories"
    
    DIR_FOUND=""

    if [ $# -ne 2 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    

        FILE_SEARCH="$1" 
        LIST_SEARCH_DIRECTORIES=`echo "$2" | ${TR} ':' ' '`

        DisplayTrace "    FILE_SEARCH             : \"${FILE_SEARCH}\"" 
        DisplayTrace "    LIST_SEARCH_DIRECTORIES : \"${LIST_SEARCH_DIRECTORIES}\"" 

        FILE_FOUND=0 
        for DIR_SEARCH in ${LIST_SEARCH_DIRECTORIES}
        do
            DisplayTrace ""
            DisplayTrace "    DIR_SEARCH              : \"${DIR_SEARCH}\"" 

            if [ ! -d "${DIR_SEARCH}" ]
            then
                # Erreur non fatale
                DisplayTrace "    Search directory \"${DIR_SEARCH}\" not found" 
                continue
            fi    

            if [ -f "${DIR_SEARCH}/${FILE_SEARCH}" ]
            then
                DisplayTrace "    File \"${FILE_SEARCH}\" found in directory \"${DIR_SEARCH}\""
                FILE_FOUND=1
                DIR_FOUND="${DIR_SEARCH}"
            else
                DisplayTrace "    File \"${FILE_SEARCH}\" not found in directory \"${DIR_SEARCH}\""
            fi

        done

        if [ "${FILE_FOUND}" -eq 0 ]
        then
            DisplayTrace ""
            DisplayTrace "    File \"${FILE_SEARCH}\" not found in any directory"
            RETURN_CODE=2
        fi
        
        DisplayTrace ""
    fi    
    
    DisplayTrace "    DIR_FOUND : \"${DIR_FOUND}\"" 
    return ${RETURN_CODE}
}


###########################################################################
# MaskRegularExpressionCharacters >>> OK 
###########################################################################

# 
# Masquage des caracteres speciaux (pour des expressions regulieres) d'une chaine
# 
# 
# Argument 1 : chaine de caracteres
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# 
# 
# 
# 
# 

MaskRegularExpressionCharacters ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> MaskRegularExpressionCharacters"
    
    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    

        echo "$@" | ${SED} 's/\([][*.\\\/?+|^$]\)/\\\1/g'
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# GetPIDFromName >>> OK
###########################################################################

# 
# Recherche du PID du processus en fonction du nom de la commande lancee
# 
# 
# Argument 1 : nom du processus (commande executee)
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# 
# Attention : actuellement, cette fonction n'affiche pas de messages d'erreur 
# sur stderr. Le code d'erreur est positionne, mais aucun message d'erreur.
# 
# 

GetPIDFromName ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> GetPIDFromName"
    

    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        if [ -z "$1" ]
        then
            DisplayError "Empty argument"
            echo '\07Error'
            RETURN_CODE=2
        else
            # Recherche des processus avec suppression du processus "grep" lui-meme
            NUMBER_PROCESSES=`${PS} -fe | ${GREP} $1 | ${GREP} -v grep | ${AWK} '{print $2}' | ${WC} -l`
            if [ ${NUMBER_PROCESSES} -eq 0 ]
            then
                DisplayTrace "No process \"$1\" found" 
                RETURN_CODE=3
            else
                if [ ${NUMBER_PROCESSES} -eq 1 ]
                then
                    PID=`${PS} -fe | ${GREP} $1 | ${GREP} -v grep | ${AWK} '{print $2}'`
                    echo ${PID}
                else
                    # Pas de message dans stderr
                    # echo "More than one process \"$1\" found : ${NUMBER_PROCESSES}" 1>&2
                    DisplayTrace "More than one process \"$1\" found : ${NUMBER_PROCESSES}" 
                    RETURN_CODE=3            
                fi    
            fi    
        fi
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# NumberElementsList >>> OK
###########################################################################

# 
# Determination du nombre d'elements d'une liste
# 
# 
# Argument 1 : liste
# Argument 2 : separateur
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# 
# Tous les cas ne sont pas geres
# 
# 
# 

NumberElementsList ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> NumberElementsList"
    

    if [ $# -ne 2 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        echo "$1" | \awk -F"$2" '{print NF}'
        RETURN_CODE=$?
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# GetFieldSublist >>> OK
###########################################################################

# 
# Recuperation d'un element d'une sous-liste
# 
# 
# Argument 1 : liste
# Argument 2 : separateur liste
# Argument 3 : separateur sous-liste
# Argument 4 : clef
# Argument 5 : indice element
# 
# Si la recherche est infructueuse le code retour reste a 0 !!
# 
# Exemple : 
# - Separateur de liste : "|" 
# - Separateur de sous-liste : ":" 
# La clef est l'element 1 d'une sous-liste
#  
# clef1:elt2:elt3|clef2:elt2:elt3:elt4|clef3...
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# Aucun affichage parasite sur stdout ne doit etre fait dans cette fonction.
# 
# Tous les cas ne sont pas geres
# Il n'y a pas de controle sur l'indice de l'element
# 
# 

GetFieldSublist ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> GetFieldSublist"
    

    if [ $# -ne 5 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        echo "${1}" | \tr "$2" '\n' | \grep "^${4}${3}" | \cut -d"$3" -f"$5"
        RETURN_CODE=$?
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# GetFieldList >>> OK
###########################################################################

# 
# Recuperation d'un element d'une liste
# 
# 
# Argument 1 : liste
# Argument 2 : separateur liste
# Argument 3 : indice element
# 
# Si la recherche est infructueuse le code retour reste a 0 !!
# 
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# Aucun affichage parasite sur stdout ne doit etre fait dans cette fonction.
#
# Tous les cas ne sont pas geres
# Il n'y a pas de controle sur l'indice de l'element
# 
# 
# 

GetFieldList ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> GetFieldList"
    

    if [ $# -ne 3 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        echo "${1}" | \cut -d"$2" -f"$3"
        RETURN_CODE=$?
    fi    
    
    return ${RETURN_CODE}
}



###########################################################################
# ReverseString >>> OK
###########################################################################

# 
# Inversion des mots d'une chaine
# 
# Argument 1 : chaine a inverser (elle doit etre entre guillemets lors de 
#              l'appel)
# 
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection
# Aucun affichage parasite sur stdout ne doit etre fait dans cette fonction.
#

ReverseString ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> ReverseString"
    

    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        RESULT_STRING=""    
        
        for WORD in ${1}
        do
            RESULT_STRING="${WORD} ${RESULT_STRING}"
        done
         
        echo "${RESULT_STRING}"
    fi    
    
    return ${RETURN_CODE}
}



###########################################################################
# ReverseFile >>> OK
###########################################################################

# 
# Creation d'un nouveau fichier a partir d'un autre en inversant l'ordre de ses lignes
# 
# Argument 1 : nom de fichier en entree
# Argument 2 : nom de fichier genere
# 
#

ReverseFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ReverseFile"


    if [ $# -ne 2 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    

        CheckReadableFile $1
        RETURN_CODE=$?

        if [ ${RETURN_CODE} -eq 0 ]
        then
    
            # Creation du fichier des actions de desinstallation : inversion
            # de l'ordre des lignes du fichier d'installation

            ${CAT} -n ${1} | ${SORT} -nr | ${CUT} -f2- > "$2"
            RETURN_CODE=$?
         
        fi 
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckEmptyDir >>> OK
###########################################################################

# 
# Indication si un repertoire est vide
# 
# Argument 1 : repertoire
# 
# Retourne 0 si repertoire vide 
# Retourne 3 si repertoire non vide (ou inexistant)
#
# On considere que si le repertoire n'existe pas, il est non vide (car cette 
# fonction est souvent utilisee en vue de la suppression du repertoire)
#
# Actuellement la variable (globale) REURN_CODE est utilisee : attention aux 
# effets de bord avec l'appelant.
# 
#

CheckEmptyDir ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckEmptyDir"



    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        if [ -d $1 ]
        then
            ${LS} -f $1 | ${AWK} 'NR >= 3 {exit 2}'
            if [ $? -eq 2 ]
            then
                RETURN_CODE=3
            else    
                RETURN_CODE=0
            fi
        else
            RETURN_CODE=3
        fi    
    fi
    
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckUser >>> OK
###########################################################################

# 
# Verification de l'existence d'un utilisateur
# 
# Argument 1 : nom d'utilisateur
# 
# Si l'argument est numerique, on considere que c'est un UID, sinon un nom 
# d'utilisateur
# 
# Utilisation de getent (1/1M) ? Commande inexistante sur HP-UX
#

CheckUser ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckUser"


    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        DisplayTrace "    USER : \"$1\"" 

        case "$1" in

        ''|*[!0-9]*) 
            # Valeur alphanumerique : nom d'utilisateur
            ${GREP} '^'"$1:" ${FILE_PASSWD} >/dev/null 2>&1
            if [ $? -eq 0 ]
            then
                RETURN_CODE=0
            else
                DisplayError "User name \"$1\" not found"
                RETURN_CODE=2
            fi
            ;;

        *)
            # Valeur numerique : numero d'utilisateur (UID)

            # Argument numerique : UID

            ${GREP} "^[^:]*:[^:]*:$1:"   ${FILE_PASSWD} >/dev/null 2>&1
            if [ $? -eq 0 ]
            then
                RETURN_CODE=0
            else
                DisplayError "User ID \"$1\" not found"
                RETURN_CODE=2
            fi
            ;;

        esac
    fi    

    return ${RETURN_CODE}
}



###########################################################################
# CheckGroup >>> OK
###########################################################################

# 
# Verification de l'existence d'un groupe
# 
# Argument 1 : nom de groupe
# 
# Si l'argument est numerique, on considere que c'est un GID, sinon un nom de
# groupe
#
# Utilisation de getent (1/1M) ? Mais commande inexistante sur HP-UX
#

CheckGroup ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckGroup"


    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        DisplayTrace "    GROUP : \"$1\"" 

        case "$1" in

        ''|*[!0-9]*) 
            # Valeur alphanumerique : nom de groupe
            ${GREP} '^'"$1:" ${FILE_GROUP} >/dev/null 2>&1
            if [ $? -eq 0 ]
            then
                RETURN_CODE=0
            else
                DisplayError "Group name \"$1\" not found"
                RETURN_CODE=2
            fi
            ;;

        *)
            # Valeur numerique : numero de groupe (GID)

            # Argument numerique : GID
            ${GREP} "^[^:]*:[^:]*:$1:"   ${FILE_GROUP} >/dev/null 2>&1
            if [ $? -eq 0 ]
            then
                RETURN_CODE=0
            else
                DisplayError "Group ID \"$1\" not found"
                RETURN_CODE=2
            fi
            ;;

        esac

    fi    

    return ${RETURN_CODE}
}



###########################################################################
# SetPermissionsFile >>> OK
###########################################################################

# 
# Modification des permissions d'un fichier
# 
# Argument 1 : permissions (notation octale)
# Argument 2 : nom de fichier
# 
# Controles supplementaires ? (existence fichier, notation octale ?)
#

SetPermissionsFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> SetPermissionsFile"

    
    if [ $# -ne 2 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
        ### PERMISSIONS="$1"
        ### FILE="$2"
    
    

        DisplayTrace "    PERMISSIONS : \"$1\"" 
        DisplayTrace "    FILE        : \"$2\"" 

    
    
        ${CHMOD} $1 $2
        if [ $? -ne 0 ]
        then
            DisplayError "Error setting permissions \"$1\" for file \"$2\""
            RETURN_CODE=1
        fi
    fi    

    return ${RETURN_CODE}
}


###########################################################################
# SetOwnerFile >>> OK
###########################################################################

# 
# Modification du proprietaire d'un fichier
# 
# Argument 1 : proprietaire
# Argument 2 : nom de fichier
# 
# Controles supplementaires ? (existence fichier, ?)
#

SetOwnerFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> SetOwnerFile"

    
    if [ $# -ne 2 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    


        DisplayTrace "    OWNER       : \"$1\"" 
        DisplayTrace "    FILE        : \"$2\"" 

    
        if CheckUser $1
        then    
        
            ${CHOWN} $1 "$2"
            if [ $? -ne 0 ]
            then
                DisplayError "Error setting owner \"$1\" for file \"$2\""
                RETURN_CODE=1
            fi
        fi
    fi    

    return ${RETURN_CODE}
}


###########################################################################
# SetGroupFile >>> OK 
###########################################################################

# 
# Modification du groupe d'un fichier
# 
# Argument 1 : groupe
# Argument 2 : nom de fichier
# 
# Controles supplementaires ? (existence fichier, ?)
#

SetGroupFile ()
{
    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> SetGroupFile"
    
    
    if [ $# -ne 2 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=1
    else    
    
        
        DisplayTrace "    GROUP       : \"$1\"" 
        DisplayTrace "    FILE        : \"$2\"" 
        
    
        if CheckGroup $1
        then    
    
            ${CHGRP} $1 "$2"
            if [ $? -ne 0 ]
            then
                DisplayError "Error setting group \"$1\" for file \"$2\"" 
                RETURN_CODE=1
            fi
        fi    
    fi    

    return ${RETURN_CODE}
}


###########################################################################
# EvaluateIdentifier >>> OK
###########################################################################

# 
# Recuperation de la valeur (evaluation) d'un identificateur de variable 
# 
# Argument 1 : nom variable
# 
# 
# Pas de trace sur stdout car redirection exploitee !
# 
#

EvaluateIdentifier ()
{
    RETURN_CODE=0
    
    
    # Pas de trace car redirection exploitee !
    # echo ""
    # DisplayTrace ">>> EvaluateIdentifier"
    

    if [ $# -ne 1 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else   
    
        # Seconde evaluation 
        VALUE=`\eval \echo '\$'"${1}"`

        
        # echo "VALUE After evaluation of action arguments : \"${VALUE}\"" 
        

        # Affichage en retour
        echo "${VALUE}"
        
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# GetINIValue >>> OK 
###########################################################################

# 
# Recuperation de la valeur d'une variable, dans une section d'un fichier .INI
# 
# Argument 1 : nom de fichier .INI
# Argument 2 : Nom de la variable
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection.
# 
# Normaliser la valeur retournee en cas d'erreur (chaine TRES specifique) 
# Si on choisit de retourner la chaine vide, pas de difference avec une valeur
# volontaire de "chaine vide".
#
#
# Code de retour : 
# 0 : aucune erreur (valeur trouvee qui peut etre vide)
# 1 : erreur sur les arguments d'appel
# 2 : variable non trouvee
#
#
#
#

GetINIValue ()
{
    RETURN_CODE=0
    
    # Pas de trace car redirection exploitee !
    # DisplayTrace ">>> GetINIValue"
    
    
    if [ $# -ne 2 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        FILE="$1"
        IDENTIFIER="$2"
    
        LINE=`grep '^'"${IDENTIFIER}=" "${FILE}"`
        if [ $? -eq 0 ]
        then
            echo "${LINE}" | cut -d'=' -f2-
            RETURN_CODE=0
        else
            echo '\07Error'
            RETURN_CODE=2
        fi
    fi
    
    return ${RETURN_CODE}
}


##########################################################################
# GetUserID
###########################################################################

# 
# Obtention du numero d'utilisateur courant (UID)
# 
# Aucun argument 
# 
# 
# 
#

GetUserID ()
{
    RETURN_CODE=0
    

    # DisplayTrace ""
    # DisplayTrace ">>> GetUserID"


    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        TEMP_USER_ID=`LANG=C ${ID} | ${CUT} -d'=' -f2 | ${CUT} -d'(' -f1`
        if [ $? -eq 0 ]
        then
            echo "${TEMP_USER_ID}"
        else
            echo '\07Error'
        fi
        
        # DisplayTrace "    TEMP_USER_ID : \"${TEMP_USER_ID}\"" 
    fi    

    return ${RETURN_CODE}
}


###########################################################################
# InitCommandsVariables
###########################################################################

# 
# Initialisation des variables correspondant commandes systeme
# 
# 
# Mettre a jour la fonction CheckEnvironment () si de nouvelles commandes sont 
# ajoutees.
#

InitCommandsVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitCommandsVariables"
       
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset AWK
        AWK=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "awk"` 
        RETURN_CODE=$?

        DisplayTrace "    AWK : ${AWK}"
    fi
        
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset CAT
        CAT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "cat"` 
        RETURN_CODE=$?

        DisplayTrace "    CAT : ${CAT}"
    fi
    

    if [ ${RETURN_CODE} -eq 0 ]
    then    
        unset CHGRP
        CHGRP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "chgrp"` 
        RETURN_CODE=$?

        DisplayTrace "    CHGRP : ${CHGRP}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset CHMOD
        CHMOD=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "chmod"` 
        RETURN_CODE=$?

        DisplayTrace "    CHMOD : ${CHMOD}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset CHOWN
        CHOWN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "chown"` 
        RETURN_CODE=$?

        DisplayTrace "    CHOWN : ${CHOWN}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset COMM
        COMM=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "comm"` 
        RETURN_CODE=$?

        DisplayTrace "    COMM : ${COMM}"
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset CP
        CP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "cp"` 
        RETURN_CODE=$?

        DisplayTrace "    CP : ${CP}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset CUT
        CUT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "cut"` 
        RETURN_CODE=$?

        DisplayTrace "    CUT : ${CUT}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset DF
        DF=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "df"` 
        RETURN_CODE=$?

        DisplayTrace "    DF : ${DF}"
    fi
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset EXPR
        EXPR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "expr"` 
        RETURN_CODE=$?

        DisplayTrace "    EXPR : ${EXPR}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset FIND
        FIND=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "find"` 
        RETURN_CODE=$?

        DisplayTrace "    FIND : ${FIND}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset GREP
        GREP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "grep"` 
        RETURN_CODE=$?

        DisplayTrace "    GREP : ${GREP}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset HEAD
        HEAD=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "head"` 
        RETURN_CODE=$?

        DisplayTrace "    HEAD : ${HEAD}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset ID
        ID=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "id"` 
        RETURN_CODE=$?

        DisplayTrace "    ID : ${ID}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset KILL
        KILL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "kill"` 
        RETURN_CODE=$?

        DisplayTrace "    KILL : ${KILL}"
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset LN
        LN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "ln"` 
        RETURN_CODE=$?

        DisplayTrace "    LN : ${LN}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset LS
        LS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "ls"` 
        RETURN_CODE=$?

        DisplayTrace "    LS : ${LS}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset MKDIR
        MKDIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "mkdir"` 
        RETURN_CODE=$?

        DisplayTrace "    MKDIR : ${MKDIR}"
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset MORE
        MORE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "more"` 
        RETURN_CODE=$?

        DisplayTrace "    MORE : ${MORE}"
    fi
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset MV
        MV=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "mv"` 
        RETURN_CODE=$?

        DisplayTrace "    MV : ${MV}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset PRINTF
        PRINTF=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "printf"` 
        RETURN_CODE=$?

        DisplayTrace "    PRINTF : ${PRINTF}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset PS
        PS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "ps"` 
        RETURN_CODE=$?

        DisplayTrace "    PS : ${PS}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset RM
        RM=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "rm"` 
        RETURN_CODE=$?

        DisplayTrace "    RM : ${RM}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset RMDIR
        RMDIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "rmdir"` 
        RETURN_CODE=$?

        DisplayTrace "    RMDIR : ${RMDIR}"
    fi

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset SED
        SED=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "sed"` 
        RETURN_CODE=$?

        DisplayTrace "    SED : ${SED}"
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset SORT
        SORT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "sort"` 
        RETURN_CODE=$?

        DisplayTrace "    SORT : ${SORT}"
    fi

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset TAR
        TAR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "tar"` 
        RETURN_CODE=$?

        DisplayTrace "    TAR : ${TAR}"
    fi

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset TR
        TR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "tr"` 
        RETURN_CODE=$?

        DisplayTrace "    TR : ${TR}"
    fi
    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset UNAME
        UNAME=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "uname"` 
        RETURN_CODE=$?

        DisplayTrace "    UNAME : ${UNAME}"
    fi
    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset WC
        WC=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "wc"` 
        RETURN_CODE=$?

        DisplayTrace "    WC : ${WC}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        unset XARGS
        XARGS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "xargs"` 
        RETURN_CODE=$?

        DisplayTrace "    XARGS : ${XARGS}"
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# InitSystemFilesVariables
###########################################################################

# 
# Initialisation des variables correspondant aux fichiers systeme
# 
# 
# Etiquette de la section : "system_files"
#
#
#

InitSystemFilesVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitSystemFilesVariables"
    
        
    FILE_SERVICES=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_services"` 
    if [ $? -ne 0 ]
    then
        FILE_SERVICES=""
    fi
    DisplayTrace "    FILE_SERVICES : ${FILE_SERVICES}"


    FILE_PROFILE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_profile"` 
    if [ $? -ne 0 ]
    then
        FILE_PROFILE=""
    fi
    DisplayTrace "    FILE_PROFILE : ${FILE_PROFILE}"


    FILE_STARTUP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_startup"` 
    if [ $? -ne 0 ]
    then
        FILE_STARTUP=""
    fi
    DisplayTrace "    FILE_STARTUP : ${FILE_STARTUP}"

    
    FILE_PASSWD=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_passwd"` 
    if [ $? -ne 0 ]
    then
        FILE_PASSWD=""
    fi
    DisplayTrace "    FILE_PASSWD : ${FILE_PASSWD}"


    FILE_GROUP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_group"` 
    if [ $? -ne 0 ]
    then
        FILE_GROUP=""
    fi
    DisplayTrace "    FILE_GROUP : ${FILE_GROUP}"


    FILE_INET_SERVICES=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_inet_services"` 
    if [ $? -ne 0 ]
    then
        FILE_INET_SERVICES=""
    fi
    DisplayTrace "    FILE_INET_SERVICES : ${FILE_INET_SERVICES}"

    return ${RETURN_CODE}
}


###########################################################################
# InitOtherSystemVariables
###########################################################################

#
# Initialisation des variables correspondant des informations systeme diverses
#
#
# Etiquette de la section : "system_other"
#
# 
#
#

InitOtherSystemVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitOtherSystemVariables"
    
        
    DAEMON_INETD=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_other" "daemon_inetd"` 
    if [ $? -ne 0 ]
    then
        DAEMON_INETD=""
    fi
    DisplayTrace "    DAEMON_INETD : ${DAEMON_INETD}"


    DAEMON_INETD_RECONFIG_SIGNAL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_other" "daemon_inetd_reconfig_signal"` 
    if [ $? -ne 0 ]
    then
        DAEMON_INETD_RECONFIG_SIGNAL=""
    fi
    DisplayTrace "    DAEMON_INETD_RECONFIG_SIGNAL : ${DAEMON_INETD_RECONFIG_SIGNAL}"


    # La variable DAEMON_INETD_RECONFIG_COMMAND sera evaluee mais juste avant
    # son utilisation afin que la variable utilisee existe et soit initialisee
    DAEMON_INETD_RECONFIG_COMMAND=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_other" "daemon_inetd_reconfig_command"` 
    if [ $? -ne 0 ]
    then
        DAEMON_INETD_RECONFIG_COMMAND=""
    fi
    DisplayTrace "    DAEMON_INETD_RECONFIG_COMMAND : ${DAEMON_INETD_RECONFIG_COMMAND}"
    DisplayTrace "    Evaluation of DAEMON_INETD_RECONFIG_COMMAND delayed until effective use"


    # La variable DAEMON_INETD_RECONFIG_PARAMETERS sera evaluee mais juste avant
    # son utilisation afin que la variable utilisee existe et soit initialisee
    DAEMON_INETD_RECONFIG_PARAMETERS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_other" "daemon_inetd_reconfig_parameters"` 
    if [ $? -ne 0 ]
    then
        DAEMON_INETD_RECONFIG_PARAMETERS=""
    fi
    DisplayTrace "    DAEMON_INETD_RECONFIG_PARAMETERS : ${DAEMON_INETD_RECONFIG_PARAMETERS}"
    DisplayTrace "    Evaluation of DAEMON_INETD_RECONFIG_PARAMETERS delayed until effective use"

    
    # La variable DF_OPTIONS contient les options necessaires pour un affichage
    # standard avec espace disponible en Ko en quatrieme colonne 
    DF_OPTIONS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_other" "df_options"` 

    DisplayTrace "    DF_OPTIONS : ${DF_OPTIONS}"

    return ${RETURN_CODE}
}


###########################################################################
# PrepareINIFile >>> OK
###########################################################################

# Preparation du fichier .INI (par decomposition) en vue de son exploitation
# 
# Argument 1 : nom de fichier
# Argument 2 : prefixe de nom fichier pour les fichiers generes par decomposition
#
#
# Les commandes (csplit, mv, ...) sont appeles avec leur vrai nom et pas via 
# une variable car on n'a pas encore lu le fichier de configuration qui 
# contient les variables.
#
# Attention : on suppose qu'il n'y a pas plus de 99 sections dans le fichier
# .INI.
#

PrepareINIFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> PrepareINIFile"

    
    if [ $# -ne 2 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=2
    else    

        # PREFIX_TMP est le prefixe des fichiers temporaires generes par csplit.
        # Ils sont ensuite renommes avec PREFIX et le nom de la section qu'il contiennent
        FILE="$1"
        FILENAME=`\basename $1`
        PREFIX_TEMP="tmp_${FILENAME}_"
        PREFIX="$2"
        
        if [ ! -f "${FILE}" ]
        then
            DisplayError "Error file not found \"${FILE}\"."
            RETURN_CODE=3      
        else
            
            \csplit -ks -f ${PREFIX_TEMP} ${FILE} '/^\[.*\]/' '{100}' 2>/dev/null
            RETURN_CODE=$?
            
            # Deplacement des fichiers generes dans le repertoire temporaire
            \mv ${PREFIX_TEMP}?? ${DIR_TEMP}
            RETURN_CODE=$?
            
              
            if [ ${RETURN_CODE} -eq 0 ]
            then
                for NOM in ${DIR_TEMP}/${PREFIX_TEMP}??
                do
                    # DisplayTrace ${NOM}


                    LINE=`\grep '^\[.*\]' ${NOM} | \tr -d ' '`
                    if [ ! -z "${LINE}" ]
                    then

                        # DisplayTrace "    LINE : \"${LINE}\"" 


                        SECTION=`\expr "${LINE}" ':' '\[\(.*\)\]'`

                        DisplayTrace "    Section found : \"${SECTION}\"" 

                        if [ -f "${DIR_TEMP}/${PREFIX}${SECTION}" ]
                        then
                            # Erreur fatale : une section est presente deux fois dans le fichier
                            DisplayError "Section file \"${PREFIX}${SECTION}\" already existing !" 
                            RETURN_CODE=1
                            break
                        else
                            \mv "${NOM}" "${DIR_TEMP}/${PREFIX}${SECTION}"
                        fi    
                    else
                        # Declarations de variables hors de toute section (en debut de fichier)
                        # Normalement il n'y en pas
                        \mv "${NOM}" "${DIR_TEMP}/${PREFIX}global"
                    fi    

                done
            else
                DisplayError "Error moving files to directory \"${DIR_TEMP}\" "
                RETURN_CODE=1
            fi
        fi
    
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# ReplaceVariable >>> OK
###########################################################################

# Remplacement de la valeur d'une variable presente sous la forme variable=valeur
# dans un fichier
# 
# La variable doit etre deja presente dans le fichier
# Detection de la ligne contenant la variable peut-etre pas assez stricte.
#
# Argument 1 : nom de fichier
# Argument 2 : nom de variable
# Argument 3 : nouvelle valeur de la variable
#
# Remplacement FILE_TEMP par _RV_FILE_TEMP 
# Suppression de la variable en sortie
#

ReplaceVariable ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ReplaceVariable"

    
    if [ $# -ne 3 ]
    then
        DisplayError "Missing or too many arguments ($#)"
        RETURN_CODE=2
    else    
  
        FILE="$1"
        VARIABLE="$2"
        VALUE="$3"

        # Fichier temporaire vide cree avec les memes droits et proprietaire que
        # l'original
        _RV_FILE_TEMP="${FILE}.REPLACE_$$"
        ${CP} -p "${FILE}" "${FILE_TEMP}"
        > "${FILE_TEMP}"


        ${AWK} -v VAR="${VARIABLE}" -v VAL="${VALUE}" -F'=' \
'{RE=VAR} \
$1 == VAR {print $1  "=" VAL} \
$1 != VAR {print $0} ' \
        "${FILE}" > "${_RV_FILE_TEMP}"

        ${MV} "${_RV_FILE_TEMP}" "${FILE}"
   
        unset _RV_FILE_TEMP
    fi
    
    
    return ${RETURN_CODE}
}


###########################################################################
# ReplaceConfigVariable >>> OK
###########################################################################

# Remplacement de la valeur dans un fichier de configuration au format INI
# 
# Pas de gestion des sections.
# La variable doit etre deja presente dans le fichier
# Detection de la ligne contenant la variable peut-etre pas assez stricte.
#
# Argument 1 : nom de fichier
# Argument 2 : nom de variable
# Argument 3 : nouvelle valeur de la variable
#
#

ReplaceConfigVariable ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ReplaceConfigVariable"

    
    if [ $# -ne 3 ]
    then
        DisplayError "Missing or too many arguments ($#)" 
        RETURN_CODE=2
    else    
    
  
    FILE="$1"
    VARIABLE="$2"
    VALUE="$3"

    # Fichier temporaire vide cree avec les memes droits et proprietaire que
    # l'original
    FILE_TEMP="${FILE}.$$"
    ${CP} -p "${FILE}" "${FILE_TEMP}"
    > "${FILE_TEMP}"
    

    ${AWK} -v VAR="${VARIABLE}" -v VAL="${VALUE}" -F'=' \
'{RE=VAR} \
$1 == VAR {print $1  "=" VAL} \
$1 != VAR {print $0} ' \
    "${FILE}" > "${FILE_TEMP}"

    ${MV} "${FILE_TEMP}" "${FILE}"
   
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckEmptyDirectory
###########################################################################

# Verification si repertoire vide
#
# Retourne :
# 0 si repertoire vide
# 1 sinon
#
#
#

CheckEmptyDirectory ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckEmptyDirectory"

    _NUMBER_LINES=`${FIND} "$1" | ${WC} -l`
    if [ $? -eq 0 ]
    then
        if [ "${_NUMBER_LINES}" -eq 1 ]
        then
            RETURN_CODE=0
        else    
            RETURN_CODE=1
        fi 
    else
        DisplayError "Error checking empty directory \"$1\"."
        RETURN_CODE=2
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckExistingDirectory >>> OK
###########################################################################

# Verification si repertoire existant
#
# Pas de verification si droits en ecriture !
#

CheckExistingDirectory ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckExistingDirectory"

    
    if [ ! -d "$1" ]
    then
        DisplayError "Error checking existing directory \"$1\"."
        RETURN_CODE=1
    
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckExecutableFile >>> OK
###########################################################################

# Verification si fichier executable
#

CheckExecutableFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckExecutableFile"

    
    if [ ! -x "$1" ]
    then
        DisplayError "Error checking executable file \"$1\"."
        RETURN_CODE=1
    
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckReadableFile >>> OK
###########################################################################

# Verification si fichier existant et lisible
#

CheckReadableFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckReadableFile"

    
    if [ ! -r "$1" ]
    then
        DisplayError "Error checking readable file \"$1\"." 
        RETURN_CODE=1
    
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckWriteableFile >>> OK
###########################################################################

#
# Verification si fichier modifiable
#

CheckWriteableFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckWriteableFile"

    
    if [ ! -w $1 ]
    then
        DisplayError "Error checking writeable file \"$1\"."
        RETURN_CODE=1
    
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# InitDefaultVariables >>> OK
###########################################################################

#
# Initialisation variables avec des valeur par defaut
#
# 

InitDefaultVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitDefaultVariables"


    # Initialisation au cas ou la lecture du .INI echoue
    # Il y a ensuite suppression de fichiers temporaires avec ${RM} qui doir
    # etre alors defini
    
    AWK=awk
    CAT=cat
    CHGRP=chgrp
    CHMOD=chmod
    CHOWN=chown
    COMM=comm
    CP=cp
    CUT=cut
    DF=df
    EXPR=expr
    FIND=find
    GREP=grep
    HEAD=head
    ID=id
    KILL=kill
    LN=ln
    LS=ls
    MKDIR=mkdir
    MORE=more
    MV=mv
    PRINTF=printf
    PS=ps
    RM=rm
    RMDIR=rmdir
    SED=sed
    SORT=sort
    TAR=tar
    TR=tr
    UNAME=uname
    WC=wc
    XARGS=xargs

    return ${RETURN_CODE}
}


###########################################################################
# CheckEnvironment
###########################################################################

#
# Verification des chemins d'acces aux commandes et fichiers
#
# 

CheckEnvironment ()

{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> CheckEnvironment"

        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Not always at the same location on Linux and PowerPC Linux and zlinux
        if [ "${PLATFORM}" != "linux" ] && [ "${PLATFORM}" != "plinuxle" ] && [ "${PLATFORM}" != "zlinux" ]
        then
            CheckExecutableFile ${AWK}
            RETURN_CODE=$?
        fi
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CAT}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CHGRP}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CHMOD}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CHOWN}
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${COMM}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CP}
        RETURN_CODE=$?
    fi

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Not always at the same location on Linux and PowerPC Linux and zlinux
        if [ "${PLATFORM}" != "linux" ] && [ "${PLATFORM}" != "plinuxle" ] && [ "${PLATFORM}" != "zlinux" ]
        then
            CheckExecutableFile ${CUT}
            RETURN_CODE=$?
        fi
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${DF}
        RETURN_CODE=$?
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${EXPR}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${FIND}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${GREP}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${HEAD}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${ID}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${KILL}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${LN}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${LS}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${MKDIR}
        RETURN_CODE=$?
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${MORE}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${MV}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${PRINTF}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${PS}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${RM}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${RMDIR}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${SED}
        RETURN_CODE=$?
    fi
        
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Not always at the same location on Linux and PowerPC Linux and zlinux
        if [ "${PLATFORM}" != "linux" ] && [ "${PLATFORM}" != "plinuxle" ] && [ "${PLATFORM}" != "zlinux" ]
        then
            CheckExecutableFile ${SORT}
            RETURN_CODE=$?
        fi
    fi
        

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${TAR}
        RETURN_CODE=$?
    fi
        
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${TR}
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${UNAME}
        RETURN_CODE=$?
    fi

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        if [ "${FILE_SERVICES}" != "" ]
        then
            CheckReadableFile ${FILE_SERVICES}
            RETURN_CODE=$?
        fi
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        ###         CheckWriteableFile ${FILE_PROFILE}
        CheckReadableFile ${FILE_PROFILE}
        RETURN_CODE=$?
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Pas de verification existence de ce fichier sur Linux, car il
        # n'existe pas forcement selon les distributions
        if [ "${PLATFORM}" != "linux" -a "${PLATFORM}" != "zlinux" -a "${PLATFORM}" != "plinux" -a "${PLATFORM}" != "plinuxle" -a "${PLATFORM}" != "ialinux" ]
        then
            ###         CheckWriteableFile ${FILE_STARTUP}
            CheckReadableFile ${FILE_STARTUP}
            RETURN_CODE=$?
        fi
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        
        # Le fichier de configuration inetd n'est pas le meme selon la distribution
        # Linux : son nom est reconfigure ulterieurement
        if [ "${PLATFORM}" != "linux" -a "${PLATFORM}" != "zlinux" -a "${PLATFORM}" != "plinux" -a "${PLATFORM}" != "plinuxle" -a "${PLATFORM}" != "ialinux" ]
        then
            if [ "${FILE_INET_SERVICES}" != "" ]
            then
                CheckReadableFile ${FILE_INET_SERVICES}
                RETURN_CODE=$?
            fi
        fi
    fi


    # Pas de verification de DAEMON_INETD_RECONFIG_COMMAND a ce niveau car la 
    # variable peut contenir des variables a evaluer dont les valeurs ne seront
    # connues qu'ulterieurement
    # if [ ${RETURN_CODE} -eq 0 ]
    # then
    #     CheckExecutableFile ${DAEMON_INETD_RECONFIG_COMMAND}
    #     RETURN_CODE=$?
    # fi
    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckReadableFile ${SCRIPT_INIT}
        RETURN_CODE=$?
    fi


    return ${RETURN_CODE}
}


###########################################################################
# DeleteTemporaryFiles
###########################################################################

#
# Suppression des fichiers temporaires 
#
# 
# 
# A faire : ou mettre les fichiers de trace apres desinstallation ?????
# Dans /tmp ???
# 
# 

DeleteTemporaryFiles ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> DeleteTemporaryFiles"
    # DisplayTrace "FILE_ACTIONS_UNINSTALL : \"${FILE_ACTIONS_UNINSTALL}\" "


    # Fichiers du repertoire temp
    ${RM} -R "${DIR_TEMP}"

    if [ "${CONTEXT}" = "uninst" ]
    then
        
        # Fichiers du repertoire install
        # Ces deux fichiers sont crees par les installer et desinstaller de 
        # InstallShield
        # A faire : modifier car un des noms est deja en parametre d'appel 
        ${RM} "${LOCATION_PATH}/install/is_variables"
        ${RM} "${LOCATION_PATH}/install/is_variables_uninstall"

        # Fichiers des actions de desinstallation de chaque feature
        # (Ils ont tous ete systematiquement crees)
        for NAME in ${LIST_FEATURES}
        do
            FILE_UNINSTALL=`GetFieldSublist "${FILE_ACTIONS_UNINSTALL}" '|' ':' "${NAME}" 2`
            ${RM} "${FILE_UNINSTALL}"
        done
    fi

    return ${RETURN_CODE}
}


###########################################################################
# DeleteTraceFiles
###########################################################################

#
# Suppression des fichiers de trace 
#
# 
# Les fichiers de traces relatifs au traitement en cours (installation ou 
# desinstallation) ont leur contenu vide car ils sont en cours d'utilisation par
# InstallShield.
# 
# On supprime les fichiers resultant d'une installation/desinstallation precedente
# dans le cas ou ils n'avaient pas ete supprimes.
# 
# 

DeleteTraceFiles ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> DeleteTraceFiles"

     
    case "${CONTEXT}" in
   "inst")
        ${RM} "${TEMPORARY_PATH}/trace_uninstall" >/dev/null 2>&1
        ${RM} "${TEMPORARY_PATH}/trace_error_uninstall" >/dev/null 2>&1
        ${RM} "${TEMPORARY_PATH}/trace_warning_uninstall" >/dev/null 2>&1
        ;;
        

    "uninst")
        ${RM} "${LOCATION_PATH}/install/trace_install" >/dev/null 2>&1
        ${RM} "${LOCATION_PATH}/install/trace_error_install" >/dev/null 2>&1
        ${RM} "${LOCATION_PATH}/install/trace_warning_install" >/dev/null 2>&1
        ;;
        
    *)
        DisplayError "Unknown context \"${CONTEXT}\" "
        RETURN_CODE=2
        ;;  
    esac
    
    return ${RETURN_CODE}
}




