#! /bin/sh

#
# @(#) Sample tests execution
#

# set -x
# set -u


if [ $# -ne 4 ]
then
    echo "Usage: sample.sh unload_directory user password backup_directory" 1>&2
    echo "       unload_directory: directory where all tables will be unloaded." 1>&2
    echo "       user            : instance user for database SAMPLE." 1>&2
    echo "       password        : password of the previous user." 1>&2
    echo "       backup_directory: directory location of a SAMPLE database backup file." 1>&2
    exit 1
fi



DB_DIR=$1
DB_USR=$2
DB_PWD=$3
DB_BACKUP=$4

echo ""

if [ ! -d ${DB_DIR} ] 
then
    mkdir ${DB_DIR}
fi
echo "Tables will be unloaded to ${DB_DIR}"

if [ ! -d ${DB_DIR}/lobs ] 
then
    mkdir ${DB_DIR}/lobs
fi
echo "LOBs will be unloaded to ${DB_DIR}/lobs"

echo ""


echo "Generation of local control files..."

cat > sample01.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE USERSPACE1
SELECT * FROM ${DB_USR}.EMPLOYEE;
PART(0)
OUTPUT("${DB_DIR}/sample01.out");
EOF

cat > sample02.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE
SELECT * FROM EMPLOYEE;
PART(0, 1)
OUTPUT("${DB_DIR}/sample02.out")
FORMAT DEL;
EOF

cat > sample03.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE
SELECT * FROM EMPLOYEE;
PART (ALL)
OUTPUT("${DB_DIR}/sample03.out")
FORMAT DEL;
EOF

cat > sample04.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE USERSPACE1
OUTPUT("${DB_DIR}/sample04")
LOB IN ("${DB_DIR}/lobs")
LOBFILE("sample04")
FORMAT DEL;
EOF

cat > sample05.ctl << EOF
GLOBAL CONNECT TO SAMPLE;

UNLOAD TABLESPACE USERSPACE1
OUTPUT("${DB_DIR}/sample05_del")
FORMAT DELIMITED ;

UNLOAD TABLESPACE USERSPACE1
OUTPUT("${DB_DIR}/sample05_ixf")
FORMAT IXF;

UNLOAD TABLESPACE USERSPACE1          
OUTPUT("${DB_DIR}/sample05_del_nolob")       
LOB IN ("${DB_DIR}/lobs")                    
LOBFILE("sample05_del")               
FORMAT DELIMITED ; 

UNLOAD TABLESPACE USERSPACE1          
OUTPUT("${DB_DIR}/sample05_ixf_nolob")       
LOB IN ("${DB_DIR}/lobs")                    
LOBFILE("sample05_ixf")               
FORMAT IXF;                           

EOF


cat > sample06.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE
OPTIONS DATE DATE_B
SELECT * FROM PROJECT;
OUTPUT("${DB_DIR}/sample06.out" REPLACE)
FORMAT DELIMITED;
EOF

cat > sample09.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE
SELECT * FROM PROJECT;
OUTPUT("${DB_DIR}/sample09.out")
FORMAT DEL;
EOF

cat > sample11.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE
SELECT FIRSTNME, SALARY
FROM EMPLOYEE
WHERE (SALARY BETWEEN 30000 AND 500000)
      OR 
      (JOB = 'MANAGER')
ORDER BY SALARY ASC;
OUTPUT("${DB_DIR}/sample11.out" REPLACE)
FORMAT DEL
;
EOF

cat > sample12.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE
SELECT FIRSTNME,LASTNAME,SALARY,
       BONUS,SALARY/BONUS
FROM EMPLOYEE
WHERE (SALARY/BONUS BETWEEN 50 AND 5000);
OUTPUT("${DB_DIR}/sample12.out" REPLACE)
FORMAT DEL
;
EOF

cat > sample13.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE
SELECT FIRSTNME, LASTNAME
  FROM EMPLOYEE
 WHERE CONCAT (FIRSTNME,LASTNAME) = 'DOLORESQUINTANA';
OUTPUT("${DB_DIR}/sample13.out" REPLACE)
FORMAT DEL
;
EOF

cat > sample14.ctl << EOF
GLOBAL CONNECT TO SAMPLE;
UNLOAD TABLESPACE
SELECT * from emp_photo;
OUTPUT("${DB_DIR}/sample14.out" REPLACE)
LOB IN ("${DB_DIR}/lobs" FOR PARTS(1))
LOBFILE("sample14")
;
EOF

cat > sample15.ctl << EOF
UNLOAD TABLESPACE SYSCATSPACE
OUTPUT("${DB_DIR}/sample15.out" REPLACE)
QUIESCE NO
LOB IN ("${DB_DIR}/lobs" FOR PARTS(0))
LOBFILE("sample15")
FORMAT DEL
;
EOF

cat > sample16.ctl << EOF
UNLOAD DATABASE SAMPLE
OUTPUT("${DB_DIR}/sample16" REPLACE)
LOB IN ("${DB_DIR}/lobs" FOR PARTS(0))
LOBFILE("sample16")
FORMAT DEL
;
EOF


cat > sample17.ctl << EOF       
GLOBAL CONNECT TO SAMPLE ;
UNLOAD TABLESPACE USERSPACE1           
USING BACKUP DATABASE SAMPLE FROM "${DB_BACKUP}" ;
SELECT * FROM ${DB_USR}.EMPLOYEE;            
OUTPUT ("${DB_DIR}/sample17.out" REPLACE)    
FORMAT DEL                            
;                                     
EOF


cat > sample18.ctl << EOF       
UNLOAD TABLESPACE SYSCATSPACE
QUIESCE NO
OUTPUT("${DB_DIR}/sample18s.out" REPLACE)
LOB IN ("${DB_DIR}/lobs")
LOBFILE("sample18s")
FORMAT IXF
;
UNLOAD TABLESPACE USERSPACE1
OUTPUT("${DB_DIR}/sample18u.out" REPLACE)
LOB IN ("${DB_DIR}/lobs")
LOBFILE("sample18u")
FORMAT IXF
;                                     
EOF


\rm -f sample.log 1>/dev/null 2>&1


echo "Executing sample01.ctl"
${TOPHPU}/bin/db2hpu -f sample01.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample02.ctl"
${TOPHPU}/bin/db2hpu -f sample02.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample03.ctl"
${TOPHPU}/bin/db2hpu -f sample03.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample04.ctl"
${TOPHPU}/bin/db2hpu -f sample04.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample05.ctl"
${TOPHPU}/bin/db2hpu -f sample05.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample06.ctl"
${TOPHPU}/bin/db2hpu -f sample06.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample09.ctl"
${TOPHPU}/bin/db2hpu -f sample09.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample11.ctl"
${TOPHPU}/bin/db2hpu -f sample11.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample12.ctl"
${TOPHPU}/bin/db2hpu -f sample12.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample13.ctl"
${TOPHPU}/bin/db2hpu -f sample13.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample14.ctl"
${TOPHPU}/bin/db2hpu -f sample14.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample15.ctl"
${TOPHPU}/bin/db2hpu -f sample15.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample16.ctl"
${TOPHPU}/bin/db2hpu -f sample16.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample17.ctl"
${TOPHPU}/bin/db2hpu -f sample17.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Executing sample18.ctl"
${TOPHPU}/bin/db2hpu -f sample18.ctl -i ${DB_USR} -u ${DB_USR} -p ${DB_PWD} -m sample.log
echo "Return code: $?"
echo ""

echo "Messages are in sample.log"


exit 0

